// -*- c -*-

#include "MyVector.h"

#include <math.h>

#define X 0
#define Y 1
#define Z 2
#define LOOP_XYZ for (int xyz = 0; xyz < 3; xyz++)

#ifndef TRUE
#define TRUE true
#define FALSE false
#endif

#ifdef _DEBUG
#include <stdlib.h>
#include <crtdbg.h>
#define new new(_NORMAL_BLOCK, __FILE__, __LINE__)
#endif

namespace MyVector{
    double GetScalarProduct(double vec0[3], double vec1[3]){
	return vec0[X]*vec1[X] + vec0[Y]*vec1[Y] + vec0[Z]*vec1[Z];
    }
    float GetScalarProduct(float vec0[3], float vec1[3]){
	return vec0[X]*vec1[X] + vec0[Y]*vec1[Y] + vec0[Z]*vec1[Z];
    }
    
    void   GetCrossedVector(double vec0[3], double vec1[3], double vecOut[3]){
	vecOut[X] = vec0[Y]*vec1[Z] - vec0[Z]*vec1[Y];
	vecOut[Y] = vec0[Z]*vec1[X] - vec0[X]*vec1[Z];
	vecOut[Z] = vec0[X]*vec1[Y] - vec0[Y]*vec1[X];
    }
    void   GetCrossedVector(float vec0[3], float vec1[3], float vecOut[3]){
	vecOut[X] = vec0[Y]*vec1[Z] - vec0[Z]*vec1[Y];
	vecOut[Y] = vec0[Z]*vec1[X] - vec0[X]*vec1[Z];
	vecOut[Z] = vec0[X]*vec1[Y] - vec0[Y]*vec1[X];
    }

    double  GetNormRelative(double vec0[3], double vec1[3]){
	return sqrt((vec0[X]-vec1[X])*(vec0[X]-vec1[X]) +
		    (vec0[Y]-vec1[Y])*(vec0[Y]-vec1[Y]) +
		    (vec0[Z]-vec1[Z])*(vec0[Z]-vec1[Z]));
    }
    
    double  GetNorm(double vec[3]){
	return sqrt(vec[X]*vec[X] + vec[Y]*vec[Y] + vec[Z]*vec[Z]);
    }

    double  GetNorm(float vec[3]){
	return sqrt(vec[X]*vec[X] + vec[Y]*vec[Y] + vec[Z]*vec[Z]);
    }

    void Normalize(double vec[3], double length){
	double toMul = length / 
	    sqrt(vec[X]*vec[X] + vec[Y]*vec[Y] + vec[Z]*vec[Z]);
	vec[X] *= toMul;
	vec[Y] *= toMul;
	vec[Z] *= toMul;
    };

    void Normalize(float vec[3], float length){
	float toMul = length / 
	    sqrt(vec[X]*vec[X] + vec[Y]*vec[Y] + vec[Z]*vec[Z]);
	vec[X] *= toMul;
	vec[Y] *= toMul;
	vec[Z] *= toMul;
    };

    // iv1āAv0ƒsj
    bool Orthogonalize(double vec0[3], double vec1[3]){
	double product = GetScalarProduct(vec0, vec1);
	LOOP_XYZ
	    vec1[xyz] -= vec0[xyz] * product;
	bool bValid = FALSE;
	LOOP_XYZ if (vec1[xyz] > 1e-12 || vec1[xyz] < -1e-12) bValid = TRUE;
	return bValid;
    }

    bool Orthogonalize(float vec0[3], float vec1[3]){
	float product = GetScalarProduct(vec0, vec1);
	LOOP_XYZ
	    vec1[xyz] -= vec0[xyz] * product;
	bool bValid = FALSE;
	LOOP_XYZ if (vec1[xyz] > 1e-12 || vec1[xyz] < -1e-12) bValid = TRUE;
	return bValid;
    }

    void Rotate(double vec[3], double angle, int axis){
	double org[3];
	LOOP_XYZ org[xyz] = vec[xyz];
	double cosA = cos(angle * 3.14159 / 180);
	double sinA = sin(angle * 3.14159 / 180);
	switch (axis){
	case X:
	    vec[Y] = cosA * org[Y] - sinA * org[Z];
	    vec[Z] = sinA * org[Y] + cosA * org[Z];
	    break;
	case Y:
	    vec[Z] = cosA * org[Z] - sinA * org[X];
	    vec[X] = sinA * org[Z] + cosA * org[X];
	    break;
	case Z:
	    vec[X] = cosA * org[X] - sinA * org[Y];
	    vec[Y] = sinA * org[X] + cosA * org[Y];
	    break;
	}
    } 

    void RotateAround(double vec[3], double axis[3], double angle){
	double mat[3][3];
	double cosA = cos(angle * 3.14159 /180);
	double sinA = sin(angle * 3.14159 /180);
	double org[3];
	LOOP_XYZ org[xyz] = vec[xyz];
	double orgaxis[3];
	LOOP_XYZ orgaxis[xyz] = axis[xyz];

	mat[X][X] = (1.0-cosA) * axis[X]*axis[X] + cosA;
	mat[Y][X] = axis[X]*axis[Y] * (1.0-cosA) + axis[Z]*sinA;
	mat[Z][X] = axis[X]*axis[Z] * (1.0-cosA) - axis[Y]*sinA;
	mat[X][Y] = axis[X]*axis[Y] * (1.0-cosA) - axis[Z]*sinA;
	mat[Y][Y] = (1.0-cosA) * axis[Y]*axis[Y] + cosA;
	mat[Z][Y] = axis[Y]*axis[Z] * (1.0-cosA) + axis[X]*sinA;
	mat[X][Z] = axis[X]*axis[Z] * (1.0-cosA) + axis[Y]*sinA;
	mat[Y][Z] = axis[Y]*axis[Z] * (1.0-cosA) - axis[X]*sinA;
	mat[Z][Z] = (1.0-cosA) * axis[Z]*axis[Z] + cosA;

	vec[X] = mat[X][X]*org[X] + mat[Y][X]*org[Y] + mat[Z][X]*org[Z];
	vec[Y] = mat[X][Y]*org[X] + mat[Y][Y]*org[Y] + mat[Z][Y]*org[Z];
	vec[Z] = mat[X][Z]*org[X] + mat[Y][Z]*org[Y] + mat[Z][Z]*org[Z];
    }
}
