// -*- c -*- 
// "clsColorCurve.h"

#include "clsColorCurve.h"
#include "clsColorCurveGUI.h"

#include "../WIDGETCTRL/EX/clsColourDialogEX.h"
extern wxColourDialogEX* GpCClrDialog;

#include "../UNDO/clsUndoItemConstParam.h"
extern UndoManager_cls GCUndoList;

#include "../../wxWidgetEnumMenu.h"

#include <wx/wx.h>
#include <wx/colordlg.h>

extern wxBitmap *GpCBitmapGraphP;
extern wxBitmap *GpCBitmapGraphN;

#define LOOP_RGB for (int rgb=0; rgb < 3; rgb++)
#define R 0
#define G 1
#define B 2
#define A 3

BEGIN_EVENT_TABLE(ColorCurveBtn_cls, wxPanel)    
EVT_LEFT_DOWN(ColorCurveBtn_cls::OnMouse)
EVT_PAINT    (ColorCurveBtn_cls::OnPaint)
END_EVENT_TABLE()

ColorCurveBtn_cls::
ColorCurveBtn_cls(wxWindow *pCParent, ColorCurve_cls *pCColorLinked,
		  bool bUseButton, bool bUseCurve, bool bWorkAsButton) : 
wxPanel(pCParent, wxID_ANY, wxDefaultPosition, /*wxDefaultSize*/wxSize(-1,20)) {
    mpCColorLinked = pCColorLinked;
    mbUseButton = bUseButton;
    mbUseCurve = bUseCurve;
    mbWorkAsButton = bWorkAsButton;
}

void ColorCurveBtn_cls::
LinkColorCurve(ColorCurve_cls *pCColorLinked){
    mpCColorLinked = pCColorLinked;
}

void ColorCurveBtn_cls::
OnMouse(wxMouseEvent &event){
    if (mbWorkAsButton == TRUE){
	wxCommandEvent com(wxEVT_COMMAND_BUTTON_CLICKED, GetId());
	GetParent()->AddPendingEvent(com);
	return;
    }

    if (mpCColorLinked == NULL || mbUseButton == FALSE) return;
    int x = event.GetX();
    if (x < 20){
	mpCColorLinked->mCbColorCurve.mbBody = !mpCColorLinked->mCbColorCurve.mbBody;
	Refresh(FALSE);

	GetParent()->AddPendingEvent(event);
	if (mpCColorLinked->mCbColorCurve.mbBody == FALSE) return;
    }

    if (mpCColorLinked->mCbColorCurve.mbBody == TRUE){
	mspCColorCurveCtrlLinked->LinkColorCurve(mpCColorLinked);
	wxCommandEvent com(wxEVT_COMMAND_MENU_SELECTED, menuDialogColorCurve);
	wxGetTopLevelParent(this)->AddPendingEvent(com);
	return; 
    }

    GpCClrDialog->SetColourFloat(mpCColorLinked->mCConstantColor.mpBody);
    if (GpCClrDialog->ShowModal() == wxID_OK){
	float newclr[4] = {0, 0, 0, 1};
	LOOP_RGB newclr[rgb] = GpCClrDialog->GetPColorFloat()[rgb];
	GCUndoList.SetNewCommand
	    (new CommandSetColor_cls(&mpCColorLinked->mCConstantColor, newclr,
				     _COMMAND_UPDATE_DISPLAY_ATTRIB | _COMMAND_UPDATE_DISPLAY_GL),
	     new CommandSetColor_cls(&mpCColorLinked->mCConstantColor, mpCColorLinked->mCConstantColor.mpBody, 
				     _COMMAND_UPDATE_DISPLAY_ATTRIB | _COMMAND_UPDATE_DISPLAY_GL));
	GCUndoList.ExecCurrent();
    }
    
    GetParent()->AddPendingEvent(event);
}

void ColorCurveBtn_cls::
OnPaint(wxPaintEvent &event){
    if (mpCColorLinked != NULL){
	wxPaintDC dc(this);
	mpCColorLinked->DrawColorOnPaintDC(dc, this, mbUseButton & mbUseCurve);
    }
    event.Skip();
}

ColorCurveCtrl_cls *ColorCurveBtn_cls::mspCColorCurveCtrlLinked = NULL;

// -----

ColorCurve_cls::
ColorCurve_cls(wxString strTag, wxString strLabel, wxString strInit, int keynum, double (*comp)[5]){
    mstrTagBase = strTag;
    mstrLabel = strLabel;

    mCConfList.Add(&mCbColorCurve, strTag + wxT("UseColorCurve="), wxT("0"));
    mCConfList.Add(&mCConstantColor, strTag + wxT("ConstantColor="), strInit);
    mCConfList.Add(&mCScalingMin, strTag + wxT("ScalingMin="), wxT("0"));
    mCConfList.Add(&mCScalingMax, strTag + wxT("ScalingMax="), wxT("1"));
    mCConfList.Add(&mCDependType, strTag + wxT("DependType="), wxT("0"));

    wxString listTag[4] = {wxT("R"), wxT("G"), wxT("B"), wxT("A")};
    wxString strClrTag = strTag;
    strClrTag << wxT(" ColorCurve");
    int updateBit = _COMMAND_UPDATE_DISPLAY_GL | _COMMAND_UPDATE_DISPLAY_COLOR |
	_COMMAND_UPDATE_DISPLAY_ATTRIB;
    mpCClrCurve = new ParamGroup_cls(strClrTag, 4, listTag, updateBit, TRUE);
    
    if (keynum == 0){
	double pClrInit[4] = {0, 1, 0, 1};
	mpCClrCurve->SetKeyFrame(pClrInit, TRUE);
	double pClrFin[4] = {1, 0, 0, 1};
	mpCClrCurve->SetKeyFrame(pClrFin, TRUE, 1);
    }
    else
	for (int i = 0; i < keynum; i++)
	    mpCClrCurve->SetKeyFrame(&comp[i][1], TRUE, comp[i][0]);
}

ColorCurve_cls::
~ColorCurve_cls(){
    delete mpCClrCurve;
}

void ColorCurve_cls::
CopyColorFromP(ColorCurve_cls *pCClrCurve){
    mpCClrCurve->CopyFromP(pCClrCurve->mpCClrCurve);
}

void ColorCurve_cls::
PreCalcScaleFactor(){
    mMin = mCScalingMin.mBody;
    mScale = mCScalingMax.mBody - mCScalingMin.mBody;
    if (mScale == 0){
	mCScalingMax.mBody = mCScalingMin.mBody + 1;
	mScale = 1;
    }
}

void ColorCurve_cls::
GetColor4fScaled(float *pColor, float value){
    if (mCbColorCurve.mbBody == TRUE){
	value = (value - mMin) / mScale;
	mpCClrCurve->GetInterpolatedValue4Fast(pColor, value, 0);
    }
    else
	memcpy(pColor, mCConstantColor.mpBody, sizeof(float)*4);
}

void ColorCurve_cls::
GetColor4fNormal(float *pColor, float value){
    if (mCbColorCurve.mbBody == TRUE){
	mpCClrCurve->GetInterpolatedValue4Fast(pColor, value, 0);
    }
    else
	memcpy(pColor, mCConstantColor.mpBody, sizeof(float)*4);
}

wxColour ColorCurve_cls::
GetWXColor(wxColour ClrBG, double valueX){
    if (mCbColorCurve.mbBody == TRUE){
	std::vector < double > VClr = mpCClrCurve->GetInterpolatedValue(valueX, 0);
	return wxColour
	    (ClrBG.Red()   + (VClr[R] * 255 - ClrBG.Red()) * VClr[A],
	     ClrBG.Green() + (VClr[G] * 255 - ClrBG.Green()) * VClr[A],
	     ClrBG.Blue()  + (VClr[B] * 255 - ClrBG.Blue()) * VClr[A]);
    }


    return wxColour(mCConstantColor.mpBody[R] * 255,
		    mCConstantColor.mpBody[G] * 255,
		    mCConstantColor.mpBody[B] * 255);
}

void ColorCurve_cls::
DrawOnDC(wxDC &dc, bool bGraphButton, wxRect rectToDraw, int offset,
	 bool bIgnoreAlpha){
    int rgt = rectToDraw.GetRight();
    int btm = rectToDraw.GetBottom();
    if (mCbColorCurve.mbBody == TRUE){
	dc.SetPen(*wxBLACK_PEN);
	dc.SetBrush(*wxTRANSPARENT_BRUSH);
	dc.DrawRectangle(rectToDraw);
	dc.SetPen(*wxTRANSPARENT_PEN);
	for (int x = offset+1; x < rgt; x++){
	    wxColour ClrBG0, ClrBG1;
	    if (bIgnoreAlpha == TRUE){
		ClrBG0.Set(0, 0, 0);
		ClrBG1.Set(0, 0, 0);
	    }
	    else {if ((x % 20) > 10){
		    ClrBG0.Set(192, 192, 192);
		    ClrBG1.Set(64,  64,   64);
		}
		else{
		    ClrBG0.Set(64,  64,  64);
		    ClrBG1.Set(192, 192, 192);
		}
	    }
	    double valueX = (double)(x-offset)/(rgt-offset);
	    dc.SetBrush(wxBrush(GetWXColor(ClrBG0, valueX)));
	    wxRect rectToPaint0(x, 1, 1, btm/2);
	    dc.DrawRectangle(rectToPaint0);
	    dc.SetBrush(wxBrush(GetWXColor(ClrBG1, valueX)));
	    wxRect rectToPaint1(x, btm/2+1, 1, btm/2);
	    dc.DrawRectangle(rectToPaint1);
	} 
	if (bGraphButton == TRUE) dc.DrawBitmap(*GpCBitmapGraphP, 0, 0);
    }
    else{
	dc.SetPen(*wxBLACK_PEN);
	dc.SetBrush(wxBrush(GetWXColor()));
	dc.DrawRectangle(rectToDraw);
	if (bGraphButton == TRUE) dc.DrawBitmap(*GpCBitmapGraphN, 0, 0);
    }
}

void ColorCurve_cls::
DrawColorOnPaintDC(wxPaintDC &dc, wxWindow *pCWindow,
		   bool bGraphButton){
    int offset = 0;
    if (bGraphButton == TRUE) offset = 20;
    wxSize sz = pCWindow->GetClientSize();
    wxRect rectToDraw(offset, 0, sz.x-offset, sz.y);

    // For efficiency, do not draw if not exposed
    if (pCWindow->IsExposed(rectToDraw))
	DrawOnDC(dc, bGraphButton, rectToDraw, offset, FALSE);
}

void ColorCurve_cls::
DrawColorOnMemoryDC(wxMemoryDC &dc, bool bGraphButton, int height,
		    bool bIgnoreAlpha){
    int offset = 0;
    if (bGraphButton == TRUE) offset = 20;
    wxSize sz = dc.GetSize();
    if (height ==0) height = sz.y;
    wxRect rectToDraw(offset, 0, sz.x-offset, height);

    DrawOnDC(dc, bGraphButton, rectToDraw, offset, bIgnoreAlpha);
}

// ---------------------------------------

wxString ColorCurve_cls::
Serialize(){
    wxString strOut;
    strOut << mCConfList.SerializeAll();
    strOut << mpCClrCurve->Serialize();
    return strOut;
}

void ColorCurve_cls::
ReadStrLine(std::wistream &istream, wxString strLine){
    mCConfList.ReadStrLineAll(strLine);
    mpCClrCurve->ReadStrLine(istream, strLine);
}
