// -*- c -*- 
// "clsShaderCtrl.h"

#include "clsShaderCtrl.h"

#include "../../GL/glsl.h"
#include "../MYAPI/MsgBox.h"

#ifdef WIN32
PFNGLPROGRAMPARAMETERIEXTPROC glProgramParameteriEXT;
#endif

ShaderCtrl_cls::
ShaderCtrl_cls(){
    mCConfList.Clear();
    mCConfList.Add(&mCCartoonBoldness, wxT("CartoonBoldness="), wxT("0.3"));
}

void ShaderCtrl_cls::
InitializeShader(wxWindow *pCParent){
    
#ifdef WIN32
    glProgramParameteriEXT = (PFNGLPROGRAMPARAMETERIEXTPROC)wglGetProcAddress("glProgramParameteriEXT");
#endif
    
    MakeShader(pCParent, 
	       mGL2ProgramGouraud, mVertShaderGouraud, mFragShaderGouraud,
	       wxT("GL/Gouraud.vert"), wxT("GL/Gouraud.frag"));

    MakeShader(pCParent,
	       mGL2ProgramPhong, mVertShaderPhong, mFragShaderPhong,
	       wxT("GL/Phong.vert"), wxT("GL/Phong.frag"));

    MakeShader(pCParent,
	       mGL2ProgramCartoon, mVertShaderCartoon, mFragShaderCartoon,
	       wxT("GL/Cartoon.vert"), wxT("GL/Cartoon.frag"));
    
    SetShaderParamIDs();
    SetShaderParamCartoon(0, mCCartoonBoldness.mBody);

    glUseProgram(0);
}

void ShaderCtrl_cls::
MakeShader(wxWindow *pCParent,
	   GLuint &program, GLuint &vertShader, GLuint &fragShader,
	   wxString strVertShader, wxString strFragShader){
    // VF[_vÕRpC^Nʂ𓾂ϐ
    GLint compiled, linked;

    vertShader = glCreateShader(GL_VERTEX_SHADER);
    fragShader = glCreateShader(GL_FRAGMENT_SHADER);
 
    // VF[_̃\[XvO̓ǂݍ 
    if (readShaderSource(vertShader, strVertShader.char_str())) exit(1);
    if (readShaderSource(fragShader, strFragShader.char_str())) exit(1);   

    // o[ebNXVF[_̃\[XvÕRpC 
    glCompileShader(vertShader);
    glGetShaderiv(mVertShaderGouraud, GL_COMPILE_STATUS, &compiled);
    if (compiled == GL_FALSE) {
	ShowLog(pCParent, vertShader);	//exit(1);
    }   

    // tOgVF[_̃\[XvÕRpC 
    glCompileShader(fragShader);
    glGetShaderiv(mFragShaderGouraud, GL_COMPILE_STATUS, &compiled);
    if (compiled == GL_FALSE) {
	ShowLog(pCParent, fragShader);	//exit(1);
    }

    // vOIuWFNg̍쐬 
    program = glCreateProgram();

    // VF[_IuWFNg̃VF[_vOւ̓o^ 
    glAttachShader(program, vertShader);
    glAttachShader(program, fragShader);

    // VF[_IuWFNg̍폜 
    glDeleteShader(vertShader);
    glDeleteShader(fragShader);

    // VF[_vÕN 
    glLinkProgram(program);
    glGetProgramiv(program, GL_LINK_STATUS, &linked);
    if (linked == GL_FALSE) {
       	//ShowLog(pCParent, mGeomShader);
	exit(1);
    }
}

void ShaderCtrl_cls::
ShowLog(wxWindow *pCParent, GLuint shader){
    int logSize;
    int length;
    glGetShaderiv(shader, GL_INFO_LOG_LENGTH , &logSize);
    char *pLog = new char [logSize];
    glGetShaderInfoLog(shader, logSize, &length, pLog);
    wxString strLog(pLog, wxConvFile);
    wxString strMes = wxT("Compile error in fragment shader.\n");
    strMes << strLog;
    MsgBox(pCParent, strLog, wxT("!!"));
    delete [] pLog;
}

void ShaderCtrl_cls::
BeginShader(int shaderType, bool bShadowMode, int lightNumber){
    glUseProgram(0);
    switch(shaderType){
    case 0:
	glUseProgram(0);
	break;
    case 1:
	glUseProgram(mGL2ProgramGouraud);
	SetShaderParamGouraud(lightNumber);
	break;
    case 2:
	glUseProgram(mGL2ProgramPhong);
	SetShaderParamPhong(lightNumber);
	break;
    case 3: {
	glUseProgram(mGL2ProgramCartoon);
	SetShaderParamCartoon(lightNumber, mCCartoonBoldness.mBody);
	float ambient[4]; 
	glGetLightfv(GL_LIGHT0, GL_AMBIENT, ambient);
	if (lightNumber == 1 && ambient[0] == 0)
	    SetShaderParamCartoonShadowMode(1);
	else
	    SetShaderParamCartoonShadowMode(0);
	break; }
    }
}

void ShaderCtrl_cls::
EndShader(int shaderType){
    glUseProgram(0);
    if (shaderType)
	glDisable(GL_TEXTURE_2D);
}

void ShaderCtrl_cls::
SetShaderParamIDs(){
    glUseProgram(mGL2ProgramGouraud);
    mParamIDLNumGouraud = 
	glGetUniformLocation(mGL2ProgramGouraud, "lightNumberGouraud");

    glUseProgram(mGL2ProgramPhong);
    mParamIDLNumPhong   = 
	glGetUniformLocation(mGL2ProgramPhong, "lightNumberPhong");
    
    glUseProgram(mGL2ProgramCartoon);
    mParamIDLNumCartoon = 
	glGetUniformLocation(mGL2ProgramCartoon, "lightNumberCartoon");
    mParamIDShadowModeCartoon = 
	glGetUniformLocation(mGL2ProgramCartoon, "shadowVolumeModeCartoon");
    mParamIDBoldnessCartoon = 
	glGetUniformLocation(mGL2ProgramCartoon, "boldnessCartoon");
}

void ShaderCtrl_cls::
SetShaderParamGouraud(int value){
    glUniform1i(mParamIDLNumGouraud, value);
}

void ShaderCtrl_cls::
SetShaderParamPhong(int value){
    glUniform1i(mParamIDLNumPhong, value);
}

void ShaderCtrl_cls::
SetShaderParamCartoon(int value, float boldness){
    glUniform1i(mParamIDLNumCartoon, value);
    glUniform1f(mParamIDBoldnessCartoon, boldness);
}

void ShaderCtrl_cls::
SetShaderParamCartoonShadowMode(bool bShadowMode){
    if (bShadowMode == 0)
	glUniform1i(mParamIDShadowModeCartoon, 1);
    else
	glUniform1i(mParamIDShadowModeCartoon, 0);
}
