// -*- c -*-
// "clsDolly.h"

#include "clsDolly.h"
#include "clsMouseCtrl.h"
#include "clsMainGLCanvas.h"
#include "../PANELS/clsPreference.h"
#include "../SCENEOBJ/clsSceneObj.h"

#define LOOP_XYZ for (int xyz = 0; xyz < 3; xyz++)
#define X 0
#define Y 1
#define Z 2

Dolly_cls::
Dolly_cls(Preference_cls *pCPreferenceLink){
    mpCPreferenceLink = pCPreferenceLink;

    mpItemPosLink    = MainGLCanvas_cls::msSelectedItemPos;
    mpItemPosOrgLink = MainGLCanvas_cls::msSelectedItemPosOrg;
    mpItemScaleLink  = MainGLCanvas_cls::msSelectedItemScale;
    mpItemDRotLink   = MainGLCanvas_cls::msSelectedItemDRot;
}

void Dolly_cls::
SetParameter(double sensitivity, MouseNormalized_cls *pCMP, 
	     wxSize sz, int displayMode){
    mSensitivity = sensitivity;
    mpCMP = pCMP;
    mAspectRatio = (double)sz.x / sz.y;
    mDisplayMode = displayMode;
}

bool Dolly_cls::
Gyration(PositionAndDirection_cls *pCCamPos){
    PositionAndDirection_cls CamPosBK = *pCCamPos;

    if (mpCPreferenceLink->mCbGyrationInvert.mbBody == TRUE) 
	mpCMP->Invert();

    pCCamPos->SetSensitivity(mSensitivity);
    if (mDisplayMode == _DIRECTION_FREE ||
	mDisplayMode >= _DOLLY_LINKED)
	pCCamPos->TrackBall(mpCMP->mPrevX, mpCMP->mPrevY, 
			    mpCMP->mCurrX, mpCMP->mCurrY);
    else{
	mSensitivity = mSensitivity * pCCamPos->GetProjectedDistanceToAnc();
	pCCamPos->SetSensitivity(mSensitivity);
	pCCamPos->SetAspectRatio(mAspectRatio);
	pCCamPos->Translate(mpCMP->GetDX(), mpCMP->GetDY());
    }

    return TRUE;
}

bool Dolly_cls::
Zoom(PositionAndDirection_cls *pCCamPos, float *pOrthoScale){
    if (mpCPreferenceLink->mCbZoomInvert.mbBody == TRUE) 
	mpCMP->Invert();
    
    if (mDisplayMode >= _DIRECTION_X_PLUS &&
	mDisplayMode <= _DIRECTION_Z_MINUS){
	*pOrthoScale *= (1.0 + mpCMP->GetDY() - mpCMP->GetDX() * 0.2);
    }
    else {
	pCCamPos->SetSensitivity(1);
	pCCamPos->Zoom(mpCMP->GetDX(), mpCMP->GetDY());
    }

    return TRUE;
}

bool Dolly_cls::
MoveCenter(PositionAndDirection_cls *pCCamPos,
	   Position_cls *pCAncPos){
    PositionAndDirection_cls CAncTmp(*pCAncPos, *pCCamPos);
    
    mSensitivity = 0.5 * mSensitivity * pCCamPos->GetProjectedDistanceToAnc();
    
    pCCamPos->SetSensitivity(mSensitivity);
    pCCamPos->SetAspectRatio(mAspectRatio);
    
    if (/*mpCLinkedPath == NULL*/TRUE)
	pCCamPos->Translate(mpCMP->GetDX(), mpCMP->GetDY());
    else mSensitivity *= -1;
    
    CAncTmp.SetAncPos(pCCamPos->mPos);
    CAncTmp.SetSensitivity(mSensitivity);
    CAncTmp.SetAspectRatio(mAspectRatio);
    CAncTmp.Translate(mpCMP->GetDX(), mpCMP->GetDY());
    pCAncPos->Set(CAncTmp);
    
    //*trigAnchor = TRUE;
    return TRUE;
}

bool Dolly_cls::
MoveItem(PositionAndDirection_cls *pCCamPos, bool bDragging, wxSize sz,
	 int dragMode){
    if (MainGLCanvas_cls::mspCSelectedObject == NULL) return FALSE;

    if (bDragging == FALSE){
	LOOP_XYZ mpItemPosLink[xyz] = 
	    MainGLCanvas_cls::mspCSelectedObject->mpCTranslate->mVCParam[xyz].GetInterpolatedValue();
	LOOP_XYZ mpItemPosOrgLink[xyz] = mpItemPosLink[xyz];
	LOOP_XYZ mpItemScaleLink[xyz] = 1.0;
	LOOP_XYZ mpItemDRotLink[xyz] = 0;
    }

    gluProject(mpItemPosLink[X], mpItemPosLink[Y], mpItemPosLink[Z],
	       mMatrixModelView, mMatrixProjection, mMatrixViewport,
	       &mPosItemOnW[0], &mPosItemOnW[1], &mPosItemOnW[2]);
    double normalPosX = (2*mPosItemOnW[X] - sz.x)/sz.x;
    double normalPosY = (2*mPosItemOnW[Y] - sz.y)/sz.y;
    double dragX = mpCMP->mCurrX - mpCMP->mPrevX;
    double dragY = mpCMP->mCurrY - mpCMP->mPrevY;
    double relX  = mpCMP->mCurrX - normalPosX;
    double relY  = mpCMP->mCurrY - normalPosY;

    switch (0){
    case _DOLLY_TRANSLATE:
	Position_cls CPosTmp(mpItemPosLink[0],
			     mpItemPosLink[1],
			     mpItemPosLink[2], 0, 0, 0);
	PositionAndDirection_cls CPos(CPosTmp, *pCCamPos);
	PositionAndDirection_cls CParentCoord;

	LOOP_XYZ CPos.mATranslateScale[xyz] = 1;

	//double sensitivity = -0.5 * pCCamPos->GetProjectedDistanceToAnc();
	double sensitivity = -0.3 * pCCamPos->GetProjectedDistance(mpItemPosLink);
	CPos.SetAncPos(pCCamPos->mPos);
	CPos.SetSensitivity(sensitivity);
	CPos.SetAspectRatio((double)sz.x / sz.y);
	CPos.Translate(dragX, dragY, 
		       _POS_AND_DIR_MOVE_FREE, &CParentCoord);
	LOOP_XYZ mpItemPosLink[xyz] = CPos.mPos[xyz];

	if (dragMode == X){
	    mpItemPosLink[Y] = mpItemPosOrgLink[Y];
	    mpItemPosLink[Z] = mpItemPosOrgLink[Z];
	}
	if (dragMode == Y){
	    mpItemPosLink[Z] = mpItemPosOrgLink[Z];
	    mpItemPosLink[X] = mpItemPosOrgLink[X];
	}
	if (dragMode == Z){
	    mpItemPosLink[X] = mpItemPosOrgLink[X];
	    mpItemPosLink[Y] = mpItemPosOrgLink[Y];
	}

	break;

    }

    return TRUE;
}

bool Dolly_cls::
ZoomWheel(PositionAndDirection_cls *pCCamPos,
	  double wheelRotation, float *pOrthoScale){
    if (mpCPreferenceLink->mCbZoomInvert.mbBody == TRUE) 
	wheelRotation *= -1;

    if (mDisplayMode >= _DIRECTION_X_PLUS &&
	mDisplayMode <= _DIRECTION_Z_MINUS){
	*pOrthoScale *= (1.0 + wheelRotation * 0.1);
    }
    else{
	pCCamPos->SetSensitivity(1);
	
	pCCamPos->Zoom(0, wheelRotation * 0.1);
    }
    return TRUE;
}
