//(setq c-basic-offset 4)

#include "clsGLCanvasCtrl.h"

#include "../SCENEOBJ/clsCameraPath.h"

#include <wx/tglbtn.h>
#include <wx/valgen.h>

#include "clsMainGLCanvas.h"
#include "../MYGL/clsFBO.h"
#include "../../wxWidgetEnum.h"
#include "../../wxWidgetEnumMenu.h"

BEGIN_EVENT_TABLE(GLCanvasCtrl_cls, wxPanel)
     EVT_PAINT       (GLCanvasCtrl_cls::OnPaint)
     EVT_KEY_DOWN    (GLCanvasCtrl_cls::OnKeyDown)
     EVT_RIGHT_DOWN  (GLCanvasCtrl_cls::OnMouse)
//EVT_MENU        (GLMenuHidePanel, GLCanvasCtrl_cls::OnMenuHide)
END_EVENT_TABLE()

#define _POV_PERSPECTIVE  0
#define _POV_ORTHOGRAPHIC 1
#define _POV_FISHEYE      2
#define _POV_PANORAMIC    3

GLCanvasCtrl_cls::
GLCanvasCtrl_cls(wxWindow *parent, FrameMain_cls *pCParentFrame,
		 wxWindowID id, int* attribList,
		 const wxPoint& pos, const wxSize& size,
		 long style, const wxString& name,
		 const wxGLContext *pContextToShare,
		 unsigned int selfIndex, int displayMode) :
wxPanel(parent, id, pos, size, style, name)
{
    mSelfIndex = selfIndex;
    mbPanelShown = TRUE;

    if (pContextToShare == NULL)
	mpCGLCanvas = new MainGLCanvas_cls
	    (this, pCParentFrame,
	     wxID_ANY, attribList, pos, size,
	     style, name, selfIndex, displayMode);
    else
	mpCGLCanvas = new MainGLCanvas_cls
	    (this, pCParentFrame,
	     wxID_ANY, attribList, pos, size,
	     style, name, pContextToShare, selfIndex, displayMode);
					 
    wxBoxSizer *pBoxSizerSrc = new wxBoxSizer(wxVERTICAL);
    mpBoxSizerBar = new wxBoxSizer(wxHORIZONTAL);
    
    wxString choice[_GL_CANVAS_LINKED_CAM0 + 1] = { 
	wxT("Free"), 
	wxT("From -X to +X"),	wxT("From +X to -X"),
	wxT("From -Y to +Y"),	wxT("From +Y to -Y"),
	wxT("From -Z to +Z"),	wxT("From +Z to -Z")
    };

    for (int i = 0; i < _GL_CANVAS_LINKED_CAM0; i++) mAChoiceStr[i] = choice[i];
    for (int i = 0; i < _PATH_NUM_MAX; i++) 
	mAChoiceStr[_GL_CANVAS_LINKED_CAM0 + i] << wxT("Cam") << i;
    
    mpCComboDirection = new wxComboBox(this, wxID_ANY, wxT("Free"), wxDefaultPosition, wxSize(108, -1), 8, mAChoiceStr, wxCB_DROPDOWN | wxCB_READONLY);
   
    this->Connect(mpCComboDirection->GetId(), wxEVT_COMMAND_COMBOBOX_SELECTED,
		  wxCommandEventHandler(GLCanvasCtrl_cls::OnComboBox));
    mpCComboDirection->SetToolTip(_("Point of View"));

    wxPoint defPos = wxDefaultPosition;

    mpCBoolPivot = new wxCheckBoxEX
	(this, wxID_ANY, _("Pivot Follows Anchor"), 
	 wxGenericValidator(&mpCGLCanvas->mCbPivotMode.mbBody));
    Connect(mpCBoolPivot->GetId(), wxEVT_COMMAND_CHECKBOX_CLICKED,
	    wxCommandEventHandler(GLCanvasCtrl_cls::OnCheckBox));

    //mpCBoolFlip->SetToopTip(wxT("Flip in Stereo Gram"));

    /*
    mpCFBO = new wxCheckBoxEX
	(this, wxID_ANY, _("FBO(HDR)"), 
	 wxGenericValidator(&mpCGLCanvas->mbCFBO.mbBody));
    Connect(mpCFBO->GetId(), wxEVT_COMMAND_CHECKBOX_CLICKED,
	    wxCommandEventHandler(GLCanvasCtrl_cls::OnCheckBox));
    */
    
    SetButtons(&mpCToggleMessage, wxT("M"), &GLCanvasCtrl_cls::OnToggleMessage,
	       _("Show/Hide Message"));
    SetButtons(&mpCToggleItem, wxT("I"), &GLCanvasCtrl_cls::OnToggleItem,
	       _("Show/Hide Item"));
    SetButtons(&mpCToggleGrid, wxT("G"), &GLCanvasCtrl_cls::OnToggleGrid,
	       _("Show/Hide Grid"));
    SetButtons(&mpCToggleData, wxT("D"), &GLCanvasCtrl_cls::OnToggleData,
	       _("Show/Hide Data"));

    UpdateWindow();
    UpdateToggleButtonsAndParams();

    mpBoxSizerBar->Add(mpCComboDirection, 0, 0);
    mpBoxSizerBar->Add(-1, -1, 1, wxEXPAND);
    //mpBoxSizerBar->Add(mpCFBO, 0, wxALIGN_CENTER_VERTICAL);
    mpBoxSizerBar->Add(mpCBoolPivot, 0, wxALIGN_CENTER_VERTICAL);
    /*
    mpBoxSizerBar->Add(mpCToggleWire, 0, 0);
    mpBoxSizerBar->Add(mpCToggleBox, 0, 0);
    */
    mpBoxSizerBar->Add(mpCToggleMessage, 0, 0);
    mpBoxSizerBar->Add(mpCToggleItem, 0, 0);
    mpBoxSizerBar->Add(mpCToggleGrid, 0, 0);
    mpBoxSizerBar->Add(mpCToggleData, 0, 0);

    pBoxSizerSrc->Add(mpBoxSizerBar, 0, wxEXPAND);
    pBoxSizerSrc->Add(mpCGLCanvas, 1, wxEXPAND);

    SetSizer(pBoxSizerSrc);

    mpCPopupMenu = new wxMenu;
    //mpCPopupMenu->Append(GLMenuHidePanel, _("Hide Control Panel"));
}

GLCanvasCtrl_cls::
~GLCanvasCtrl_cls(){
    delete mpCPopupMenu;
}

void GLCanvasCtrl_cls::
SetButtons(wxToggleButton **ppButton, wxString strChar,
	   void (GLCanvasCtrl_cls::*pFunc)(wxCommandEvent&), wxString strTips){
    *ppButton = new wxToggleButton
	(this, wxID_ANY, strChar, wxDefaultPosition, wxSize(24,-1));
    Connect((*ppButton)->GetId(), wxEVT_COMMAND_TOGGLEBUTTON_CLICKED,
	    wxObjectEventFunction(pFunc));
    (*ppButton)->SetToolTip(strTips);
    (*ppButton)->SetValue(TRUE);
}

int GLCanvasCtrl_cls::
GetDisplayMode(){
    return mpCGLCanvas->mCDisplayMode.mBody;
}

bool GLCanvasCtrl_cls::
IsPanelShown(){
    return mbPanelShown;
}

void GLCanvasCtrl_cls::
ShowPanel(){
    size_t itemNum = mpBoxSizerBar->GetChildren().GetCount();
    for (size_t index = 0; index < itemNum; index++)
	mpBoxSizerBar->Show(index);
    mbPanelShown = TRUE;
    Layout();
}

void GLCanvasCtrl_cls::
UpdateWindow(){
    switch (mpCGLCanvas->mCDisplayMode.mBody){
    case _GL_CANVAS_DISPLAY_FREE:
	mpCComboDirection->SetValue(wxT("Free")); break;
    case _GL_CANVAS_DISPLAY_X_PLUS:
	mpCComboDirection->SetValue(wxT("From -X to +X")); break;
    case _GL_CANVAS_DISPLAY_X_MINUS:
	mpCComboDirection->SetValue(wxT("From +X to -X")); break;
    case _GL_CANVAS_DISPLAY_Y_PLUS:
	mpCComboDirection->SetValue(wxT("From -Y to +Y")); break;
    case _GL_CANVAS_DISPLAY_Y_MINUS:
	mpCComboDirection->SetValue(wxT("From +Y to -Y")); break;
    case _GL_CANVAS_DISPLAY_Z_PLUS:
	mpCComboDirection->SetValue(wxT("From -Z to +Z")); break;
    case _GL_CANVAS_DISPLAY_Z_MINUS:
	mpCComboDirection->SetValue(wxT("From +Z to -Z")); break;
    default: {
	for (int i = 0; i < _PATH_NUM_MAX; i++)
	    if (mpCGLCanvas->mCDisplayMode.mBody == _GL_CANVAS_LINKED_CAM0 + i){
		wxString strCam; strCam << wxT("Cam") << i;
		mpCComboDirection->SetValue(strCam);
	    }
	break; }
    }

    //mpCFBO->GetValidator()->TransferToWindow();
    mpCBoolPivot->GetValidator()->TransferToWindow();
}

void GLCanvasCtrl_cls::
UpdateComboBoxList(int camNum){
    mpCComboDirection->Freeze();
    mpCComboDirection->Clear();
    for (int i = 0; i < _GL_CANVAS_LINKED_CAM0; i++) 
	mpCComboDirection->Append(mAChoiceStr[i]);
    for (int i = 0; i < camNum; i++) 
	mpCComboDirection->Append(mAChoiceStr[_GL_CANVAS_LINKED_CAM0 + i]);
    mpCComboDirection->Thaw();
    UpdateWindow();

    //mpCFBO->Enable(FBO_cls::msbSupported);
}

void GLCanvasCtrl_cls::
OnPaint(wxPaintEvent& event){
    event.Skip();
    //SkipAԈړȂǂinvalidateŖ̂AppendB
    //@\邯ǖʂeventłHƖʂȂłH
    mpCGLCanvas->AddPendingEvent(event);
    mpCComboDirection->SetValue(mAChoiceStr[GetDisplayMode()]);
}

void GLCanvasCtrl_cls::OnSize(wxSizeEvent& event){
    event.Skip();
    //UpdateWindowSizeInChildClass();
}

#include "../MYAPI/MsgBox.h"

void GLCanvasCtrl_cls::
OnKeyDown(wxKeyEvent& event){
    event.SetId(wxGetTopLevelParent(this)->GetId());
    wxGetTopLevelParent(this)->AddPendingEvent(event);
}

void GLCanvasCtrl_cls::
OnComboBox(wxCommandEvent& event){
    SyncDisplayModeToCombobox();
    mpCGLCanvas->SetFocus();
}

void GLCanvasCtrl_cls::
SyncDisplayModeToCombobox(){
    if (mpCComboDirection->GetValue() == wxT("Free"))
	mpCGLCanvas->SetDisplayMode(_GL_CANVAS_DISPLAY_FREE);
    if (mpCComboDirection->GetValue() == wxT("From -X to +X"))
	mpCGLCanvas->SetDisplayMode(_GL_CANVAS_DISPLAY_X_PLUS);
    if (mpCComboDirection->GetValue() == wxT("From +X to -X"))
	mpCGLCanvas->SetDisplayMode(_GL_CANVAS_DISPLAY_X_MINUS);
    if (mpCComboDirection->GetValue() == wxT("From -Y to +Y"))
	mpCGLCanvas->SetDisplayMode(_GL_CANVAS_DISPLAY_Y_PLUS);
    if (mpCComboDirection->GetValue() == wxT("From +Y to -Y"))
	mpCGLCanvas->SetDisplayMode(_GL_CANVAS_DISPLAY_Y_MINUS);
    if (mpCComboDirection->GetValue() == wxT("From -Z to +Z"))
	mpCGLCanvas->SetDisplayMode(_GL_CANVAS_DISPLAY_Z_PLUS);
    if (mpCComboDirection->GetValue() == wxT("From +Z to -Z"))
	mpCGLCanvas->SetDisplayMode(_GL_CANVAS_DISPLAY_Z_MINUS);

    for (int i = 0; i < _PATH_NUM_MAX; i++){
	wxString strIndex; strIndex << wxT("Cam") << i;
	if (mpCComboDirection->GetValue() == strIndex)
	    mpCGLCanvas->LinkageConnect();
    }

    mpCGLCanvas->ForceUpdateCameraPos();
    mpCGLCanvas->SetThisToBeActiveGL();
    mpCGLCanvas->Refresh(FALSE);
}

void GLCanvasCtrl_cls::
OnCheckBox(wxCommandEvent& event){
    /*
    wxCommandEvent command = wxCommandEvent(wxEVT_COMMAND_MENU_SELECTED, menuItemRefresh);
    wxGetTopLevelParent(mpCGLCanvas)->AddPendingEvent(command);
    */
    mpCGLCanvas->Refresh(FALSE);
}

/*
void GLCanvasCtrl_cls::
OnToggleWire(wxCommandEvent& event){
    mpCGLCanvas->mCbShowWire.mbBody = mpCToggleWire->GetValue();
    mpCGLCanvas->Refresh(FALSE);
}

void GLCanvasCtrl_cls::
OnToggleBox(wxCommandEvent& event){
    mpCGLCanvas->mCbShowBox.mbBody = mpCToggleBox->GetValue();
    mpCGLCanvas->Refresh(FALSE);
}
*/

void GLCanvasCtrl_cls::
OnToggleMessage(wxCommandEvent& event){
    mpCGLCanvas->mCbShowMessage.mbBody = mpCToggleMessage->GetValue();
    mpCGLCanvas->Refresh(TRUE);
}

void GLCanvasCtrl_cls::
OnToggleItem(wxCommandEvent& event){
    mpCGLCanvas->mCbShowItems.mbBody = mpCToggleItem->GetValue();
    mpCGLCanvas->Refresh(FALSE);
}

void GLCanvasCtrl_cls::
OnToggleGrid(wxCommandEvent& event){
    mpCGLCanvas->mCbShowGrid.mbBody = mpCToggleGrid->GetValue();
    mpCGLCanvas->Refresh(FALSE);
}

void GLCanvasCtrl_cls::
OnToggleData(wxCommandEvent& event){
    mpCGLCanvas->mCbShowData.mbBody = mpCToggleData->GetValue();
    mpCGLCanvas->Refresh(FALSE);
}

void GLCanvasCtrl_cls::
OnMenuHide(wxCommandEvent& event){
    size_t itemNum = mpBoxSizerBar->GetChildren().GetCount();
    for (size_t index = 0; index < itemNum; index++)
	mpBoxSizerBar->Hide(index);
    Layout();
    mbPanelShown = FALSE;
}

void GLCanvasCtrl_cls::
OnMouse(wxMouseEvent& event){
    PopupMenu(mpCPopupMenu, wxDefaultPosition);
}

void GLCanvasCtrl_cls::
UpdateToggleButtonsAndParams(){
    mpCToggleItem->SetValue(mpCGLCanvas->mCbShowItems.mbBody);
    mpCToggleGrid->SetValue(mpCGLCanvas->mCbShowGrid.mbBody);
    mpCToggleData->SetValue(mpCGLCanvas->mCbShowData.mbBody);
    if (mpCGLCanvas->mCGLGeometry.mBody == _POV_ORTHOGRAPHIC)
	mpCGLCanvas->mCGLPerspective.mbPerspective = FALSE;
    else
	mpCGLCanvas->mCGLPerspective.mbPerspective = TRUE;
}

// ----

void GLCanvasCtrl_cls::
ReadStrLine(wxString strLine){
    mpCGLCanvas->ReadStrLine(strLine);
}

wxString GLCanvasCtrl_cls::
Serialize(){
    return mpCGLCanvas->Serialize();
}
