// -*- c -*-
// "clsMainGLCanvas.cpp"

#ifndef _MAIN_GL_CANVAS_H_
#define _MAIN_GL_CANVAS_H_

#define GL_GLEXT_PROTOTYPES 
#define _MANIP_TRANSLATE 0
#define _MANIP_SCALE 1
#define _MANIP_ROTATE 2

#define _GEOM_PERSPECTIVE  0
#define _GEOM_ORTHOGRAPHIC 1
#define _GEOM_FISHEYE      2
#define _GEOM_PANORAMIC    3

#include <wx/wx.h>
#include <wx/glcanvas.h>
#include <wx/colordlg.h>
#include <wx/stopwatch.h>

class DF3SeqCtrl_cls;
class DF3Sequence_cls;
class TreeBoxRule_cls;

class AnchorPath_cls;
class CameraPath_cls; 
class Canvas_cls; 
class DataExporter_cls;
class DrawingBlock_cls;
class DrawTimeIndex_cls; 
class DrawClrLegend_cls;
class DrawOption_cls;
class FilterCtrl_cls;
class GLGuides_cls;
class ImgSequence_cls;
class MainGLCanvas_cls; 
class MouseCtrl_cls;
class MultiNumericalInputs_cls; 
class Overlay_cls;
class Preference_cls; 
class SceneObj_cls;
class TimeCtrl_cls;

#include "../CONFITEM/clsConfItem.h"
#include "../GLCANVAS/clsStructDrawInfo.h"
#include "../MYGL/clsPositionAndDirection.h"
#include "../MYGL/clsGLPerspective.h"
//#include "../MYGL/clsRedrawPBO.h"
//#include "../PARAMETER/clsMultiParameters.h"
//#include "../POVRAY/clsPovSerialize.h"

#include "clsGLButton.h"
#include "clsHitTest.h"

#include <vector>

class FrameMain_cls; 

//#include "../WIDGETCTRL/clsPendEventList.h"

class ColorCurve_cls;
class Dolly_cls;
class FBO_cls;
class GLStrings2D_cls;
class LightSRC_cls; 
class LightSRCCtrl_cls;
class Parameters_cls; 
class GLCanvasCtrl_cls;
class MouseNormalized_cls;

#define _GL_CANVAS_DISPLAY_FREE    _DIRECTION_FREE
#define _GL_CANVAS_DISPLAY_X_PLUS  _DIRECTION_X_PLUS
#define _GL_CANVAS_DISPLAY_X_MINUS _DIRECTION_X_MINUS
#define _GL_CANVAS_DISPLAY_Y_PLUS  _DIRECTION_Y_PLUS
#define _GL_CANVAS_DISPLAY_Y_MINUS _DIRECTION_Y_MINUS
#define _GL_CANVAS_DISPLAY_Z_PLUS  _DIRECTION_Z_PLUS
#define _GL_CANVAS_DISPLAY_Z_MINUS _DIRECTION_Z_MINUS
#define _GL_CANVAS_LINKED_CAM0 7

#define _GL_CANVAS_GRID_BASIC 0
#define _GL_CANVAS_GRID_BASIC_CIRCULAR 1
#define _GL_CANVAS_GRID_POLAR 2

#define _GL_CANVAS_NOASSIN  0
#define _GL_CANVAS_GYRATION 1

/*
#define _GL_PERSPECTIVE 0
#define _GL_ORTHOGRAPHIC 1
#define _GL_FISHEYE 2
#define _GL_PANORAMIC 3
*/

#define _GYRATION_FREE 0
#define _GYRATION_FIXPOS 1

#define _PIVOT_FOLLOW_PATH 1

#ifdef _DEBUG
#include <stdlib.h>
#include <crtdbg.h>
#define new new(_NORMAL_BLOCK, __FILE__, __LINE__)
#endif

class MainGLCanvas_cls : 
public wxGLCanvas
{
 public:
    MainGLCanvas_cls
	(GLCanvasCtrl_cls *pCParentCtrl, FrameMain_cls *pCParentFrame,
	 wxWindowID id = wxID_ANY, int* attribList = 0,
	 const wxPoint& pos = wxDefaultPosition,
	 const wxSize& size = wxDefaultSize, 
	 long style = 0, const wxString& name = wxT("MainGLCanvas"),
	 const wxGLContext *pContextToShare = NULL,
	 unsigned int selfIndex = 0,
	 int displayMode = _GL_CANVAS_DISPLAY_FREE);

    MainGLCanvas_cls
	(GLCanvasCtrl_cls *pCParentCtrl, FrameMain_cls *pCParentFrame,
	 wxWindowID id = wxID_ANY, int* attribList = 0,
	 const wxPoint& pos = wxDefaultPosition,
	 const wxSize& size = wxDefaultSize, 
	 long style = 0, const wxString& name = wxT("MainGLCanvas"),
	 unsigned int selfIndex = 0,
	 int displayMode = _GL_CANVAS_DISPLAY_FREE);

    void ConstructorMain
	(GLCanvasCtrl_cls *pCParentCtrl, FrameMain_cls *pCParentFrame,
	 unsigned int selfIndex, int displayMode, 
	 const wxGLContext *pContextToShare = NULL);
	
    ~MainGLCanvas_cls();
    
    GLCanvasCtrl_cls  *mpCGLCtrl;
    MouseCtrl_cls     *mpCMouseCtrl;
    Dolly_cls         *mpCDolly;
    GLStrings2D_cls   *mpCGLString2D;
    static DrawTimeIndex_cls *mspCDrawTimeIndex;
    static DrawClrLegend_cls *mspCDrawClrLegend;

    std::vector < MainGLCanvas_cls* > mVPOtherCanvas;

    wxString         mstrWarning;
    unsigned int     mSelfIndex;
    int              mPanelWidth, mPanelHeight;
    wxStopWatch      mCStopWatchSnap;
    double mMatrixModelView[16];
    double mMatrixProjection[16];
    int    mMatrixViewport[4];
    
    double mPosItemOnW[3];
    double mPosCoordinateTag[2][3][3];
    double mPosCameraTag[4][3];
    bool   mbForceUpdateCameraPos;
    bool   mbHitTestItemSelect;
    bool   mbHitTestGuideSelect;
    int    mMouseX, mMouseY;
    bool   mbShowCoordinateTagXYZ[3];
    int    mLightNumber;
    static int msGLLightIndex[8];
    static GLButton_cls msCGLBTNRot, msCGLBTNTrans, msCGLBTNScale;
    static HitTest_cls  msCHittest;
    
 private:
    bool SetGLContext();
    void InitGL();
    void InitGLUT();
    void InitializeShader();
    void InitializeGLIfNot();
    void UpdateWindowSizeInChildClass();
    
    // ----

 public:
    void SetCameraToDefaultPos();
    void UpdateDrawInfo(int width, int height);
    PositionAndDirection_cls* GetPCCameraPosition();
    Position_cls* GetPCAnchorPosition();
    void ForceUpdateCameraPos();
    void UpdateCameraPos();
    CameraPath_cls* GetPLinkedCameraPath();
    AnchorPath_cls* GetPLinkedAnchorPath();
    LightSRCCtrl_cls* GetPLinkLightSourceCtrl();
    void UpdateAnc();
    Position_cls GetAncPos();

    bool IsStereo();
    bool IsOrtographic();
    bool IsDisplayModeOrtho();
    bool IsFishEye();
    //bool IsFBOUse();
    void ToggleOrthographic();
    void SetAngleIfLinked();
    void ModifyByStereogram(int forceType = -1);
    void ModifyByDomeTilt();
    void ModifyByDome(PositionAndDirection_cls *pCCam, int index);
    double* GetAnchorPosForRender();

    void SetKeyPos();
    void SetKeyAnc();
    void SetViewport();
    void SetPerspective(PositionAndDirection_cls *pCCam);
    void SetModelView(PositionAndDirection_cls *pCCam);
    void SetLight();

    void ClearBuffer();
    
    void SetGLLightSource(double doubleIndex);
    void DrawDataLitDiffuse(double doubleIndex, int fboIndex, int lightIndex);
    void DrawDataMain(double doubleIndex, int fboIndex, int shadowNumber);
    void DrawCameraPos(double scale, PositionAndDirection_cls CPos);
    void DrawGrid(double scale, double scale10);
    void DrawCameraPath(double doubleIndex, double scale);
    void DrawLightSource(double doubleIndex, double scale, bool bHittest = FALSE);
    void DrawAnchor(double scale);
    void DrawMessage();
    void DrawModeAxis(double scale);
    void DrawSelectedItem(double scale);
    void ShowArrow();
    void ShowNavigator(PositionAndDirection_cls CCamPos,
		       int width, int height);
    
    void ShowInformation();

    bool IsMouseOnClient();
    bool CheckFocusOnTherCanvas();
    bool CheckGLButton(int canvasWidth, int canvasHeight,
		       int mouseX, int mouseY);

    //std::vector < double > GetEulerAngelOfView();
    void ShowPreviousFramePBO();
    //    void DrawScence(PositionAndDirection_cls CCameraPos);
    void DrawOtherObjects(double doubleIndex, double scale, double scale10);
    void RefreshOtherOrthoCanvas();
    void CheckGLRefresh(int frame, int interpStep);
    void DrawOnFBO(FBO_cls *pCFBO[4], FBO_cls *pCFBOShadow,
		   int stereo, double doubleIndex,
		   double scale, int bufferSize);
    void GatherFBOImages(FBO_cls *pCFBO[4], int width, int height, bool bOnFBO);
    void OnPaint(wxPaintEvent& event);
    void FinishPaint(wxPaintEvent &event);
    void ProcReadImage(int width, int height, 
		       FBO_cls *pCFBOTotal, FBO_cls *pCFBOOverlay);
    void ProcReadImageSequence(int width, int height, 
			       FBO_cls *pCFBO, FBO_cls *pCFBOOverlay);
    void StartSaveImgSeqProcess(ImgSequence_cls* pCImgSeq);
    void StopSaveImgSeqProcess();
    void StartExportDataProcess(DataExporter_cls* pCDataExporter);
    void StopExportDataProcess();

    void OnSize (wxSizeEvent& event);

    bool (*mApFuncMove[3])(double);

    void ShowMenu(int x, int y);
    void ShowMenuPanel(int type, int x, int y);
    
    // $B%^%&%9A`:n4X78(B
    bool SetThisToBeActiveGL();
    bool SetOtherCanvasToBeActiveGL();
    bool ProcViewOrItemDragging(wxSize size, bool& trigAnchor);
    void ProcSelectItemForDragging(int mouseX, int mouseY);
    void ProcItemDraggingEnd();
    bool ProcCaptureMouse();
    bool StartHitTest(int mouseX, int mouseY);
    
    void OnMouse(wxMouseEvent& event);
    void OnMouseRelease(wxMouseCaptureLostEvent& event);
    void OnMouseMotion(wxMouseEvent& event);
    void OnKeyDown(wxKeyEvent& event);
    void RotationMode();
    void TranslationMode();
    void ScaleMode();


    static void InitializeStaticMembers(Overlay_cls *pCOverlayLink);
    static void DestructStaticMembers();
    static bool IsGLBlendEquationEXOK();

 private:
    static FilterCtrl_cls* mspCFilterCtrl;
    static DF3SeqCtrl_cls* mspCDF3SeqCtrl;

 public:
    static Preference_cls* mspCPreference;
    
    void SetDefaultPosition(int displayMode);
    void TogglePerspective();
    void SetDisplayMode(int displayMode, bool bUpdateCombobox = FALSE);
    void LinkageConnect(/*int pathIndex*/);
    void OnPanelConfigure(wxCommandEvent &event);
    void OnMenuPers(wxCommandEvent& event);
    void OnMenuGridAnti(wxCommandEvent& event);
    void OnMenuOtherCam(wxCommandEvent& event);
    void OnMenuMoveToCamPos(wxCommandEvent& event);
    void OnMenuSetKeyPos   (wxCommandEvent& event);
    void OnMenuSetKeyAnc   (wxCommandEvent& event);
    void OnMenuToggleLink  (wxCommandEvent& event);
    void OnMenuTogglePerspective(wxCommandEvent& event);
    void OnMenuPassToParent(wxCommandEvent& event);
    void OnMenuAddPrimitive(wxCommandEvent& event);
    void OnMenuShowPanel   (wxCommandEvent& event);

    void CopySettingToAll();
   
    static void SelectItem(Parameters_cls *pCParam);
    static void HookGridComboBox(void* pArg, void *pEvent);

    void   ModifyOrthoCamera();
    double CalcAutoScale();

    FrameMain_cls* mpCParentFrameLink;
    DrawOption_cls* mpCDrawOptionLink;
    
    wxMenu *mpPopupMenu;
    wxMenu *mpMenuConfigure;
    wxMenu *mpMenuAdd;

    wxMenuItem *mpMenuItemShowPanel;

    bool mbInitialized;
    unsigned int mDrawingFrame;
    unsigned int mDrawingStep;
    unsigned int mPrevFrame;
    unsigned int mPrevStep;
    
 public:
    bool mbMouseDragging;
    bool mbItemDragging;
    std::vector < int > mVHitObj; //
    
    int  mIndexForDebug;

    static ColorCurve_cls *mspCClrCurveBG, *mspCClrCurveOut;
    
 public:
    ConfItemList_cls      mCConfList;

    ConfItemBOOL_cls      mCbShowMessage, mCbShowItems;
    ConfItemBOOL_cls      mCbShowGrid, mCbShowData;
    ConfItemBOOL_cls      mCbPivotMode;
    
    ConfItemFLOAT_cls     mCGLPersAngle;
    ConfItemFLOAT_cls     mCGLOrthoScale;
    
    ConfItemINT_cls       mCGLGeometry;
    ConfItemINT_cls       mCDisplayMode;
    ConfItemINT_cls       mCPathIndex;
    ConfItemUINT_cls      mCGridType;
    ConfItemBOOL_cls      mCbShowOtherCam;
    ConfItemBOOL_cls      mCbXOR;
    ConfItemBOOL_cls      mCbGridAnti;
    ConfItemBOOL_cls      mCbShowGridTag;
    bool mbSaveImageQueue;
    bool mbSaveImgSeqQueue;
    bool mbExportDataQueue;
    bool mbRefreshQueue;

 public:
    wxGLContext*              mpCGLContext;

    //RedrawPBO_cls             mCRedrawPBO[4];
    StructDrawInfo            mCDrawInfo, mCDrawInfoPrev;
    
    GLPerspective_cls         mCGLPerspective;
    static GLGuides_cls       *mspCGLGuide, *mspCGLGuideScrew, *mspCGLGuideYPR;
    PositionAndDirection_cls  mCCameraPos;

    PositionAndDirection_cls  mCCameraPosFree, mCCameraPosCamera;
    PositionAndDirection_cls  mCCameraPosOrtho;
    
    PositionAndDirection_cls  mCDataParentCoordinate;
    static Position_cls       msCAnchorPos, msCAnchorPosOrtho; 
    static int                msMoveMode;
    static int                msManipulatorMode;

    static SceneObj_cls      *mspCSelectedObject;
    static double             msSelectedItemPos[3];
    static double             msSelectedItemPosOrg[3];
    static double             msSelectedItemScale[3];
    static double             msSelectedItemDRot[3];
    double                    mdPosTmp[3];
    
    static int                msAFBOSize[5];

    static ConfItemList_cls   msCConfListStatic;
    //static ConfItemBOOL_cls msbCFBO;
    static ConfItemUINT_cls   msCFBOSizeIndex;
    
    static float              msLineWidth;

    // -- $B%j%s%/(B --
    CameraPath_cls*           mpCCameraPathLinked;
    AnchorPath_cls*           mpCAnchorPathLinked;
    ImgSequence_cls*          mpCImgSeqLinked;
    DataExporter_cls*         mpCDataExporterLinked;
    // ----

    void ReadStrLine(wxString strLine);
    wxString Serialize();

    DECLARE_EVENT_TABLE()
};

#endif //_MAIN_GL_CANVAS_H_
