// -*- c -*-
// "clsMouseCtrl.h"

#include "clsMouseCtrl.h"
#include "../PANELS/clsPreference.h"

MouseCtrl_cls::
MouseCtrl_cls(Preference_cls* pCPreferenceLink){
    mpCPreferenceLink = pCPreferenceLink;
    mPrevX = 0;
    mPrevY = 0;
}

void MouseCtrl_cls::
SetMouseInformation(wxMouseEvent &event){
    
    mbAModifier[0] = event.m_shiftDown;
    mbAModifier[1] = event.m_controlDown;
    mbAModifier[2] = event.m_altDown;
    mbAModifier[3] = FALSE;
    mbAMouseBTN[0] = event.LeftIsDown();
    mbAMouseBTN[1] = event.MiddleIsDown();
    mbAMouseBTN[2] = event.RightIsDown();
    mbAMouseBTN[3] = FALSE;

    mMouseX = event.GetX();
    mMouseY = event.GetY();

    mbButtonDown = event.ButtonDown();
    mbButtonUp   = event.ButtonUp();
    mbMouseWheel = (event.GetEventType() == wxEVT_MOUSEWHEEL);
    if (mbMouseWheel)
	mWheelRotationNormalized =
	    (float)(event.m_wheelRotation)/ event.GetWheelDelta();
	    
    mbMouseDragging = event.Dragging() & 
	((mPrevX != mMouseX) | (mPrevY != mMouseY));

    mpCPreferenceLink->ModulateBTN(mbAModifier, mbAMouseBTN);
}

void MouseCtrl_cls::
SetPreviousPosition(){
    mPrevX = mMouseX;
    mPrevY = mMouseY;
}

void MouseCtrl_cls::
SetGyrationMode(wxSize sz){
    float lX = (float)(2.0 * mMouseX - sz.x)     / sz.x;
    float lY = (float)(sz.y - 2.0 * mMouseY)     / sz.y;
    if (lX > 0.85 || lX < -0.85 || lY > 0.85 || lY < -0.85)
	mGyrationMode = _GYRATION_FIXPOS;
    else
	mGyrationMode = _GYRATION_FREE;
}

bool MouseCtrl_cls::
IsGyration(){
    return mpCPreferenceLink->IsMouse4Gyration(mbAModifier, mbAMouseBTN);
}

bool MouseCtrl_cls::
IsZoom(){
    return mpCPreferenceLink->IsMouse4Zoom(mbAModifier, mbAMouseBTN);
}

bool MouseCtrl_cls::
IsMoveCenter(){
    return mpCPreferenceLink->IsMouse4MoveCenter(mbAModifier, mbAMouseBTN);
}

bool MouseCtrl_cls::
IsMoveItem(){
    return mpCPreferenceLink->IsMouse4MoveItem(mbAModifier, mbAMouseBTN);
}

bool MouseCtrl_cls::
IsWheel(){
    return mbMouseWheel;
}

bool MouseCtrl_cls::
IsShowMenu(){
    if (mbButtonDown == FALSE) 
	return FALSE;
    
    if (mpCPreferenceLink->IsMouse4Menu(mbAModifier, mbAMouseBTN) == FALSE)
	return FALSE;
    
    return TRUE;
}

bool MouseCtrl_cls::
IsHitTest(){
    return mpCPreferenceLink->IsMouse4HitTest(mbAModifier, mbAMouseBTN);
}

bool MouseCtrl_cls::
IsButton(int btnType){
    return mbAMouseBTN[btnType];
}

bool MouseCtrl_cls::
IsButtonDown(){
    return mbButtonDown;
}

bool MouseCtrl_cls::
IsButtonUp(){
    return mbButtonUp;
}

bool MouseCtrl_cls::
IsMouseDragging(){
    return mbMouseDragging;
}

// -----------------------------

MouseNormalized_cls::
MouseNormalized_cls(MouseCtrl_cls *pCMouseCtrl, wxSize sz){
    mPrevX = (float)(2.0 * pCMouseCtrl->mPrevX - sz.x)  / sz.x;
    mPrevY = (float)(sz.y - 2.0 * pCMouseCtrl->mPrevY)  / sz.y;
    mCurrX = (float)(2.0 * pCMouseCtrl->mMouseX - sz.x) / sz.x;
    mCurrY = (float)(sz.y - 2.0 * pCMouseCtrl->mMouseY) / sz.y;
    mGyrationMode = pCMouseCtrl->mGyrationMode;
}

void MouseNormalized_cls::
LimitMove(){
    if (mPrevX < mCurrX - 0.2) mPrevX = mCurrX - 0.2;
    if (mPrevX > mCurrX + 0.2) mPrevX = mCurrX + 0.2;
    if (mPrevY < mCurrY - 0.2) mPrevY = mCurrY - 0.2;
    if (mPrevY > mCurrY + 0.2) mPrevY = mCurrY + 0.2;
}

void MouseNormalized_cls::
Invert(){
    mPrevX *= -1; mCurrX *= -1;
    mPrevY *= -1; mCurrY *= -1;
}

double MouseNormalized_cls::
GetDX(){
    return mCurrX - mPrevX;
}

double MouseNormalized_cls::
GetDY(){
    return mCurrY - mPrevY;
}
