// -*- c -*-
// "clsGrid.h"
// (setq c-basic-offset 4)

#include "clsCone.h"

#include "../COMMON/MyVector.h"

#include <math.h>

#ifdef WIN32
#include <windows.h>
#include <GL/gl.h>
#include <GL/glu.h>
#elif defined __WXMAC__
#include <OpenGL/glu.h>
#else
#include <GL/glu.h>
#endif

#ifndef TRUE
#define TRUE true
#define FALSE false
#endif

#ifndef M_PI
#define M_PI 3.131593f
#endif

#define X 0
#define Y 1
#define Z 2

bool Cone_cls::mbInitialized = FALSE;
int  Cone_cls::mDList = -1;

void Cone_cls::
DrawMain(int quality, float baseScale){
    int face; 
    switch(quality){
    case 0:
	face = 8;
	break;
    case 1:
	face = 16;
	break;
    case 2:
	face = 24;
	break;
    }
    double factor = M_PI * 2 / face;
    double cosAngle = baseScale / sqrt(1.0 + baseScale*baseScale);
    double sinAngle = 1.0 / sqrt(1.0 + baseScale*baseScale);
    
    glBegin(GL_QUAD_STRIP);
    double normal[3] = {cosAngle, sinAngle, 0};
    double vert0[3]  = {0, baseScale, 0};
    double vert1[3]  = {1, 0, 0};
    glNormal3dv(normal);
    glVertex3dv(vert0);
    glVertex3dv(vert1);
    
    for (int i = face-1; i >= 0 ; i--){
	double normal[3] = {cosAngle, sinAngle * baseScale*cos(factor*i), sinAngle * baseScale*sin(factor*i)};
	double vert0[3] =  {0, baseScale*cos(factor*i), baseScale*sin(factor*i)};
	double vert1[3] =  {1, 0, 0};
	glNormal3dv(normal);
	glVertex3dv(vert0);
	glVertex3dv(vert1);
    }
    glEnd();
    glBegin(GL_POLYGON);
    for (int i = face-1; i>=0; i--){
	double normal[3] = {-1, 0, 0};
	double vert[3] = {0, baseScale*cos(factor*i), baseScale*sin(factor*i)};
	glNormal3dv(normal);
	glVertex3dv(vert);
    }
    glEnd();
}

int Cone_cls::
DListDraw(int GLMode){
    int    listName = 0;
	
    if (GLMode == GL_COMPILE || GLMode == GL_COMPILE_AND_EXECUTE){
	listName = glGenLists(1); // VKXg̔ԍ
	glNewList (listName, GLMode); 
    }

    DrawMain(1.0, 0.3);

    if (GLMode == GL_COMPILE || GLMode == GL_COMPILE_AND_EXECUTE)
	glEndList(); 
    
    return listName;
}

void Cone_cls::
InitializeIfNot(){
    if (mbInitialized == TRUE) return;
    mDList = DListDraw(GL_COMPILE);
    mbInitialized = TRUE;
}

void Cone_cls::
Draw(){
    glCallList(mDList);
}
