// "clsMainGLCanvas.h"
// (setq c-basic-offset 4)

#include "clsImageIO.h"
#include "clsStereoGram.h"
#include "../MYAPI/MsgBox.h"
#include "../COMMON/clsFilename.h"
#include "../WIDGETCTRL/EX/clsFileDialogEX.h"
#include <wx/wx.h>
#ifdef WIN32
#include <windows.h>
#include <GL/gl.h>
#elif defined __WXMAC__
#include <OpenGL/gl.h>
#else
#include <GL/gl.h>
#endif

extern wxFileDialogEX*   GpCFileDialog;

ThreadSave_cls::
ThreadSave_cls(wxThreadKind kind) :
 wxThread(kind){
}

void ThreadSave_cls::
SetParameters(bool *pbFinishedFlag, 
	      ImageIO_cls *pCParent, int width, int height,
	      wxString strFileOut, bool bUseOverlay,
	      unsigned char *pBuffer, 
	      bool bFreeDataAtEnd){
    mpbFinishedFlagLink = pbFinishedFlag;
    mpCParent = pCParent;
    mWidth = width; mHeight = height;
    mstrFileOut = strFileOut;
    
    if (pBuffer == NULL){
	mpBuffer = (unsigned char*)malloc(mWidth * mHeight * 3);
	mbAllocLocal = TRUE;
    }
    else{
	mpBuffer = pBuffer;
	mbAllocLocal = bFreeDataAtEnd;
    }

    mbUseOverlay = bUseOverlay;
    if (mbUseOverlay == TRUE)
	mpOverlay = (unsigned char*)malloc(mWidth * mHeight * 4);

    mpBufferSRC = NULL;
    mbAllocLocalSRC = FALSE;

    if ( Create() != wxTHREAD_NO_ERROR) 
	MsgBoxExit(NULL, "Error!", "Can't create thread");
}

void ThreadSave_cls::
DeclareResizing(int bufferWidth, int bufferHeight){
    mBufferWidth = bufferWidth;
    mBufferHeight = bufferHeight;
    
    mpBufferSRC = (unsigned char*)malloc(mBufferWidth * mBufferHeight * 3);
    mbAllocLocalSRC = TRUE;
}

void ThreadSave_cls::
GetAverageRGB(int indexTGT, int widthBuffer, int heightBuffer,
	      double dIndexX, double dIndexY,
	      double dIndexX2, double dIndexY2,
	      unsigned char* pBufferSRC,
	      unsigned char* pBufferTGT){
    double RSum = 0, GSum = 0, BSum = 0;
    double areaSum = 0;
    if (dIndexY2 >= heightBuffer) dIndexY2 = heightBuffer - 0.001;
    if (dIndexX2 >= widthBuffer) dIndexX2 = widthBuffer - 0.001;
    for (int srcY = (int)dIndexY; srcY <= dIndexY2; srcY++){
	double factorY = 1;
	if (dIndexY - srcY > 0)
	    factorY -= dIndexY - srcY;
	if (dIndexY2 - srcY < 1)
	    factorY -= (srcY + 1 - dIndexY2);
	int indexY = srcY * widthBuffer;
	for (int srcX = (int)dIndexX; srcX <= dIndexX2; srcX++){
	    double factorX = 1;
	    if (dIndexX - srcX > 0)
		factorX -= dIndexX - srcX;
	    if (dIndexX2 - srcX < 1)
		factorX -= (srcX + 1 - dIndexX2);
	    int index = (indexY + srcX) * 3;
	    double area = factorX * factorY;
	    RSum += pBufferSRC[index + 0] * area;
	    GSum += pBufferSRC[index + 1] * area;
	    BSum += pBufferSRC[index + 2] * area;
	    areaSum += area;
	}
    }
    
    pBufferTGT[indexTGT + 0] = (unsigned char)(RSum / areaSum);
    pBufferTGT[indexTGT + 1] = (unsigned char)(GSum / areaSum);
    pBufferTGT[indexTGT + 2] = (unsigned char)(BSum / areaSum);
}

wxThread::ExitCode 
ThreadSave_cls::
Entry(){
    unsigned char *pBuffer = mpBuffer;
    
    // resizing;
    if (mbAllocLocalSRC == TRUE){
	if (mWidth < mHeight){
	    double dOffsetX = (1.0 - (double)mWidth/mHeight) / 2;
	    for (int y = 0; y < mHeight; y++){
		double dIndexY = (double)y / mHeight * mBufferHeight;
		double dHeight = 1.0 / mHeight * mBufferHeight;
		int indexy = y * mWidth;
		for (int x = 0; x < mWidth; x++){
		    double dIndexX = ((double)x/mHeight + dOffsetX) * mBufferWidth;
		    double dWidth = 1.0 / mHeight * mBufferWidth;
		    GetAverageRGB((indexy + x) * 3, 
				  mBufferWidth, mBufferHeight,
				  dIndexX, dIndexY, 
				  dIndexX + dWidth, 
				  dIndexY + dHeight,
				  mpBufferSRC, pBuffer);
		}
	    }
	}
	else{
	    double dOffsetY = (1.0 - (double)mHeight/mWidth) / 2;
	    for (int y = 0; y < mHeight; y++){
		double dIndexY = ((double)y / mWidth + dOffsetY) * mBufferHeight;
		double dHeight = 1.0 / mWidth * mBufferHeight;
		int indexy = y * mWidth;
		for (int x = 0; x < mWidth; x++){
		    double dIndexX = (double)x /mWidth * mBufferWidth;
		    double dWidth = 1.0 / mWidth * mBufferWidth;
		    GetAverageRGB((indexy + x) * 3, mBufferWidth, mBufferHeight,
				  dIndexX, dIndexY, 
				  dIndexX + dWidth, 
				  dIndexY + dHeight,
				  mpBufferSRC,
				  pBuffer);
		}
	    }
	}
    }

    unsigned char *pOverlay = mpOverlay;

    if (mbUseOverlay == TRUE)
	for (int y = 0; y < mHeight; y++){
	    int indexy = y * mWidth;
	    for (int x = 0; x < mWidth; x++){
		int index = indexy + x;
		float alpha = (float)pOverlay[index * 4 + 3]/255;
		for (int rgb = 0; rgb < 3; rgb++)
		    pBuffer[index * 3 + rgb] = 
			pOverlay[index * 4 + rgb] * alpha +
			pBuffer[index * 3 + rgb] * (1.0f-alpha);
	    }
	}
    
    wxImage CImage(mWidth, mHeight, mpBuffer, TRUE);

    SplitPath_cls CPath(mstrFileOut);
    if (CPath.GetPathStr(_FN_EXT) == wxT("")) mstrFileOut += wxT(".png");
    
    mpCParent->mstrLastSavedFile = mstrFileOut; //Xbh瑀댯
    
    wxImage CMirror(CImage.Mirror(FALSE)); // ]ftH
    CMirror.SetOption(wxT("quality"), mpCParent->msJPEGQuality);
    CMirror.SaveFile(mstrFileOut);

    if (mbAllocLocal == TRUE) free(mpBuffer);
    if (mbUseOverlay == TRUE) free(mpOverlay);
    if (mbAllocLocalSRC == TRUE) free(mpBufferSRC);

    *mpbFinishedFlagLink = TRUE;
    return 0;
}

// -----

ImageIO_cls::
ImageIO_cls(){
    for (int i = 0; i < _IMAGE_OUT_MAX_THREAD; i++){
	mbSaveThreadActive[i] = FALSE;
	mbSaveThreadFinished[i] = FALSE;
    }
    mActiveThread = 0;
}

ImageIO_cls::
~ImageIO_cls(){
    /*
    for (int i = 0; i < _IMAGE_OUT_MAX_THREAD; i++)
	if (mbSaveThreadActive[i] == TRUE){
	    mpCThreadSave[i]->Wait();
	    delete mpCThreadSave[i]; }
    */
}

bool ImageIO_cls::
SaveImageFromData(int width, int height, wxString strFileOut, 
		  unsigned char* pBuffer){
    if (mbSaveThreadActive[mActiveThread] == TRUE){
	while (mbSaveThreadFinished[mActiveThread] == FALSE)
	    wxMilliSleep(3);
	mbSaveThreadActive[mActiveThread] = FALSE;
	mbSaveThreadFinished[mActiveThread] = FALSE;
    }

    bool retVal = FALSE;

    mpCThreadSave[mActiveThread] = new ThreadSave_cls(wxTHREAD_DETACHED);
    mpCThreadSave[mActiveThread]->
	SetParameters(&mbSaveThreadFinished[mActiveThread],
		      this, width, height, strFileOut, FALSE,
		      pBuffer, TRUE);
 
    mbSaveThreadActive[mActiveThread] = TRUE;    
    mpCThreadSave[mActiveThread]->Run();
    mActiveThread++;
    if (mActiveThread >= _IMAGE_OUT_MAX_THREAD) mActiveThread = 0;
    
    return TRUE;
}

bool ImageIO_cls::
SaveImageFromGL(int width, int height){
    return SaveImageFromGL(width, height, mstrFile);
}

bool ImageIO_cls::
SaveImageFromGL(int width, int height, wxString strFileOut,
		int offsetX, int offsetY){

    if (mbSaveThreadActive[mActiveThread] == TRUE){
	while (mbSaveThreadFinished[mActiveThread] == FALSE)
	    wxMilliSleep(3);
	//mpCThreadSave[mActiveThread]->Wait();
	//delete mpCThreadSave[mActiveThread]; // 
	mbSaveThreadActive[mActiveThread] = FALSE;
	mbSaveThreadFinished[mActiveThread] = FALSE;
    }

    bool retVal = FALSE;
    int trimWidth = width - (width % 8);
    
    mpCThreadSave[mActiveThread] = new ThreadSave_cls(wxTHREAD_DETACHED);
    mpCThreadSave[mActiveThread]->
	SetParameters(&mbSaveThreadFinished[mActiveThread],
		      this, trimWidth, height, strFileOut, FALSE);

    glReadPixels(offsetX, offsetY, trimWidth, height, GL_RGB, GL_UNSIGNED_BYTE, mpCThreadSave[mActiveThread]->mpBuffer);

    mbSaveThreadActive[mActiveThread] = TRUE;    
    mpCThreadSave[mActiveThread]->Run();
    mActiveThread++;
    if (mActiveThread >= _IMAGE_OUT_MAX_THREAD) mActiveThread = 0;
    
    return TRUE;
}

void ImageIO_cls::
GetAverageRGB(int indexTGT, int widthBuffer, int heightBuffer,
	      double dIndexX, double dIndexY,
	      double dIndexX2, double dIndexY2,
	      unsigned char* pBufferSRC,
	      unsigned char* pBufferTGT){
    double RSum = 0, GSum = 0, BSum = 0;
    double areaSum = 0;
    if (dIndexY2 >= heightBuffer) dIndexY2 = heightBuffer - 0.001;
    if (dIndexX2 >= widthBuffer) dIndexX2 = widthBuffer - 0.001;
    for (int srcY = (int)dIndexY; srcY <= dIndexY2; srcY++){
	double factorY = 1;
	if (dIndexY - srcY > 0)
	    factorY -= dIndexY - srcY;
	if (dIndexY2 - srcY < 1)
	    factorY -= (srcY + 1 - dIndexY2);
	int indexY = srcY * widthBuffer;
	for (int srcX = (int)dIndexX; srcX <= dIndexX2; srcX++){
	    double factorX = 1;
	    if (dIndexX - srcX > 0)
		factorX -= dIndexX - srcX;
	    if (dIndexX2 - srcX < 1)
		factorX -= (srcX + 1 - dIndexX2);
	    int index = (indexY + srcX) * 3;
	    double area = factorX * factorY;
	    RSum += pBufferSRC[index + 0] * area;
	    GSum += pBufferSRC[index + 1] * area;
	    BSum += pBufferSRC[index + 2] * area;
	    areaSum += area;
	}
    }
    
    pBufferTGT[indexTGT + 0] = (unsigned char)(RSum / areaSum);
    pBufferTGT[indexTGT + 1] = (unsigned char)(GSum / areaSum);
    pBufferTGT[indexTGT + 2] = (unsigned char)(BSum / areaSum);
}

bool ImageIO_cls::
SaveImageFromTexture(int widthBuffer, int heightBuffer, 
		     int width, int height, int texID, int texIDOverlay){
    return SaveImageFromTexture(widthBuffer, heightBuffer,
				width, height, texID, texIDOverlay, mstrFile);
}

bool ImageIO_cls::
SaveImageFromTexture(int widthBuffer, int heightBuffer, 
		     int width, int height, int texID, int texIDOverlay,
		     wxString strFileOut,
		     int offsetX, int offsetY){
    if (mbSaveThreadActive[mActiveThread] == TRUE){
	while (mbSaveThreadFinished[mActiveThread] == FALSE)
	    wxMilliSleep(3);
	mbSaveThreadActive[mActiveThread] = FALSE;
	mbSaveThreadFinished[mActiveThread] = FALSE;
    }

    bool retVal = FALSE;
    int trimWidth = width - (width % 8);
    
    mpCThreadSave[mActiveThread] = new ThreadSave_cls(wxTHREAD_DETACHED);
    mpCThreadSave[mActiveThread]->
	SetParameters(&mbSaveThreadFinished[mActiveThread],
		      this, /*trimWidth*/width, height, strFileOut, TRUE);
    
    unsigned char *pBuffer = mpCThreadSave[mActiveThread]->mpBuffer;

    glEnable(GL_TEXTURE_2D);

    glBindTexture(GL_TEXTURE_2D, texIDOverlay);
    unsigned char *pBufferOver = mpCThreadSave[mActiveThread]->mpOverlay;
    glGetTexImage( GL_TEXTURE_2D, 0, GL_RGBA, GL_UNSIGNED_BYTE, pBufferOver);
    
    glBindTexture(GL_TEXTURE_2D, texID);
    if (widthBuffer == width && heightBuffer == height){
	glGetTexImage( GL_TEXTURE_2D, 0, GL_RGB, GL_UNSIGNED_BYTE, pBuffer);
    }
    else if (widthBuffer >= width && widthBuffer >= height){
	mpCThreadSave[mActiveThread]->DeclareResizing(widthBuffer, heightBuffer);
	unsigned char *pBufferSRC = mpCThreadSave[mActiveThread]->mpBufferSRC;
	    //(unsigned char*)malloc(widthBuffer * heightBuffer * 3);
	glGetTexImage( GL_TEXTURE_2D, 0, GL_RGB, GL_UNSIGNED_BYTE, pBufferSRC);

	// {̓TCYʃXbh̕
	// Ƃ肠Ă
	
	/*
	if (width < height){
	    double dOffsetX = (1.0 - (double)width/height) / 2;
	    for (int y = 0; y < height; y++){
		double dIndexY = (double)y / height * heightBuffer;
		double dHeight = 1.0 / height * heightBuffer;
		int indexy = y * width;
		for (int x = 0; x < width; x++){
		    double dIndexX = ((double)x/height + dOffsetX) * widthBuffer;
		    double dWidth = 1.0 / height * widthBuffer;
		    GetAverageRGB((indexy + x) * 3, widthBuffer, heightBuffer,
				  dIndexX, dIndexY, 
				  dIndexX + dWidth, 
				  dIndexY + dHeight,
				  pBufferSRC,
				  pBuffer);
		}
	    }
	}
	else{
	    double dOffsetY = (1.0 - (double)height/width) / 2;
	    for (int y = 0; y < height; y++){
		double dIndexY = ((double)y / width + dOffsetY) * heightBuffer;
		double dHeight = 1.0 / width * heightBuffer;
		int indexy = y * width;
		for (int x = 0; x < width; x++){
		    double dIndexX = (double)x /width * widthBuffer;
		    double dWidth = 1.0 / width * widthBuffer;
		    GetAverageRGB((indexy + x) * 3, widthBuffer, heightBuffer,
				  dIndexX, dIndexY, 
				  dIndexX + dWidth, 
				  dIndexY + dHeight,
				  pBufferSRC,
				  pBuffer);
		}
	    }
	}
	*/

	//free(pBufferSRC);
    }
    
    mbSaveThreadActive[mActiveThread] = TRUE;    
    mpCThreadSave[mActiveThread]->Run();
    mActiveThread++;
    if (mActiveThread >= _IMAGE_OUT_MAX_THREAD) mActiveThread = 0;
    
    glDisable(GL_TEXTURE_2D);
    return TRUE;
}

bool ImageIO_cls::
SelectFile(wxString strDefaultDir, wxWindow *pC){
    static bool sbState = FALSE; // Ƀ_CAOoĂ邩
    if (sbState == TRUE) {sbState = FALSE; return FALSE;}
    sbState = TRUE; 

    GpCFileDialog->SetMessage(_("Select a image file to save"));
    GpCFileDialog->SetWildcard(wxT("image(*.png;*.jpg;*.tiff)|*.png;*.jpg;*.tiff|png (*.png)|*.png|jpg (*.jpg)|*.jpg|tiff (*.tiff)|*.tiff"));
    GpCFileDialog->SetStyle(wxFD_SAVE | wxFD_OVERWRITE_PROMPT);
    
    if (strDefaultDir == wxT("None"))
	GpCFileDialog->SetInitPath(wxT(""), wxT("snapshot.png"));
    else if (strDefaultDir != wxT("")){
	GpCFileDialog->SetInitPathFull(strDefaultDir);
    }
    
    bool bRetval = (GpCFileDialog->ShowModal() == wxID_OK);
    if (bRetval == TRUE) mstrFile = GpCFileDialog->GetPath();
    
    sbState = FALSE;
    return bRetval;
}

void ImageIO_cls::
SetFilenameStereo(int eyeType){
    SplitPath_cls CSplit(mstrFile);

    wxString strLast = CSplit.mstrFilename.Right(1);
    if (strLast.IsSameAs(wxT("L")) || strLast.IsSameAs(wxT("R")))
	CSplit.mstrFilename.RemoveLast();
    
    switch(eyeType){
    case _STEREO_LFT:
	CSplit.mstrFilename.Append(wxT("L"));
	break;
    case _STEREO_RGT:
	CSplit.mstrFilename.Append(wxT("R"));
	break;
    }
			 
    mstrFile = CSplit.GetPathStr();
}

ImageIO_cls GCImgIO;

int ImageIO_cls::msJPEGQuality = 95;
