// -*- c++ -*- 
// (setq c-basic-offset 4)
// "clsStereoGram.h"

#include "clsStereoGram.h"

#include "../DF3/clsDF3Sequence.h"
#include "../MYAPI/MsgBox.h"
#include "../WIDGETCTRL/EX/clsButtonEX.h"

#ifdef __WXMAC__
#include <GLUT/glut.h>
#else
#include <GL/glut.h>
#endif

#include "../GLOBJ/GLOBJECTS.h"

#include "../../wxWidgetEnum.h"
#include "../../wxWidgetEnumMenu.h"
#include "../../clsMain.h"
#include "../../clsTimeCtrl.h"

#ifndef M_PI
#define M_PI 3.14159
#endif

#define LOOP_XYZ for (int xyz = 0; xyz < 3; xyz++)

// ÓIɃCxgт
BEGIN_EVENT_TABLE(StereoGramGUI_cls, wxDialog)
     EVT_BUTTON(buttonCtrlStereoDefault, StereoGramGUI_cls::OnButtonDefault)
     EVT_BUTTON(buttonCtrlStereoClose,   StereoGramGUI_cls::OnButtonClose)
END_EVENT_TABLE()
     
StereoGramGUI_cls::
    StereoGramGUI_cls(StereoGram_cls* pCParent,
		      wxWindow *pC,
		      wxPoint pos, wxSize size) :
    wxDialogEXKey(pC, wxID_ANY, _("Stereogram Configure"),
		  pos, size, pCParent->mpBitmapLink)
{
    mspCParent = pCParent;
    SetCloseButtonID(buttonCtrlStereoClose);
    SetScreenRectPos(&pCParent->mCScreenRect);

    // TCU[
    wxBoxSizer       *pBoxSizerSRC   = new wxBoxSizer(wxVERTICAL);
    wxBoxSizer       *pBoxSizerTOP   = new wxBoxSizer(wxHORIZONTAL);
    wxBoxSizer       *pBoxSizerParamL = new wxBoxSizer(wxVERTICAL);
    wxBoxSizer       *pBoxSizerBTM    = new wxBoxSizer(wxHORIZONTAL);

    SetMNumMainParam(pBoxSizerParamL);

    wxButtonEX *pCButtonDEF   = new wxButtonEX (this, buttonCtrlStereoDefault, 
						_("Set to Defalt"));
    wxButtonEX *pCButtonClose = new wxButtonEX (this, buttonCtrlStereoClose,
						wxT("Close"));

    pBoxSizerBTM->Add( pCButtonDEF,    0, wxEXPAND);
    pBoxSizerBTM->Add( -1, -1, 1, wxEXPAND);
    pBoxSizerBTM->Add( pCButtonClose,  0, wxEXPAND);

    pBoxSizerTOP->Add (pBoxSizerParamL, 0, wxEXPAND);
    pBoxSizerSRC->Add (pBoxSizerTOP, 0, wxEXPAND);
    pBoxSizerSRC->Add (pBoxSizerBTM, 0, wxEXPAND);

    SetSizerAndFit(pBoxSizerSRC);

    pCButtonClose->SetFocus();
}

StereoGramGUI_cls::
~StereoGramGUI_cls(){
}

void StereoGramGUI_cls::
SetMNumMainParam(wxBoxSizer *pBoxSizer){
    MultiNumericalInputsPanel_cls* pCurr = 
	InitializeMNumPanel(&mpCMNumMainParam, this);
    pCurr->mTextWidth = 140;

    pCurr->mComboWidth = 108;
    wxString strListType[3] = {
	_("0: Monogram"), wxT("1: Off-Axis"),wxT("2: Toe-In")};
    pCurr->CombinePointer(_GP_UNSIGNED_INT_LIST, &mspCParent->mCType.mBody,
			  _("Stereogram"), _GP_ENABLE, 3, strListType);

    wxString strListEye[6] = {
	_("No Parallax"), _("LEFT"), _("RIGHT"), _("LINE BY LINE"), _("RED-CYAN"), _("YELLOW-BLUE") };
    mIndexEye = pCurr->CombinePointer
	(_GP_UNSIGNED_INT_LIST, &mspCParent->mCEye.mBody,
	 _("Eye (Cam0)"), _GP_ENABLE, 6, strListEye);
    
    mIndexToeIn = pCurr->CombinePointer
	(_GP_DOUBLE, &mspCParent->mCToeInAngle.mBody, _("Toe-In Angle"));
    pCurr->SetRange(0, 45);

    mIndexDistance = pCurr->CombinePointer
	(_GP_DOUBLE, mspCParent->mCDistanceOfEyes.mpInterpolatedValue, _("Parallax Distance"));
    mIndexDistanceFocus = pCurr->CombinePointer
	(_GP_DOUBLE, &mspCParent->mDistanceToFocus, _("Distance to Focus"), _GP_DISABLE);

#ifdef WIN32
    pCurr->mMinimumWidth = 240;
#else
    pCurr->mMinimumWidth = 300;
#endif
    pCurr->FinishSetting(pBoxSizer, TRUE);

    pCurr->SetFuncHook(&FuncHookAssign, _MN_HOOK_ASSIGN);
    pCurr->SetFuncHook(&FuncHookComboBox, _MN_HOOK_COMBOBOX);
}

void StereoGramGUI_cls::
HideAndShowItems(){
    if (mspCParent->mCType.mBody == _STEREO_NONE){
	mpCMNumMainParam->ShowItems(mIndexEye, 1, FALSE);
	mpCMNumMainParam->ShowItems(mIndexToeIn, 1, FALSE);
	mpCMNumMainParam->ShowItems(mIndexDistance, 1, FALSE);
	mpCMNumMainParam->ShowItems(mIndexDistanceFocus, 1, FALSE);
    }
    else{
	mpCMNumMainParam->ShowItems(mIndexEye, 1, TRUE);
	if (mspCParent->mCEye.mBody == _STEREO_NONE){
	    mpCMNumMainParam->ShowItems(mIndexToeIn, 1, FALSE);
	    mpCMNumMainParam->ShowItems(mIndexDistance, 1, FALSE);
	    mpCMNumMainParam->ShowItems(mIndexDistanceFocus, 1, FALSE);
	}
	else{
	    if (mspCParent->mCType.mBody == _STEREO_TOEIN){
		mpCMNumMainParam->ShowItems(mIndexToeIn, 1, TRUE);
		mpCMNumMainParam->ShowItems(mIndexDistance, 1, TRUE);
		mpCMNumMainParam->ShowItems(mIndexDistanceFocus, 1, TRUE);
	    }
	    else{
		mpCMNumMainParam->ShowItems(mIndexToeIn, 1, FALSE);
		mpCMNumMainParam->ShowItems(mIndexDistance, 1, TRUE);
		mpCMNumMainParam->ShowItems(mIndexDistanceFocus, 1, FALSE);
	    }
	}
    }

    ResizeDialog();
}

void StereoGramGUI_cls::
OnButtonDefault(wxCommandEvent &event){
    if (MsgBoxYesno(this, _("Query"), _("Set to default ?")) == FALSE) return;
    mspCParent->SetToDefault();
    mspCParent->UpdateItems(_MN_NOFORCE);
    HideAndShowItems();
}


void StereoGramGUI_cls::
OnButtonClose(wxCommandEvent &event){
    mspCParent->UpdateItems(_MN_NOFORCE);
    CloseEXKey();
}

void StereoGramGUI_cls::
FuncHookComboBox(void *pArg, void *pEvent){
    mspCParent->UpdateItems();
    mspCParent->mpCGUI->HideAndShowItems();
    wxCommandEvent com1(wxEVT_COMMAND_MENU_SELECTED, menuItemRefreshCamCom);
    mspCParent->mpCParentFrame->AddPendingEvent(com1);
    wxCommandEvent com2(wxEVT_COMMAND_MENU_SELECTED, menuItemRefresh);
    mspCParent->mpCParentFrame->AddPendingEvent(com2);
}

void StereoGramGUI_cls::
FuncHookAssign(void *pArg, void *pEvent){
    mspCParent->mCDistanceOfEyes.SetKeyIfChanged(_PARAM_CURRENT_DINDEX);
    mspCParent->UpdateItems();
    mspCParent->mpCParentFrame->mbPathChanged = TRUE;
    wxCommandEvent dummy1(wxEVT_COMMAND_MENU_SELECTED, menuItemRefreshCamera);
    mspCParent->mpCParentFrame->AddPendingEvent(dummy1);
    wxCommandEvent dummy2(wxEVT_COMMAND_MENU_SELECTED, menuItemRefresh);
    mspCParent->mpCParentFrame->AddPendingEvent(dummy2);
}

StereoGram_cls    *StereoGramGUI_cls::mspCParent;

MultiNumericalInputsPanel_cls* StereoGramGUI_cls::mpCMNumMainParam;

// ---------------------

StereoGram_cls::
StereoGram_cls(FrameMain_cls *pC, wxBitmap *pBitmap) :
    mCDistanceOfEyes(1, L"DistanceOfEyes"),
    mCScreenRect(wxDefaultPosition, wxDefaultSize)
{
    mpCParentFrame = pC;
    mpBitmapLink = pBitmap;
    mpCGUI  = NULL;
    SetToDefault();
    double distanceOfEyes = 0.1;
    mCDistanceOfEyes.ExecCommandInitSet(&distanceOfEyes);
    InitGUI(pC); // GUI ̃CX^X
}

StereoGram_cls::
~StereoGram_cls(){
    if (mpCGUI != NULL) delete mpCGUI;
}

void StereoGram_cls::
UpdateItems(bool bForce){
    mDistanceToFocus = GetDistanceToFocus
	(mpCParentFrame->mpCParent->mpCTime->GetDoubleIndex());
    if (mpCGUI != NULL)
	mpCGUI->mpCMNumMainParam->UpdateValuesAll(bForce);
}

void StereoGram_cls::
SetToDefault(){
    mCConfList.Clear();
    mCConfList.Add(&mCType, L"StereogramType=", L"0");
    mCConfList.Add(&mCEye,  L"StereogramEye=", L"0");
    mCConfList.Add(&mCToeInAngle, L"StereoGramToeInAngle=", L"0.5");
    mCConfList.Add(&mCPitchAngle, L"StereoGramPitchAngle=", L"0");
    //mCConfList.Add(&mCDome, L"DomeDype=", L"0");
}

void StereoGram_cls::
InitGUI(wxWindow *pC){
    mpCGUI = new StereoGramGUI_cls(this, pC,
				   mCScreenRect.GetPosition(),
				   mCScreenRect.GetSize());
    SetPDialog(mpCGUI);
}


double StereoGram_cls::
GetStereogramYawAngle(){
    double angle = mCToeInAngle.mBody;
    if (mCType.mBody == _STEREO_NONE || mCType.mBody == _STEREO_OFFAXIS)
	angle = 0;
    if (mCEye.mBody == _STEREO_RGT) angle *= -1;
    return angle;
}

double StereoGram_cls::
GetDomePitchAngle(){
    return mCPitchAngle.mBody;
}

void StereoGram_cls::
DrawFocus(PositionAndDirection_cls CCamPos,
	  double doubleIndex, double scale){
    if (mCType.mBody == _STEREO_NONE) return;
    if (mCType.mBody == _STEREO_OFFAXIS) return;
    if (mCEye.mBody  == _STEREO_NONE) return;
    if (mCDistanceOfEyes.mCbVisible.mbBody == FALSE) return;
    double posFocus[3];
    LOOP_XYZ posFocus[xyz] = CCamPos.mPos[xyz] + 
	CCamPos.mTgt[xyz] * GetDistanceToFocus(doubleIndex);
    glPushMatrix();
    glTranslated(posFocus[0], posFocus[1], posFocus[2]);
    glScaled(scale * 0.03, scale * 0.03, scale * 0.03);
    GLOBJ::Object::Draw(_CAMERA_CROSSHAIR);
    glPopMatrix();
}

unsigned int StereoGram_cls::
GetEye(){
    return mCEye.mBody;
}

double StereoGram_cls::
GetDistanceToFocus(double doubleIndex){
    return *(mCDistanceOfEyes.GetPInterpolatedValue(doubleIndex)) * 0.5 / tan (mCToeInAngle.mBody * M_PI / 180);
}

double StereoGram_cls::
GetOffsetRight(){
    if (mCEye.mBody == _STEREO_NONE) return 0;
    double offset = *(mCDistanceOfEyes.mpInterpolatedValue);
    if (mCEye.mBody == _STEREO_LFT) offset *= -1;
    return offset;
}

void StereoGram_cls::
SetColorMask(int stereo, bool bFlip){
    if (bFlip == TRUE) stereo = 1 - stereo;
    
    if (mCType.mBody != _STEREO_NONE){
	if (mCEye.mBody == _STEREO_RED_CYAN)
	    switch(stereo){
	    case 0:
		glColorMask(1, 0, 0, 1); return;
	    case 1:
		glColorMask(0, 1, 1, 1); return;
	    }
	if (mCEye.mBody == _STEREO_YELLOW_BLUE)
	    switch(stereo){
	    case 0:
		glColorMask(1, 1, 0, 1); return;
	    case 1:
		glColorMask(0, 0, 1, 1); return;
	    }
    }
    glColorMask(1, 1, 1, 1);
}

bool StereoGram_cls::
IsGUIShownOnScreen(){
    if (mpCGUI == NULL) return FALSE;
    return mpCGUI->IsShownOnScreen();
}

void StereoGram_cls::
ReadStrLine(std::wistream &istream, std::wstring strLine){
    std::wstring::size_type loc = strLine.find(L"--StereoGram");
    if (loc != std::wstring::npos){
	std::wstring strLine;
	std::getline(istream, strLine);
	mCDistanceOfEyes.ReadStrLine(istream, strLine);
    }
    mCConfList.ReadStrLineAll(strLine);
}

wxString StereoGram_cls::
Serialize(){
    wxString sOut;
    sOut << wxT("--StereoGram\n");
    sOut << mCDistanceOfEyes.Serialize();
    sOut << mCConfList.SerializeAll();
    return sOut;
}
