
#include "clsExposure.h"
#include "clsExposureGUI.h"

#include "../../GL/glsl.h"

#include "../MYAPI/MsgBox.h"

#ifdef WIN32
#include <windows.h>
#include <GL/glu.h>
#include <GL/glext.h>
//PFNGLPROGRAMPARAMETERIEXTPROC glProgramParameteriEXT;
#elif defined __WXMAC__
#define GL_GLEXT_PROTOTYPES 
#include <OpenGL/glu.h>
#else
#define GL_GLEXT_PROTOTYPES 
#include <GL/glu.h>
#endif


Exposure_cls::
Exposure_cls() : mpCGUI(NULL){
    mCConfList.Clear();
    //mCConfList.Add(&mCExposure, wxT("ExposureExposure="), wxT("1"));
    mCConfList.Add(&mCLevel, wxT("ExposureLevel="), wxT("0 1"));
    mCConfList.Add(&mCGamma, wxT("ExposureGamma="), wxT("1"));
    mCConfList.Add(&mCHueOffset, wxT("ExposureHueOffset="), wxT("0"));
    mCConfList.Add(&mCSaturation, wxT("ExposureSaturation="), wxT("0"));
}

Exposure_cls::
~Exposure_cls(){
    if (mpCGUI != NULL) delete mpCGUI;
}

void Exposure_cls::
InstantiateGUI(wxWindow *pCParent){
    mpCGUI = new ExposureGUI_cls(this, pCParent);
}

void Exposure_cls::
Initialize(wxWindow *pCParent){
    
    //glProgramParameteriEXT = (PFNGLPROGRAMPARAMETERIEXTPROC)wglGetProcAddress("glProgramParameteriEXT");

    // VF[_vÕRpC^Nʂ𓾂ϐ
    GLint compiled, linked;
    
    mVertShader = glCreateShader(GL_VERTEX_SHADER);
    mFragShader = glCreateShader(GL_FRAGMENT_SHADER);
    //mGeomShader = glCreateShader(GL_GEOMETRY_SHADER);

    // VF[_̃\[XvO̓ǂݍ 
    if (readShaderSource(mVertShader, "GL/exposure.vert")) exit(1);
    if (readShaderSource(mFragShader, "GL/exposure.frag")) exit(1);   
    //if (readShaderSource(mGeomShader, "GL/simple.geom")) exit(1);

    // o[ebNXVF[_̃\[XvÕRpC 
    glCompileShader(mVertShader);
    glGetShaderiv(mVertShader, GL_COMPILE_STATUS, &compiled);
    if (compiled == GL_FALSE) {
	ShowLog(pCParent, mVertShader);	exit(1);
    }   

    // tOgVF[_̃\[XvÕRpC 
    glCompileShader(mFragShader);
    glGetShaderiv(mFragShader, GL_COMPILE_STATUS, &compiled);
    if (compiled == GL_FALSE) {
	ShowLog(pCParent, mFragShader);	exit(1);
    }

    /*
    // WIgVF[_̃\[XvÕRpC
    glCompileShader(mGeomShader);
    glGetShaderiv(mGeomShader, GL_COMPILE_STATUS, &compiled);
    if (compiled == GL_FALSE) {
	ShowLog(pCParent, mFragShader);	exit(1);
    }
    */
    
    // vOIuWFNg̍쐬 
    mGL2Program = glCreateProgram( );
    
    // VF[_IuWFNg̃VF[_vOւ̓o^ 
    //glAttachShader(mGL2Program, mGeomShader);
    glAttachShader(mGL2Program, mVertShader);
    glAttachShader(mGL2Program, mFragShader);

    // VF[_IuWFNg̍폜 
    //glDeleteShader(mGeomShader);
    glDeleteShader(mVertShader);
    glDeleteShader(mFragShader);

    /*
    // Op`͂ĎOp`o͂
    glProgramParameteriEXT(mGL2Program,GL_GEOMETRY_INPUT_TYPE_EXT,GL_QUADS);
    glProgramParameteriEXT(mGL2Program,GL_GEOMETRY_OUTPUT_TYPE_EXT,GL_QUADS);
    
    // WIgVF[_ŏo͂ő咸_̎擾Ɛݒ
    int temp;
    glGetIntegerv(GL_MAX_GEOMETRY_OUTPUT_VERTICES_EXT,&temp);
    glProgramParameteriEXT(mGL2Program,GL_GEOMETRY_VERTICES_OUT_EXT,temp);
    */

    // VF[_vÕN 
    glLinkProgram(mGL2Program);
    glGetProgramiv(mGL2Program, GL_LINK_STATUS, &linked);
    if (linked == GL_FALSE) {
	ShowLog(pCParent, mGL2Program);
	exit(1);
    }

    glUseProgram(mGL2Program);

    SetParamIDs(mGL2Program);
    SetParamTexture(0);
    
    glUseProgram(0);
    
}

void Exposure_cls::
ShowLog(wxWindow *pCParent, GLuint shader){
    int logSize;
    int length;
    glGetShaderiv(shader, GL_INFO_LOG_LENGTH , &logSize);
    char *pLog = new char [logSize];
    glGetShaderInfoLog(shader, logSize, &length, pLog);
    wxString strLog(pLog, wxConvFile);
    wxString strMes = wxT("Compile error in fragment shader.\n");
    strMes << strLog;
    MsgBox(pCParent, strLog, wxT("!!"));
    delete [] pLog;
}

void Exposure_cls::
Begin(){
    glUseProgram(mGL2Program);
    SetParamExposure();
}

void Exposure_cls::
End(){
    glUseProgram(0);
};

void Exposure_cls::
SetParamIDs(GLint program){
    mParamIDTexture  = glGetUniformLocation(program, "textureImage");
    mParamIDExposure = glGetUniformLocation(program, "exposure");
    mParamIDLevelMin = glGetUniformLocation(program, "levelMin");
    mParamIDLevelMax = glGetUniformLocation(program, "levelMax");
    mParamIDGamma    = glGetUniformLocation(program, "gamma");
    mParamIDHueOffset = glGetUniformLocation(program, "hueOffset");
    mParamIDSaturation = glGetUniformLocation(program, "saturation");
}

void Exposure_cls::
SetParamTexture(int value){
    glUniform1i(mParamIDTexture, value);
}

void Exposure_cls::
SetParamExposure(){
    //float exposure = mCExposure.mBody;
    float levelMin = mCLevel.mpBody[0];
    float levelMax = mCLevel.mpBody[1];
    float gamma = mCGamma.mBody;
    float hueOffset = mCHueOffset.mBody;
    float saturation = mCSaturation.mBody;

    //glUniform4f(mParamIDExposure, exposure, exposure, exposure, exposure);
    glUniform4f(mParamIDLevelMin, levelMin, levelMin, levelMin, levelMin);
    glUniform4f(mParamIDLevelMax, levelMax, levelMax, levelMax, levelMax);
    glUniform4f(mParamIDGamma, gamma, gamma, gamma, gamma);
    glUniform1i(mParamIDHueOffset, hueOffset);
    glUniform1i(mParamIDSaturation, saturation);
}

// ----------------

void Exposure_cls::
ReadStrLine(wxString strLine){
    mCConfList.ReadStrLineAll(strLine);
}

wxString Exposure_cls::
Serialize(){
    return mCConfList.SerializeAll();
}
