
#include "clsOverlay.h"
#include "clsOverlayGUI.h"

#include "../MYGL/clsFBO.h"
#include "../GLATTRIB/clsColorCurve.h"
#include "../PARAMETER/clsTics.h"

#ifdef WIN32
#include <windows.h>
#include <GL/glu.h>
#elif defined __WXMAC__
#define GL_GLEXT_PROTOTYPES 
#include <OpenGL/glu.h>
#else
#define GL_GLEXT_PROTOTYPES 
#include <GL/glu.h>
#endif


Overlay_cls::
Overlay_cls() : mpCGUI(NULL){
    mCConfList.Clear();
    mCConfList.Add(&mCbShowTime, wxT("OverLayShowTime="), wxT("0"));
    mCConfList.Add(&mCTimePosition, wxT("OverLayTimePos="), wxT("2"));
    mCConfList.Add(&mCbShowLegend, wxT("OverLayShowLegend="), wxT("1 0 0 0"));
    mCConfList.Add(&mCLegendPosition, wxT("OverLayLegendPos="), wxT("3"));
    mCConfList.Add(&mCLegendIgnoreAlpha, 
		   wxT("OverLayLegendIgnoreAlphe="), wxT("0"));
    mCConfList.Add(&mCLegendIgnoreTics, 
		   wxT("OverLayLegendIgnoreTics="), wxT("0"));
}

Overlay_cls::
~Overlay_cls(){
    if (mpCGUI != NULL) delete mpCGUI;
}

void Overlay_cls::
InstantiateGUI(wxWindow *pCParent){
    mpCGUI = new OverlayGUI_cls(this, pCParent);
}

void Overlay_cls::
ReadStrLine(wxString strLine){
    mCConfList.ReadStrLineAll(strLine);
}

wxString Overlay_cls::
Serialize(){
    return mCConfList.SerializeAll();
}

// -----

OverlayBase_cls::
OverlayBase_cls(Overlay_cls* pCOverlayLink){
    mpBufferRGBA = NULL;
    mpCOverlayLink = pCOverlayLink;
}

void OverlayBase_cls::
SetOffsetPos(int postype, int index, int width, int height){
    switch(postype){
    case _TOP_LEFT:
	mOffsetX = 20;
	mOffsetY = height - 36 - index * 36;
	break;
    case _TOP_RIGHT:
	mOffsetX = width - 240 - 20;
	mOffsetY = height - 36 - index * 36;
	break;
    case _BTM_LEFT:
	mOffsetX = 20;
	mOffsetY = 20 + index * 36;
	break;
    case _BTM_RIGHT:
	mOffsetX = width - 240 - 20;
	mOffsetY = 20 + index * 36;
	break;
    }
}

// cɕׂ邱ƂB͂
void OverlayBase_cls::
GenerateBufferRGBA(wxImage *pCImage, wxImage *pCImage2){
    int imgWidth   = pCImage->GetWidth();
    int imgHeight  = pCImage->GetHeight();
    int imgHeight2 = 0;
    mBufferWidth  = pCImage->GetWidth();
    mBufferHeight = pCImage->GetHeight();
    if (pCImage2 != NULL){
	imgHeight2 = pCImage2->GetHeight();
	mBufferHeight += pCImage2->GetHeight();
    }
    mpBufferRGBA = new unsigned char [mBufferWidth * mBufferHeight * 4];
    unsigned char *pData  = pCImage->GetData();
    for (int y = 0; y < imgHeight; y++){
	int indexy = mBufferWidth * y;
	for (int x = 0; x <  imgWidth; x++){
	    int index = indexy + x;
	    mpBufferRGBA[index * 4 + 0] = pData[index * 3 + 0];
	    mpBufferRGBA[index * 4 + 1] = pData[index * 3 + 1];
	    mpBufferRGBA[index * 4 + 2] = pData[index * 3 + 2];
	    if (pData[index * 3 + 0] == 0&&
		pData[index * 3 + 1] == 0&&
		pData[index * 3 + 2] == 0)
		mpBufferRGBA[index * 4 + 3] = 0;
	    else
		mpBufferRGBA[index * 4 + 3] = 255;
	}
    }

    if (pCImage2 == NULL) return;
    pData  = pCImage2->GetData();
    for (int y = 0; y < imgHeight2; y++){
	int indexy = mBufferWidth * y;
	int indexyBuf = mBufferWidth * (y + imgHeight);
	for (int x = 0; x < imgWidth; x++){
	    int index = indexy + x;
	    int indexBuf = indexyBuf + x;
	    mpBufferRGBA[indexBuf * 4 + 0] = pData[index * 3 + 0];
	    mpBufferRGBA[indexBuf * 4 + 1] = pData[index * 3 + 1];
	    mpBufferRGBA[indexBuf * 4 + 2] = pData[index * 3 + 2];
	    if (pData[index * 3 + 0] == 0&&
		pData[index * 3 + 1] == 0&&
		pData[index * 3 + 2] == 0)
		mpBufferRGBA[indexBuf * 4 + 3] = 0;
	    else
		mpBufferRGBA[indexBuf * 4 + 3] = 255;
	}
    }
}

void OverlayBase_cls::
DeleteBufferRGBA(){
    if (mpBufferRGBA != NULL) delete mpBufferRGBA;
    mpBufferRGBA  = NULL;
    mBufferWidth  = 0;
    mBufferHeight = 0;
}

// -----

DrawTimeIndex_cls::
DrawTimeIndex_cls(Overlay_cls* pCOverlayLink) :
    OverlayBase_cls(pCOverlayLink) 
{
    mOffsetX = 20;
    mOffsetY = 20;
}

DrawTimeIndex_cls::
~DrawTimeIndex_cls(){
    DeleteBufferRGBA();
}

void DrawTimeIndex_cls::
GetTimeOnBitmap(double time, wxBitmap *pCBitmap){

    wxString strTime;
    if (time == -9999)
	strTime << _("Time=") << wxT("Not Defined");
    else
	strTime << _("Time=") << time;
    
    wxMemoryDC memDC(*pCBitmap);
    memDC.SetBackground(*wxBLACK_BRUSH);
    memDC.Clear();
    memDC.SetTextForeground(*wxWHITE);
    memDC.DrawText(strTime, 0, 0);
}

void DrawTimeIndex_cls::
MakeTimeTexture(double time){
    wxBitmap CTimeBitmap(240, 40, -1);
    GetTimeOnBitmap(time, &CTimeBitmap);

    wxImage CTimeImage = CTimeBitmap.ConvertToImage();

    GenerateBufferRGBA(&CTimeImage);

    glBindTexture(GL_TEXTURE_2D, mTexID);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);

    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, 
		 mBufferWidth, mBufferHeight,
		 0, GL_RGBA, GL_UNSIGNED_BYTE, mpBufferRGBA);
    DeleteBufferRGBA();
}

void DrawTimeIndex_cls::
DrawTimeInfo(double time, int width, int height){
    if (mpCOverlayLink->mCbShowTime.mbBody == FALSE) return;

    MakeTimeTexture(time);

    glEnable(GL_TEXTURE_2D);
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE);
    glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
    
    FBO_cls::Push2DMatrix(width, height);
    
    SetOffsetPos(mpCOverlayLink->mCTimePosition.mBody, 0, width, height);
    
    glDisable(GL_DEPTH_TEST);
    glBegin(GL_QUADS);
    glTexCoord2f(1, 0); glVertex2d(mOffsetX+240, mOffsetY+40-14);
    glTexCoord2f(0, 0); glVertex2d(mOffsetX    , mOffsetY+40-14);
    glTexCoord2f(0, 1); glVertex2d(mOffsetX    , mOffsetY-14   );
    glTexCoord2f(1, 1); glVertex2d(mOffsetY+240, mOffsetY-14   );
    glEnd();
    
    FBO_cls::Pop2DMatrix();

    glDisable(GL_TEXTURE_2D);   
    glDisable(GL_BLEND);
}

void DrawTimeIndex_cls::
Initialize(){
    glGenTextures(1, &mTexID);
}

// -------

DrawClrLegend_cls::
DrawClrLegend_cls(Overlay_cls* pCOverlayLink) :
    OverlayBase_cls(pCOverlayLink){
    mOffsetX = 20;
    mOffsetY = 80;
}

DrawClrLegend_cls::
~DrawClrLegend_cls(){
    DeleteBufferRGBA();
}

void DrawClrLegend_cls::
GetClrCurveOnBitmap(ColorCurve_cls *pCClrCurve, wxBitmap *pCBitmap){
    wxMemoryDC memDC(*pCBitmap);
    pCClrCurve->DrawColorOnMemoryDC(memDC, FALSE, 20,
				    mpCOverlayLink->mCLegendIgnoreAlpha.mBody);
}

void DrawClrLegend_cls::
GetTicsOnBitmap(Tics_cls *pCTics, wxBitmap *pCBitmap){
    wxMemoryDC memDC(*pCBitmap);
    if (mpCOverlayLink->mCLegendIgnoreTics.mBody == TRUE){
	memDC.SetBackground(*wxBLACK_BRUSH);
	memDC.Clear();
	return;
    }
    pCTics->DrawTics(memDC);
}

void DrawClrLegend_cls::
MakeClrCurveTexture(ColorCurve_cls *pCClrCurve, int texID){
    wxBitmap CClrCurveBitmap(240, 20, -1);
    GetClrCurveOnBitmap(pCClrCurve, &CClrCurveBitmap);

    wxImage CClrCurveImage = CClrCurveBitmap.ConvertToImage();
    
    Tics_cls CTics;
    CTics.SetParameter(wxSize(240, 14), 
		       pCClrCurve->mCScalingMin.mBody,
		       pCClrCurve->mCScalingMax.mBody, 1);
    CTics.SetColorParameter(wxColour(0, 0, 0), wxColour(255, 255, 255), wxColour(192, 192, 192));
    wxBitmap CTicsBitmap(240, 14, -1);
    GetTicsOnBitmap(&CTics, &CTicsBitmap);
    
    wxImage CTicsImage = CTicsBitmap.ConvertToImage();

    GenerateBufferRGBA(&CTicsImage, &CClrCurveImage);

    glBindTexture(GL_TEXTURE_2D, texID);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);

    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, 
		 mBufferWidth, mBufferHeight,
		 0, GL_RGBA, GL_UNSIGNED_BYTE, mpBufferRGBA);
    DeleteBufferRGBA();
}

void DrawClrLegend_cls::
DrawClrLegend(ColorCurve_cls *pCClrCurve[4], int width, int height){

    glEnable(GL_TEXTURE_2D);
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
    
    FBO_cls::Push2DMatrix(width, height);

    int itemIndex = 0;
    for (int i = 0; i < 4; i++){
	if (mpCOverlayLink->mCbShowLegend.mpbBody[i] == FALSE) continue;
	MakeClrCurveTexture(pCClrCurve[i], mTexID[i]);

	SetOffsetPos(mpCOverlayLink->mCLegendPosition.mBody, itemIndex,
		     width, height);
	glDisable(GL_DEPTH_TEST);
	glBegin(GL_QUADS);
	glTexCoord2f(1, 0); glVertex2d(mOffsetX+240, mOffsetY+34);
	glTexCoord2f(0, 0); glVertex2d(mOffsetX    , mOffsetY+34);
	glTexCoord2f(0, 1); glVertex2d(mOffsetX    , mOffsetY);
	glTexCoord2f(1, 1); glVertex2d(mOffsetX+240, mOffsetY);
	glEnd();
	itemIndex++;
    }
    
    FBO_cls::Pop2DMatrix();
	
    glDisable(GL_TEXTURE_2D);   
    glDisable(GL_BLEND);
}

void DrawClrLegend_cls::
Initialize(){
    glGenTextures(4, mTexID);
}
