// "clsFBO.h"
// (setq c-basic-offset 4)

#ifdef WIN32
#include <windows.h>
#include <GL/glu.h>
#include <GL/glext.h>
PFNGLGENFRAMEBUFFERSEXTPROC glGenFramebuffersEXT;
PFNGLDELETEFRAMEBUFFERSEXTPROC glDeleteFramebuffersEXT;
PFNGLBINDFRAMEBUFFEREXTPROC glBindFramebufferEXT;
PFNGLFRAMEBUFFERTEXTURE2DEXTPROC glFramebufferTexture2DEXT;
PFNGLGENRENDERBUFFERSEXTPROC glGenRenderbuffersEXT;
PFNGLDELETERENDERBUFFERSEXTPROC glDeleteRenderbuffersEXT;
PFNGLBINDRENDERBUFFEREXTPROC glBindRenderbufferEXT;
PFNGLRENDERBUFFERSTORAGEEXTPROC glRenderbufferStorageEXT;
PFNGLFRAMEBUFFERRENDERBUFFEREXTPROC glFramebufferRenderbufferEXT;
PFNGLCHECKFRAMEBUFFERSTATUSEXTPROC glCheckFramebufferStatusEXT;
#elif defined __WXMAC__
#define GL_GLEXT_PROTOTYPES 
#include <OpenGL/glu.h>
#else
#define GL_GLEXT_PROTOTYPES 
#include <GL/glu.h>
#endif

#include "clsFBO.h"
#include "clsPositionAndDirection.h"

#ifndef TRUE
#define TRUE true
#define FALSE false
#endif

FBO_cls::
FBO_cls(int width, int height){
    if (msbInitialized == FALSE) Initialize();
    SetGeometryPanel(width, height);
    mWidthBuffer = 1024; mHeightBuffer = 1024;
    mbUsed = FALSE;
    mDepth = 16;
}

FBO_cls::
~FBO_cls(){
    // gpς eNX`AFBO 폜
    if (mbUsed == TRUE){
	glDeleteTextures(1, &mTexRGBId);
	glDeleteRenderbuffersEXT(1, &mRBId);
	glDeleteFramebuffersEXT(1, &mFBOId);
    }
}

void FBO_cls::
Initialize(){
#ifdef WIN32
    glGenFramebuffersEXT =
	(PFNGLGENFRAMEBUFFERSEXTPROC)wglGetProcAddress("glGenFramebuffersEXT");
    glDeleteFramebuffersEXT =
	(PFNGLDELETEFRAMEBUFFERSEXTPROC)wglGetProcAddress("glDeleteFramebuffersEXT");
    glBindFramebufferEXT =
	(PFNGLBINDFRAMEBUFFEREXTPROC)wglGetProcAddress("glBindFramebufferEXT");
    glFramebufferTexture2DEXT =
	(PFNGLFRAMEBUFFERTEXTURE2DEXTPROC)wglGetProcAddress("glFramebufferTexture2DEXT");
    glGenRenderbuffersEXT =
	(PFNGLGENRENDERBUFFERSEXTPROC)wglGetProcAddress("glGenRenderbuffersEXT");
    glDeleteRenderbuffersEXT =
	(PFNGLDELETERENDERBUFFERSEXTPROC)wglGetProcAddress("glDeleteRenderbuffersEXT");
    
    glBindRenderbufferEXT =
	(PFNGLBINDRENDERBUFFEREXTPROC)wglGetProcAddress("glBindRenderbufferEXT");
    glRenderbufferStorageEXT =
	(PFNGLRENDERBUFFERSTORAGEEXTPROC)wglGetProcAddress("glRenderbufferStorageEXT");
    glFramebufferRenderbufferEXT =
	(PFNGLFRAMEBUFFERRENDERBUFFEREXTPROC)wglGetProcAddress("glFramebufferRenderbufferEXT");
    glCheckFramebufferStatusEXT =
	(PFNGLCHECKFRAMEBUFFERSTATUSEXTPROC)wglGetProcAddress("glCheckFramebufferStatusEXT");
    if (glCheckFramebufferStatusEXT == NULL) msbSupported = FALSE;
#endif
    
    if (msbSupported == TRUE){
	GLenum status;                                            
	status = glCheckFramebufferStatusEXT(GL_FRAMEBUFFER_EXT); 
	if (status != GL_FRAMEBUFFER_COMPLETE_EXT)
	    msbSupported = FALSE;
    }

    msbInitialized = TRUE;
}

void FBO_cls::
SetGeometryBuffer(int width, int height){
    mWidthBuffer = width;
    mHeightBuffer = height;
}

void FBO_cls::
SetGeometryPanel(int width, int height){
    mWidthPanel = width;
    mHeightPanel = height;
}

void FBO_cls::
SetDepth(int depth){
    mDepth = depth;
}

void FBO_cls::
MakeFBO(bool bUseRGBA){
    glGenTextures(1, &mTexRGBId);
    glBindTexture( GL_TEXTURE_2D, mTexRGBId);

    switch (mDepth){
    case 16:
	if (bUseRGBA == TRUE)
	    glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA16F_ARB, 
			  mWidthBuffer, mHeightBuffer, 0,
			  GL_RGBA, GL_HALF_FLOAT_ARB, 0);
	else
	    glTexImage2D( GL_TEXTURE_2D, 0, GL_RGB16F_ARB, 
			  mWidthBuffer, mHeightBuffer, 0,
			  GL_RGB, GL_HALF_FLOAT_ARB, 0);
	break;
    case 8:
	if (bUseRGBA == TRUE)
	    glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA, 
			  mWidthBuffer, mHeightBuffer, 0,
			  GL_RGBA, GL_UNSIGNED_BYTE, 0);
	else
	    glTexImage2D( GL_TEXTURE_2D, 0, GL_RGB, 
			  mWidthBuffer, mHeightBuffer, 0,
			  GL_RGB, GL_UNSIGNED_BYTE, 0);
	break;
    }
    
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP );
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP );

    // Depth ێ郌_[obt@𐶐č쐬
    glGenRenderbuffersEXT(1, &mRBId); 
    glBindRenderbufferEXT(GL_RENDERBUFFER_EXT, mRBId);
    glRenderbufferStorageEXT(GL_RENDERBUFFER_EXT, //GL_DEPTH_COMPONENT,
			     GL_DEPTH_STENCIL_EXT,
			     mWidthBuffer, mHeightBuffer);
    
    // FBO̍쐬
    glGenFramebuffersEXT(1, &mFBOId);
}

void FBO_cls::
Begin(bool bClear){
    glPushAttrib(GL_ALL_ATTRIB_BITS);

    glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, mFBOId);
    
    // FBOɃeNX`IuWFNgƃ_[obt@ 
    glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT,
			      GL_TEXTURE_2D, mTexRGBId, 0);
    glFramebufferRenderbufferEXT(GL_FRAMEBUFFER_EXT, GL_DEPTH_ATTACHMENT_EXT,
				 GL_RENDERBUFFER_EXT, mRBId);
    glFramebufferRenderbufferEXT(GL_FRAMEBUFFER_EXT, GL_STENCIL_ATTACHMENT_EXT,
				 GL_RENDERBUFFER_EXT, mRBId);

    // 
    glBindRenderbufferEXT(GL_RENDERBUFFER_EXT, 0);
    
    // ̂܂ܕ`FBOɈڂ
    glViewport(0, 0, mWidthBuffer, mHeightBuffer);

    mbUsed = TRUE;

    glClearColor(0,0,0,0);
    if (bClear == TRUE)
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_ACCUM_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);
}

void FBO_cls::
End(){
    // FBOւ̌̉
    glBindFramebufferEXT( GL_FRAMEBUFFER_EXT, mFBOId);
    glFramebufferTexture2DEXT( GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT,
			       GL_TEXTURE_2D, 0, 0 );
    glFramebufferRenderbufferEXT(GL_FRAMEBUFFER_EXT, GL_DEPTH_ATTACHMENT_EXT,
				 GL_RENDERBUFFER_EXT, 0);
    glFramebufferRenderbufferEXT(GL_FRAMEBUFFER_EXT, GL_STENCIL_ATTACHMENT_EXT,
				 GL_RENDERBUFFER_EXT, 0);
    glBindFramebufferEXT( GL_FRAMEBUFFER_EXT, 0 );

    glPopAttrib();
}

void FBO_cls::
MakeCirclePatternOnStencil(){
    //ꂩ`悷̂̃XeVlɂׂĂP^O    
    glStencilFunc( GL_ALWAYS, 1, ~0); 
    glStencilOp(GL_REPLACE,GL_REPLACE,GL_REPLACE);
    
    //XeVȊOɂ͏܂Ȃ
    glColorMask(0,0,0,0);
    glDepthMask(0);
    
    glBegin(GL_TRIANGLE_FAN);
    glVertex3d(0, 0, 0);
    for (int i = 0; i <= 72; i++){
	double rad = (double)i / 72 * 2 * 3.1416;
	glVertex3d(0.5 * cos(rad), 0.5 * sin(rad), 0);
    }
    glEnd();

    //̉
    glColorMask(1,1,1,1);
    glDepthMask(1);
    glStencilOp(GL_KEEP,GL_KEEP,GL_KEEP);
}

void FBO_cls::
DrawDomeTexSegment(double coordu, double coordv0, double coordv1,
		   double x0, double x1, double y0, double y1, 
		   double z0, double z1, double deg){
    double r0 = sqrt(x0*x0 + y0*y0 + z0*z0);
    double r1 = sqrt(x1*x1 + y1*y1 + z1*z1);
    double theta0 = asin(z0/r0);
    double theta1 = asin(z1/r1);
    double phi0 = atan2(y0, x0);
    double phi1 = atan2(y1, x1);

    glTexCoord2f(coordu, coordv1);
    glVertex3d((1.5708 - theta1) / 3.1415 * 180 / deg * cos(phi1), 
	       (1.5708 - theta1) / 3.1415 * 180 / deg * sin(phi1), 0);
    glTexCoord2f(coordu, coordv0);
    glVertex3d((1.5708 - theta0) / 3.1415 * 180 / deg * cos(phi0), 
	       (1.5708 - theta0) / 3.1415 * 180 / deg * sin(phi0), 0);
}

void FBO_cls::
Push2DMatrix(int width, int height){
    glMatrixMode(GL_PROJECTION);
    glPushMatrix();
    glViewport( 0, 0, width, height);
    glLoadIdentity();
    gluOrtho2D(0, width, 0, height);
    glMatrixMode(GL_MODELVIEW);
    glPushMatrix();
    glLoadIdentity();
}

void FBO_cls::
Pop2DMatrix(){
    glPopMatrix();
    glMatrixMode(GL_PROJECTION);
    glPopMatrix();
    glMatrixMode(GL_MODELVIEW); 
}

void FBO_cls::
DrawOnScreen2D(FBO_cls *pCFBO, 
	       int clientWidth, int clientHeight,
	       int clipWidth, int clipHeight,
	       int offsetX, int offsetY){

    // `ʏ
    //glBindFramebufferEXT( GL_FRAMEBUFFER_EXT, 0 );
    
    glClear(GL_DEPTH_BUFFER_BIT);
    glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
    
    glPushAttrib(GL_ALL_ATTRIB_BITS);
    
    Push2DMatrix(clientWidth, clientHeight);

    glEnable(GL_TEXTURE_2D);
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

    glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S , GL_CLAMP);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T , GL_CLAMP);

    glAlphaFunc(GL_GREATER, 0.5);
    glEnable(GL_ALPHA_TEST);

    // ʏ̕`
    glBindTexture( GL_TEXTURE_2D, pCFBO->mTexRGBId);
    
    glDisable(GL_CULL_FACE);

    glBegin(GL_QUADS);
    glTexCoord2f(0, 0); glVertex2d( offsetX, offsetY);
    glTexCoord2f(1, 0); glVertex2d( offsetX + clipWidth, offsetY);
    glTexCoord2f(1, 1); glVertex2d( offsetX + clipWidth, offsetY + clipHeight);
    glTexCoord2f(0, 1); glVertex2d( offsetX, offsetY + clipHeight);
    glEnd();

    glDisable(GL_ALPHA_TEST);

    Pop2DMatrix();
    glPopAttrib();
}

void FBO_cls::
DrawOnScreen(FBO_cls *pCFBO[4], int num, GLPerspective_cls CGLPerspective,
	     int clipWidth, int clipHeight, bool bAdd){
    double deg = CGLPerspective.mDeg;
    
    if (bAdd == FALSE){
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_ACCUM_BUFFER_BIT | GL_STENCIL_BUFFER_BIT );
    }
    glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
    
    glPushAttrib(GL_ALL_ATTRIB_BITS);
    
    glPushMatrix();

    CGLPerspective.SetWindowSize(clipWidth, clipHeight);
    CGLPerspective.SetClipSize(clipWidth, clipHeight);
    PositionAndDirection_cls CPos(0, 0, -1);

    // clipHeightΉ猩邱Ƃŏ̈ɕ`
    //PositionAndDirection_cls CPos(0, 0, -(double)CGLPerspective.mHeight / clipHeight);
    //PositionAndDirection_cls CPos(0, 0, -2);
    
    CGLPerspective.mbPerspective = FALSE;
    CGLPerspective.SetViewportOnGL();
    CGLPerspective.SetPerspectiveOnGL(&CPos);
    CGLPerspective.SetModelOrProjectionViewOnGL(&CPos, GL_MODELVIEW);
   
    glEnable(GL_TEXTURE_2D);

    glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S , GL_CLAMP);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T , GL_CLAMP);

    if (bAdd == TRUE){
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE);
    }

    double aspect = CGLPerspective.GetClipAspectRatio(); 
    if (num == 1){
	// ʏ̕`
	glBindTexture( GL_TEXTURE_2D, pCFBO[0]->mTexRGBId);
	
	glBegin(GL_QUADS);
	glTexCoord2f(0, 0); glVertex3d(-0.5 * aspect, -0.5 * aspect, 0);
	glTexCoord2f(1, 0); glVertex3d( 0.5 * aspect, -0.5 * aspect, 0);
	glTexCoord2f(1, 1); glVertex3d( 0.5 * aspect, 0.5 * aspect, 0);
	glTexCoord2f(0, 1); glVertex3d(-0.5 * aspect, 0.5 * aspect, 0);
	glEnd();
    }

    if (num == 4){
	// h[̕`

	glEnable(GL_STENCIL_TEST);
	MakeCirclePatternOnStencil();
	glStencilFunc( GL_EQUAL, 1, ~0); //1̃sNZɕ`

	double sqrt2 = sqrt(2.0);
	int v_max = 80, u_max = 80;

	glBindTexture( GL_TEXTURE_2D, pCFBO[0]->mTexRGBId);
	
	for (int v = 0; v < v_max; v++){
	    double coordv0 = (double)v / v_max;
	    double coordv1 = (double)(v+1) / v_max;
	    glBegin(GL_QUAD_STRIP);
	    for (int u = 0; u <= u_max; u++){
		double coordu = (double)u / u_max;
		double x  = (coordu - 1) * sqrt2; 
		double y0 = coordv0 * 2 - 1;
		double y1 = coordv1 * 2 - 1;
		double z =  coordu * sqrt2;
		DrawDomeTexSegment(coordu, coordv0, coordv1,
				   x, x, y0, y1, z, z, deg);
	    }
	    glEnd();
	}

	glBindTexture( GL_TEXTURE_2D, pCFBO[1]->mTexRGBId);
	for (int v = 0; v < v_max; v++){
	    double coordv0 = (double)v / v_max;
	    double coordv1 = (double)(v+1) / v_max;
	    glBegin(GL_QUAD_STRIP);
	    for (int u = 0; u <= u_max; u++){
		double coordu = (double)u / u_max;
		double x  = coordu * sqrt2; 
		double y0 = coordv0 * 2 - 1;
		double y1 = coordv1 * 2 - 1;
		double z = (1.0 - coordu) * sqrt2;
		DrawDomeTexSegment(coordu, coordv0, coordv1,
				   x, x, y0, y1, z, z, deg);
	    }
	    glEnd();
	}

	glBindTexture( GL_TEXTURE_2D, pCFBO[2]->mTexRGBId);
	for (int v = 0; v < v_max; v++){
	    double coordv0 = (double)v / v_max;
	    double coordv1 = (double)(v+1) / v_max;
	    glBegin(GL_QUAD_STRIP);
	    for (int u = 0; u <= u_max; u++){
		double coordu = (double)u / u_max;
		double x0 = (coordu + coordv0 - 1) * sqrt2; 
		double x1 = (coordu + coordv1 - 1) * sqrt2; 
		double y = 1;
		double z0 = (coordu - coordv0) * sqrt2;
		double z1 = (coordu - coordv1) * sqrt2;
		DrawDomeTexSegment(coordu, coordv0, coordv1,
				   x0, x1, y, y, z0, z1, deg);
	    }
	    glEnd();
	}

	glBindTexture( GL_TEXTURE_2D, pCFBO[3]->mTexRGBId);
	for (int v = 0; v < v_max; v++){
	    double coordv0 = (double)v / v_max;
	    double coordv1 = (double)(v+1) / v_max;
	    glBegin(GL_QUAD_STRIP);
	    for (int u = 0; u <= u_max; u++){
		double coordu = (double)u / u_max;
		double x0 = (coordu + coordv0 - 1) * sqrt2; 
		double x1 = (coordu + coordv1 - 1) * sqrt2; 
		double y = -1;
		double z0 = (-coordu + coordv0) * sqrt2;
		double z1 = (-coordu + coordv1) * sqrt2;
		DrawDomeTexSegment(coordu, coordv0, coordv1,
				   x0, x1, y, y, z0, z1, deg);
	    }
	    glEnd();
	}

	// EГh LFT
	glBindTexture( GL_TEXTURE_2D, pCFBO[0]->mTexRGBId);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST );
	double coordv0 = -0.001 * 0.3;
	double coordv1 = 0.003 * 0.3;
	glBegin(GL_QUAD_STRIP);
	for (int u = 0; u <= u_max; u++){
	    double coordu = (double)u / u_max;
	    double x  = (coordu - 1) * sqrt2; 
	    double y0 = coordv0 * 2 - 1;
	    double y1 = coordv1 * 2 - 1;
	    double z =  coordu * sqrt2;
	    DrawDomeTexSegment(coordu, coordv0, coordv1,
			       x, x, y0, y1, z, z, deg);
	}
	glEnd();

	// EГh RGT
	glBindTexture( GL_TEXTURE_2D, pCFBO[1]->mTexRGBId);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST );
	coordv0 = 1 - 0.001 * 0.3;
	coordv1 = 1 + 0.003 * 0.3;
	glBegin(GL_QUAD_STRIP);
	for (int u = 0; u <= u_max; u++){
	    double coordu = (double)u / u_max;
	    double x  = coordu * sqrt2; 
	    double y0 = coordv0 * 2 - 1;
	    double y1 = coordv1 * 2 - 1;
	    double z = (1.0 - coordu) * sqrt2;
	    DrawDomeTexSegment(coordu, coordv0, coordv1,
			       x, x, y0, y1, z, z, deg);
	}
	glEnd();
	
	// EГh TOP
	glBindTexture( GL_TEXTURE_2D, pCFBO[2]->mTexRGBId);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST );
	coordv0 = -0.001 * 0.3;
	coordv1 = 0.003 * 0.3;
	glBegin(GL_QUAD_STRIP);
	for (int u = 0; u <= u_max; u++){
	    double coordu = (double)u / u_max;
	    double x0 = (coordu + coordv0 - 1) * sqrt2; 
	    double x1 = (coordu + coordv1 - 1) * sqrt2; 
	    double y = 1;
	    double z0 = (coordu - coordv0) * sqrt2;
	    double z1 = (coordu - coordv1) * sqrt2;
	    DrawDomeTexSegment(coordu, coordv0, coordv1,
			       x0, x1, y, y, z0, z1, deg);
	}
	glEnd();

	// EГh BTM
	glBindTexture( GL_TEXTURE_2D, pCFBO[3]->mTexRGBId);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST );
	coordv0 = 1 - 0.001 * 0.3;
	coordv1 = 1 + 0.003 * 0.3;
	glBegin(GL_QUAD_STRIP);
	for (int u = 0; u <= u_max; u++){
	    double coordu = (double)u / u_max;
	    double x0 = (coordu + coordv0 - 1) * sqrt2; 
	    double x1 = (coordu + coordv1 - 1) * sqrt2; 
	    double y = -1;
	    double z0 = (-coordu + coordv0) * sqrt2;
	    double z1 = (-coordu + coordv1) * sqrt2;
	    DrawDomeTexSegment(coordu, coordv0, coordv1,
			       x0, x1, y, y, z0, z1, deg);
	}
	glEnd();

	glDisable(GL_STENCIL_TEST);
    }

    glPopMatrix();
    glPopAttrib();
}

float FBO_cls::
GetLineWidthFactor(){
    switch(mWidthBuffer){
    case 256:
	return 0.125;
    case 512:
	return 0.25;
    case 1024: 
	return 0.5;
    case 2048:
	return 1.0;
    case 4096:
	return 2.0;
    }
    return 1;
}

bool FBO_cls::msbInitialized = FALSE;
bool FBO_cls::msbSupported = TRUE;

