// -*- c -*-
//(setq c-basic-offset 4)

#include "clsGLPerspective.h"

#ifndef TRUE
#define TRUE true
#define FALSE false
#endif

#define X 0
#define Y 1
#define Z 2

GLPerspective_cls::
GLPerspective_cls(int width, int height, double deg, 
		  double depthMIN, double depthMAX){
    mbPerspective = TRUE;
    mDeg = deg;
    mDepthMIN = depthMIN;
    mDepthMAX = depthMAX;
    mWidth = width; 
    mHeight = height;
    mClipWidth = width;
    mClipHeight = height;
}

double GLPerspective_cls::
GetAspectRatio(){
    return (double)mWidth / mHeight;
}

double GLPerspective_cls::
GetClipAspectRatio(){
    return (double)mClipWidth / mClipHeight;
}

void GLPerspective_cls::
SetViewportOnGL(){
    glViewport(0, 0, (GLint) mWidth, (GLint) mHeight);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
}

// ^lpFBOւ̏o͂
void GLPerspective_cls::
SetPerspectiveOnGL(PositionAndDirection_cls* pCConfigure){
    if (mbPerspective){
	double degree = mDeg;
	if (GetClipAspectRatio() > 1){
	    double clipAspect = GetClipAspectRatio();
	    double tanDegHalf = tan(mDeg*3.14159/360);
	    degree = 2 * atan ( tanDegHalf * clipAspect) * 180 / 3.14159;
	}
	gluPerspective(degree, 1.0, mDepthMIN, mDepthMAX);
    }
    else{
	float vsize = (float)(pCConfigure->GetDistanceToAnc() * 0.5);
	glOrtho(-vsize * GetAspectRatio(), vsize * GetAspectRatio(),
		-vsize, vsize, 
		-100, 100);
    }
}

void GLPerspective_cls::
SetModelOrProjectionViewOnGL(PositionAndDirection_cls* pCConfigure, GLenum mode){
    double mat[16];
    glMatrixMode(mode);
    glLoadIdentity();
    
    mat[0]  =  pCConfigure->mRgt[X];
    mat[1]  =  pCConfigure->mTop[X];
    mat[2]  = -pCConfigure->mTgt[X];
    mat[3]  = 0;
    mat[4]  =  pCConfigure->mRgt[Y];
    mat[5]  =  pCConfigure->mTop[Y];
    mat[6]  = -pCConfigure->mTgt[Y];
    mat[7]  = 0;
    mat[8]  =  pCConfigure->mRgt[Z];
    mat[9]  =  pCConfigure->mTop[Z];
    mat[10] = -pCConfigure->mTgt[Z];
    mat[11] = mat[12] = mat[13] = mat[14] = 0;
    mat[15] = 1;
    glMultMatrixd(mat);
    
    glTranslated(-pCConfigure->mPos[X],
		 -pCConfigure->mPos[Y],
		 -pCConfigure->mPos[Z]);
}

void GLPerspective_cls::
SetWindowSize(int width, int height){
    mWidth  = width;
    mHeight = height;
}

void GLPerspective_cls::
SetClipSize(int width, int height){
    mClipWidth = width;
    mClipHeight = height;
}

void GLPerspective_cls::
Push2DMatrix(int width, int height){
    glMatrixMode(GL_PROJECTION);
    glPushMatrix();
    glViewport( 0, 0, width, height);
    glLoadIdentity();
    gluOrtho2D(0, width, 0, height);
    glMatrixMode(GL_MODELVIEW);
    glPushMatrix();
    glLoadIdentity();
}

void GLPerspective_cls::
Pop2DMatrix(){
    glPopMatrix();
    glMatrixMode(GL_PROJECTION);
    glPopMatrix();
    glMatrixMode(GL_MODELVIEW); 
}
