// -*- c -*-
// "clsKeyFrame.h"

#include "clsKeyFrame.h"

#include <fstream>
#include <sstream>

#ifndef TRUE
#define TRUE true
#define FALSE false
#endif

#ifdef WIN32
#define ENDLWX wxT("\r\n")
#else
#define ENDLWX wxT("\n")
#endif

#ifdef _DEBUG
#include <stdlib.h>
#include <crtdbg.h>
#define new new(_NORMAL_BLOCK, __FILE__, __LINE__)
#endif

KeyFrame_cls::
KeyFrame_cls() : 
mType(_PARAM_HERMITE_AUTO) 
{
    mValue   = 0;
    mDValue1 = 0;
    mDValue2 = 0;
    mbSelected = FALSE;
}

void KeyFrame_cls::
FullCopy(const KeyFrame_cls& obj){
    mbSelected   = obj.mbSelected;
    mType        = obj.mType;
    mDoubleIndex = obj.mDoubleIndex;
    mValue  = obj.mValue;
    mDValue1 = obj.mDValue1;
    mDValue2 = obj.mDValue2;
}

int KeyFrame_cls::operator > (const KeyFrame_cls obj){
    return mDoubleIndex > obj.mDoubleIndex; 
}

void KeyFrame_cls::
Set(double doubleIndex, unsigned int type, 
    double value, double DValue1, double DValue2, bool bSelected){
    mDoubleIndex = doubleIndex;
    mType = type;
    mbSelected = bSelected;
    mValue = value;
    mDValue1 = DValue1;
    mDValue2 = DValue2;
}

void KeyFrame_cls::
SetValue(double value, double DValue1, double DValue2){
    mValue = value;
    mDValue1 = DValue1;
    mDValue2 = DValue2;
}

bool KeyFrame_cls::
Select(double doubleIndex, double resolution, bool bTest){
    if (doubleIndex < mDoubleIndex + resolution &&
	doubleIndex > mDoubleIndex - resolution){
	if (bTest == FALSE) mbSelected = TRUE;
	return TRUE; }
    return FALSE;
}

bool KeyFrame_cls::
Toggle(double doubleIndex, double resolution){
    if (doubleIndex < mDoubleIndex + resolution &&
	doubleIndex > mDoubleIndex - resolution){
	mbSelected = !mbSelected;
	return TRUE;
    }
    return FALSE;
}

void KeyFrame_cls::
DrawKey(wxAutoBufferedPaintDC &dc, int x, int mode, bool bOnPanel){
    switch (mode){
    case _KEY_NORMAL:
	if (mbSelected == TRUE)
	    dc.SetBrush(wxBrush(wxT("YELLOW")));
	else
	    dc.SetBrush(*wxGREY_BRUSH);
	break;
    case _KEY_DRAG:
	if (bOnPanel == TRUE){
	    dc.SetPen(*wxGREY_PEN);
	    dc.SetBrush(*wxGREY_BRUSH);
	}
	else{
	    dc.SetPen(*wxLIGHT_GREY_PEN);
	    dc.SetBrush(*wxTRANSPARENT_BRUSH);
	}
    }

    static wxPoint ADiaPoints[4] = {
	wxPoint(-4, 0), wxPoint(0, -4), wxPoint(4, 0), wxPoint(0, 4)};

    switch(mType){
    case _PARAM_LINEAR:
	dc.DrawPolygon(4, ADiaPoints, x, 8); 
	break;
    case _PARAM_HERMITE_AUTO:
	dc.DrawRoundedRectangle(x-3, 5, 6, 6, 2); 
	break;
    case _PARAM_HERMITE_ADJ:
	dc.DrawCircle(x, 8, 4); 
	break;
    case _PARAM_HERMITE:
	dc.DrawCircle(x, 8, 4); dc.DrawCircle(x, 8, 2); 
	break;
    case _PARAM_STEP:
	dc.DrawRectangle(x-4, 5, 8, 8); 
	break;
    }
};

wxString KeyFrame_cls::
Serialize(){
    wxString strOut;
    wxString strDim, strType, strValue, strDValue1, strDValue2;
    strDim    .Printf(wxT("%.9lf"), mDoubleIndex);
    strType   .Printf(wxT("%d"), mType);
    strValue  .Printf(wxT("%f"), mValue);
    strDValue1.Printf(wxT("%f"), mDValue1);
    strDValue2.Printf(wxT("%f"), mDValue2);
    strOut << strDim << wxT(" ");
    strOut << strType << wxT(" ");
    strOut << strValue << wxT(" ");
    strOut << strDValue1 << wxT(" ");
    strOut << strDValue2 << wxT(" ") << ENDLWX;
    return strOut;
}

void KeyFrame_cls::
ReadStrLine(wxString strLine){
    std::wistringstream istream(strLine.c_str());
    istream >> mDoubleIndex >> mType >> mValue >> mDValue1 >> mDValue2;
}

wxString KeyFrame_cls::msAstrInterp[5] = {
    _("LINEAR"), _("HERMITE (AUTO)"), wxT("HERMITE (SEMIAUTO)"),
    _("HERMITE"), _("STEP")};
