// -*- c -*-
// "clsParamGroup.h"

#include "clsParamGroup.h"
#include "clsParamGroupGUI.h"

#include "clsParameters.h"
#include "clsParametersPanel.h"
#include "clsParamTL.h"

#include "../MAINCANVAS/clsParamGUICtrl.h"
#include "../SCENEOBJ/clsSceneObj.h"
#include "../WIDGETCTRL/clsBitmapPanel.h"
#include "../WIDGETCTRL/clsMultiInputPanel.h"
#include "../WIDGETCTRL/clsHookCtrl.h"
#include "../UNDO/clsUndoManager.h"
extern UndoManager_cls GCUndoList;

#include "../../wxWidgetEnumMenu.h"

#ifdef WIN32
#define ENDLWX wxT("\r\n")
#else
#define ENDLWX wxT("\n")
#endif

ParamGroup_cls::
ParamGroup_cls(wxString strGroutpLabel,
	       int dimension, wxString *pstrSubLabel, 
	       int updateBit, bool bTimeDependent) {
    mpCGUI = NULL;
    mControlMode = -1;
    mstrGroupLabel = strGroutpLabel;
    wxString strDimension; 
    strDimension << dimension;
    mCConfList.Add(&mCDimension, wxT("dimension="), strDimension);
    
    for (int i = 0; i < dimension; i++){
	mVCParam.push_back(Parameters_cls(pstrSubLabel[i], updateBit, bTimeDependent));
	mVCParam[i].mpCParentGroupLink = this;
    }
    
    mbGroupingGroup = FALSE;
    mbLinking = TRUE;
    //mbVisible = TRUE;
    mbShownOnTL = TRUE;
    if (dimension == 0){
	mbGroupingGroup = TRUE;
	//mbShownOnTL = TRUE;
    }
    mpCSceneObjLink = NULL;
}

void ParamGroup_cls::
MakeParamGUI(ParamGUICtrl_cls *pCParent, wxBoxSizer *pCBoxSizer, int depth){
    mpCGUI = new ParamGroupGUI_cls
	(this, pCParent, mstrGroupLabel, depth, wxID_ANY, 
	 wxDefaultPosition, wxSize(-1, 12));
    
    for (int i = 0; i < mCDimension.mBody; i++){
	mVpCParamGUI.push_back
	    (new ParametersPanel_cls
	     (&mVCParam[i], pCParent, TRUE,
	      wxDefaultPosition, wxSize(-1, 16), wxID_ANY));
    }

    pCBoxSizer->Add(mpCGUI, 0, wxEXPAND);
    for (int i = 0; i < mCDimension.mBody; i++){
	pCBoxSizer->Add(mVpCParamGUI[i], 0, wxEXPAND);
    }
}

void ParamGroup_cls::
HideItems(){
    for (int i = 0; i < mCDimension.mBody; i++){
	mVpCParamGUI[i]->Show(FALSE);
    }

    for (int i = 0; i < mVpCParamGroup.size(); i++){
	mVpCParamGroup[i]->mpCGUI->Show(FALSE);
	mVpCParamGroup[i]->HideItems();
    }

    mpCGUI->mpCBPanelFolder->mState = 1;

    mpCGUI->mpCCtrlLink->ResizeTL();
}

void ParamGroup_cls::
ShowItems(){
    for (int i = 0; i < mCDimension.mBody; i++){
	mVpCParamGUI[i]->Show(TRUE);
    }

    for (int i = 0; i < mVpCParamGroup.size(); i++){
	mVpCParamGroup[i]->mpCGUI->Show(TRUE);
	if (mVpCParamGroup[i]->mpCGUI->mpCBPanelFolder->mState == 0)
	    mVpCParamGroup[i]->ShowItems();
    }

    mpCGUI->mpCCtrlLink->ResizeTL();
}

void ParamGroup_cls::
RefreshPanels(){
    for (int i = 0; i < mVpCParamGroup.size(); i++)
	mVpCParamGroup[i]->RefreshPanels();

    if (mbShownOnTL == FALSE) return;

    for (int i = 0; i < mCDimension.mBody; i++){
	if (mpCGUI->mpCBPanelFolder->mState == 0){
	    mVpCParamGUI[i]->Refresh();
	    mVpCParamGUI[i]->mpCParamTL->Refresh();
	}
    }
}

bool ParamGroup_cls::
SelectObject(SceneObj_cls* pCSceneObj){
    bool bRet = FALSE;
    
    if (mbGroupingGroup == TRUE)
	for (unsigned int i = 0; i < mVpCParamGroup.size(); i++)
	    bRet |= mVpCParamGroup[i]->SelectObject(pCSceneObj);
    
    if (mpCSceneObjLink == NULL) return bRet;

    if (pCSceneObj == mpCSceneObjLink){
	if (mpCSceneObjLink->mbSelected == FALSE) {
	    mpCSceneObjLink->mbSelected = TRUE;
	    mpCSceneObjLink->mbDragging = FALSE;
	    mpCGUI->SetParamBackgroundColour();
	    bRet = TRUE;
	}
    }
    else{ 
	if (mpCSceneObjLink->mbSelected == TRUE){
	    mpCSceneObjLink->mbSelected = FALSE;
	    mpCGUI->SetParamBackgroundColour();
	    bRet = TRUE;
	}
    }
    
    return bRet;
}

SceneObj_cls* ParamGroup_cls::
GetPCObject(int objectIndex){
    SceneObj_cls *pCSelectedObject = NULL;
    
    if (mbGroupingGroup == TRUE)
	for (unsigned int i = 0; i < mVpCParamGroup.size(); i++)
	    pCSelectedObject = mVpCParamGroup[i]->GetPCObject(objectIndex);
    
    if (mpCSceneObjLink == NULL) return pCSelectedObject;

    if (objectIndex == mpCSceneObjLink->mObjIndex)
	pCSelectedObject = mpCSceneObjLink;

    return pCSelectedObject;
}

bool ParamGroup_cls::
SelectObject(int objectIndex, int guideIndex){
    bool bRet = FALSE;
    
    if (mbGroupingGroup == TRUE)
	for (unsigned int i = 0; i < mVpCParamGroup.size(); i++)
	    bRet |= mVpCParamGroup[i]->SelectObject(objectIndex, guideIndex);
    
    if (mpCSceneObjLink == NULL) return bRet;

    if (objectIndex == mpCSceneObjLink->mObjIndex){
	if (mpCSceneObjLink->mbSelected == FALSE) {
	    mpCSceneObjLink->mbSelected = TRUE;
	    mpCSceneObjLink->mbDragging = FALSE;
	    mpCGUI->SetParamBackgroundColour();
	    bRet = TRUE;
	}
    }
    else{ 
	if (mpCSceneObjLink->mbSelected == TRUE){
	    mpCSceneObjLink->mbSelected = FALSE;
	    mpCGUI->SetParamBackgroundColour();
	    bRet = TRUE;
	}
    }
    
    return bRet;
}

void ParamGroup_cls::
CheckSnippingTag(std::vector < bool > *pVbSnipTag){
    for (int i = 0; i < mCDimension.mBody; i++)
	mVCParam[i].CheckSnippingTag(pVbSnipTag);
}

bool ParamGroup_cls::
IsKeyFrameAll(double doubleIndex){
    bool bIsKeyFrame = TRUE;
    for (int i = 0; i < mCDimension.mBody; i++)
	bIsKeyFrame &= mVCParam[i].IsKeyFrame(doubleIndex);
    return bIsKeyFrame;
}

void ParamGroup_cls::
CreateKeyDLG(wxPoint posPanel, int xpos, int ypos, 
	     int frame, double doubleFrame){

    wxString strLabel = mstrGroupLabel;
    MultiInputDialog_cls *pCDLG = new
	MultiInputDialog_cls(&pCDLG, mpCGUI, wxID_ANY, strLabel,
			     posPanel + wxPoint(xpos, ypos));
    pCDLG->SetIcon(ParamTL_cls::msCIcon);
    pCDLG->mpCUserDefinedClue = this;

    //int frame = (msMaxFrame * doubleFrame) + 1e-5;

    double paramCache[4]; // FIX_ME!!
    for (int i = 0; i < mCDimension.mBody; i++)
	paramCache[i] = mVCParam[i].GetInterpolatedValue(doubleFrame);

    pCDLG->mpCPanel->mTextWidth = 60;
    pCDLG->mpCPanel->mItemWidth = -1;

    pCDLG->CombinePointer(_ITEM_INTEGER, &frame, _("Frame"), FALSE);
    for (int i = 0; i < mCDimension.mBody; i++)
	pCDLG->CombinePointer(_ITEM_DOUBLE, &paramCache[i], 
			      mVCParam[i].mstrLabel);

    mTypeCache = mVCParam[0].GetInterpTypeFromDouble();
    mTypeCacheOrg = mTypeCache;
    pCDLG->CombinePointerList(_ITEM_UINT_LIST, &mTypeCache, 
			      5, KeyFrame_cls::msAstrInterp, _("Interpolate Type"));
    
    int boxWidth = 2;
    int    keyIndex[4];
    mpDValue2Cache = new double [mCDimension.mBody];
    for (int i = 0; i < mCDimension.mBody; i++){
	keyIndex[i] = 
	    mVCParam[i].GetKeyFrameIndexFromDouble(doubleFrame);
	if (keyIndex[i] == -1)
	    mpDValue2Cache[i] = 0;
	else
	    mpDValue2Cache[i] = mVCParam[i].mVCKeys[keyIndex[i]].mDValue2;
	int boxWidth = 2;
	int index = pCDLG->CombinePointer(_ITEM_BOXSIZERHORIZONTAL, &boxWidth, wxT("Ease Factor"));
	if (i == 0) mIndexHermiteParam = index;
	pCDLG->CombinePointer(_ITEM_DOUBLE_SLIDER, &mpDValue2Cache[i]); 
	pCDLG->SetRange(_ITEM_LASTITEM, 0, 30, 10);
	pCDLG->CombinePointer(_ITEM_DOUBLE, &mpDValue2Cache[i]);
    }
    
    pCDLG->MakeGeometry();

    pCDLG->mpCHookCtrlLink->SetFuncHook(&ShowAndHideParameters, _HOOK_COMBOBOX);
    ShowAndHideParameters(pCDLG->mpCPanel, NULL);
    
    pCDLG->UpdateValues();

    if (pCDLG->ShowModal() != wxID_OK){ 
	delete [] mpDValue2Cache;
	return;
    }

    bool bChanged = FALSE;
    bChanged |= (mTypeCache != mTypeCacheOrg);
    for (int i = 0; i < mCDimension.mBody; i++){
	mVCParam[i].mInterpolatedValue = paramCache[i];
	bChanged |= (mVCParam[i].IsValueChanged() == TRUE);
	bChanged |= (mVCParam[i].mVCKeys[keyIndex[i]].mDValue2 != mpDValue2Cache[i]);
    }
    
    if (bChanged == TRUE){
	GCUndoList.OpenBlock();
	for (int i = 0; i < mCDimension.mBody; i++){
	    mVCParam[i].SetChangedValueKey(_PARAM_CURRENT, 0, mpDValue2Cache[i]);
	    mVCParam[i].SetKeyType(keyIndex[i], mTypeCache);
	}
	GCUndoList.CloseBlock();
    }

    delete [] mpDValue2Cache;
}

void ParamGroup_cls::
ShowAndHideParameters(void* pArg, void *pEvent){
    MultiInputDialog_cls* pCDLG = ((MultiInputPanel_cls*)pArg)->mpCParentDLG;
    ParamGroup_cls* pCThis = (ParamGroup_cls*)pCDLG->mpCUserDefinedClue;
    
    int dimension = (int)pCThis->mVCParam.size();
    
    for (int i = 0; i < dimension; i++){
	pCDLG->HideItem(pCThis->mIndexHermiteParam + i*3 +0); //sizer
	//pCDLG->HideItem(pCThis->mIndexHermiteParam + i*3 +1); //sizer
	//pCDLG->HideItem(pCThis->mIndexHermiteParam + i*3 +2); //sizer
	//pCDLG->HideItem(pC->mIndexHermiteParam + i*3 +1); //slider
	//pCDLG->HideItem(pC->mIndexHermiteParam + i*3 +2); //value
    }

    if (pCThis->mTypeCache == _PARAM_HERMITE_ADJ){
	for (int i = 0; i < dimension; i++)
	    pCThis->mpDValue2Cache[i] = 0;
	
	int candid = pCThis->mVCParam[0].GetKeyFrameIndexFromDouble();
	if (candid != -1)
	    for (int i = 0; i < dimension; i++){
		if (pCThis->mVCParam[i].mVCKeys[candid].mType == _PARAM_HERMITE_ADJ)
		    pCThis->mpDValue2Cache[i] = pCThis->mVCParam[i].mVCKeys[candid].mDValue2;
	    }
	
	for (int i = 0; i < dimension; i++)
	    pCDLG->ShowItem(pCThis->mIndexHermiteParam + i*3);
    }

    pCDLG->Layout();
    pCDLG->ResizeDialog();
    pCDLG->UpdateValues();
}

std::vector < double > ParamGroup_cls::
GetInterpolatedValue(double valueX, double resolution){
    std::vector < double > VValue;
    for (int i = 0; i < mCDimension.mBody; i++)
	VValue.push_back(mVCParam[i].GetInterpolatedValue(valueX, resolution));
    return VValue;
}

void ParamGroup_cls::
GetInterpolatedValue4Fast(float *pValue, 
			  double valueX, double resolution){
    for (int i = 0; i < 4; i++)
	pValue[i] = mVCParam[i].GetInterpolatedValue(valueX, resolution);
}

void ParamGroup_cls::
SetKeyFrame(double *pValue, bool bForce, double doubleIndex){
    bool bChanged = bForce;
    for (int i = 0; i < mCDimension.mBody; i++){
	mVCParam[i].mInterpolatedValue = pValue[i];
	bChanged |= mVCParam[i].IsValueChanged(doubleIndex);
    }
    
    if (bChanged == TRUE){
	for (int i = 0; i < mCDimension.mBody; i++)
	    mVCParam[i].SetChangedValueKey(doubleIndex);
    }
}

void ParamGroup_cls::
SetKeyFrameIfChanged(double doubleIndex){
    for (int i = 0; i < mCDimension.mBody; i++){
	if (mVCParam[i].IsValueChanged(doubleIndex) == TRUE)
	    mVCParam[i].SetChangedValueKey(doubleIndex);
    }
}

// ----

void ParamGroup_cls::
DeselectAll(){
    for (int i = 0; i < mCDimension.mBody; i++)
	mVCParam[i].DeselectAll();
}

void ParamGroup_cls::
InsertKey(int index, double valueX, double valueY){
    if (index == _PARAM_ALL){
	GCUndoList.OpenBlock();
	for (int i = 0; i < mCDimension.mBody; i++)
	    mVCParam[i].InsertKey(valueX, mVCParam[i].GetInterpolatedValue(valueX));
	GCUndoList.CloseBlock();
    }
    else if (valueY != _PARAM_MAGICNUM){
	// łɂ̈ʒȕꏊɂׂL[
	// XLbv
	if (mVCParam[index].IsKeyFrame(valueX, 0.03) == TRUE &&
	    mVCParam[index].GetInterpolatedValue(valueX) == valueY)
	    return;
	mVCParam[index].InsertKey(valueX, valueY);
	
    }
    else {
	if (mVCParam[index].IsKeyFrame(valueX, 0.03) == TRUE)
	    return;
	mVCParam[index].InsertKey(valueX, mVCParam[index].GetInterpolatedValue(valueX));
    }
}

void ParamGroup_cls::
DeleteSelectedKey(){
    GCUndoList.OpenBlock();
    for (int i = 0; i < mCDimension.mBody; i++)
	mVCParam[i].DeleteSelectedKey();
    GCUndoList.CloseBlock();
}

bool ParamGroup_cls::
DeleteKey(double doubleIndex){
    bool bDeleted = FALSE;
    GCUndoList.OpenBlock();
    for (int i = 0; i < mCDimension.mBody; i++)
	bDeleted |= mVCParam[i].DeleteKey(doubleIndex);
    GCUndoList.CloseBlock();
    return bDeleted;
}

bool ParamGroup_cls::
SelectKeyFrameRegion(double doubleIndexA, double doubleIndexB,
		     bool bSelect = TRUE){
    bool bSelected = FALSE;
    for (int i = 0; i < mCDimension.mBody; i++)
	bSelected |= mVCParam[i].SelectKeyFrameRegion
	    (doubleIndexA, doubleIndexB, bSelect);
    return bSelected;
}

void ParamGroup_cls::
SelectKeyFrame(double doubleIndex,  double resolutionByPixel,
	       bool   bMultiSelect, bool bDeselect){
    for (int i = 0; i < mCDimension.mBody; i++)
	mVCParam[i].SelectKeyFrame
	    (doubleIndex, resolutionByPixel, bMultiSelect, bDeselect);
}

bool ParamGroup_cls::
SelectKeyByXY(double valueX, double resolutionX,
	      double valueY, double resolutionY, bool bSingleSelect){
    bool bSelected = FALSE;
    for (int i = 0; i < mCDimension.mBody; i++){
	if ((mControlMode != -1) && (i != mControlMode)) continue;
	bSelected |= mVCParam[i].SelectKeyByXY(valueX, resolutionX,
					       valueY, resolutionY);
	if (bSelected == TRUE && bSingleSelect == TRUE)
	    return TRUE;
    }
    return bSelected;
}

bool ParamGroup_cls::
MoveSelectedKeyByXY(double dX, double dY, double resolution,
		    double minX, double maxX, double minY, double maxY){
    bool bRetval = FALSE;
    for (int i = 0; i < mCDimension.mBody; i++)
	bRetval |= mVCParam[i].MoveSelectedKeyByXY(dX, dY, resolution,
						   minX, maxX, minY, maxY);
    return bRetval;
}

void ParamGroup_cls::
CopyFromP(ParamGroup_cls *pCParamGroup){
    if (mCDimension.mBody != pCParamGroup->mCDimension.mBody) return;
    GCUndoList.OpenBlock();
    for (int i = 0; i < mCDimension.mBody; i++){
	mVCParam[i].CopyParameterFromP(&pCParamGroup->mVCParam[i]);
    }
    GCUndoList.CloseBlock();
}

void ParamGroup_cls::
ProcStartDragging(double origin, bool bCtrl, bool bAlt){
    for (int i = 0; i < mCDimension.mBody; i++)
	mVCParam[i].mpCParamTLLink->ProcStartDragging(origin, bCtrl, bAlt);
}

void ParamGroup_cls::
ProcDragging(wxMouseEvent& event){
    for (int i = 0; i < mCDimension.mBody; i++)
	mVCParam[i].mpCParamTLLink->ProcDragging(event);
}

void ParamGroup_cls::
SetRegionOrigin(double origin){
    for (int i = 0; i < mCDimension.mBody; i++)
	mVCParam[i].mpCParamTLLink->SetRegionOrigin(origin);
}

void ParamGroup_cls::
TurnOffDragging(){
   for (int i = 0; i < mCDimension.mBody; i++)
	mVCParam[i].mpCParamTLLink->TurnOffDragging();
}

// ----

wxString ParamGroup_cls::
Serialize(){
    wxString strOut;
    strOut << wxT("-ParamGroup=") << mstrGroupLabel << ENDLWX;
    strOut << mCConfList.SerializeAll();
    strOut << wxT("Start") << ENDLWX;
    for (int i = 0; i < mCDimension.mBody; i++)
	strOut << mVCParam[i].Serialize();
    return strOut;
}

void ParamGroup_cls::
ReadStrLine(std::wistream &istream, wxString strLineTag){
    wxString strTag;
    strTag << wxT("-ParamGroup=") << mstrGroupLabel;
    if (strLineTag == strTag){
	std::wstring wstrLine;
	wxString strLine;
	while ( std::getline (istream, wstrLine)){
	    strLine = wstrLine.c_str();
	    if (strLine == wxT("Start")) break;
	    mCConfList.ReadStrLineAll(strLine);
	}
	for (int i = 0; i < mCDimension.mBody; i++){
	    std::getline (istream, wstrLine);
	    strLine = wstrLine.c_str();
	    mVCParam[i].ReadStrLine(istream, strLine);
	}
    }
}
