// -*- c -*-
// "clsParaGroupGUI.h"
// (setq c-basic-offset 4)

#include "clsParamGroupGUI.h"
#include "clsParamGroup.h"
#include "clsParametersPanel.h"
#include "../GLCANVAS/clsMainGLCanvas.h"
#include "../MAINCANVAS/clsParamGUICtrl.h"
#include "../SCENEOBJ/clsSceneObj.h"
#include "../WIDGETCTRL/clsBitmapPanel.h"
#include "../WIDGETCTRL/EX/clsStaticTextEX.h"

#include "../../wxWidgetEnumMenu.h"

ParamGroupGUI_cls::
ParamGroupGUI_cls
(ParamGroup_cls *pCParamGroup, ParamGUICtrl_cls *pCParentCtrl, 
 wxString strLabel, int depth, wxWindowID id, wxPoint pos, wxSize size) :
wxPanel(pCParentCtrl, id, pos, size)
{
    mpCCtrlLink = pCParentCtrl;
    mpCParentGroup = pCParamGroup;
    mDepth = depth;

    SetParamBackgroundColour();
    
    if (msbInitialized == FALSE){
	LoadBitmap();
	msbInitialized = TRUE;
    }

    wxBoxSizer *pCBoxSizerSRC = new wxBoxSizer(wxHORIZONTAL);

    mpCBPanelFolder = new BitmapPanel_cls(this, wxID_ANY, wxDefaultPosition, wxSize(14,16));
    mpCBPanelFolder->AddBitmapP(&msCBitmapOpen);
    mpCBPanelFolder->AddBitmapP(&msCBitmapClose);
    mpCBPanelFolder->SetBitmapPDisable(&msCBitmapClose);
    Connect(mpCBPanelFolder->GetId(), wxEVT_LEFT_UP,
	    wxMouseEventHandler(ParamGroupGUI_cls::OnMouseLOpen));
    
    mpCLabelText = new wxStaticTextEX(this, wxID_ANY, strLabel,
				      wxDefaultPosition, wxSize(-1, 12));

    if (mDepth > 0) pCBoxSizerSRC->Add(12, -1, 0, wxEXPAND);
    pCBoxSizerSRC->Add(mpCBPanelFolder, 0, wxEXPAND);
    pCBoxSizerSRC->Add(mpCLabelText, 0, wxEXPAND);

    Connect(mpCLabelText->GetId(), wxEVT_LEFT_DOWN,
	    wxMouseEventHandler(ParamGroupGUI_cls::OnMouseLSelect));
    Connect(GetId(), wxEVT_LEFT_DOWN,
	    wxMouseEventHandler(ParamGroupGUI_cls::OnMouseLSelect));

    if (mDepth > 0){
	mpCBPanelLink = new BitmapPanel_cls(this, wxID_ANY, wxDefaultPosition, wxSize(14,16));
	mpCBPanelLink->AddBitmapP(&msCBitmapLinked);
	mpCBPanelLink->AddBitmapP(&msCBitmapUnLinked);
	mpCBPanelLink->SetBitmapPDisable(&msCBitmapLinked);
	Connect(mpCBPanelLink->GetId(), wxEVT_LEFT_UP,
		wxMouseEventHandler(ParamGroupGUI_cls::OnMouseLLink));
	pCBoxSizerSRC->Add(2, -1, 0, wxEXPAND);
	pCBoxSizerSRC->Add(mpCBPanelLink, 0, wxEXPAND);
	mpCBPanelVisible = NULL;
    }
    else {
	mpCBPanelLink = NULL;
	mpCBPanelVisible = new BitmapPanel_cls(this, wxID_ANY, wxDefaultPosition, wxSize(14,16));
	mpCBPanelVisible->AddBitmapP(&ParametersPanel_cls::msCBitmapVisible);
	mpCBPanelVisible->AddBitmapP(&ParametersPanel_cls::msCBitmapInvisible);
	mpCBPanelVisible->SetBitmapPDisable(&ParametersPanel_cls::msCBitmapVisible);
	Connect(mpCBPanelVisible->GetId(), wxEVT_LEFT_UP,
		wxMouseEventHandler(ParamGroupGUI_cls::OnMouseLVisible));
	pCBoxSizerSRC->Add(2, -1, 0, wxEXPAND);
	pCBoxSizerSRC->Add(mpCBPanelVisible, 0, wxEXPAND);
    }

    SetSizerAndFit(pCBoxSizerSRC);
    if (mpCBPanelLink != NULL) mpCBPanelLink->Hide();
}

void ParamGroupGUI_cls::
SetParamBackgroundColour(){
    if (mpCParentGroup->mpCSceneObjLink != NULL &&
	mpCParentGroup->mpCSceneObjLink->mbSelected == TRUE){
	if (mDepth == 0)
	    SetBackgroundColour(wxColour(192, 192, 255));
	else
	    SetBackgroundColour(wxColour(192, 192, 255));
    }
    else{
	if (mDepth == 0)
	    SetBackgroundColour(wxColour(255, 255, 32));
	else
	    SetBackgroundColour(wxColour(255, 255, 128));
    }
    Refresh(TRUE);
}

void ParamGroupGUI_cls::
LoadBitmap(){
    msCBitmapOpen  = wxBitmap(wxT("ICONS/open.png") ,wxBITMAP_TYPE_PNG);
    msCBitmapClose = wxBitmap(wxT("ICONS/close.png"),wxBITMAP_TYPE_PNG);
    msCBitmapLinked   = wxBitmap(wxT("ICONS/Linked.png"), wxBITMAP_TYPE_PNG);
    msCBitmapUnLinked = wxBitmap(wxT("ICONS/UnLinked.png"), wxBITMAP_TYPE_PNG);
}

void ParamGroupGUI_cls::
OnMouseLOpen(wxMouseEvent& event){
    if (mpCBPanelFolder->mState == 1){
	mpCParentGroup->HideItems();
	if (mpCBPanelLink != NULL) mpCBPanelLink->Hide();
    }
    else{
	mpCParentGroup->ShowItems();
	if (mpCBPanelLink != NULL) mpCBPanelLink->Show();
    }
}

void ParamGroupGUI_cls::
OnMouseLLink(wxMouseEvent& event){
    if (mpCBPanelLink->mState == 1)
	mpCParentGroup->mbLinking = FALSE;
    else
	mpCParentGroup->mbLinking = TRUE;
}

void ParamGroupGUI_cls::
OnMouseLVisible(wxMouseEvent& event){
    if (mpCParentGroup->mpCSceneObjLink != NULL){
	if (mpCBPanelVisible->mState == 1)
	    mpCParentGroup->mpCSceneObjLink->mCbVisible.mbBody = FALSE;
	else
	    mpCParentGroup->mpCSceneObjLink->mCbVisible.mbBody = TRUE;
	
	wxCommandEvent dummy(wxEVT_COMMAND_MENU_SELECTED, menuEditRefreshGL);
	wxGetTopLevelParent(this)->AddPendingEvent(dummy);
    }
    
}

void ParamGroupGUI_cls::
OnMouseLSelect(wxMouseEvent& event){
    if (mpCParentGroup->mpCSceneObjLink != NULL)
	MainGLCanvas_cls::mspCSelectedObject = 
	    mpCCtrlLink->SelectObject(mpCParentGroup->mpCSceneObjLink);
}

wxBitmap ParamGroupGUI_cls::msCBitmapOpen;
wxBitmap ParamGroupGUI_cls::msCBitmapClose;
wxBitmap ParamGroupGUI_cls::msCBitmapLinked;
wxBitmap ParamGroupGUI_cls::msCBitmapUnLinked;
bool     ParamGroupGUI_cls::msbInitialized = FALSE;
