// -*- c -*-
// "clsParameters.h"
// (setq c-basic-offset 4)

#include "clsParameters.h"
#include "clsParamCommands.h"
#include "clsParamGroup.h"
#include "clsParamTL.h"

#include <math.h>
#include <algorithm>
#include <fstream>
#include <sstream>

#include "../GLCANVAS/clsMainGLCanvas.h"
#include "../MYAPI/MsgBox.h"
#include "../PARTICLEDATA/clsParticleDataSeqCtrl.h"
#include "../TIMELINE/clsTimeCtrl.h"
#include "../UNDO/clsUndoManager.h"
extern UndoManager_cls GCUndoList;

#include <wx/clipbrd.h>


extern std::vector < Parameters_cls* > GpVPInstancePrim;

#define X 0
#define Y 1
#define Z 2
#define LOOP_XYZ for (int xyz = 0; xyz < 3; xyz++)
#define LOOP_RGBA for (int rgba = 0; rgba < 4; rgba++)
#define LOOP_YPRY for (int ypry = 0; ypry < 4; ypry++)

#ifdef WIN32
#define ENDLWX wxT("\r\n")
#else
#define ENDLWX wxT("\n")
#endif

#ifdef _DEBUG
#include <stdlib.h>
#include <crtdbg.h>
#define new new(_NORMAL_BLOCK, __FILE__, __LINE__)
#endif

// ------

Parameters_cls::
Parameters_cls(wxString strLabel, int updateBit, bool bTimeDependent):
    mstrLabel(strLabel), mVCKeys(0)
{
    mpCParentGroupLink = NULL;
    mpCParamTLLink = NULL;
    mUpdateBit = updateBit;
    mInterpolatedValue = 0;

    //wchar_t buf[8];
    //swprintf(buf, 8, L"%d", dimension);
    //mCConfList.Add(&mCDimension, L"dimension ", buf);
    wxString strTimeDependent;
    strTimeDependent << bTimeDependent;
    mCConfList.Add(&mCbTimeDependent, wxT("timeDepend="), strTimeDependent);
    mCConfList.Add(&mCPeriodic, wxT("periodic="), wxT("0"));
    
    //mCConfList.Add(&mCbVisible, L"visible=", L"1");
    //mCConfList.Add(&mCbShow, L"show=", L"1");
    mbUpdateCamera = FALSE;

    bool bPrim = FALSE;

    //if (bPrim == TRUE && mspVPInstancePrim != NULL) mspVPInstancePrim->push_back(this);
}

Parameters_cls:: // $B%3%T!<%3%s%9%H%i%/%?(B
Parameters_cls(const Parameters_cls &obj){

    mpCParentGroupLink = obj.mpCParentGroupLink;
    mstrLabel = obj.mstrLabel;
    mUpdateBit = obj.mUpdateBit;

    mCbTimeDependent = obj.mCbTimeDependent;
    mCPeriodic       = obj.mCPeriodic;
    //mCbVisible       = obj.mCbVisible;
    //mCbShow          = obj.mCbShow;
    mbUpdateCamera   = obj.mbUpdateCamera;

    DefineConfList();
    
    mInterpolatedValue = obj.mInterpolatedValue;
    
    mVCKeys.clear();
    size_t keySize = obj.mVCKeys.size();
    for (size_t index = 0; index < keySize; index++)
	mVCKeys.push_back(obj.mVCKeys[index]);

    /*
    for (unsigned int i = 0; i < mspVPInstancePrim->size(); i++)
	if ((*mspVPInstancePrim)[i] == &obj)
	    (*mspVPInstancePrim)[i] = this;
    */
}    

void Parameters_cls::
CopyFromP(const Parameters_cls *pObj){

    mpCParentGroupLink = pObj->mpCParentGroupLink;
    mstrLabel = pObj->mstrLabel;
    mUpdateBit = pObj->mUpdateBit;

    //_ASSERT(mCDimension.mBody == pObj->mCDimension.mBody);    

    mCbTimeDependent = pObj->mCbTimeDependent;
    mCPeriodic       = pObj->mCPeriodic;
    //mCbVisible       = pObj->mCbVisible;
    //mCbShow          = pObj->mCbShow;
    mbUpdateCamera   = pObj->mbUpdateCamera;
    
    DefineConfList();

    mInterpolatedValue = pObj->mInterpolatedValue;
    
    mVCKeys.clear();
    size_t keySize = pObj->mVCKeys.size();
    for (size_t index = 0; index < keySize; index++)
	mVCKeys.push_back(pObj->mVCKeys[index]);
}

void Parameters_cls::
DefineConfList(){
    mCConfList.Clear();
    //mCConfList.Add(&mCDimension, L"dimension ", mCDimension.GetItemStr());
    mCConfList.Add(&mCbTimeDependent, L"timeDepend=", mCbTimeDependent.GetItemStr());
    mCConfList.Add(&mCPeriodic, wxT("periodic="), mCPeriodic.GetItemStr());
    //mCConfList.Add(&mCbVisible, L"visible ", mCbVisible.GetItemStr());
    //mCConfList.Add(&mCbShow, L"show ", mCbShow.GetItemStr());
}

Parameters_cls::
~Parameters_cls(){
}

void Parameters_cls::
GroupRefresh(){
    if (mpCParentGroupLink != NULL) mpCParentGroupLink->RefreshPanels();
}

void Parameters_cls::
RemapKeyFrame(int oldMax, int newMax){
    for (int i = 0; i < GetKeyNumber(); i++)
	mVCKeys[i].mDoubleIndex *= (double) oldMax / newMax;
}

void Parameters_cls::
RemapKeyFrameAll(int oldMax, int newMax){
    /*
    for (unsigned int i = 0; i < mspVPInstancePrim->size(); i++){
	MsgBox(NULL, (*mspVPInstancePrim)[i]->mLabelStr, L"!!");
	(*mspVPInstancePrim)[i]->RemapKeyFrame(oldMax, newMax);
    }
    */
}

void Parameters_cls::
SetFrameNumber(int frame, int animInterp/*, unsigned int quantum*/){
    msFrameNumber = frame;
    msAnimInterp = animInterp;
    if (frame >= 1)
	msResolution = 1.0/msFrameNumber/msAnimInterp;
    else
	msResolution = 1.0;
    //msStepQuantum = quantum;
}

/*
void Parameters_cls::
SetCurrentDoubleIndex(double doubleIndex){
    msCurrentDoubleIndex = doubleIndex;
}
*/

double Parameters_cls::
Quantize(double doubleIndex, unsigned int frameNumber){
    if (frameNumber == _PARAM_USE_COMMON) frameNumber = msFrameNumber;
    int index = doubleIndex * frameNumber * 2;
    index = (index + 1) / 2;
    if (frameNumber == 0) 
	doubleIndex = 0;
    else
	doubleIndex = (double)index/ frameNumber;
    return doubleIndex;
};

void Parameters_cls::
SetTimeDependent(bool bTimeDependent){
    GCUndoList.OpenBlock();

    double tmp = mInterpolatedValue;
    GCUndoList.SetNewCommand
	(new ComParamClearKey_cls(this, tmp),
	 new ComParamMemento_cls(this));
    GCUndoList.ExecCurrent();
    GCUndoList.PushNewCommand
	(new ComParamSetTimeDependent_cls(this, bTimeDependent),
	 new ComParamSetTimeDependent_cls(this, !bTimeDependent));
    GCUndoList.ExecCurrent();
    GCUndoList.CloseBlock();
}

void Parameters_cls::
CopyParameterFromP(Parameters_cls *pCParamFrom){
    GCUndoList.OpenBlock();
    
    GCUndoList.SetNewCommand
	(new ComParamMemento_cls(this, pCParamFrom),
	 new ComParamMemento_cls(this));
    GCUndoList.ExecCurrent();

    GCUndoList.CloseBlock();
}

void Parameters_cls::
ExecCommandTimeDependent(bool bTimeDependent){
    mCbTimeDependent.mbBody = bTimeDependent;
}

int Parameters_cls::
GetKeyNumber(){
    return (unsigned int)mVCKeys.size();
}

int Parameters_cls::
GetSelectedKeyNum(){
    int num = 0;
    for (int i = 0; i < GetKeyNumber(); i++)
	if (mVCKeys[i].mbSelected == TRUE) num++;
    return num;
}

void Parameters_cls::
SetKeyToValueAt(KeyFrame_cls *pCKey, double doubleIndex){
    int prev = GetAdjacentKey(doubleIndex, -1);
    int next = GetAdjacentKey(doubleIndex, +1);
    double vel = CalcVelocity(prev, next);
    
    pCKey->Set(doubleIndex, GetInterpTypeFromDouble(doubleIndex),
	       GetInterpolatedValue(doubleIndex), vel);
}

TimeCtrl_cls* Parameters_cls::
GetPCTimeCtrl(){
    return mspCTimeCtrlLink;
}

PDataSeqCtrl_cls* Parameters_cls::
GetPCPDataSeqCtrl(){
    return mspCPDataSeqCtrlLink;
}

void Parameters_cls::
CheckSnippingTag(std::vector < bool > *pVbSnipTag){
    for (int i = 0; i < GetKeyNumber(); i++){
	int frame = (mVCKeys[i].mDoubleIndex * msFrameNumber + 0.1);
	(*pVbSnipTag)[frame] = TRUE;
	unsigned int index = GetPCTimeCtrl()->GetRemappedIndex(frame);
	GetPCPDataSeqCtrl()->SetSnipping(index);
    }
}

void Parameters_cls::
SortKey(){
    int status = 1;
    // $BD>@\%/%i%9$r%=!<%H$9$k$N$OF0E*%a%b%j$N4X78$GLq2p!#<jF0$G(B
    while (status == 1){
	status = 0;
	size_t keySize = mVCKeys.size();
	for (size_t index = 0; index < keySize - 1; index++){
	    if (mVCKeys[index] > mVCKeys[index + 1]){
		status = 1;
		KeyFrame_cls tmpKey;
		tmpKey.FullCopy(mVCKeys[index]);
		mVCKeys[index].FullCopy(mVCKeys[index+1]);
		mVCKeys[index+1].FullCopy(tmpKey);
		index = 0;
	    }
	}
    }
}

double Parameters_cls::
CalcVelocity(int prev, int next){
    double vel = 0;
    if (prev != next && prev != -1 && next != -1){
	double value0 = mVCKeys[prev].mValue;
	double value1 = mVCKeys[next].mValue;
	double period = mCPeriodic.mBody;
	if (mCPeriodic.mBody == 0)
	    vel = (value1 - value0)/
		(mVCKeys[next].mDoubleIndex - mVCKeys[prev].mDoubleIndex);
	else{
	    double value1periodic = GetValue1Periodic(value0, value1, period);
	    vel = (value1periodic - value0)/
		(mVCKeys[next].mDoubleIndex - mVCKeys[prev].mDoubleIndex);
	}
    }
    return vel;
}

void Parameters_cls::
AdjustAutoControl(){
    size_t keySize = mVCKeys.size();
    if (keySize == 1) return;
    //unsigned int dimension = mCDimension.mBody;

    for (unsigned int index = 0; index < keySize; index++){
	unsigned int prev = 
	    (index != 0)                ? index - 1 : 0;
	unsigned int next = 
	    (index != keySize-1) ? index + 1 : keySize-1;
	double vel = CalcVelocity(prev, next);
	switch (mVCKeys[index].mType){
	case _PARAM_HERMITE_AUTO:
	    mVCKeys[index].mDValue1 = vel;
	    mVCKeys[index].mDValue2 = vel;
	    break;
	case _PARAM_HERMITE_ADJ:
	    mVCKeys[index].mDValue1 = vel;
	    break;
	default:
	    break;
	}
    }
}

void Parameters_cls::
InitSet(double value){
    GCUndoList.SetNewCommand
	(new ComParamClearKey_cls(this, value),
	 new ComParamMemento_cls(this));
    GCUndoList.ExecCurrent();
}

void Parameters_cls::
ExecCommandInitSet(double value){
    mVCKeys.clear();
    KeyFrame_cls newKey;
    newKey.Set(0, _PARAM_HERMITE_AUTO, value);
    mVCKeys.push_back(newKey);
}

void Parameters_cls::
InsertKeyQueue(double doubleIndex){
    if (mpCParentGroupLink != NULL && 
	mpCParentGroupLink->mbLinking == TRUE){
	mpCParentGroupLink->InsertKey(_PARAM_ALL, doubleIndex);
	return;
    }
    InsertKey(doubleIndex);
}

void Parameters_cls::
InsertKey(double doubleIndex, double value, double DValue1, double DValue2){
    if (doubleIndex == -1) {
	doubleIndex = msCurrentDoubleIndex; 
	value = GetInterpolatedValue(msCurrentDoubleIndex);
    }
    KeyFrame_cls CKey;
    SetKeyToValueAt(&CKey, doubleIndex);
    CKey.SetValue(value, DValue1, DValue2);
    SetKey(CKey);
}

int Parameters_cls::
CheckOverWrite(double doubleIndex, double resolution){
    int overwrite = -1;
    size_t keySize = mVCKeys.size();
    for (size_t index = 0; index < keySize; index++){
	double difference = doubleIndex - mVCKeys[index].mDoubleIndex;
	difference = sqrt(difference * difference);
	if (difference < resolution / 2.0)
	    overwrite = (int)index;
    }
    return overwrite;
}

void Parameters_cls::
SetKey(KeyFrame_cls CKey, double resolution){

    if (resolution == _PARAM_USE_COMMON) resolution = msResolution;

    //Check Overwrite
    int overwrite = CheckOverWrite(CKey.mDoubleIndex, resolution);

    if (overwrite == -1)
	GCUndoList.SetNewCommand
	    (new ComParamSetKeyC_cls(this, CKey),
	     new ComParamDelKey_cls(this, 
				    GetKeyFrameIndexToInsert(CKey.mDoubleIndex)));
    else
	GCUndoList.SetNewCommand
	    (new ComParamOvwtKeyC_cls(this, overwrite, CKey),
	     new ComParamOvwtKeyC_cls(this, overwrite, mVCKeys[overwrite]));
    GCUndoList.ExecCurrent();
}

void Parameters_cls::
ExecCommandSetKey(KeyFrame_cls CKey){
    mVCKeys.push_back(CKey);
    SortKey();
    AdjustAutoControl();
}

void Parameters_cls::
ExecCommandSetKey(double doubleIndex, double value, unsigned int type, 
		  double DValue1, double DValue2){
    KeyFrame_cls newKey;
    newKey.Set(doubleIndex, type, value, DValue1, DValue2);
    mVCKeys.push_back(newKey);
    SortKey();
    AdjustAutoControl();
}

void Parameters_cls::
ExecCommandOvwtKey(unsigned int index, KeyFrame_cls CKey){
    //CKey.mDoubleIndex = mVCKeys[index].mDoubleIndex;
    mVCKeys[index].FullCopy(CKey);
    SortKey();
    AdjustAutoControl();
}

bool Parameters_cls::
IsValueChanged(double doubleIndex){
    if (doubleIndex == _PARAM_CURRENT) doubleIndex = msCurrentDoubleIndex;
    bool bChanged = FALSE;
    if (mCbTimeDependent.mbBody == FALSE)
	doubleIndex = 0;
    double cache = mInterpolatedValue;
    if (cache != GetInterpolatedValue(doubleIndex)) bChanged = TRUE;
    mInterpolatedValue = cache;
    return bChanged;
}

void Parameters_cls::
SetChangedValueKey(double doubleIndex, double DValue1, double DValue2, double resolution){
    if (doubleIndex == _PARAM_CURRENT) doubleIndex = msCurrentDoubleIndex;
    int keyframe = GetKeyFrameIndexFromDouble(doubleIndex, resolution, FALSE);
    if (keyframe == -1)
	InsertKey(doubleIndex, mInterpolatedValue, DValue1, DValue2);
    else{
	//$B%$%s%5!<%H$G$O$J$/?tCM$NJQ99(B
	bool bSelected = mVCKeys[keyframe].mbSelected;
	InsertKey(doubleIndex, mInterpolatedValue, DValue1, DValue2);
	mVCKeys[keyframe].mbSelected = bSelected;
    }
    GetInterpolatedValue(doubleIndex);
}

bool Parameters_cls::
MoveSelectedKeyByXYQueue(double dX, double dY, double resolution,
		    double minX, double maxX, double minY, double maxY){
    if (mpCParentGroupLink != NULL && 
	mpCParentGroupLink->mbLinking == TRUE)
	return mpCParentGroupLink->MoveSelectedKeyByXY
	    (dX, dY, resolution, minX, maxX, minY, maxY);

    return MoveSelectedKeyByXY
	(dX, dY, resolution, minX, maxX, minY, maxY);
}

bool Parameters_cls::
MoveSelectedKeyByXY(double dX, double dY, double resolution,
		    double minX, double maxX, double minY, double maxY){
    bool retval = FALSE;
    bool bSkipMove = FALSE;
    if (resolution == -1) resolution = msResolution;
    if ((dX < resolution * 0.5) && (dX > -resolution * 0.5))
	bSkipMove = TRUE;
    if (dY == 0 && bSkipMove == TRUE) return FALSE;
    if (mVCKeys.size() == 0) return FALSE;
    if (GetSelectedKeyNum() == 0) return FALSE;

    GCUndoList.OpenBlock();
    if (dX < 0){
	for (int i = 0; i < GetKeyNumber(); i++)
	    if (mVCKeys[i].mbSelected == TRUE){
		if (bSkipMove == FALSE){
		    int processBit = MoveKey(i, dX, 1.0/resolution);
		    if ((processBit & 1) != 0) retval = TRUE;
		    if ((processBit & _PARAM_PROCESS_OVRW) != 0) i--; 
		}
		// key$B$,(B1$B$D8:$C$?(B
		if (dY != 0){
		    retval = TRUE;
		    mInterpolatedValue = mVCKeys[i].mValue + dY;
		    SetChangedValueKey(mVCKeys[i].mDoubleIndex);
		}
	    }
    }
    else{
	for (int i = (int)mVCKeys.size() - 1; i >= 0; i--)
	    if (mVCKeys[i].mbSelected == TRUE){
		if (bSkipMove == FALSE)
		    if ((MoveKey(i, dX, 1.0/resolution) & 1) == TRUE) retval = TRUE;
		if (dY != 0){
		    retval = TRUE;
		    mInterpolatedValue = mVCKeys[i].mValue + dY;
		    SetChangedValueKey(mVCKeys[i].mDoubleIndex);
		}
	    }
    }

    if (minX != 0 || maxX != 0)
	for (int i = 0; i < GetKeyNumber(); i++){
	    if (mVCKeys[i].mDoubleIndex > maxX){
		MoveKey(i, maxX - mVCKeys[i].mDoubleIndex, 1.0/resolution);
		i = 0; //$B%-!<$,8:$k2DG=@-$,$"$k$N$G(B
	    }
	    if (mVCKeys[i].mDoubleIndex < minX){
		MoveKey(i, minX - mVCKeys[i].mDoubleIndex, 1.0/resolution);
		i = 0; //$B%-!<$,8:$k2DG=@-$,$"$k$N$G(B
	    }
	}

    if (minY != 0 || maxY != 0)
	for (int i = 0; i < GetKeyNumber(); i++){
	    if (mVCKeys[i].mValue > maxY){
		mInterpolatedValue = maxY;
		SetChangedValueKey(mVCKeys[i].mDoubleIndex, resolution);
	    }
	    if (mVCKeys[i].mValue < minY){
		mInterpolatedValue = minY;
		SetChangedValueKey(mVCKeys[i].mDoubleIndex, resolution);
	    }

	}

    GCUndoList.CloseBlock();

    if (retval == FALSE) GCUndoList.UndoCurrent(); 

    return retval;
}

void Parameters_cls::
RemapKeys(double rate){
    size_t keySize = mVCKeys.size();
    if (rate < 1){
	for (size_t i = 0; i < keySize; i++)
	    if (mVCKeys[i].mDoubleIndex != 0)
		MoveKey((int)i, mVCKeys[i].mDoubleIndex * (rate - 1.0));
    }
    else{
	for (int i = (int)keySize - 1; i >= 0; i--)
	    if (mVCKeys[i].mDoubleIndex != 0)
		MoveKey(i, mVCKeys[i].mDoubleIndex * (rate - 1.0));
    }
}

int Parameters_cls::
MoveKey(int indexToMove, double doubleIndexDiff,
	unsigned int frameNumber){

    double resolution;
    if (frameNumber == _PARAM_USE_COMMON){
	frameNumber = msFrameNumber;
	resolution = msResolution;
    }
    else
	resolution = 1.0 / frameNumber;

    double orgDoubleIndex = mVCKeys[indexToMove].mDoubleIndex;
    unsigned int orgType = mVCKeys[indexToMove].mType;
    double newDoubleIndex = orgDoubleIndex + doubleIndexDiff;
    newDoubleIndex = Quantize(newDoubleIndex, frameNumber);
    if (newDoubleIndex < 0) newDoubleIndex = 0;
    if (newDoubleIndex > 1) newDoubleIndex = 1;

    if (newDoubleIndex == orgDoubleIndex) return 0;

    // Check Overwrite
    int overwrite = CheckOverWrite(newDoubleIndex, resolution);

    GCUndoList.OpenBlock();
    
    int retFlag = 1;

    if (overwrite == -1){
	GCUndoList.SetNewCommand
	    (new ComParamMoveKey_cls(this, orgDoubleIndex, newDoubleIndex, resolution),
	     new ComParamMoveKey_cls(this, newDoubleIndex, orgDoubleIndex, resolution));
	GCUndoList.ExecCurrent();
    }
    else{
	retFlag += _PARAM_PROCESS_OVRW;
	int typeOvrwted = mVCKeys[overwrite].mType;
	
	KeyFrame_cls CKey(mVCKeys[indexToMove]);
	CKey.mDoubleIndex = mVCKeys[overwrite].mDoubleIndex;

	GCUndoList.SetNewCommand
	    (new ComParamOvwtKeyC_cls(this, overwrite, CKey),
	     new ComParamOvwtKeyC_cls(this, overwrite, mVCKeys[overwrite]));

	GCUndoList.PushNewCommand
	    (new ComParamDelKey_cls(this, indexToMove),
	     new ComParamSetKey_cls(this, orgDoubleIndex, 
				    GetKeyFrameValue(indexToMove),
				    GetKeyFrameValueD1(indexToMove),
				    GetKeyFrameValueD2(indexToMove),
				    orgType));
	GCUndoList.ExecCurrent();
	//$B>e=q$-$5$l$?(Bkey$B$OA*Br:Q$_(B

	if (indexToMove < overwrite)
	    mVCKeys[overwrite-1].mbSelected = TRUE;
	else
	    mVCKeys[overwrite].mbSelected = TRUE;
    }

    GCUndoList.CloseBlock();

    return retFlag;
}

void Parameters_cls::
ExecCommandMoveKey(double doubleIndexFrom, double doubleIndexTo, double resolution){
    int index = GetKeyFrameIndexFromDouble(doubleIndexFrom, resolution);
    if (index == -1) return;
    mVCKeys[index].mDoubleIndex = doubleIndexTo;
    SortKey();
    AdjustAutoControl();
}

void Parameters_cls::
SetKeyType(unsigned int index, unsigned int mode){

    if (mVCKeys[index].mType == mode) return;
   
    GCUndoList.SetNewCommand
	(new ComParamSetKeyType_cls(this, index, mode),
	 new ComParamSetKeyType_cls(this, index, mVCKeys[index].mType));
    GCUndoList.ExecCurrent();
}

void Parameters_cls::
SetKeyTypeMulti(unsigned int mode){

    unsigned int num = 0;
    for (int i = 0; i < GetKeyNumber(); i++)
	if (mVCKeys[i].mbSelected == TRUE) num++;

    if (num == 0) return;

    unsigned int *listIndexFront = new unsigned int [num];
    unsigned int *listTypeFront  = new unsigned int [num];
    unsigned int *listIndexBack  = new unsigned int [num];
    unsigned int *listTypeBack   = new unsigned int [num];
    // UndoList$B$K<u$1EO$7$F!"8~$&$G(Bdelte$B$7$F$b$i$&(B

    unsigned int index = 0;
    for (int i = 0; i < GetKeyNumber(); i++)
	if (mVCKeys[i].mbSelected == TRUE){
	    listIndexFront[index] = i;
	    listIndexBack[index] = i;
	    listTypeFront[index] = mode;
	    listTypeBack[index] = mVCKeys[i].mType;
	    index++;
	}
   
    GCUndoList.SetNewCommand
	(new ComParamSetKeyTypeMulti_cls(this, num, listIndexFront, listTypeFront),
	 new ComParamSetKeyTypeMulti_cls(this, num, listIndexBack,  listTypeBack));
    GCUndoList.ExecCurrent();
}


void Parameters_cls::
ExecCommandSetKeyType(unsigned int index, unsigned int mode){
    mVCKeys[index].mType = mode;
    AdjustAutoControl();
}

void Parameters_cls::
ExecCommandSetKeyTypeMulti(unsigned int num, 
			   unsigned int *pIndex, unsigned int *pType){
    for (unsigned int i = 0; i < num; i++)
	mVCKeys[pIndex[i]].mType = pType[i];
    AdjustAutoControl();
}

bool Parameters_cls::
DeleteSelectedKey(){
    bool retval = FALSE;
    GCUndoList.OpenBlock();
    // FIX_ME $BMWAG$,5M$^$k:o=|$O5U=g$G%k!<%W$9$l$P$9$s$J$j!)(B
    for (int i = 0; i < (int)mVCKeys.size(); i++)
	if (mVCKeys[i].mbSelected == TRUE)
	    if (DeleteKey(mVCKeys[i].mDoubleIndex) == TRUE){
		retval = TRUE;
		i--;
	    }
    GCUndoList.CloseBlock();
    return retval;
}

#include "../MYAPI/MsgBox.h"

bool Parameters_cls::
DeleteKeyQueue(double doubleIndex){
    if (mpCParentGroupLink != NULL && 
	mpCParentGroupLink->mbLinking == TRUE)
	return mpCParentGroupLink->DeleteKey(doubleIndex);
    return DeleteKey(doubleIndex);
}

bool Parameters_cls::
DeleteKey(double doubleIndex, double resolution){
    if (mVCKeys.size() == 1) {
	mVCKeys[0].mDoubleIndex = 0;
	return FALSE; 
    }

    int index = GetKeyFrameIndexFromDouble(doubleIndex, resolution);
    if (index == -1) return FALSE;
    GCUndoList.SetNewCommand
	(new ComParamDelKey_cls(this, index),
	 new ComParamSetKey_cls(this, doubleIndex, 
				GetKeyFrameValue(index),
				GetKeyFrameValueD1(index), 
				GetKeyFrameValueD2(index), 
				mVCKeys[index].mType)
	 );
    GCUndoList.ExecCurrent();
    return TRUE;
}

void Parameters_cls::
ExecCommandDeleteKey(int index){
    if (index == -1) return;
    mVCKeys[index].mDoubleIndex = 255;
    SortKey();
    AdjustAutoControl();
    mVCKeys.pop_back();
}

bool Parameters_cls::
IsKeyFrame(double doubleIndex, double resolution){
    if (resolution == _PARAM_USE_COMMON) resolution = msResolution;
    bool isTrue = FALSE;
    for (int index = 0; index < GetKeyNumber(); index++)
	if (mVCKeys[index].mDoubleIndex < doubleIndex + resolution / 2.0 &&
	    mVCKeys[index].mDoubleIndex >= doubleIndex - resolution / 2.0)
	    isTrue = TRUE;
    return isTrue;
}

double Parameters_cls::
GetNextKeyPosDoubleIndex(double cur, double resolution){
    if (resolution == _PARAM_USE_COMMON) resolution = msResolution;
    cur = cur + resolution / 2.0;
    if (cur < mVCKeys[0].mDoubleIndex)
	return mVCKeys[0].mDoubleIndex;
    for (int index = 0; index < GetKeyNumber() - 1; index++)
	if (cur < mVCKeys[index + 1].mDoubleIndex) 
	    return mVCKeys[index + 1].mDoubleIndex;
    return mVCKeys[GetKeyNumber() - 1].mDoubleIndex;
}

double Parameters_cls::
GetPrevKeyPosDoubleIndex(double cur, double resolution){
    if (resolution == _PARAM_USE_COMMON) resolution = msResolution;
    cur = cur - resolution / 2.0;
    if (cur > mVCKeys[GetKeyNumber()-1].mDoubleIndex)
	return mVCKeys[GetKeyNumber()-1].mDoubleIndex;
    for (unsigned int index = GetKeyNumber(); index > 0; index--) 
	if (cur > mVCKeys[index - 1].mDoubleIndex)
	    return mVCKeys[index - 1].mDoubleIndex; 
    return 0;
}

int Parameters_cls::
GetKeyFrameIndexFromDouble(double doubleIndex, double resolution,
			   bool bFlagGetKeyEvenNotKeyFrame){
    if (doubleIndex == _PARAM_CURRENT) doubleIndex = msCurrentDoubleIndex;
    if (resolution == _PARAM_USE_COMMON) resolution = msResolution;

    for (int index = 0; index < GetKeyNumber(); index++){
	if (mVCKeys[index].mDoubleIndex < doubleIndex + resolution / 2.0 &&
	    mVCKeys[index].mDoubleIndex >= doubleIndex - resolution / 2.0)
	    return (int)index;
    }

    if (bFlagGetKeyEvenNotKeyFrame == TRUE){
	for (int index = 0; index < GetKeyNumber(); index++)
	    if (mVCKeys[index].mDoubleIndex > doubleIndex + resolution / 2.0)
		if (index == 0)
		    return 0;
		else
		    return (int)index - 1;
	return GetKeyNumber() - 1;
    }

    return -1;
}

int Parameters_cls::
GetKeyFrameIndexToInsert(double doubleIndex, double resolution){
    if (resolution == _PARAM_USE_COMMON) resolution = msResolution;

    for (int index = 0; index < GetKeyNumber(); index++)
	if (mVCKeys[index].mDoubleIndex > doubleIndex + resolution / 2.0)
	    return (int)index;
    return GetKeyNumber();
}

int Parameters_cls::
GetAdjacentKey(double doubleIndex, int direction){
    int index = GetKeyFrameIndexFromDouble(doubleIndex);
    if (index != -1){
	index += direction;
	if (index < 0) return 0;
	if (index >= (int)GetKeyNumber()) return GetKeyNumber() - 1;
	return index;
    }
    
    switch(direction){
    case 1:
	for (index = 0; index < (int)GetKeyNumber(); index++)
	    if (mVCKeys[index].mDoubleIndex > doubleIndex)
		return (int)index;
	return GetKeyNumber() - 1;
    case -1:
	for (index = GetKeyNumber() - 1; index >= 0; index--)
	    if (mVCKeys[index].mDoubleIndex < doubleIndex)
		return (int)index;
	return 0;
    }
    
    return 0;
}

int Parameters_cls::
GetSelectedKeyIndex(){
    size_t keySize = mVCKeys.size();
    for (size_t i = 0; i < keySize; i++)
	if (mVCKeys[i].mbSelected == TRUE) return (int)i;
    return -1;
}

double Parameters_cls::
GetKeyFrameDoubleIndex(int index){
    if (index >= GetKeyNumber()) return -1;
    return mVCKeys[index].mDoubleIndex;
}

double Parameters_cls::
GetKeyFrameValue(int index){
    if (index >= GetKeyNumber()) return -9999;
    return mVCKeys[index].mValue;
}

double Parameters_cls::
GetKeyFrameValueD1(int index){
    if (index >= GetKeyNumber()) return -9999;
    return mVCKeys[index].mDValue1;
}

double Parameters_cls::
GetKeyFrameValueD2(int index){
    if (index >= GetKeyNumber()) return -9999;
    return mVCKeys[index].mDValue2;
}

unsigned int Parameters_cls::
GetInterpTypeFromDouble(double doubleIndex){
    if (doubleIndex == _PARAM_CURRENT) doubleIndex = msCurrentDoubleIndex;
    unsigned int type = _PARAM_LINEAR;
    int candid = GetKeyFrameIndexFromDouble(doubleIndex, _PARAM_USE_COMMON,
					    TRUE);
    if (candid != -1) type = mVCKeys[candid].mType;
    return type;
}

void Parameters_cls::
CalcInterpolateStep(double interpolateRate, int index){
    mInterpolatedValue = mVCKeys[index].mValue;
}

double Parameters_cls::
GetValue1Periodic(double value0, double value1, double period){
    double value1copy = value1;
    if (value1 > value0){
	while (value1copy > value0) value1copy -= period;
	if (value1copy + period - value0 < value0 - value1copy)
	    value1copy += period;
    }
    else{
	while (value1copy < value0) value1copy += period;
	if (value0 - value1copy + period < value1copy - value0)
	    value1copy -= period;
    }
    return value1copy;
}

void Parameters_cls::
CalcInterpolateLinear(double interpolateRate, int index){
    double value0 = mVCKeys[index].mValue;
    double value1 = mVCKeys[index+1].mValue;
    double period = mCPeriodic.mBody;
    if (mCPeriodic.mBody == 0){
	mInterpolatedValue = value0 + interpolateRate * (value1 - value0);
	return;
    }

    double value1periodic = GetValue1Periodic(value0, value1, period);
    //value1;
    /*
    if (value1 > value0){
	while (value1copy > value0) value1copy -= period;
	if (value1copy + period - value0 < value0 - value1copy)
	    value1copy += period;
    }
    else{
	while (value1copy < value0) value1copy += period;
	if (value0 - value1copy + period < value1copy - value0)
	    value1copy -= period;
    }
    */
    
    mInterpolatedValue = value0 + interpolateRate * (value1periodic - value0);
}

void Parameters_cls::
CalcInterpolateHermite(double interpolateRate, int index){
    double scale = mVCKeys[index+1].mDoubleIndex - mVCKeys[index].mDoubleIndex;
    
    double vin, vout;
    
    if (mVCKeys[index].mType == _PARAM_HERMITE_ADJ)
	vin  = mVCKeys[index].mDValue1 * mVCKeys[index].mDValue2;
    else
	vin  = mVCKeys[index].mDValue1;
    if (mVCKeys[index+1].mType == _PARAM_HERMITE_ADJ)
	vout = mVCKeys[index+1].mDValue1 * mVCKeys[index+1].mDValue2;
    else
	vout = mVCKeys[index+1].mDValue1;

    double value0 = mVCKeys[index].mValue;
    double value1 = mVCKeys[index+1].mValue;
    double period = mCPeriodic.mBody;
    if (period != 0)
	value1 = GetValue1Periodic(value0, value1, period);

    mInterpolatedValue = 
	value0 + 
	vin * scale * interpolateRate + 
	(-3 * value0 - 2 * vin * scale + 3 * value1 - vout * scale) * interpolateRate * interpolateRate + 
	(2 * value0 + vin * scale - 2 * value1 + vout * scale) * interpolateRate * interpolateRate * interpolateRate;

    /*
    mInterpolatedValue = 
	mVCKeys[index].mValue + 
	vin * scale * interpolateRate + 
	(-3 * mVCKeys[index].mValue - 2 * vin * scale + 3 * mVCKeys[index+1].mValue - vout * scale) * interpolateRate * interpolateRate + 
	(2 * mVCKeys[index].mValue + vin * scale - 2 * mVCKeys[index+1].mValue + vout * scale) * interpolateRate * interpolateRate * interpolateRate;
    */
}

double Parameters_cls::
GetInterpolatedValue(double doubleIndex, double resolution){
    if (doubleIndex == _PARAM_CURRENT) doubleIndex = msCurrentDoubleIndex;
    if (resolution  == _PARAM_USE_COMMON) resolution = msResolution;
    
    if (mVCKeys.size() == 0) {
	mInterpolatedValue = 0;
	return mInterpolatedValue;
    }
    if (mCbTimeDependent.mbBody == FALSE ||
	doubleIndex <= mVCKeys[0].mDoubleIndex){
	mInterpolatedValue = mVCKeys[0].mValue;
	return mInterpolatedValue;
    }

    if (doubleIndex >= mVCKeys[mVCKeys.size() - 1].mDoubleIndex){
	mInterpolatedValue = mVCKeys[mVCKeys.size() - 1].mValue;
	return mInterpolatedValue;
    }
    
    for (unsigned int index = 0; index < mVCKeys.size() - 1; index++){
	if (doubleIndex < mVCKeys[index + 1].mDoubleIndex - resolution / 2){
	    double interpolateRate = 
		(doubleIndex - mVCKeys[index].mDoubleIndex) /
		(mVCKeys[index + 1].mDoubleIndex - mVCKeys[index].mDoubleIndex);
	    switch(mVCKeys[index].mType){
	    case _PARAM_LINEAR:
		CalcInterpolateLinear(interpolateRate, index);
		break;
	    case _PARAM_HERMITE_AUTO:
	    case _PARAM_HERMITE:
	    case _PARAM_HERMITE_ADJ:
		CalcInterpolateHermite(interpolateRate, index);
		break;
	    case _PARAM_STEP:
		CalcInterpolateStep(interpolateRate, index);
		break;
	    default:
		CalcInterpolateLinear(interpolateRate, index);
		break;
	    }
	    return mInterpolatedValue;
	}
    }
    
    mInterpolatedValue = mVCKeys[mVCKeys.size()-1].mValue;
    return mInterpolatedValue;
}

bool Parameters_cls::
SelectKeyFrameRegionQueue(double doubleIndexA, double doubleIndexB, bool bSelected){
    if (mpCParentGroupLink != NULL && 
	mpCParentGroupLink->mbLinking == TRUE)
	return mpCParentGroupLink->SelectKeyFrameRegion
	    (doubleIndexA, doubleIndexB, bSelected);
    return SelectKeyFrameRegion(doubleIndexA, doubleIndexB, bSelected);
}

bool Parameters_cls::
SelectKeyFrameRegion(double doubleIndexA, double doubleIndexB, bool bSelected){
    bool bRet = FALSE;
    if (doubleIndexB < doubleIndexA){
	double tmp = doubleIndexB;
	doubleIndexB = doubleIndexA;
	doubleIndexA = tmp;}
    size_t keySize = mVCKeys.size();
    for (size_t i = 0; i < keySize; i++)
	if (mVCKeys[i].mDoubleIndex >= doubleIndexA &&
	    mVCKeys[i].mDoubleIndex <= doubleIndexB){
	    mVCKeys[i].mbSelected = bSelected;
	    bRet = TRUE;
	}
    return bRet;
}

bool Parameters_cls::
SelectKeyByXY(double valueX, double resolutionX,
	      double valueY, double resolutionY,
	      bool bSingleSelect){
    bool bRet = FALSE;
    size_t keySize = mVCKeys.size();
    for (size_t i = 0; i < keySize; i++)
	if (mVCKeys[i].mDoubleIndex > valueX - resolutionX &&
	    mVCKeys[i].mDoubleIndex < valueX + resolutionX &&
	    mVCKeys[i].mValue > valueY - resolutionY &&
	    mVCKeys[i].mValue < valueY + resolutionY){
	    mVCKeys[i].mbSelected = TRUE;
	    bRet = TRUE;
	    if (bSingleSelect) return TRUE;
	}
    return bRet;
}

int Parameters_cls::
OnWhichKeyFrame(double doubleIndex,  double resolutionByPixel){
    int  selected = -1;
    size_t keySize = mVCKeys.size();
    for (size_t i = 0; i < keySize; i++)
	if (mVCKeys[i].Select(doubleIndex, resolutionByPixel, TRUE) == TRUE)
	    selected = (int)i;
    return selected;
}

void Parameters_cls::
SelectKeyFrameQueue(double doubleIndex,  double resolutionByPixel,
		    bool   bMultiSelect, bool bDeselect){
    if (mpCParentGroupLink != NULL && 
	mpCParentGroupLink->mbLinking == TRUE){
	mpCParentGroupLink->SelectKeyFrame
	    (doubleIndex, resolutionByPixel, bMultiSelect, bDeselect);
	return;
    }
    
    SelectKeyFrame(doubleIndex, resolutionByPixel,
		   bMultiSelect, bDeselect);
}

void Parameters_cls::
SelectKeyFrame(double doubleIndex,  double resolutionByPixel,
	       bool   bMultiSelect, bool bDeselect){
    int  selected = -1;
    bool bSelectSelected = FALSE;
    
    int keySize = GetKeyNumber();
    for (int i = 0; i < keySize; i++){
	bool state = mVCKeys[i].mbSelected;
	if (mVCKeys[i].Select(doubleIndex, resolutionByPixel) == TRUE){
	    selected = (int)i;
	    if (state == TRUE) bSelectSelected = TRUE;
	    i = GetKeyNumber();
	}
    }
    
    if (bMultiSelect == TRUE){
	if (bDeselect == FALSE || selected == -1){
	    if (mpCParamTLLink != NULL) mpCParamTLLink->mSelected = selected;
	    return;// selected;
	}
    }

    if (bDeselect == TRUE && bSelectSelected == FALSE)
	for (int i = 0; i < keySize; i++)
	    if (i != selected)
		mVCKeys[i].mbSelected = FALSE;
    
    if (mpCParamTLLink != NULL) mpCParamTLLink->mSelected = selected;
    return;// selected;
}

void Parameters_cls::
DeselectAll(){
    for (int i = 0; i < GetKeyNumber(); i++)
	mVCKeys[i].mbSelected = FALSE;
}

void Parameters_cls::
ProcStartDraggingQueue(double origin, bool bCtrl, bool bAlt){
    if (mpCParentGroupLink != NULL && 
	mpCParentGroupLink->mbLinking == TRUE){
	mpCParentGroupLink->ProcStartDragging(origin, bCtrl, bAlt);
	return;
    }
    mpCParamTLLink->ProcStartDragging(origin, bCtrl, bAlt);
}

void Parameters_cls::
ProcDraggingQueue(wxMouseEvent& event){
    if (mpCParentGroupLink != NULL && 
	mpCParentGroupLink->mbLinking == TRUE){
	mpCParentGroupLink->ProcDragging(event);
	return;
    }
    mpCParamTLLink->ProcDragging(event);
}

void Parameters_cls::
SetRegionOriginQueue(double origin){
    if (mpCParentGroupLink != NULL &&
	mpCParentGroupLink->mbLinking == TRUE){
	mpCParentGroupLink->SetRegionOrigin(origin);
    }
    mpCParamTLLink->SetRegionOrigin(origin);
}

void Parameters_cls::
TurnOffDraggingQueue(){
    if (mpCParentGroupLink != NULL && 
	mpCParentGroupLink->mbLinking == TRUE){
	mpCParentGroupLink->TurnOffDragging();
	return;
    }
    mpCParamTLLink->TurnOffDragging();
}

// -----
void Parameters_cls::
CopyKeyFrameToClipBoard(){

    int itemnum = 0;
    size_t keySize = mVCKeys.size();
    for (size_t i = 0; i < keySize; i++)
	if (mVCKeys[i].mbSelected == TRUE) itemnum++;
    if (itemnum == 0) return;
    
    wxString strInfo;
    strInfo << wxT("Parameter_") << mstrLabel << ENDLWX;
    //strInfo << wxT("dimension ") << mCDimension.mBody << ENDLWX;
    strInfo << wxT("itemnum ") << itemnum << ENDLWX;
	    
    for (size_t i = 0; i < keySize; i++)
	if (mVCKeys[i].mbSelected == TRUE)
	    strInfo << mVCKeys[i].Serialize();
    
    if (wxTheClipboard->Open()){
	// This data objects are held by the clipboard,
	// so do not delete them in the app.
	wxTheClipboard->SetData( new wxTextDataObject(strInfo) );
	wxTheClipboard->Close();
    }
}

void Parameters_cls::
PasteKeyFrameFromClipBoard(){
    /*
    wxStringEX strData;
    wxString strLine;
    if (wxTheClipboard->Open())	{
	if (wxTheClipboard->IsSupported( wxDF_TEXT )){
	    wxTextDataObject data;
	    wxTheClipboard->GetData( data );
	    strData = data.GetText();
	}
	wxTheClipboard->Close();
    }
    
    strData.GetLine(strLine);
    wxString tagStr; tagStr << wxT("Parameter_") << mLabelStr;
    int  loc = strLine.Find(tagStr);
    if (loc == wxNOT_FOUND) return;
    
    //MsgBoxf(NULL, "!!", "Find\n%s", strLine.ToAscii());

    strData.GetLine(strLine);
    wxString strDimension = strLine.c_str() + strlen("dimension ");
    unsigned int dimension = atoi(strDimension.ToAscii());

    strData.GetLine(strLine);
    wxString strItemNum = strLine.c_str() + strlen("itemnum ");
    unsigned int itemnum = atoi(strItemNum.ToAscii());
    
    //MsgBoxf(NULL, "!!", "Find\n%s dim=%d item=%d", strLine.ToAscii(), dimension, itemnum);

    double initDoubleIndex;
    GCUndoList.OpenBlock();
    for (unsigned int i = 0; i < itemnum; i++){
	strData.GetLine(strLine);
	KeyFrame_cls CKey(mCDimension.mBody);
	CKey.ReadStrLine(strLine.c_str());
	if (i == 0) initDoubleIndex = CKey.mDoubleIndex;
	CKey.mDoubleIndex += -initDoubleIndex + msCurrentDoubleIndex;

	if (CKey.mDoubleIndex > 1) continue;

	SetKey(CKey);
    }
    GCUndoList.CloseBlock();
    AdjustAutoControl();
    */
}

wxString Parameters_cls::
Serialize(){
    wxString strOut;
    strOut << wxT("Parameter_") << mstrLabel << ENDLWX;
    strOut << mCConfList.SerializeAll();
    strOut << wxT("itemnum ") << (int)mVCKeys.size() << ENDLWX;
    size_t keySize = mVCKeys.size();
    for (size_t i = 0; i < keySize; i++)
	strOut << mVCKeys[i].Serialize();
    return strOut;
}

void Parameters_cls::
ReadStrLine(std::wistream &istream, wxString strLineTag){
    wxString strTag = wxT("Parameter_");
    strTag << mstrLabel;
    int loc = strLineTag.Find(strTag);
    if (loc != wxNOT_FOUND){
	mVCKeys.clear();
	std::wstring wstrLine;
	wxString strLine;
	while (TRUE){
	    std::getline(istream, wstrLine);
	    strLine = wstrLine.c_str();
	    if (mCConfList.ReadStrLineAll(strLine) == FALSE) break;
	}
	// $B%3%3$O7h$aBG$A(B
	long itemnum;
	int loc = strLine.Find(wxT("itemnum "));
	if (loc == wxNOT_FOUND) itemnum = 0; //ERROR!!
	else strLine.Mid(8).ToLong(&itemnum);

	for (unsigned int i = 0; i < itemnum; i++){
	    std::getline(istream, wstrLine);
	    strLine = wstrLine.c_str();
	    mVCKeys.push_back(KeyFrame_cls());
	    mVCKeys[mVCKeys.size() - 1].ReadStrLine(strLine);
	}
	
	AdjustAutoControl();
    }
}

int    Parameters_cls::msFrameNumber = 100;
int    Parameters_cls::msAnimInterp = 1;
double Parameters_cls::msResolution = 0.01;
//unsigned int Parameters_cls::msStepQuantum = 1;
double Parameters_cls::msCurrentDoubleIndex = 0;
//std::vector < Parameters_cls* > *Parameters_cls::mspVPInstancePrim;
TimeCtrl_cls*     Parameters_cls::mspCTimeCtrlLink = NULL;
PDataSeqCtrl_cls* Parameters_cls::mspCPDataSeqCtrlLink = NULL;

// ------------------------

ComParamBase_cls::
ComParamBase_cls(Parameters_cls* pCTgt)  
{
    mpCTgt = pCTgt;
    mUpdateBit = pCTgt->mUpdateBit;
}

ComParamSetKey_cls::
ComParamSetKey_cls(Parameters_cls* pCTgt, double doubleIndex,
		   double value, double DValue1, double DValue2,
		   unsigned int type) : 
    ComParamBase_cls(pCTgt)
{
    mDoubleIndex = doubleIndex;
    mType = type;
    mValue = value;
    mDValue1 = DValue1;
    mDValue2 = DValue2;
}

ComParamSetKeyC_cls::
ComParamSetKeyC_cls(Parameters_cls* pCTgt, KeyFrame_cls CKey) :
ComParamBase_cls(pCTgt)
{
    mCKey.FullCopy(CKey);
}

void ComParamSetKey_cls::
Execute(bool bFlag){
    mpCTgt->ExecCommandSetKey(mDoubleIndex, mValue, mType,
			      mDValue1, mDValue2);
    UpdateBit();
}

void ComParamSetKeyC_cls::
Execute(bool bFlag){
    mpCTgt->ExecCommandSetKey(mCKey);
    UpdateBit();
}

ComParamDelKey_cls::
ComParamDelKey_cls(Parameters_cls *pCTgt, int unsigned index) :
ComParamBase_cls(pCTgt)
{
    mIndex = index;
}

void ComParamDelKey_cls::
Execute(bool bFlag){
    mpCTgt->ExecCommandDeleteKey(mIndex);
    UpdateBit();
}

ComParamOvwtKeyC_cls::
ComParamOvwtKeyC_cls(Parameters_cls* pCTgt, unsigned int index,
		     KeyFrame_cls CKey):
    ComParamBase_cls(pCTgt)
{
    mIndex = index;
    mCKey.FullCopy(CKey);
}

void ComParamOvwtKeyC_cls::
Execute(bool bFlag){
    mpCTgt->ExecCommandOvwtKey(mIndex, mCKey);
    UpdateBit();
}

ComParamMoveKey_cls::
ComParamMoveKey_cls(Parameters_cls* pCTgt, 
		    double doubleIndexFrom, double doubleIndexTo, 
		    double resolution) :
    ComParamBase_cls(pCTgt)
{
    mDoubleIndexFrom = doubleIndexFrom;
    mDoubleIndexTo   = doubleIndexTo;
    mResolution      = resolution;
}

void ComParamMoveKey_cls::
Execute(bool bFlag){
    mpCTgt->ExecCommandMoveKey(mDoubleIndexFrom, mDoubleIndexTo, mResolution);
    UpdateBit();
}

ComParamSetKeyType_cls::
ComParamSetKeyType_cls(Parameters_cls *pCTgt, 
		       unsigned int index, unsigned int type) : 
    ComParamBase_cls(pCTgt)
{
    mIndex = index;
    mType = type;
}

void ComParamSetKeyType_cls::
Execute(bool bFlag){
    mpCTgt->ExecCommandSetKeyType(mIndex, mType);
    UpdateBit();
}

ComParamSetKeyTypeMulti_cls::
ComParamSetKeyTypeMulti_cls(Parameters_cls *pCTgt,
			    unsigned int listnum,
			    unsigned int *listIndex,
			    unsigned int *listType) :
    ComParamBase_cls(pCTgt)
{
    mListnum = listnum;
    mAIndex = listIndex;
    mAType = listType;
}

ComParamSetKeyTypeMulti_cls::
~ComParamSetKeyTypeMulti_cls(){
    delete mAIndex;
    delete mAType;
}

void ComParamSetKeyTypeMulti_cls::
Execute(bool bFlag){
    mpCTgt->ExecCommandSetKeyTypeMulti(mListnum, mAIndex, mAType);
    UpdateBit();
}

ComParamClearKey_cls::
ComParamClearKey_cls(Parameters_cls* pCTgt, double value) :
    ComParamBase_cls(pCTgt)
{
    mValue = value;
}

ComParamClearKey_cls::
~ComParamClearKey_cls(){
}

void ComParamClearKey_cls::
Execute(bool bFlag){
    mpCTgt->ExecCommandInitSet(mValue);
    UpdateBit();
}

ComParamSetTimeDependent_cls::
ComParamSetTimeDependent_cls(Parameters_cls* pCTgt, bool bDependent) :
    ComParamBase_cls(pCTgt)
{
    mbDependent = bDependent;
}

void ComParamSetTimeDependent_cls::
Execute(bool bFlag){
    mpCTgt->ExecCommandTimeDependent(mbDependent);
    UpdateBit();
}

ComParamMemento_cls::
ComParamMemento_cls(Parameters_cls* pCTgt, Parameters_cls *pCFrom) :
    ComParamBase_cls(pCTgt)
{
    if (pCFrom == NULL){
	mpCParamMemento = new Parameters_cls();
	mpCParamMemento->CopyFromP(pCTgt);
    }
    else{
	mpCParamMemento = new Parameters_cls();
	mpCParamMemento->CopyFromP(pCFrom);
    }
}

ComParamMemento_cls::
~ComParamMemento_cls(){
    delete mpCParamMemento;
}

void ComParamMemento_cls::
Execute(bool bFlag){
    mpCTgt->CopyFromP(mpCParamMemento);
    UpdateBit();
}
