// -*- c -*- 
// "clsTics.h"

#include "clsTics.h"

Tics_cls::
Tics_cls(){
    mBGColor  = wxColour(216, 216, 216);
    mFGColor1 = wxColour(32, 32, 32);
    mFGColor2 = wxColour(128, 128, 128);
}

void Tics_cls::
SetParameter(wxSize size, float min, float max, int offset){
    
    mTicWidth  = size.GetWidth();
    mTicHeight = size.GetHeight();

    float width = max - min;
    mRegionMin = min;
    mRegionMax = max;
    mTicOffset = offset;
    
    if (width < 2) mbUseFloat = TRUE;
    else mbUseFloat = FALSE;

    float ratio = width / mTicWidth;

    if (mRegionMax > 1000) ratio *= 2;
		       
    mStepLarge = 1;
    mStepFloatScaling = 1;
    mStepFloatOrder = 1;
    if (ratio < 0.005)  mStepFloatScaling = 10;
    if (ratio < 0.001)  {mStepFloatScaling = 20; mStepFloatOrder = 2;}
    if (ratio < 0.0005) {mStepFloatScaling = 100; mStepFloatOrder = 2;}
    if (ratio < 0.0001) {mStepFloatScaling = 200; mStepFloatOrder = 3;}
    if (ratio < 0.00005) {mStepFloatScaling = 1000; mStepFloatOrder = 3;}
    if (ratio < 0.00001) {mStepFloatScaling = 2000; mStepFloatOrder = 4;}
    if (ratio > 0.02)  mStepLarge = 2;
    if (ratio > 0.05)  mStepLarge = 5;
    if (ratio > 0.10)  mStepLarge = 10;
    if (ratio > 0.20)  mStepLarge = 20;
    if (ratio > 0.50)  mStepLarge = 50;
    if (ratio > 1.00)  mStepLarge = 100;
    if (ratio > 2.00)  mStepLarge = 200;
    if (ratio > 5.00)  mStepLarge = 500;
    if (ratio > 10.00)  mStepLarge = 1000;
    if (ratio > 20.00)  mStepLarge = 2000;
    if (ratio > 100.00)  mStepLarge = 10000;
    if (ratio > 200.00)  mStepLarge = 20000;
    if (ratio > 500.00)  mStepLarge = 50000;
    if (ratio > 1000.00)  mStepLarge = 100000;
    mStepMiddle = mStepLarge / 2;
    mStepSmall = mStepLarge / 10; 
    if (mStepLarge == 5) mStepMiddle = 5;
    if (mStepSmall == 0) mStepSmall = 1;
    if (mStepMiddle == 0) mStepMiddle = 1;
}

void Tics_cls::
SetColorParameter(wxColour BGColor, wxColour FGColor1, wxColour FGColor2){
    mBGColor = BGColor;
    mFGColor1 = FGColor1;
    mFGColor2 = FGColor2;
}

int Tics_cls::
GetTicDeviceX(float x){
    return (int)(x * (mTicWidth - 2. * mTicOffset) + mTicOffset);
}

void Tics_cls::
CalcTicDeviceCoordinate(int *pX0, int *pX1,
			float frame0, float frame1){
    if (mRegionMax == mRegionMin){
	*pX0 = 0; *pX1 = 1;
	return;
    }

    float diff0 = frame0 - mRegionMin;
    float diff1 = frame1 - mRegionMin;
    float width = mRegionMax - mRegionMin;
    float x0 = diff0/width;
    float x1 = diff1/width;
    *pX0 = GetTicDeviceX(x0); *pX1 = GetTicDeviceX(x1);
}

void Tics_cls::
DrawTics(wxDC &dc){
    dc.SetBackground(wxBrush(mBGColor, wxSOLID));
    dc.Clear();

    dc.SetFont(*wxSMALL_FONT);
    dc.SetPen(mFGColor2);
    dc.SetTextForeground(mFGColor2);
    if (mbUseFloat == TRUE){
	for (int i = mRegionMin * mStepFloatScaling; i <= mRegionMax * mStepFloatScaling; i++){
	    CalcTicDeviceCoordinate(&mTicsX0, &mTicsX1, 
				    (float)i/mStepFloatScaling, (float)i/mStepFloatScaling);
	    dc.SetPen(mFGColor1);

	    wxString Mes;
	    wxString strFormat;
	    strFormat << wxT("%0.") << mStepFloatOrder << wxT("f");
	    Mes << wxString::Format(strFormat, (float)i/mStepFloatScaling);
	    int mesOffset = 2;
	    dc.DrawText(Mes, mTicsX0+mesOffset, +0);
	    dc.DrawRectangle(mTicsX0, mTicHeight-6, 1, 6);
	}
    }
    else
	for (int i = (int)mRegionMin; i <= (int)mRegionMax; i++){
	    CalcTicDeviceCoordinate(&mTicsX0, &mTicsX1, 
				    i, i);
	    if (i % mStepSmall != 0) continue;
	    if (mTicsX1 < 0 || mTicsX0 > mTicWidth) continue;
	    if (i % mStepMiddle == 0) 
		dc.SetPen(mFGColor1);
	    else
		dc.SetPen(mFGColor2);
	    if (i % mStepLarge == 0){
		wxString Mes;
		Mes << i;
		int mesOffset = 2;
		if (i > 0.95 * (mRegionMax - mRegionMin) + mRegionMin){
		    if (i < 10) mesOffset = -12;
		    else mesOffset = -18;
		}
		dc.DrawText(Mes, mTicsX0+mesOffset, +0);
		dc.DrawRectangle(mTicsX0, mTicHeight-6, 1, 6);
		continue;
	    }
	    dc.DrawRectangle(mTicsX0, mTicHeight-3, 1, 3);
	}
}
