// -*- c -*- 
// "clsGLDrawData.cpp"

#ifndef _GL_DRAW_DATA_H_
#define _GL_DRAW_DATA_H_

#include "../MYGL/clsPositionAndDirection.h"

#ifdef WIN32
#include <windows.h>
#include <GL/glu.h>
#include <GL/glext.h>
#elif defined __WXMAC__
#define GL_GLEXT_PROTOTYPES 
#include <OpenGL/glu.h>
#else
#define GL_GLEXT_PROTOTYPES 
#include <GL/glu.h>
#endif

#include <wx/thread.h>

class PDataBlock_cls;
class ShadowVolume_cls;
class SnapData_cls;
class GLAttribute_cls;

#define _SORT_RADIAL 0
#define _SORT_ORTHO  1

#ifdef _DEBUG
#include <stdlib.h>
#include <crtdbg.h>
#define new new(_NORMAL_BLOCK, __FILE__, __LINE__)
#endif

#define PACKNUM 300

class ArrayAndIndex_cls{ 
public:
    int index;
    float z;
    int operator < (ArrayAndIndex_cls rhs) const{
	return (z < rhs.z);}
    int operator > (ArrayAndIndex_cls rhs) const{
	return (z > rhs.z);}
};

class ThreadA_cls : public wxThread {
 public:
    ThreadA_cls(wxThreadKind kind);

    void SetParameters(bool *pbFinishedFlag, float *pX, int offset, int pnumber, 
		       float *vertexArray, float *pRotatedArray);
    
    bool  *mpbFinishedFlagLink;
    float *mpX;
    int    mOffset;
    int    mPNumber;
    float *mvertexArray;
    float *mpRotatedArray;
    
    virtual ExitCode Entry();
};

class GLDrawData_cls{
 public:
    GLDrawData_cls();
    ~GLDrawData_cls();

    static unsigned int mspIndices[PACKNUM * 4];
    static float        msTexCoordArray[PACKNUM * 4 * 2];
    static float        msVertexArray[PACKNUM * 4 * 3];
    static float        msVertexArrayOrg[12];
    static bool         msbInitialized;
    static SnapData_cls *mspCSnapShot;
    static ArrayAndIndex_cls *mspIndexAndZ;
    
    ShadowVolume_cls *mpCShadowVolume;
    
    float  mpZero4[4];
    float  mCameraPos[3], mCameraDir[3];

    int    mLightNum;
    double mLightPos[3];
    //float  mLightClr[4];
    double mLightPhi, mLightTheta;
    //float  mLightStrength;
    //float mLightTgt[3], mLightRgt[3], mLightTop[3];
    PositionAndDirection_cls mCLightPos;
    PositionAndDirection_cls mCParticlePos;

    int   mDownsizedAtFBO0;
    bool  mbShadowVolume;
    bool  mbShadowMode;
    
    
    void SetCameraPos(double *pCamPos);
    void SetCameraDir(double *pCamDir);
    void SetLightSourceInfo(int lightNumber,
			    float *pLightPos, //float *pLightClr,
			    double lightPhi, double lightTheta);

    void GLDraw(int fboIndex, PDataBlock_cls *pCDataBlock, 
		GLAttribute_cls **ppCGLAttribute, double dT);

    void GLDrawPointArrayElement(int fboIndex,
				 SnapData_cls *pCDataSnap,
				 GLAttribute_cls *pCGLAttribute);
    void GLDrawPointAll(int fboIndex,
			SnapData_cls *pCDataSnap,
			GLAttribute_cls *pCGLAttribute);
    void GLDrawBillboardBlock(float (*pX)[3], float *pRadius, int offset, int pnumber, 
			      float *pRotatedArray, float radiusFactor,
			      bool bConstantRadius, bool bConstantColor,
			      GLAttribute_cls *pCGLAttribute, float *pValue1);
    void MultiplyMatrixOnVertexArrayOrg(GLfloat matrix[16], float array[3]);
    void GLDrawBillboardAll(int fboIndex, SnapData_cls *pCDataSnap, 
			    GLAttribute_cls *pCGLAttribute);

    void GLDrawSphereConstRadiusConstantColor
	(int downsized, float (*pPos)[3], 
	 float radiusFactor, float quality);
    void GLDrawSphereConstRadiusVariableColor
	(int downsized, float (*pPos)[3], float *pValue1,
	 float radiusFactor, float quality, float steppingAlphaFactor,
	 GLAttribute_cls *pCGLAttribute);
    void GLDrawSphereVariableRadiusConstColor
	(int downsized, float (*pPos)[3], float *pRadius,
	 float radiusFactor, float quality);
    void GLDrawSphereVariableRadiusVariableColor
	(int downsized, float (*pPos)[3], float *pRadius, float *pValue1,
	 float radiusFactor, float quality, float steppingAlphaFactor,
	 GLAttribute_cls *pCGLAttribute);
    void GLDrawCylindeConstantRadius
	(int downsized, float (*pPos)[3], float *pRadius,
	 float radiusFactor, float quality);
    void GLDrawCylinderVariableRadius
	(int downsized, float (*pPos)[3], float *pRadius,
	 float radiusFactor, float quality);
    void DrawCylinder(float pos[3], float scale, int quality);
	 
    void GLDrawSphereAll(int fboIndex, SnapData_cls *pCDataBlock, 
			 GLAttribute_cls *pCGLAttribute);
    
    
    void DrawSphere(float pos[3], float scale, int quality);
    int  ZSortOrthA(int PNum, int step, float (*pX)[3], int sortType);
    int  ZSortSphericalA(int PNum, int step, float (*pX)[3], int sortType);
    void ZSortAtFBO0(int PNum, int step, float (*pX)[3],
		     int sortOrder, int sortType);
    int  ZSort(int fboIndex, 
	       SnapData_cls* pCDataSnap, GLAttribute_cls *pCGLAttribute);

    void GLDrawSequential(int fboIndex, 
			  SnapData_cls *pCDataSnap, 
			  GLAttribute_cls **ppCGLAttribute);
    void GLDrawShadowVolumeSequential
	(SnapData_cls *pCDataSnap, int downsized, int drawTypeList[4],
	 int radiusTypeList[4], float radiusFactorList[4], 
	 unsigned char *pType, float (*pPos)[3], float *pRadius);
};

#endif
