
#include "clsParticleDataBlock.h"
#include "clsParticleDataSnap.h"

#include "clsThreadFileReader.h"
extern ThreadFileReader_cls   *GpCThreadReader;
extern bool                    GbThreadReaderFinished;
extern int                     GThreadReaderIndex;
#include "clsThreadInterpolate.h"
extern ThreadInterpolator_cls *GpCThreadInterpolate;
extern bool                    GbThreadInterpolateFinished;

extern wxFrame       *GpCMainFrameCLS;

#include "../PANELS/clsDataLoader.h"

#include "stdio.h"
#include <fstream>

#include <omp.h>

#ifndef TRUE
#define TRUE true
#define FALSE false
#endif

#define X 0
#define Y 1
#define Z 2

PDataBlock_cls::
PDataBlock_cls(PDataSnap_cls *pCParentSnap){
    mpCParentSnap = pCParentSnap;
    mPNumBlock = _PNUM_UNDEFINED;
    mbAllocated = FALSE;
    mInterpolateType = _INTERPOLATE_HERMITE;
    mVariablePNum    = _INTERPOLATE_PNUM_CONSTANT;
    mbUseID = TRUE;
}

PDataBlock_cls::
~PDataBlock_cls(){
    ReleaseData();
}

void PDataBlock_cls::
Allocate(int pNum){
    if (mbAllocated == TRUE) ReleaseData();

    printf("Allocate %d\n", mPNumBlock);
	
    mPNumBlock = pNum;
    if (pNum == 0) return;

    mpX = new float [mPNumBlock][3];
    mpV = new float [mPNumBlock][3];
    
    if (mInterpolateType == _INTERPOLATE_HERMITE){
	mpA = new float [mPNumBlock][3];
	mpJ = new float [mPNumBlock][3];
    }

    mpRadius  = new float [mPNumBlock];
    mpDRadius = new float [mPNumBlock];
    mpType    = new unsigned char [mPNumBlock];
    mpValue1  = new float [mPNumBlock];
    mpDValue1 = new float [mPNumBlock];

    if (mbUseID == TRUE)
	mpID = new int [mPNumBlock];

    mbAllocated = TRUE;
}

void PDataBlock_cls::
ReleaseData(){
    if (mbAllocated == FALSE){
	;//printf("Release failed\n");
	return;
    }
    else
	;//printf("Release\n");

    delete[] mpX;
    delete[] mpV;
    
    if (mInterpolateType == _INTERPOLATE_HERMITE){
	printf("ReleaseAJ\n");
	delete[] mpA;
	delete[] mpJ;
    }

    delete[] mpRadius;
    delete[] mpDRadius;
    delete[] mpType;
    delete[] mpValue1;
    delete[] mpDValue1;

    if (mbUseID == TRUE)
	delete[] mpID;

    mbAllocated = FALSE;
    mPNumBlock = _PNUM_UNDEFINED;
}

int PDataBlock_cls::
GetUsedMemoryKB(){
    if (mPNumBlock == _PNUM_UNDEFINED) return 0;
    return (12 + 12 + 12 + 12 + 4 + 4) * mPNumBlock / 1024;
}

void PDataBlock_cls::
LoadData(wxString strFilename){
    if (mPNumBlock == _PNUM_UNDEFINED){
	switch(mspCDataLoaderLink->mCDataType.mBody){
	case _DATA_SIMPLEBINARY:
	    LoadHeaderBinary(strFilename);
	    break;
	case _DATA_SIMPLEASCII:
	    LoadHeaderAscii(strFilename);
	    break;
	}
	mInterpolateType = mspCDataLoaderLink->mCInterpType.mBody;
	mVariablePNum = mspCDataLoaderLink->mCVariablePNum.mBody;
    }
    
    // _HERE_ ReleaseMemoryIfNessesary!!
    if (mbAllocated == FALSE) Allocate(mPNumBlock);
    StartLoadingData(strFilename);
}

void PDataBlock_cls::
LoadHeaderBinary(wxString strFilename){
    std::fstream ifstream(wxFNCONV(strFilename),
			  std::ios::in | std::ios::binary);

    if (ifstream.fail()){
	printf("Read Header Fails!!\n");
	exit(0);
    }
    
    float time;
    int number;
    float dummy[10];

    int itemNumber = mspCDataLoaderLink->mCGrammerHead.mDimension;
    for (int i = 0; i < itemNumber; i++){
	switch(mspCDataLoaderLink->mCGrammerHead.mpBody[i]){
	case _GRAMMER_TIME:
	    ifstream.read((char*)&time, sizeof(float));
	    break;
	case _GRAMMER_NUMBER:
	    ifstream.read((char*)&number, sizeof(int));
	    break;
	case _GRAMMER_DUMMY1:
	    ifstream.read((char*)dummy, sizeof(float));
	    break;
	case _GRAMMER_DUMMY3:
	    ifstream.read((char*)dummy, sizeof(float) * 3);
	    break;
	case _GRAMMER_DUMMY10:
	    ifstream.read((char*)dummy, sizeof(float) * 10);
	    break;
	}
    }

    printf("time=%f\n", time);
    printf("number=%d\n", number);

    mPNumBlock = number;
    mpCParentSnap->mTime = time;
}

#define MAXLINE 255

void PDataBlock_cls::
LoadHeaderAscii(wxString strFilename){
    FILE *fp;

    fp = fopen(strFilename.mb_str(), "rt");

    if (fp == NULL){
	printf("Load Header Ascii Error\n");
	exit(0);
    }

    char buf[MAXLINE+1];
    int itemNumber = mspCDataLoaderLink->mCGrammerHead.mDimension;
    for (int i = 0; i < itemNumber; i++){
	if (fgets(buf, MAXLINE, fp) == NULL){
	    //ErrorProcess!! ;
	}
	switch(mspCDataLoaderLink->mCGrammerHead.mpBody[i]){
	case _GRAMMER_TIME:
	    mpCParentSnap->mTime = atof(buf);
	    break;
	case _GRAMMER_NUMBER:
	    mPNumBlock = atoi(buf);
	    break;
	case _GRAMMER_DUMMY1:
	    break;
	case _GRAMMER_DUMMY3:
	    break;
	case _GRAMMER_DUMMY10:
	    break;
	}
    }

    fclose(fp);
}

void PDataBlock_cls::
StartLoadingData(wxString strFilename){
    if (GpCThreadReader != NULL) return;

    GThreadReaderIndex = mpCParentSnap->mSelfIndex;
    GbThreadReaderFinished = FALSE;
    GpCThreadReader = new ThreadFileReader_cls(wxTHREAD_DETACHED);
    GpCThreadReader->SetParameters(&mpCParentSnap->mbLoadFinished,
				   mspCDataLoaderLink,
				   strFilename, mPNumBlock,
				   mpX, mpV, mpA, mpJ,
				   mpRadius, mpType, mpID, mpValue1,
				   mbUseID);
    GpCThreadReader->Run();
}

void PDataBlock_cls::
InterpolateData(PDataBlock_cls *pCNext){
    if (GpCThreadInterpolate != NULL) return;

    float dtime = pCNext->mpCParentSnap->mTime - mpCParentSnap->mTime;
    if (dtime == 0) dtime = 1;
    
    GbThreadInterpolateFinished = FALSE;
    GpCThreadInterpolate = new ThreadInterpolator_cls(wxTHREAD_DETACHED);
    GpCThreadInterpolate->SetParameters
	(&mpCParentSnap->mbInterpolateFinishd, 
	 mPNumBlock, pCNext->mPNumBlock, dtime,
	 mInterpolateType, mVariablePNum,
	 1, 0, mpX, mpV, mpA, mpJ,
	 mpRadius, mpDRadius, 
	 mpValue1, mpDValue1, 
	 mpType, mpID,
	 pCNext->mpX, pCNext->mpV, pCNext->mpA, pCNext->mpJ,
	 pCNext->mpRadius, pCNext->mpValue1, 
	 pCNext->mpType, pCNext->mpID);
    GpCThreadInterpolate->Run();
}

// ----

DataLoader_cls *PDataBlock_cls::mspCDataLoaderLink = NULL;
