
#include "clsParticleDataSequence.h"
#include "clsParticleDataSnap.h"
#include "../COMMON/clsFilename.h"

#ifdef WIN32
#define ENDLWX wxT("\r\n")
#else
#define ENDLWX wxT("\n")
#endif

#define LOOP_XYZ for (int xyz = 0; xyz < 3; xyz++)

PDataSequence_cls::
PDataSequence_cls(int index, DrawOption_cls *pCDrawOptionLink){
    mSelfIndex = index;
    for (int i = 0; i < _MAX_PDATA_INDEX; i++){
	mpCPDataSnap[i] = new PDataSnap_cls(this, i);
    }

    for (int i = 0; i < _MAX_PDATA_INDEX - 1; i++){
	mpCPDataSnap[i]->SetPNextSnap(mpCPDataSnap[i+1]);
	mpCPDataSnap[i+1]->SetPPrevSnap(mpCPDataSnap[i]);
    }

    mpCGLDrawData = new GLDrawData_cls;
    for (int i = 0; i < _MAX_PARTICLE_TYPE; i++)
	mpCGLAttribute[i] = new GLAttribute_cls(this, i, pCDrawOptionLink);
    
    wxString strTag = wxT("DataSequence"); strTag << mSelfIndex << wxT("=");
    wxString strSelfIndex; strSelfIndex << mSelfIndex;
    mCConfSelfIndex.SetItem(strTag, strSelfIndex);
}

PDataSequence_cls::
~PDataSequence_cls(){
    for (int i = 0; i < _MAX_PDATA_INDEX; i++)
	delete mpCPDataSnap[i];
    delete mpCGLDrawData;
    for (int i = 0; i < _MAX_PARTICLE_TYPE; i++)
	delete mpCGLAttribute[i];
}

void PDataSequence_cls::
InstantiateGLAttribGUI(wxWindow *pCParent){
    for (int i = 0; i < _MAX_PARTICLE_TYPE; i++)
	mpCGLAttribute[i]->InstantiateGUI(pCParent);
}

void PDataSequence_cls::
RefreshGLAttribTex(){
    for (int i = 0; i < _MAX_PARTICLE_TYPE; i++)
	mpCGLAttribute[i]->RefreshTex();
}

void PDataSequence_cls::
Clear(int index, bool bClearFileInfo){
    if (index == _SEQUENCE_ALL_INDEX){
	if (bClearFileInfo == TRUE)
	    mCFileSequence.Set(wxT(""));
	for (int i = 0; i < _MAX_PDATA_INDEX; i++)
	    mpCPDataSnap[i]->Clear(bClearFileInfo);
    }
    else
	mpCPDataSnap[index]->Clear(bClearFileInfo);
}

void PDataSequence_cls::
ReleaseMemoryOutofStep(int animStep){
    // FIX_ME!!
    // TimeRemap邽߂ɁA
    // {́AanimStepԊuFrameŕKvIndex𓾂
    // sKvIndexReleaseȂƂȂ
    for (int index = 0; index < mCFileSequence.mSeqLength; index++){
	if (index % animStep != 0 && index % animStep != 1 &&
	    mpCPDataSnap[index]->mbLoadStarted == TRUE &&
	    mpCPDataSnap[index]->mbLoadFinished == TRUE)
	    mpCPDataSnap[index]->ReleaseData();
    }
}

int  PDataSequence_cls::
SearchLowNeedStepLarge(int currentIndex, bool bSnipped){
    int candidateL = -1;
    for (int index = mCFileSequence.mSeqLength - 1; index > currentIndex; index--)
	if (mpCPDataSnap[index]->mbLoadStarted == TRUE &&
	    mpCPDataSnap[index]->mbLoadFinished == TRUE &&
	    mpCPDataSnap[index]->mbSnipping == bSnipped){
	    candidateL = index;
	    break;
	}
    return candidateL;
}

int  PDataSequence_cls::
SearchLowNeedStepSmall(int currentIndex, bool bSnipped){
    int candidateS = -1;
    for (int index = 0; index < currentIndex; index++)
	if (mpCPDataSnap[index]->mbLoadStarted == TRUE &&
	    mpCPDataSnap[index]->mbLoadFinished == TRUE &&
	    mpCPDataSnap[index]->mbSnipping == bSnipped){
	    candidateS = index;
	    break;
	}
    return candidateS;
}

bool PDataSequence_cls::
ReleaseMemoryAStep(int currentIndex, int animStep){
    int diffL0 = 0; // L[̂Ȃ폜
    int candidateL0 = SearchLowNeedStepLarge(currentIndex, FALSE);
    if (candidateL0 != -1)
	diffL0 = candidateL0 - currentIndex;

    int diffS0 = 0; // L[̂Ȃ폜
    int candidateS0 = SearchLowNeedStepSmall(currentIndex, FALSE);
    if (candidateS0 != -1)
	diffS0 = currentIndex - candidateS0;

    int diffL1 = 0; // L[̂폜
    int candidateL1 = SearchLowNeedStepLarge(currentIndex, TRUE);
    if (candidateL1 != -1)
	diffL1 = candidateL1 - currentIndex;

    int diffS1 = 0; // L[̂폜
    int candidateS1 = SearchLowNeedStepSmall(currentIndex, TRUE);
    if (candidateS1 != -1)
	diffS1 = currentIndex - candidateS1;

    bool bRelease = FALSE;
    if (diffL0 > diffS0 * 2 && diffL0 > 3 * animStep && diffL0 * 2 > diffL1){
	mpCPDataSnap[candidateL0]->ReleaseData(); bRelease = TRUE;
    }
    
    if (bRelease == FALSE && candidateS0 != -1 && diffS0 * 2 > diffS1){
	mpCPDataSnap[candidateS0]->ReleaseData(); bRelease = TRUE;
    }

    if (bRelease == FALSE && diffL1 > diffS1 * 2 && diffL1 > 3 * animStep){
	mpCPDataSnap[candidateL1]->ReleaseData(); bRelease = TRUE;
    }

    if (bRelease == FALSE && candidateS1 != -1){
	mpCPDataSnap[candidateS1]->ReleaseData(); bRelease = TRUE;
    }

    return bRelease;
}

void PDataSequence_cls::
SetIndividualFilenames(){
    int index = 0;
    for (int offset = 0; offset < mCFileSequence.mSeqLength; offset++){
	mpCPDataSnap[index]->SetFilePath(mCFileSequence.GetModifiedStr(offset));
	index++;
    }
}

void PDataSequence_cls::
OpenDataFile(wxString strFilePathInit){
    mCFileSequence.Set(strFilePathInit);

    mCFileSequence.mSeqLength = 0;
    
    // if (LoadSnapShot) mSeqLength = 1

    if (mCFileSequence.mSeqLength == 0) mCFileSequence.CalcMaxOffset();
    
    // SetMaxIndex(CFileSequence.mSeqLength)

    SetIndividualFilenames();
}

int PDataSequence_cls::
ClampIndexWithinRange(int index){
    if (mCFileSequence.mSeqLength > 0 &&
	index >= mCFileSequence.mSeqLength) 
	index = mCFileSequence.mSeqLength - 1;
    return index;
}

wxString PDataSequence_cls::
GetFilename(int index){
    index = ClampIndexWithinRange(index);
    return mCFileSequence.GetModifiedStr(index);
}

int      PDataSequence_cls::
GetFilenameStatus(int index){
    index = ClampIndexWithinRange(index);
    return mpCPDataSnap[index]->mFilenameStatus;
}

int      PDataSequence_cls::
GetParticleNumber(int index){
    index = ClampIndexWithinRange(index);
    return mpCPDataSnap[index]->GetParticleNumber();
}

double   PDataSequence_cls::
GetTime(int index){
    index = ClampIndexWithinRange(index);
    return mpCPDataSnap[index]->mTime;
}

double   PDataSequence_cls::
GetDT(int index, double interpStepDouble){
    double dT = GetTime(index+1) - GetTime(index);
    return dT * interpStepDouble;
}

void PDataSequence_cls::
ClearSnipping(){
    for (int index = 0; index < mCFileSequence.mSeqLength; index++)
	mpCPDataSnap[index]->mbSnipping = FALSE;
}

void PDataSequence_cls::
SetSnipping(int index){
    index = ClampIndexWithinRange(index);
    mpCPDataSnap[index]->mbSnipping = TRUE;
}

void PDataSequence_cls::
PrepareIndex(int index, double indexmod, bool bWaitFinish){
    index = ClampIndexWithinRange(index);
    bool bUseInterpolate = (indexmod != 0);
    mpCPDataSnap[index]->PrepareData(bUseInterpolate, bWaitFinish);
}

int PDataSequence_cls::
GetUsedTotalMemoryMB(){
    int totalMemoryKB = 0;
    for (int index = 0; index < mCFileSequence.mSeqLength; index++)
	totalMemoryKB += mpCPDataSnap[index]->GetUsedMemoryKB();
    return totalMemoryKB/1024;
}

int PDataSequence_cls::
GetLoadedFrameNumber(){
    int totalFrameNumber = 0;
    for (int index = 0; index < mCFileSequence.mSeqLength; index++)
	if (mpCPDataSnap[index]->mbLoadFinished == TRUE)
	    totalFrameNumber++;
    return totalFrameNumber;
}

bool PDataSequence_cls::
IsPreparing(int index, double indexmod){
    index = ClampIndexWithinRange(index);
    bool bUseInterpolate = (indexmod != 0);
    return mpCPDataSnap[index]->IsPreparing(bUseInterpolate);
}

bool PDataSequence_cls::
IsTheIndexReady(int index, double indexmod){
    index = ClampIndexWithinRange(index);
    bool bUseInterpolate = (indexmod != 0);
    return mpCPDataSnap[index]->IsItReady(bUseInterpolate);
}

bool PDataSequence_cls::
IsTheIndexLoaded(int index){
    index = ClampIndexWithinRange(index);
    return mpCPDataSnap[index]->IsItLoaded();
}


// -----

void PDataSequence_cls::
SetCameraInfo(PositionAndDirection_cls CCameraPos){
    mpCGLDrawData->SetCameraPos(CCameraPos.mPos);
    mpCGLDrawData->SetCameraDir(CCameraPos.mTgt);
}

void PDataSequence_cls::
GLDrawData(int fboIndex, int index, double dT, 
	   int lightNumber, float *pLightPos, 
	   double lightPhi, double lightTheta,
	   bool bShadowVolume, bool bShadowMode){
    index = ClampIndexWithinRange(index);
    mpCGLDrawData->mbShadowVolume = bShadowVolume;
    mpCGLDrawData->mbShadowMode   = bShadowMode;
    mpCGLDrawData->SetLightSourceInfo(lightNumber, pLightPos, lightPhi, lightTheta);
    mpCPDataSnap[index]->GLDrawData
	(fboIndex, mpCGLDrawData, mpCGLAttribute, dT); 
}

void PDataSequence_cls::
ExportData(DataExporter_cls *pCDataExporter, int index){
    index = ClampIndexWithinRange(index);
    mpCPDataSnap[index]->ExportData(pCDataExporter);
}

wxString PDataSequence_cls::
SerializeFile(){
    wxString strOut;
    strOut << mCConfSelfIndex.Serialize();
    strOut << mCFileSequence.mstrPath << ENDLWX;
    return strOut;
}

wxString PDataSequence_cls::
Serialize(){
    wxString strOut;

    strOut << SerializeFile();

    for (int i = 0; i < _MAX_PARTICLE_TYPE; i++){
	strOut << mpCGLAttribute[i]->Serialize();
    }
    
    return strOut;
}

void PDataSequence_cls::
ReadStrLine(std::wistream &istream, wxString strLine){
    if (mCConfSelfIndex.ReadStrLine(strLine) == TRUE){
	std::wstring wstr;
	std::getline (istream, wstr);
	strLine = wstr.c_str();
	if (strLine != wxT("")) OpenDataFile(strLine);
    }
    for (int i = 0; i < _MAX_PARTICLE_TYPE; i++){
	mpCGLAttribute[i]->ReadStrLine(istream, strLine);
    }
}
