
#include "clsGLDrawData.h"
#include "clsParticleDataSnap.h"
#include "clsParticleDataBlock.h"

#include "clsThreadFileReader.h"
extern ThreadFileReader_cls   *GpCThreadReader;
extern bool                   GbThreadReaderFinished;
#include "clsThreadInterpolate.h"
extern ThreadInterpolator_cls *GpCThreadInterpolate;
extern bool                   GbThreadInterpolateFinished;

#include "../PANELS/clsDataExporter.h"

#include "../../wxWidgetEnumMenu.h"

extern bool GbGLLockForLoadingData;
extern bool GbGLLockForInterplateData;

extern wxFrame  *GpCMainFrameCLS;

#include "fstream"

PDataSnap_cls::
PDataSnap_cls(PDataSequence_cls *pCParentSequence, int index){
    mpCParentSequence = pCParentSequence;
    mSelfIndex = index;
    mTime = -9999;
    mbSnipping = FALSE;
    for (int i = 0; i < _MAX_PDATA_BLOCK; i++)
	mpCPDataBlock[i] = new PDataBlock_cls(this);
    mFilenameStatus = _FN_STATUS_UNDEF;
    mbLoadStarted  = FALSE;
    mbLoadFinished = FALSE;
    mbInterpolateStarted = FALSE;
    mbInterpolateFinishd = FALSE;
    mpCNextPDSnapLink = NULL;
    mpCPrevPDSnapLink = NULL;
}

PDataSnap_cls::
~PDataSnap_cls(){
    for (int i = 0; i < _MAX_PDATA_BLOCK; i++)
	delete mpCPDataBlock[i];
}

void PDataSnap_cls::
SetPNextSnap(PDataSnap_cls *pCPDataSnap){
    mpCNextPDSnapLink = pCPDataSnap;
}

void PDataSnap_cls::
SetPPrevSnap(PDataSnap_cls *pCPDataSnap){
    mpCPrevPDSnapLink = pCPDataSnap;
}

void PDataSnap_cls::
Clear(bool bClearFileInfo){
    ReleaseData();
    if (bClearFileInfo == FALSE) return;
    mTime = -9999;
    mstrFilePath = wxT("");
    mFilenameStatus = _FN_STATUS_UNDEF;
}

void PDataSnap_cls::
ReleaseData(){
    WaitThreadReading(this, TRUE); // ̃t[̓ǂݍݒҋ@
    for (int i = 0; i < _MAX_PDATA_BLOCK; i++)
	mpCPDataBlock[i]->ReleaseData();
    mbLoadStarted  = FALSE;
    mbLoadFinished = FALSE;
    mbInterpolateStarted = FALSE;
    mbInterpolateFinishd = FALSE;
}

void PDataSnap_cls::
SetFilePath(wxString strFilePath){
    mstrFilePath = strFilePath;
    mFilenameStatus = _FN_STATUS_UNDEF;
}

int  PDataSnap_cls::
GetParticleNumber(){
    int number = 0;
    for (int i = 0; i < _MAX_PDATA_BLOCK; i++)
	if (mpCPDataBlock[i]->mPNumBlock != _PNUM_UNDEFINED)
	    number += mpCPDataBlock[i]->mPNumBlock;
    return number;
}

bool PDataSnap_cls::
IsFileValid(){
    if (mstrFilePath == wxT("")) return FALSE;
    if (mFilenameStatus == _FN_STATUS_INVALID) return FALSE;
    if (mFilenameStatus == _FN_STATUS_VALID) return TRUE;

    std::fstream ifstream(wxFNCONV(mstrFilePath),
			  std::ios::in | std::ios::binary);
    if (ifstream.fail() == TRUE){
	mFilenameStatus = _FN_STATUS_INVALID; return FALSE;
    }
    mFilenameStatus = _FN_STATUS_VALID; return TRUE;
}

void PDataSnap_cls::
WaitThreadReading(PDataSnap_cls *pCSnap, bool bPreparing){
    if (GpCThreadReader != NULL &&
	pCSnap->IsPreparing(FALSE) == bPreparing){
	GbGLLockForLoadingData = TRUE;
	while (GbThreadReaderFinished == FALSE){
	    wxMilliSleep(5);
	    wxCommandEvent com(wxEVT_COMMAND_MENU_SELECTED, menuEditRefreshSB);
	    GpCMainFrameCLS->AddPendingEvent(com);
	    while (wxTheApp->Pending())
		wxTheApp->Dispatch();
	}
	GpCThreadReader = NULL;
	GbGLLockForLoadingData = FALSE;
    }
}
 
void PDataSnap_cls::
WaitThreadInterpolate(PDataSnap_cls *pCSnap, bool bPreparing){
    if (GpCThreadInterpolate != NULL &&
	pCSnap->IsPreparing(TRUE) == bPreparing){
	GbGLLockForInterplateData = TRUE;
	while (GbThreadInterpolateFinished == FALSE)
	    wxMilliSleep(3);
	GpCThreadInterpolate = NULL;
	GbGLLockForInterplateData = FALSE;
    }
}

void PDataSnap_cls::
PrepareData(bool bUseInterpolate, bool bWaitFinish){
    if (mbLoadFinished == TRUE && mbInterpolateFinishd == TRUE) return;
    if (mbLoadFinished == FALSE && mbLoadStarted == FALSE)
	LoadData();
    
    if (bWaitFinish == TRUE)
	WaitThreadReading(this, TRUE);	// ̃t[̓ǂݍݒҋ@

    if (bUseInterpolate == FALSE) return;

    if (mbInterpolateStarted == FALSE && mbInterpolateFinishd == FALSE){
	InterpolateData(bWaitFinish);
    }

    if (bWaitFinish == TRUE)
	WaitThreadInterpolate(this, TRUE); // ̃t[̕Ԓҋ@
}

void PDataSnap_cls::
LoadData(){
    WaitThreadReading(this, FALSE); // ʃt[̓ǂݍݒҋ@

    if (IsFileValid() == TRUE){
	mbLoadStarted = TRUE;
	mpCPDataBlock[0]->LoadData(mstrFilePath);
    }
}

void PDataSnap_cls::
InterpolateData(bool bWaitFinish){
    if (mbLoadFinished == FALSE) return; // ̃pXł͖߂Ȃ͂

    if (mpCNextPDSnapLink->mbLoadFinished == FALSE) {
	mpCNextPDSnapLink->LoadData();

	if (bWaitFinish == FALSE) return;
	WaitThreadReading(mpCNextPDSnapLink, TRUE); // t[ǂݍݒҋ@
    }

    WaitThreadInterpolate(this, FALSE); //ʃt[̕Ԓҋ@
    
    // ̃t@C_Ȃ̂͏ĨXebv̂͂
    if (mpCNextPDSnapLink->IsFileValid() == FALSE){
	mbInterpolateStarted = TRUE;
	mpCPDataBlock[0]->InterpolateData(mpCPDataBlock[0]);
	return;
    }

    mbInterpolateStarted = TRUE;
    mpCPDataBlock[0]->InterpolateData(mpCNextPDSnapLink->mpCPDataBlock[0]);
}

int PDataSnap_cls::
GetUsedMemoryKB(){
    int usedMemoryKB = 0;
    for (int i = 0; i < _MAX_PDATA_BLOCK; i++){
	usedMemoryKB += mpCPDataBlock[i]->GetUsedMemoryKB();
    }
    return usedMemoryKB;
}

bool PDataSnap_cls::
IsPreparing(bool bUseInterpolate){
    if (bUseInterpolate == FALSE){
	if (mbLoadFinished == TRUE) return FALSE;
	if (mbLoadStarted == TRUE) return TRUE;
	return FALSE;
    }
    if (mbLoadFinished == TRUE && mbInterpolateFinishd == TRUE) return FALSE;
    if (mbLoadFinished == TRUE && mbInterpolateFinishd == FALSE) return TRUE;
    if (mbLoadStarted == TRUE) return TRUE;
    if (mbInterpolateStarted == TRUE) return TRUE;
    return FALSE;
}

bool PDataSnap_cls::
IsItReady(bool bUseInterpolate){
    // `ĂȂΕ\O.K.
    if (IsFileValid() == FALSE) return TRUE;
    if (bUseInterpolate == FALSE)
	return mbLoadStarted & mbLoadFinished;
    return mbLoadStarted & mbLoadFinished & mbInterpolateStarted & mbInterpolateFinishd;
}

bool PDataSnap_cls::
IsItLoaded(){
    return mbLoadStarted & mbLoadFinished;
}

// -----

void PDataSnap_cls::
GLDrawData(int fboIndex, 
	   GLDrawData_cls *pCDrawData, GLAttribute_cls **ppCGLAttribute,
	   double dT){
    for (int i = 0; i < _MAX_PDATA_BLOCK; i++){
	if (mpCPDataBlock[i]->mPNumBlock == 0 || 
	    mpCPDataBlock[i]->mPNumBlock == _PNUM_UNDEFINED) continue;
	pCDrawData->GLDraw(fboIndex, mpCPDataBlock[i], ppCGLAttribute, dT);
    }
}

void PDataSnap_cls::
ExportData(DataExporter_cls *pCDataExporter){
    pCDataExporter->ExportData(mpCPDataBlock[0]); //!! _AD_HOC_
}
