// -*- c -*-
// "clsShadowVolume.h"

#include "clsShadowVolume.h"

#include "../GLATTRIB/clsGLAttribute.h"
#include "../GLOBJECTS/clsCylinder.h"

#include "../../wxWidgetEnumMenu.h"

extern int  GDrawingPercentage;
extern bool GbAbort;

extern wxFrame  *GpCMainFrameCLS;

#ifdef WIN32
#include <windows.h>
#include <GL/glu.h>
#include <GL/glext.h>
#elif defined __WXMAC__
#define GL_GLEXT_PROTOTYPES 
#include <OpenGL/glu.h>
#else
#define GL_GLEXT_PROTOTYPES 
#include <GL/glu.h>
#endif

#define LOOP_XYZ for (int xyz = 0; xyz < 3; xyz++)
#define X 0
#define Y 1
#define Z 2

ShadowVolume_cls::
ShadowVolume_cls(ArrayAndIndex_cls **sppIndexAndZ,
		 PositionAndDirection_cls *pCParticlePos){
    msppIndexAndZLink = sppIndexAndZ;
    mpCParticlePosLink = pCParticlePos;
    
    mCylinderDList[0] = 0;
    mCylinderDList[1] = 0;

    mpbInShadowList = NULL;
}

void ShadowVolume_cls::
InitializeStaticMembers(){
    msCConfListStatic.Add(&msCShadowQuality, wxT("ShadowQuality="), wxT("1"));
    msCConfListStatic.Add(&msCShadowLength, wxT("ShadowLength="), wxT("20"));
}

void ShadowVolume_cls::
UpdateDisplayList(double phi, double theta){
    if (mCylinderDList[0] == 0) mCylinderDList[0] = glGenLists(1);
    if (mCylinderDList[1] == 0) mCylinderDList[1] = glGenLists(1);

    glNewList (mCylinderDList[0], GL_COMPILE);
    Cylinder_cls::DrawMain(0, -phi*3.14149/180, theta*3.14149/180, 
			   msCShadowQuality.mBody, msCShadowLength.mBody);
    glEndList();

    glNewList (mCylinderDList[1], GL_COMPILE);
    Cylinder_cls::DrawMain(1, -phi*3.14149/180, theta*3.14149/180, 
			   msCShadowQuality.mBody, msCShadowLength.mBody);
    glEndList();
}

// XN[ɉeƁAʏShadow-Volume͔j]B
// Qshadow-volumegĉł邪xB
// eXN[ɂ\eς
// }`XbhΑȂ]n͂邩

void ShadowVolume_cls::
CheckShadow(int downsized, float (*pPos)[3], float *pRadius,
	    float radiusFactor,
	    float cameraPos[3], float lightDir[3],
	    double lightPhi, double lightTheta){

    UpdateDisplayList(lightPhi, lightTheta);

    if (mpbInShadowList == NULL)
	mpbInShadowList = new bool [downsized];
    // {̓XN[ʒuɂ邩
    // Ƃ肠JʒuŌς

    ArrayAndIndex_cls* spIndexAndZ = *msppIndexAndZLink;

    float lightNormed[3];
    float lightNorm = sqrt(lightDir[X]*lightDir[X] + lightDir[Y]*lightDir[Y] + lightDir[Z]*lightDir[Z]);
    lightNormed[X] = lightDir[X]/lightNorm;
    lightNormed[Y] = lightDir[Y]/lightNorm;
    lightNormed[Z] = lightDir[Z]/lightNorm;

    float radius2xfac = radiusFactor * radiusFactor * 400;
    float rel[3];
    float projected[3];
    for (int i_ = 0; i_ < downsized; i_++){
	
	int i = (spIndexAndZ+i_)->index;
	
	mpbInShadowList[i_] = FALSE;
	rel[X] = cameraPos[X] - pPos[i][X];
	rel[Y] = cameraPos[Y] - pPos[i][Y];
	rel[Z] = cameraPos[Z] - pPos[i][Z];
	
	float norm = lightDir[X]*rel[X] + lightDir[Y]*rel[Y] + lightDir[Z]*rel[Z];
	projected[X] = rel[X] - norm * lightNormed[X];
	projected[Y] = rel[Y] - norm * lightNormed[Y];
	projected[Z] = rel[Z] - norm * lightNormed[Z];

	float projectedNorm2 = 
	    projected[X] * projected[X] + projected[Y] * projected[Y] + projected[Z] * projected[Z];

	if ( projectedNorm2 < radius2xfac)
	    mpbInShadowList[i_] = TRUE;
    }
}

void ShadowVolume_cls::
CheckShadowSequential(int downsized, int drawTypeList[4],
		      int radiusTypeList[4], float radiusFactorList[4],
		      unsigned char *pType, float (*pPos)[3], float *pRadius,
		      float cameraPos[3], float lightDir[3],
		      double lightPhi, double lightTheta){
    UpdateDisplayList(lightPhi, lightTheta);

    if (mpbInShadowList == NULL)
	mpbInShadowList = new bool [downsized];

    ArrayAndIndex_cls* spIndexAndZ = *msppIndexAndZLink;

    float lightNormed[3];
    float lightNorm = sqrt(lightDir[X]*lightDir[X] + lightDir[Y]*lightDir[Y] + lightDir[Z]*lightDir[Z]);
    lightNormed[X] = lightDir[X]/lightNorm;
    lightNormed[Y] = lightDir[Y]/lightNorm;
    lightNormed[Z] = lightDir[Z]/lightNorm;

    //float radius2xfac = radiusFactor * radiusFactor * 400;
    float rel[3];
    float projected[3];
    for (int i_ = 0; i_ < downsized; i_++){
	
	int i = (spIndexAndZ+i_)->index;
	int particleType = /*pCDataSnap->mpTypeLink[i]*/pType[i];
	int drawType = drawTypeList[particleType];
	if (drawType != _GL_DRAW_SPHERE) continue;
	float radiusFactor = radiusFactorList[particleType];
	if (radiusTypeList[particleType] != _GL_RADIUS_CONSTANT)
	    radiusFactor *= pRadius[i];

	float radius2xfac = radiusFactor * radiusFactor * 400;
	
	mpbInShadowList[i_] = FALSE;
	rel[X] = cameraPos[X] - pPos[i][X];
	rel[Y] = cameraPos[Y] - pPos[i][Y];
	rel[Z] = cameraPos[Z] - pPos[i][Z];
	
	float norm = lightDir[X]*rel[X] + lightDir[Y]*rel[Y] + lightDir[Z]*rel[Z];
	projected[X] = rel[X] - norm * lightNormed[X];
	projected[Y] = rel[Y] - norm * lightNormed[Y];
	projected[Z] = rel[Z] - norm * lightNormed[Z];

	float projectedNorm2 = 
	    projected[X] * projected[X] + projected[Y] * projected[Y] + projected[Z] * projected[Z];
	if ( projectedNorm2 < radius2xfac)
	    mpbInShadowList[i_] = TRUE;
    }
}
	
void ShadowVolume_cls::
GLDrawCylinderConstantRadius(int downsized, float (*pPos)[3], float *pRadius,
			     float radiusFactor, float quality){
    ArrayAndIndex_cls* spIndexAndZ = *msppIndexAndZLink;

    glScalef(radiusFactor, radiusFactor, radiusFactor);
    for (int ibase = 0; ibase < downsized; ibase+=PACKNUM) {
	int number = PACKNUM;
	if (ibase + number >= downsized) number = downsized - ibase;

	if (ibase % (PACKNUM * 500) == 0){
	    GDrawingPercentage = 100 * ibase / downsized;
	    wxCommandEvent com(wxEVT_COMMAND_MENU_SELECTED, menuEditRefreshSB);
	    GpCMainFrameCLS->AddPendingEvent(com);
	    while (wxTheApp->Pending())
		wxTheApp->Dispatch();
	}

	glPushMatrix();
	int i = (spIndexAndZ + ibase)->index;
	glTranslatef(pPos[i][X]/radiusFactor, pPos[i][Y]/radiusFactor, pPos[i][Z]/radiusFactor);

	if (mpbInShadowList[ibase] == TRUE){
	    glStencilOp(GL_KEEP, GL_KEEP, GL_INCR);
	    glDisable(GL_DEPTH_TEST);
	    glCallList(mCylinderDList[1]);
	    glEnable(GL_DEPTH_TEST);
	    glCallList(mCylinderDList[0]);
	    
	    glStencilOp(GL_KEEP, GL_KEEP, GL_DECR);
	    glDisable(GL_DEPTH_TEST);
	    glCallList(mCylinderDList[0]);
	    glEnable(GL_DEPTH_TEST);
	    glCallList(mCylinderDList[1]);
	}
	else{
	    glStencilOp(GL_KEEP, GL_KEEP, GL_INCR);
	    glCallList(mCylinderDList[0]);
	    
	    glStencilOp(GL_KEEP, GL_KEEP, GL_DECR);
	    glCallList(mCylinderDList[1]);
	}

	int prev = i;
	
	for (int i_ = ibase+1; i_ < ibase + number; i_ ++){
	    int i = (spIndexAndZ + i_)->index;
	    glTranslatef((pPos[i][X] - pPos[prev][X])/radiusFactor,
			 (pPos[i][Y] - pPos[prev][Y])/radiusFactor,
			 (pPos[i][Z] - pPos[prev][Z])/radiusFactor);
	    
	    if (mpbInShadowList[i_] == TRUE){
		glStencilOp(GL_KEEP, GL_KEEP, GL_INCR);
		glDisable(GL_DEPTH_TEST);
		glCallList(mCylinderDList[1]);
		glEnable(GL_DEPTH_TEST);
		glCallList(mCylinderDList[0]);
		
		glStencilOp(GL_KEEP, GL_KEEP, GL_DECR);
		glDisable(GL_DEPTH_TEST);
		glCallList(mCylinderDList[0]);
		glEnable(GL_DEPTH_TEST);
		glCallList(mCylinderDList[1]);
	    }
	    else{
		glStencilOp(GL_KEEP, GL_KEEP, GL_INCR);
		glCallList(mCylinderDList[0]);
		
		glStencilOp(GL_KEEP, GL_KEEP, GL_DECR);
		glCallList(mCylinderDList[1]);
	    }
	    
	    prev = i;
	}

	glPopMatrix();

	if (GbAbort == TRUE){
	    ibase = downsized;
	    GbAbort = FALSE;
	}
	
    }

    if (mpbInShadowList != NULL){
	delete [] mpbInShadowList;
	mpbInShadowList = NULL;
    }
}

void ShadowVolume_cls::
GLDrawCylinderVariableRadius(int downsized, float (*pPos)[3], float *pRadius,
			     float radiusFactor, float quality){
    ArrayAndIndex_cls* spIndexAndZ = *msppIndexAndZLink;

    for (int i_ = 0; i_ < downsized; i_++){
	int i = (spIndexAndZ + i_)->index;
	DrawCylinder(pPos[i], pRadius[i] * radiusFactor, i, i_);
	if (GbAbort == TRUE){
	    i_ = downsized;
	    GbAbort = FALSE;
	}
    }

    if (mpbInShadowList != NULL){
	delete [] mpbInShadowList;
	mpbInShadowList = NULL;
    }
}

void ShadowVolume_cls::
DrawCylinder(float pos[3], float scale, int i, int i_){
    // Look At LightSourceInf

    //mpCParticlePosLink->mPos[0] = pos[0];
    //mpCParticlePosLink->mPos[1] = pos[1];
    //mpCParticlePosLink->mPos[2] = pos[2];
    
    glPushMatrix();

    glTranslatef(pos[0], pos[1], pos[2]);
    glScaled(scale, scale, scale);

    if (mpbInShadowList[i_] == TRUE){
	glStencilOp(GL_KEEP, GL_KEEP, GL_INCR);
	glDisable(GL_DEPTH_TEST);
	glCallList(mCylinderDList[1]);
	glEnable(GL_DEPTH_TEST);
	glCallList(mCylinderDList[0]);
	
	glStencilOp(GL_KEEP, GL_KEEP, GL_DECR);
	glDisable(GL_DEPTH_TEST);
	glCallList(mCylinderDList[0]);
	glEnable(GL_DEPTH_TEST);
	glCallList(mCylinderDList[1]);
    }
    else{
	glStencilOp(GL_KEEP, GL_KEEP, GL_INCR);
	glCallList(mCylinderDList[0]);
	
	glStencilOp(GL_KEEP, GL_KEEP, GL_DECR);
	glCallList(mCylinderDList[1]);
    }

    /*
    glStencilOp(GL_KEEP, GL_KEEP, GL_INCR);
    Cylinder_cls::Draw(1);

    glStencilOp(GL_KEEP, GL_KEEP, GL_DECR);
    Cylinder_cls::Draw(0);
    */
    
    glPopMatrix();
}

ArrayAndIndex_cls** ShadowVolume_cls::msppIndexAndZLink = NULL;

ConfItemList_cls ShadowVolume_cls::msCConfListStatic;
ConfItemUINT_cls ShadowVolume_cls::msCShadowQuality;
ConfItemFLOAT_cls ShadowVolume_cls::msCShadowLength;
