
#include "clsSnapData.h"
#include "clsParticleDataBlock.h"
#include "clsParticleDataSnap.h"

#include "../GLATTRIB/clsColorCurve.h"
#include "../GLATTRIB/clsGLAttribute.h"

bool GbSnapDataRefresh = FALSE;

#include <omp.h>

SnapData_cls::
SnapData_cls(PDataBlock_cls *pCDataBlockLink,
	     GLAttribute_cls **ppCGLAttribute,
	     double dT){
    mbAllocated = FALSE;
    Allocate(pCDataBlockLink->mPNumBlock);
    Interpolate(pCDataBlockLink, ppCGLAttribute, dT);
    mTime = pCDataBlockLink->mpCParentSnap->mTime + dT;
}

SnapData_cls::
~SnapData_cls(){
    ReleaseData();
}

void SnapData_cls::
Allocate(int pNum){
    mPNumBlock = pNum;
    if (pNum == 0) return;

    mpX = new float [mPNumBlock][3];
    mpRadius = new float [mPNumBlock];
    mpValue1 = new float [mPNumBlock];

    mpColor = new float [mPNumBlock][4];
    
    mbAllocated = TRUE;
}

void SnapData_cls::
ReleaseData(){
    if (mbAllocated == FALSE){
	printf("Release Snap failed\n");
	return;
    }

    delete[] mpX;
    delete[] mpRadius;
    delete[] mpValue1;
    delete[] mpColor;

    mbAllocated = FALSE;
}

bool SnapData_cls::
IsItSameSnap(PDataBlock_cls *pCDataBlockLink, double dT){
    if (GbSnapDataRefresh == TRUE){
	GbSnapDataRefresh = FALSE;
	return FALSE;
    }

    if (mTime == pCDataBlockLink->mpCParentSnap->mTime + dT)
	return TRUE;
    else
	return FALSE;
}

void SnapData_cls::
InterpolateNot(PDataBlock_cls *pCDataBlockLink, 
	       GLAttribute_cls **ppCGLAttribute){
    mpTypeLink = pCDataBlockLink->mpType;

    int   alphaType[4];
    float alphaDependBase[4];
    float alphaMax[4];
    float alphaFactor[4];
    ColorCurve_cls *pCClrCurveCol[4];
    for (int i = 0; i < 4; i++){
	alphaType[i]        = ppCGLAttribute[i]->mCAlphaType.mBody; 
	alphaDependBase[i]  = ppCGLAttribute[i]->mCAlphaDenendBase.mBody;
	alphaMax[i]         = ppCGLAttribute[i]->mCAlphaMax.mBody;
	alphaFactor[i]      = ppCGLAttribute[i]->mCAlphaFactor.mBody;
	ppCGLAttribute[i]->mpCClrCurveCol->PreCalcScaleFactor();
	pCClrCurveCol[i] = ppCGLAttribute[i]->mpCClrCurveCol;
    }

    memcpy(mpX, pCDataBlockLink->mpX, sizeof(float)*3*mPNumBlock);
    memcpy(mpRadius, pCDataBlockLink->mpRadius, sizeof(float)*mPNumBlock);
    memcpy(mpValue1, pCDataBlockLink->mpValue1, sizeof(float)*mPNumBlock);

    float *pValue1[4];
    for (int i = 0; i < 4; i++)
	switch(ppCGLAttribute[i]->mpCClrCurveCol->mCDependType.mBody){
	case _DEPEND_ON_VALUE1:
	    pValue1[i] = pCDataBlockLink->mpValue1; break;
	case _DEPEND_ON_RADIUS:
	    pValue1[i] = pCDataBlockLink->mpRadius; break;
	case _DEPEND_ON_VELOCITY:
	    pValue1[i] = pCDataBlockLink->mpValue1; break; //!! _FIX_ME !!
	}
    
    float (*pColorMem)[4] = mpColor;
    unsigned char *pTypeLinkMem = mpTypeLink;

    for (int i = 0; i < mPNumBlock; i++){

	int particleType = pTypeLinkMem[i];
	
	// F̊{͌vZĂ
	pCClrCurveCol[particleType]->GetColor4fScaled(pColorMem[i], pValue1[particleType][i]);
	if (alphaType[particleType] == _GL_ALPHA_INVERSE_SQUARE){
	    pColorMem[i][3] *= alphaDependBase[particleType] * alphaDependBase[particleType];
	}
	else
	    pColorMem[i][3] *= alphaFactor[particleType];
    }
    
}

void SnapData_cls::
Interpolate(PDataBlock_cls *pCDataBlockLink, 
	    GLAttribute_cls **ppCGLAttribute, 
	    double dT){
    if (dT == 0){
	InterpolateNot(pCDataBlockLink, ppCGLAttribute);
	return;
    }

    mpTypeLink = pCDataBlockLink->mpType;

    int   alphaType[4];
    float alphaDependBase[4];
    float alphaMax[4];
    float alphaFactor[4];
    ColorCurve_cls *pCClrCurveCol[4];
    for (int i = 0; i < 4; i++){
	alphaType[i]        = ppCGLAttribute[i]->mCAlphaType.mBody; 
	alphaDependBase[i]  = ppCGLAttribute[i]->mCAlphaDenendBase.mBody;
	alphaMax[i]         = ppCGLAttribute[i]->mCAlphaMax.mBody;
	alphaFactor[i]      = ppCGLAttribute[i]->mCAlphaFactor.mBody;
	ppCGLAttribute[i]->mpCClrCurveCol->PreCalcScaleFactor();
	pCClrCurveCol[i] = ppCGLAttribute[i]->mpCClrCurveCol;
    }

    float (*pX)[3]  = pCDataBlockLink->mpX;
    float (*pV)[3]  = pCDataBlockLink->mpV;
    float *pRadius  = pCDataBlockLink->mpRadius;
    float *pDRadius = pCDataBlockLink->mpDRadius;
    float *pValue1  = pCDataBlockLink->mpValue1;
    float *pDValue1 = pCDataBlockLink->mpDValue1;
    
    float (*pXMem)[3] = mpX;
    float *pRadiusMem = mpRadius;
    float *pValue1Mem = mpValue1;
    float (*pColorMem)[4] = mpColor;
    unsigned char *pTypeLinkMem = mpTypeLink;

    float *pValue1ForColor[4];
    for (int i = 0; i < 4; i++)
	switch(ppCGLAttribute[i]->mpCClrCurveCol->mCDependType.mBody){
	case _DEPEND_ON_VALUE1:
	    pValue1ForColor[i] = mpValue1; break;
	case _DEPEND_ON_RADIUS:
	    pValue1ForColor[i] = mpRadius; break;
	case _DEPEND_ON_VELOCITY:
	    pValue1ForColor[i] = mpValue1; break; //!! _FIX_ME !!
	}

#ifdef _OPENMP
    // 1%xȂۂB
    omp_set_dynamic( 0 );
    omp_set_num_threads( 2 );
#pragma omp parallel for
#endif
    for (int i = 0; i < mPNumBlock; i++){
	pXMem[i][0] = pX[i][0] + pV[i][0] * dT;
	pXMem[i][1] = pX[i][1] + pV[i][1] * dT;
	pXMem[i][2] = pX[i][2] + pV[i][2] * dT;
	pRadiusMem[i] = pRadius[i] + pDRadius[i] * dT;
	pValue1Mem[i] = pValue1[i] + pDValue1[i] * dT;
	
	int particleType = pTypeLinkMem[i];
	
	// F̊{͌vZĂ
	pCClrCurveCol[particleType]->GetColor4fScaled(pColorMem[i], pValue1ForColor[particleType][i]);
	if (alphaType[particleType] == _GL_ALPHA_INVERSE_SQUARE){
	    pColorMem[i][3] *= alphaDependBase[particleType] * alphaDependBase[particleType];
	}
	else
	    pColorMem[i][3] *= alphaFactor[particleType];
    }
}
