
#include "clsThreadFileReader.h"
#include "clsThreadFileReaderAscii.h"
#include "../MYAPI/MsgBox.h"
#include "../PANELS/clsDataLoader.h"

#include "../../wxWidgetEnumMenu.h"

#include <fstream>
#include <sstream>
#include <string.h>

ThreadFileReader_cls *GpCThreadReader = NULL;
bool                 GbThreadReaderFinished = FALSE;
int                  GThreadReaderPercentage = 0;
int                  GThreadReaderIndex = 0;
extern wxFrame       *GpCMainFrameCLS;
extern int           GThreadNumberReadAscii;

#define X 0
#define Y 1
#define Z 2
#define LOOP_XYZ for (int xyz = 0; xyz < 3; xyz++)

// ------

ThreadFileReader_cls::
ThreadFileReader_cls(wxThreadKind kind) :
    wxThread(kind) {
}

void ThreadFileReader_cls::
SetParameters(bool *pbFinishedFlagLink, 
	      DataLoader_cls *pCDataLoder,
	      wxString strFilePath, int  pNum,
	      float (*pX)[3], float (*pV)[3],
	      float (*pA)[3], float (*pJ)[3],
	      float *pRadius, unsigned char *pType, int *pID,
	      float *pValue1,
	      bool bUseID){

    mpbFinishedFlagLink = pbFinishedFlagLink;
    mstrFilePath = strFilePath;

    mpX = pX;
    mpV = pV;
    mpA = pA;
    mpJ = pJ;
    mpRadius = pRadius;
    mpType = pType;
    mpID = pID;
    mpValue1 = pValue1;

    mbUseID = bUseID;

    mPNum = pNum;
    mpCDataLoaderLink = pCDataLoder;
    mDataType = mpCDataLoaderLink->mCDataType.mBody;
    for (int i = 0; i < mpCDataLoaderLink->mCGrammerBody.mDimension; i++)
	mVGrammerBody.push_back(mpCDataLoaderLink->mCGrammerBody.mpBody[i]);
    mHeaderByte = mpCDataLoaderLink->GetHeaderByte();
    mHeaderLine = mpCDataLoaderLink->GetHeaderLine();

    LOOP_XYZ mTranslate[xyz] = 
	mpCDataLoaderLink->mCAdjustTranslate.mpBody[xyz];
    mScaling = 
	mpCDataLoaderLink->mCAdjustScale.mBody;

    mbTranslate = TRUE;
    mbScaling   = TRUE;
    if (mTranslate[X] == 0 && mTranslate[Y] == 0 && mTranslate[Z] == 0)
	mbTranslate = FALSE;
    if (mScaling == 0)
	mbScaling = FALSE;

    int defaultFormat[6] = {0, 1, 5, 6, 7, 8};
    std::vector < int > VDefaultFormat;
    VDefaultFormat.push_back(0);
    VDefaultFormat.push_back(1);
    VDefaultFormat.push_back(5);
    VDefaultFormat.push_back(6);
    VDefaultFormat.push_back(7);
    VDefaultFormat.push_back(8);
    if (mVGrammerBody == VDefaultFormat)
	mbUseDefaultFormat = TRUE;
    else
	mbUseDefaultFormat = FALSE;

    if ( Create() != wxTHREAD_NO_ERROR) 
	MsgBoxExit(NULL, "Error!", "Can't create thread");
}

void ThreadFileReader_cls::
DoAdjusting(){
    if (mbTranslate == TRUE){
	float (*pX)[3] = mpX;
	float translateX = mTranslate[X];
	float translateY = mTranslate[Y];
	float translateZ = mTranslate[Z];
	for (int i = 0; i < mPNum; i++){	
	    pX[i][X] += translateX;
	    pX[i][Y] += translateY;
	    pX[i][Z] += translateZ;
	}
    }

    if (mbScaling == TRUE){
	float (*pX)[3] = mpX;
	float (*pV)[3] = mpV;
	float *pRadius = mpRadius;
	float scaling = mScaling;
	for (int i = 0; i < mPNum; i++){
	    pX[i][X] *= scaling;
	    pX[i][Y] *= scaling;
	    pX[i][Z] *= scaling;
	    pV[i][X] *= scaling;
	    pV[i][Y] *= scaling;
	    pV[i][Z] *= scaling;
	    pRadius[i] *= scaling;
	}
    }
}

void ThreadFileReader_cls::
ReadBinaryBodyDefaultFormat(std::fstream& ifstream, int progressInterval){
    
    if (mbUseID == TRUE)
	for (int i = 0; i < mPNum; i++){
	    if (i % progressInterval == 0)
		GThreadReaderPercentage = i * 100 / mPNum;
	    ifstream.read((char*)mpX[i], sizeof(float)*3);
	    ifstream.read((char*)mpV[i], sizeof(float)*3);
	    ifstream.read((char*)&mpRadius[i], sizeof(float));
	    ifstream.read((char*)&mpID[i], sizeof(int));
	    int type;
	    ifstream.read((char*)&type, sizeof(int));
	    mpType[i] = (unsigned char)type;
	    ifstream.read((char*)&mpValue1[i], sizeof(float));
	    
	    // AD_HOC!! FIX_ME!!
	    if (mpType[i] < 0) mpType[i] = 0;
	    if (mpType[i] > 3) mpType[i] = 3;
	}
    else
	for (int i = 0; i < mPNum; i++){	
	    if (i % progressInterval == 0)
		GThreadReaderPercentage = i * 100 / mPNum;
	    ifstream.read((char*)mpX[i], sizeof(float)*3);
	    ifstream.read((char*)mpV[i], sizeof(float)*3);
	    ifstream.read((char*)&mpRadius[i], sizeof(float));
	    int   ID;
	    ifstream.read((char*)&ID, sizeof(int));
	    int   type;
	    ifstream.read((char*)&type, sizeof(int));
	    mpType[i] = (unsigned char)type;
	    ifstream.read((char*)&mpValue1[i], sizeof(float));
	    
	    // AD_HOC!! FIX_ME!!
	    if (mpType[i] < 0) mpType[i] = 0;
	    if (mpType[i] > 3) mpType[i] = 3;
	}

    DoAdjusting();
}

void ThreadFileReader_cls::
ReadBinaryBody(){
    if (this == NULL) return; 
    std::fstream ifstream(wxFNCONV(mstrFilePath),
			  std::ios::in | std::ios::binary);

    if (ifstream.fail()){
	printf("Read Main Fails!!\n");
	exit(0);
    }

    int bufferSize = 5*1024*1024;
    char *pBuffer = new char [bufferSize];
    ifstream.rdbuf()->pubsetbuf(pBuffer, bufferSize);

    size_t fileSize = (size_t)ifstream.seekg(0, std::ios::end).tellg();
    ifstream.seekg(mHeaderByte, std::ios::beg);
    
    int progressInterval = 100000;
    if (mPNum > 2000000) progressInterval = 200000;

    if (mbUseDefaultFormat == TRUE){
	ReadBinaryBodyDefaultFormat(ifstream, progressInterval);
	delete [] pBuffer;
	return; 
    }

    int   dummyType = 0;
    float dummy[10];
    char *pAPointer[24];
    int   AItemSize[24];
    int   AItemOffset[24];
    int itemNumber = mVGrammerBody.size();
    
    // !! _FIX_ME !! {͕@`FbNĕKvȂɂ
    for (int i = 0; i < mPNum; i++){
	mpValue1[i] = 0;
	mpRadius[i] = 0;
    }

    for (int i = 0; i < itemNumber; i++){
	switch(mVGrammerBody[i]){
	case _GRAMMER_XYZ:
	    pAPointer[i] = (char*)mpX[0];
	    AItemSize[i] = 12;
	    AItemOffset[i] = 12;
	    break;
	case _GRAMMER_VXVYVZ:
	    pAPointer[i] = (char*)mpV[0];
	    AItemSize[i] = 12;
	    AItemOffset[i] = 12;
	    break;
	case _GRAMMER_RADIUS:
	    pAPointer[i] = (char*)mpRadius;
	    AItemSize[i] = 4;
	    AItemOffset[i] = 4;
	    break;
	case _GRAMMER_TYPE:
	    pAPointer[i] = (char*)&dummyType;
	    AItemSize[i] = 4;
	    AItemOffset[i] = 0;
	    break;
	case _GRAMMER_ID:
	    if (mbUseID == TRUE){
		pAPointer[i] = (char*)mpID;
		AItemSize[i] = 4;
		AItemOffset[i] = 4;
	    }
	    else{
		pAPointer[i] = (char*)dummy;
		AItemSize[i] = 4;
		AItemOffset[i] = 0;
	    }
	    break;
	case _GRAMMER_VALUE1:
	    pAPointer[i] = (char*)mpValue1;
	    AItemSize[i] = 4;
	    AItemOffset[i] = 4;
	    break;
	case _GRAMMER_DUMMY1:
	    pAPointer[i] = (char*)dummy;
	    AItemSize[i] = 4;
	    AItemOffset[i] = 0;
	    break;
	case _GRAMMER_DUMMY3:
	    pAPointer[i] = (char*)dummy;
	    AItemSize[i] = 12;
	    AItemOffset[i] = 0;
	    break;
	case _GRAMMER_DUMMY10:
	    pAPointer[i] = (char*)dummy;
	    AItemSize[i] = 40;
	    AItemOffset[i] = 0;
	    break;
	}
    }
	
    for (int i = 0; i < mPNum; i++){
	if (i % progressInterval == 0)
	    GThreadReaderPercentage = i * 100 / mPNum;
	for (int j = 0; j < itemNumber; j++)
	    ifstream.read(pAPointer[j] + AItemOffset[j]*i, AItemSize[j]);
	mpType[i] = dummyType;
    }

    delete [] pBuffer;

    DoAdjusting();
}


void ThreadFileReader_cls::
ReadAsciiBody(){
    std::fstream ifstream(wxFNCONV(mstrFilePath),
			  std::ios::in);
    
    // !! _FIX_ME !! {͕@`FbNĕKvȂɂ
    for (int i = 0; i < mPNum; i++){
	mpValue1[i] = 0;
	mpRadius[i] = 0;
    }

    // USɂ߂Ă݂B
    int len;
    ifstream.seekg (0, std::ios::end); 
    len = ifstream.tellg ();
    ifstream.seekg (0, std::ios::beg);
    std::string strBuf; strBuf.reserve(len); strBuf.resize(len);
    
    ifstream.read (&(strBuf[0]), len);

    int threadNumber = GThreadNumberReadAscii;
    if (threadNumber == 0) threadNumber = 1;
    bool *bAFinished = new bool [threadNumber];
    for (int core = 0; core < threadNumber; core++)
	bAFinished[core] = FALSE;

    ThreadAsciiReader_cls **ppACAsciiReader = 
	new ThreadAsciiReader_cls* [threadNumber];
    for (int core = 0; core < threadNumber; core++){
	ppACAsciiReader[core] = new ThreadAsciiReader_cls(wxTHREAD_DETACHED);
	ppACAsciiReader[core]->SetParameters
	    (&bAFinished[core], &strBuf, mPNum, mHeaderLine, 
	     threadNumber, core, mVGrammerBody, mbUseDefaultFormat,
	     mpX, mpV, mpA, mpJ, mpRadius, mpType, mpID, mpValue1);
	ppACAsciiReader[core]->Run();
    }
    
    bool bFinishedAll = FALSE;
    while (bFinishedAll == FALSE){
	wxMilliSleep(3);
	bFinishedAll = TRUE;
	for (int core = 0; core < threadNumber; core++)
	    bFinishedAll &= bAFinished[core];
    }

    delete [] bAFinished;
    delete [] ppACAsciiReader;

    DoAdjusting();
}

wxThread::ExitCode ThreadFileReader_cls::
Entry(){
    GThreadReaderPercentage = 0;

    switch(mDataType){
    case _DATA_SIMPLEBINARY:
	this->ReadBinaryBody();
	break;
    case _DATA_SIMPLEASCII:
	this->ReadAsciiBody();
	break;
    }

    *mpbFinishedFlagLink = TRUE;
    GbThreadReaderFinished = TRUE;

    GThreadReaderPercentage = 100;
    wxCommandEvent comA(wxEVT_COMMAND_MENU_SELECTED, menuEditRefreshSB);
    GpCMainFrameCLS->AddPendingEvent(comA);
    
    wxCommandEvent comB(wxEVT_COMMAND_MENU_SELECTED, menuEditRefreshTL);
    GpCMainFrameCLS->AddPendingEvent(comB);
    GpCThreadReader = NULL;
    //Delete();
    return 0;
}
