
#include "clsThreadInterpolate.h"
#include "../MYAPI/MsgBox.h"

#include "../../wxWidgetEnumMenu.h"

#include <omp.h>

ThreadInterpolator_cls *GpCThreadInterpolate = NULL;
bool                   GbThreadInterpolateFinished = FALSE;
extern wxFrame         *GpCMainFrameCLS;

#define X 0
#define Y 1
#define Z 2

ThreadInterpolator_cls::
ThreadInterpolator_cls(wxThreadKind kind) :
    wxThread(kind) {
}

void ThreadInterpolator_cls::
SetParameters(bool *pbFinishedFlag,
	      int PNum, int PNumNext, float dtime,
	      int interpolateType, int variablePNum,
	      int threadnum, int modnumber,
	      float (*pX)[3], float (*pV)[3],
	      float (*pA)[3], float (*pJ)[3], 
	      float *pRadius, float *pDRadius,
	      float *pValue1, float *pDValue1,
	      unsigned char  *pType, int *pID,
	      float (*pXNext)[3], float (*pVNext)[3],
	      float (*pANext)[3], float (*pJNext)[3], 
	      float *pRadiusNext, float *pValue1Next, 
	      unsigned char  *pTypeNext, int *pIDNext) {
    mpbFinishedFlagLink = pbFinishedFlag;
    mInterpolateType = interpolateType;
    mValiablePNum = variablePNum;

    mPNum = PNum;
    mpX = pX;
    mpV = pV;
    mpA = pA;
    mpJ = pJ;
    mpRadius = pRadius;
    mpDRadius = pDRadius;
    mpValue1 = pValue1;
    mpDValue1 = pDValue1;
    mpType = pType;
    mpID   = pID;

    mPNumNext = PNumNext;
    mpXNext = pXNext;
    mpVNext = pVNext;
    mpANext = pANext;
    mpJNext = pJNext;
    mpRadiusNext = pRadiusNext;
    mpValue1Next = pValue1Next;
    mpTypeNext = pTypeNext;
    mpIDNext = pIDNext;


    mDTime = dtime;
    mThreadNum = threadnum;
    mModNumber = modnumber;
    
    if ( Create() != wxTHREAD_NO_ERROR) 
	MsgBoxExit(NULL, "Error!", "Can't create thread");
}

void ThreadInterpolator_cls::
InterpolateDataLinear(){
    float dtime = mDTime;

#ifndef __WXMAC__
#ifdef _OPENMP
    // 1%xȂۂB
    omp_set_dynamic( 0 );
    omp_set_num_threads( 2 );
#pragma omp parallel for
#endif
#endif
    for (int i = 0; i < mPNum; i++){
	mpV[i][X] = (mpXNext[i][X] - mpX[i][X]) / dtime;
	mpV[i][Y] = (mpXNext[i][Y] - mpX[i][Y]) / dtime;
	mpV[i][Z] = (mpXNext[i][Z] - mpX[i][Z]) / dtime;
	mpDRadius[i] = (mpRadiusNext[i] - mpRadius[i]) / dtime;
	mpDValue1[i] = (mpValue1Next[i] - mpValue1[i]) / dtime;
    }
}

int ThreadInterpolator_cls::
SearchIDNear(int index, int ID, int range){
    int ini = index - range;
    if (ini > mPNumNext) return -1;
    if (ini < 0) ini = 0;
    int fin = index + range;
    if (fin > mPNumNext) fin = mPNumNext;
    for (int i = ini; i < fin; i++)
	if (mpIDNext[i] == ID) return i;
    return -1;
}

int ThreadInterpolator_cls::
SearchIDFull(int ID){
    for (int i = 0; i < mPNumNext/10; i++)
	if (mpIDNext[i] == ID) return i;
    return -1;
}

void ThreadInterpolator_cls::
InterpolateDataLinearSearchingID(){
    float dtime = mDTime;

    for (int i = 0; i < mPNum; i++){
	int ID = mpID[i];
	int iNext = -1;
	if (i < mPNumNext){
	    if (ID == mpIDNext[i]) {
		iNext = i;
		mpV[i][X] = (mpXNext[iNext][X] - mpX[i][X]) / dtime;
		mpV[i][Y] = (mpXNext[iNext][Y] - mpX[i][Y]) / dtime;
		mpV[i][Z] = (mpXNext[iNext][Z] - mpX[i][Z]) / dtime;
		mpDRadius[i] = (mpRadiusNext[i] - mpRadius[i]) / dtime;
		mpDValue1[i] = (mpValue1Next[i] - mpValue1[i]) / dtime;
		continue;
	    }
	}

	iNext = SearchIDNear(i, ID, 10);
	if (iNext == -1)
	    iNext = SearchIDNear(i, ID, 100);
	if (iNext == -1)
	    iNext = SearchIDNear(i, ID, 1000);
	if (iNext == -1)
	    iNext = SearchIDNear(i, ID, 10000);
	if (iNext == -1)
	    iNext = SearchIDFull(ID);
	if (iNext == -1){
	    mpV[i][X] = 0;
	    mpV[i][Y] = 0;
	    mpV[i][Z] = 0;
	    mpDRadius[i] = 0;
	    mpDValue1[i] = 0;
	    continue;
	}
	
	mpV[i][X] = (mpXNext[iNext][X] - mpX[i][X]) / dtime;
	mpV[i][Y] = (mpXNext[iNext][Y] - mpX[i][Y]) / dtime;
	mpV[i][Z] = (mpXNext[iNext][Z] - mpX[i][Z]) / dtime;
	mpDRadius[i] = (mpRadiusNext[iNext] - mpRadius[i]) / dtime;
	mpDValue1[i] = (mpValue1Next[iNext] - mpValue1[i]) / dtime;
    }
}


wxThread::ExitCode ThreadInterpolator_cls::
Entry(){
    
    if (mInterpolateType == _INTERPOLATE_LINEAR && 
	mValiablePNum == _INTERPOLATE_PNUM_CONSTANT)
	InterpolateDataLinear();
    if (mInterpolateType == _INTERPOLATE_LINEAR &&
	mValiablePNum == _INTERPOLATE_PNUM_VARIABLE)
	InterpolateDataLinearSearchingID();
    
    *mpbFinishedFlagLink = TRUE;
    GbThreadInterpolateFinished = TRUE;

    wxCommandEvent com(wxEVT_COMMAND_MENU_SELECTED, menuEditRefreshTL);
    GpCMainFrameCLS->AddPendingEvent(com);
    GpCThreadInterpolate = NULL;
    
    return 0;
}
