// "clsCameraPathGUI.h"

#include "clsCameraPathGUI.h"
#include "clsCameraPath.h"

#include "../GLATTRIB/clsColorCurve.h"
#include "../GLCANVAS/clsMainGLCanvas.h"
extern MainGLCanvas_cls* GpCurrentGLPanel;
#include "../UNDO/clsUndoManager.h"

#include "../../clsFrameMain.h"
#include "../../wxWidgetEnum.h"
#include "../../wxWidgetEnumMenu.h"

#include "../WIDGETCTRL/clsMultiInputPanel.h"
#include "../WIDGETCTRL/clsHookCtrl.h"

#define X 0
#define Y 1
#define Z 2

BEGIN_EVENT_TABLE(CameraPathGUI_cls, wxDialog)
EVT_NOTEBOOK_PAGE_CHANGED(ID_BOOK_VIEW, CameraPathGUI_cls::OnNotebookChanged)
EVT_BUTTON    (btnPathAdv ,    CameraPathGUI_cls::OnButtonShowItemAdv)
EVT_BUTTON    (btnPathPath,    CameraPathGUI_cls::OnButtonShowItemPath)
EVT_BUTTON    (btnViewDefault, CameraPathGUI_cls::OnButtonViewDefault)
END_EVENT_TABLE()

CameraPathGUI_cls::
CameraPathGUI_cls(CameraPathCtrl_cls* pCCtrl,
		  wxWindow *pC, wxPoint pos, wxSize size) :
wxDialogEXKey(pC, wxID_ANY, _("View and Camera Config"), 
	      pos, size, pCCtrl->mpBitmapLink) {
    mspCThis = this;
    mpCCtrl = pCCtrl;
    
    wxBoxSizer *pBoxSizerSrc     = new wxBoxSizer(wxVERTICAL);
    //wxBoxSizer *pBoxSizerView    = new wxBoxSizer(wxVERTICAL);
    wxBoxSizer *pBoxSizerAdv     = new wxBoxSizer(wxVERTICAL);
    wxBoxSizer *pBoxSizerPath    = new wxBoxSizer(wxVERTICAL);

    mpCBook1 = new wxNotebook (this, ID_BOOK_VIEW);

    //SetMNumView(pBoxSizerView);
    for (int i = 0; i < 4; i++)
	SetMNumView(i, NULL, mpCBook1);
    SetMNumAdv (pBoxSizerAdv);
    SetMNumPath(pBoxSizerPath);

    mpCBook1->AddPage(mpCMNumView[0], wxT("View0"));
    mpCBook1->AddPage(mpCMNumView[1], wxT("View1"));
    mpCBook1->AddPage(mpCMNumView[2], wxT("View2"));
    mpCBook1->AddPage(mpCMNumView[3], wxT("View3"));

    //pBoxSizerSrc->Add(pBoxSizerView, 0, wxEXPAND);
    pBoxSizerSrc->Add(mpCBook1, 0, wxEXPAND);
    pBoxSizerSrc->Add(pBoxSizerAdv, 0, wxEXPAND);
    pBoxSizerSrc->Add(pBoxSizerPath, 0, wxEXPAND);
    
    SetSizerAndFit(pBoxSizerSrc);
    
    int minimumWidth = GetClientSize().GetWidth() - 8;
    //minimumWidth = mpCMNumView->GetPaddingWidth(minimumWidth);
    minimumWidth = mpCMNumAdv->GetPaddingWidth(minimumWidth);
    minimumWidth = mpCMNumPath->GetPaddingWidth(minimumWidth);
    //mpCMNumView->SetPaddingWidth(minimumWidth);
    mpCMNumAdv->SetPaddingWidth(minimumWidth);
    mpCMNumPath->SetPaddingWidth(minimumWidth);

    mpCMNumAdv->ShowPanel(FALSE);
    mpCMNumPath->ShowPanel(FALSE);

    ResizeDialog();
}

void CameraPathGUI_cls::
SetMNumView(int i, wxBoxSizer *pBoxSizer, wxWindow *pCWindow){
    MultiInputPanel_cls* pCurr = 
	InitializeMNumPanel(&mpCMNumView[i], pCWindow);

    MainGLCanvas_cls *pCGLCanvasLink = 
	mpCCtrl->mpCParentFrame->GetPCGLCanvas(i);

    pCurr->mUpdateBit = 
	_COMMAND_UPDATE_DISPLAY_CAMCONF | _COMMAND_UPDATE_DISPLAY_GL |
	_COMMAND_UPDATE_DISPLAY_IMGSEQ;
    
    int boxWidth = 2;
#ifdef WIN32
    pCurr->mTextWidth = 90;
#else
    pCurr->mTextWidth = 100;
#endif
    pCurr->mItemWidth = 80;
    pCurr->mComboWidth = -1;
    
    pCurr->CombinePointer
	(_ITEM_BOXSIZERHORIZONTAL, &boxWidth, _("Geometry"));

    /*
    wxString strListView[4] = {
	wxT("View0"),wxT("View1"),wxT("View2"), wxT("View3")};
    pCurr->CombinePointerList(_ITEM_UINT_LIST, &mpCCtrl->mActiveView,
			      4, strListView);
    */

    wxString strGeometry[4] = {
	_("Perspective"), wxT("Orthographic"), _("FishEye"), _("Panoramic")};
    int typeSelect = 4;
    pCurr->CombinePointerList
	(_ITEM_UINT_LIST, &pCGLCanvasLink->mCGLGeometry.mBody,
	 typeSelect, strGeometry);
    
    pCurr->CombinePointerButton(btnViewDefault, wxT("Default"));
    pCurr->GetLastItem().mAlign = _GP_ALIGN_RIGHT;
	

    boxWidth = 3;
    int indexTranslate = pCurr->CombinePointer
	(_ITEM_BOXSIZERHORIZONTAL, &boxWidth, wxT("Translate"));
    pCurr->CombinePointer(_ITEM_DOUBLE, &pCGLCanvasLink->mCCameraPos.mPos[X]);
    pCurr->CombinePointer(_ITEM_DOUBLE, &pCGLCanvasLink->mCCameraPos.mPos[Y]);
    pCurr->CombinePointer(_ITEM_DOUBLE, &pCGLCanvasLink->mCCameraPos.mPos[Z]);

    int indexRotation = pCurr->CombinePointer
	(_ITEM_BOXSIZERHORIZONTAL, &boxWidth, wxT("Rotate"));
    pCurr->CombinePointer(_ITEM_DOUBLE, &pCGLCanvasLink->mCCameraPos.mYPR[X]);
    pCurr->CombinePointer(_ITEM_DOUBLE, &pCGLCanvasLink->mCCameraPos.mYPR[Y]);
    pCurr->CombinePointer(_ITEM_DOUBLE, &pCGLCanvasLink->mCCameraPos.mYPR[Z]);

    int indexAnchor = pCurr->CombinePointer
	(_ITEM_BOXSIZERHORIZONTAL, &boxWidth, wxT("Pivot"));
    pCurr->CombinePointer(_ITEM_DOUBLE, &MainGLCanvas_cls::msCAnchorPos.mPos[X]);
    pCurr->CombinePointer(_ITEM_DOUBLE, &MainGLCanvas_cls::msCAnchorPos.mPos[Y]);
    pCurr->CombinePointer(_ITEM_DOUBLE, &MainGLCanvas_cls::msCAnchorPos.mPos[Z]);
    
    /*
    boxWidth = 2;
    mIndexPitchAngle = pCurr->CombinePointer(_GP_BOXSIZERHORIZONTAL, &boxWidth, 
					     _("Pitch Angle"));
    pCurr->CombinePointer
	(_GP_DOUBLE_SLIDER, &mspCParent->GetActivePath()->mpCStereoGramLinked->mCPitchAngle.mBody);
    pCurr->SetRange(-180, 180, _MN_LASTITEM, 1);
    pCurr->CombinePointer(_GP_DOUBLE, &mspCParent->GetActivePath()->mpCStereoGramLinked->mCPitchAngle.mBody);
    */
    
    pCurr->mItemWidth = -1;
    boxWidth = 2;
    pCurr->CombinePointer(_ITEM_BOXSIZERHORIZONTAL, &boxWidth, 
			  _("View Angle"));
    pCurr->CombinePointer
	(_ITEM_FLOAT_SLIDER, &pCGLCanvasLink->mCGLPersAngle.mBody);
    pCurr->SetRange(_ITEM_LASTITEM, 1, 180);
    pCurr->CombinePointer(_ITEM_FLOAT, &pCGLCanvasLink->mCGLPersAngle.mBody);
    
    pCurr->CombinePointer(_ITEM_BOXSIZERHORIZONTAL, &boxWidth, 
			  _("DepthMIN/MAX"));
    pCurr->CombinePointer
	(_ITEM_DOUBLE, &pCGLCanvasLink->mCGLPerspective.mDepthMIN);
    pCurr->CombinePointer
	(_ITEM_DOUBLE, &pCGLCanvasLink->mCGLPerspective.mDepthMAX);

    pCurr->FinishSetting(pBoxSizer, TRUE);

    pCurr->SetColorForXYZ(indexTranslate+1);
    pCurr->SetColorForXYZ(indexRotation+1);
    pCurr->SetColorForXYZ(indexAnchor+1);

    pCurr->mpCHookCtrl->SetFuncHook(FuncHookShow, _HOOK_SHOW);
    pCurr->mpCHookCtrl->SetFuncHook(FuncHookAssign, _HOOK_ASSIGN);
    //pCurr->SetFuncHook(&FuncHookRefresh, _MN_HOOK_CHECKBOX);
    //pCurr->SetFuncHook(&FuncHookRefresh, _MN_HOOK_COMBOBOX);
    //pCurr->SetColorForXYZ(mIndexCamPos+1);
    //pCurr->SetColorForXYZ(mIndexAncPos+1);
    
    HideAndShowView(i);
}

void CameraPathGUI_cls::
SetMNumAdv(wxBoxSizer *pBoxSizer){
    MultiInputPanel_cls* pCurr =
	InitializeMNumPanel(&mpCMNumAdv, this);
    mpCBTNAdv = new wxButtonEX(this, btnPathAdv, _("[+]View Advanced Setting"), 
				wxDefaultPosition, wxSize(-1,-1));
    pCurr->AssignMinimizeButtonBoth(mpCBTNAdv);
    
#ifdef WIN32
    pCurr->mTextWidth = 90;
#else
    pCurr->mTextWidth = 100;
#endif
    pCurr->mItemWidth = -1;
    pCurr->mComboWidth = 120;
    
    int boxWidth = 2;
    pCurr->CombinePointer(_ITEM_BOXSIZERHORIZONTAL, &boxWidth, _("BG Color"));
    ColorCurveBtn_cls *pCBtnBG = new ColorCurveBtn_cls
	(pCurr, MainGLCanvas_cls::mspCClrCurveBG, TRUE, FALSE);
    ColorCurveBtn_cls *pCBtnOut = new ColorCurveBtn_cls
	(pCurr, MainGLCanvas_cls::mspCClrCurveOut, TRUE, FALSE);
    
    pCurr->CombinePointer(_ITEM_WXPANEL, pCBtnBG,  _("BG Color"));
    pCurr->CombinePointer(_ITEM_WXPANEL, pCBtnOut, _("Outside Color"));
    
    /*
    mIndexBG0 = pCurr->CombinePointerButton(btnGLPanelBG0, _("BG Color 0"));
    pCurr->mVCItemList[mIndexBG0].mpColorFloat
	= GpCurrentGLPanel->mCColorBG0.mpBody;
    mIndexBG1 = pCurr->CombinePointerButton(btnGLPanelBG1, _("Outside Color"));
    pCurr->mVCItemList[mIndexBG1].mpColorFloat
	= GpCurrentGLPanel->mCColorBG1.mpBody;

    //pCurr->mButtonWidth = -1;
    pCurr->CombinePointerButton(btnGLPanelAll, _("Apply to All"));
    */

    pCurr->mItemWidth = 80;
    pCurr->mComboWidth = 80;
    boxWidth = 3;
    pCurr->CombinePointer(_ITEM_BOXSIZERHORIZONTAL, &boxWidth, _("Grid and Width"));
    wxString strListGrid[3] = {
	_("Grid"), _("Grid+Circle"), _("Polar")};
    mIndexGrid = pCurr->CombinePointerList
	(_ITEM_UINT_LIST, &GpCurrentGLPanel->mCGridType.mBody, 
	 3, strListGrid);
    
    pCurr->mItemWidth = 80;
    pCurr->CombinePointer(_ITEM_FLOAT, &MainGLCanvas_cls::msLineWidth);
    pCurr->CombinePointer(_ITEM_FLOAT_SLIDER, &MainGLCanvas_cls::msLineWidth);
    pCurr->SetRange(_ITEM_LASTITEM, 1, 10);
    
    pCurr->mItemWidth = -1;
    wxString listFBOSize[5] = {wxT("256x256"), wxT("512x512"), wxT("1024x1024"), wxT("2048x2048"), wxT("4096x4096")};
    mIndexFBOSize = pCurr->CombinePointerList
	(_ITEM_UINT_LIST, 
	 //&GpCurrentGLPanel->mCFBOSizeIndex.mBody,
	 &MainGLCanvas_cls::msCFBOSizeIndex.mBody,
	 5, listFBOSize, _("Buffer Size"));

    /*
    boxWidth = 2;
    pCurr->CombinePointer(_GP_BOXSIZERHORIZONTAL, &boxWidth);
    mIndexAntiAlias = pCurr->CombinePointer
	(_GP_BOOL, &GpCurrentGLPanel->mCbGridAnti.mbBody, _("AntiAlias"));
    pCurr->CombinePointer(_GP_BOOL, &GpCurrentGLPanel->mCbShowGridTag.mbBody, _("Show Coordinate"));

    pCurr->CombinePointer(_GP_BOXSIZERHORIZONTAL, &boxWidth);
    mIndexShowOtherCam = pCurr->CombinePointer
	(_GP_BOOL, &GpCurrentGLPanel->mCbShowOtherCam.mbBody, _("view point"));
    mIndexXOR = 
	pCurr->CombinePointer(_GP_BOOL, &GpCurrentGLPanel->mCbXOR.mbBody, _("Draw item in XOR"));
    */
#ifdef WIN32
    pCurr->mMinimumWidth = 280;
#else
    pCurr->mMinimumWidth = 320;
#endif
    pCurr->FinishSetting(pBoxSizer);

    pCurr->mpCHookCtrl->SetFuncHook(FuncHookShow, _HOOK_SHOW);
    pCurr->mpCHookCtrl->SetFuncHook(FuncHookAssign, _HOOK_ASSIGN);
}

void CameraPathGUI_cls::
SetMNumPath(wxBoxSizer *pBoxSizer){
    MultiInputPanel_cls* pCurr =
	InitializeMNumPanel(&mpCMNumPath, this);
    mpCBTNPath = new wxButtonEX(this, btnPathPath, _("[+]Camera Setting"), 
				wxDefaultPosition, wxSize(-1,-1));
    pCurr->AssignMinimizeButtonBoth(mpCBTNPath);
    pCurr->mUpdateBit = 
	_COMMAND_UPDATE_DISPLAY_CAMCONF | _COMMAND_UPDATE_DISPLAY_GL |
	_COMMAND_UPDATE_DISPLAY_IMGSEQ;
    
    int boxWidth = 3;
    //pCurr->mDefaultButton = _MN_NO_BUTTONS;
#ifdef WIN32
    pCurr->mTextWidth = 90;
#else
    pCurr->mTextWidth = 100;
    //pCurr->mComboWidth = 108;
#endif
    pCurr->mItemWidth = 80;
    //pCurr->mItemWidth = -1;
    pCurr->mComboWidth = 100;

    /*
    pCurr->CombinePointer(_GP_BOXSIZERHORIZONTAL, &boxWidth, _("Active Cam"));
    wxString strListView[1] = {wxT("Cam0")};
    mIndexActivePath = pCurr->CombinePointer
	(_GP_UNSIGNED_INT_LIST, &mspCParent->mActivePath,
	 _("Active Path"), TRUE, 1, strListView, 0);
    pCurr->CombinePointerButton(buttonCtrlAddPath, wxT("Add"));
    pCurr->CombinePointerButton(buttonCtrlDeletePath, wxT("Delete"));
    */

    pCurr->CombinePointer(_ITEM_BOXSIZERHORIZONTAL, &boxWidth, 
			  _("Camera Geom."));
    
    wxString strGeometry[4] = {
	_("Perspective"), wxT("Orthographic"), _("FishEye"), _("Panoramic")};
    int typeSelect = 4/* - (FBO_cls::msbSupported == FALSE) * 2*/;
    pCurr->CombinePointerList
	(_ITEM_UINT_LIST, &mpCCtrl->mpCCameraPathLinked->mCCamGeometry.mBody,
	 typeSelect, strGeometry);
    
    pCurr->mItemWidth = 100;
    pCurr->CombinePointer(_ITEM_BOOL,
			  &mpCCtrl->mpCCameraPathLinked->mCbLookAnchor.mbBody, wxT("Look Anchor"));
    
    pCurr->mItemWidth = 80;
    pCurr->CombinePointer(_ITEM_BOXSIZERHORIZONTAL, &boxWidth, _("View Angle"));
    mIndexAnglePath = pCurr->CombinePointer
	(_ITEM_DOUBLE_SLIDER, &mpCCtrl->mpCCameraPathLinked->mCVAngle.mBody);
    pCurr->SetRange(_ITEM_LASTITEM, 1, 180);
    pCurr->CombinePointer(_ITEM_DOUBLE, &mpCCtrl->mpCCameraPathLinked->mCVAngle.mBody);
    
    int indexTranslate = pCurr->CombinePointer(_ITEM_BOXSIZERHORIZONTAL, &boxWidth, wxT("Translae"));
    pCurr->CombinePointer(_ITEM_DOUBLE, &mpCCtrl->mpCCameraPathLinked->mpCTranslate->mVCParam[X].mInterpolatedValue);
    pCurr->CombinePointer(_ITEM_DOUBLE, &mpCCtrl->mpCCameraPathLinked->mpCTranslate->mVCParam[Y].mInterpolatedValue);
    pCurr->CombinePointer(_ITEM_DOUBLE, &mpCCtrl->mpCCameraPathLinked->mpCTranslate->mVCParam[Z].mInterpolatedValue);

    int indexRotation = pCurr->CombinePointer(_ITEM_BOXSIZERHORIZONTAL, &boxWidth, wxT("Rotation"));
    pCurr->CombinePointer(_ITEM_DOUBLE, &mpCCtrl->mpCCameraPathLinked->mpCRotation->mVCParam[X].mInterpolatedValue);
    pCurr->CombinePointer(_ITEM_DOUBLE, &mpCCtrl->mpCCameraPathLinked->mpCRotation->mVCParam[Y].mInterpolatedValue);
    pCurr->CombinePointer(_ITEM_DOUBLE, &mpCCtrl->mpCCameraPathLinked->mpCRotation->mVCParam[Z].mInterpolatedValue);

    int indexAnchor = pCurr->CombinePointer(_ITEM_BOXSIZERHORIZONTAL, &boxWidth, wxT("Anchor"));
    pCurr->CombinePointer(_ITEM_DOUBLE, &mpCCtrl->mpCAnchorPathLinked->mpCTranslate->mVCParam[X].mInterpolatedValue);
    pCurr->CombinePointer(_ITEM_DOUBLE, &mpCCtrl->mpCAnchorPathLinked->mpCTranslate->mVCParam[Y].mInterpolatedValue);
    pCurr->CombinePointer(_ITEM_DOUBLE, &mpCCtrl->mpCAnchorPathLinked->mpCTranslate->mVCParam[Z].mInterpolatedValue);
    
    /*
    boxWidth = 1;
    pCurr->CombinePointer(_GP_BOXSIZERHORIZONTAL, &boxWidth, wxT("Point Sky"));
    mIndexPointSky = pCurr->CombinePointer
	(_GP_BOOL, &mspCParent->GetActivePath()->mCbSky.mbBody);
    //pCurr->CombinePointer(_GP_STATIC_TEXT, NULL);
    //pCurr->CombinePointer(_GP_STATIC_TEXT, NULL);
    
    boxWidth = 3;
    pCurr->CombinePointer(_GP_BOXSIZERHORIZONTAL, &boxWidth, wxT("Sky Vector"));
    mIndexSky = pCurr->CombinePointer
	(_GP_DOUBLE, &mspCParent->GetActivePath()->mCSky.mpInterpolatedValue[0]);
    pCurr->CombinePointer(_GP_DOUBLE, &mspCParent->GetActivePath()->mCSky.mpInterpolatedValue[1]);
    pCurr->CombinePointer(_GP_DOUBLE, &mspCParent->GetActivePath()->mCSky.mpInterpolatedValue[2]);
    */
    
#ifdef WIN32
    pCurr->mMinimumWidth = 280;
#else
    pCurr->mMinimumWidth = 320;
#endif
    pCurr->FinishSetting(pBoxSizer);

    pCurr->SetColorForXYZ(indexTranslate+1);
    pCurr->SetColorForXYZ(indexRotation+1);
    pCurr->SetColorForXYZ(indexAnchor+1);

    pCurr->mpCHookCtrl->SetFuncHook(FuncHookShow, _HOOK_SHOW);
    pCurr->mpCHookCtrl->SetFuncHook(FuncHookAssign, _HOOK_ASSIGN);
    /*
    pCurr->SetFuncHook(FrameMain_cls::FuncHookParamChanged, _MN_HOOK_ASSIGN);
    pCurr->SetFuncHook(&FuncHookAssign, _MN_HOOK_ASSIGN);
    pCurr->SetFuncHook(&FuncHookRefresh, _MN_HOOK_COMBOBOX);
    pCurr->SetFuncHook(&FuncHookRefresh, _MN_HOOK_CHECKBOX);
    pCurr->SetFuncHook(&FuncHookButton, _MN_HOOK_BUTTON, _MN_HOOK_ID_ANY);
    */
}

void CameraPathGUI_cls::
ReconnectView(){
    
    mpCBook1->SetSelection(GpCurrentGLPanel->mSelfIndex);

    mpCCtrl->mActiveView = GpCurrentGLPanel->mSelfIndex; 
    
    // Adṽ̕N͕Kv FIX_ME !!

    /*
    pCurr->UpdateValues();

    HideAndShowAll();
    */
}

void CameraPathGUI_cls::
ReconnectViewAdv(){
    MultiInputPanel_cls*  pCurr = mpCMNumAdv;
    MainGLCanvas_cls *pCGLCanvasLink = GetPCActiveGLCanvas();

    pCurr->mVCItemList[mIndexGrid].
	CombinePointer(_ITEM_UINT_LIST, &pCGLCanvasLink->mCGridType.mBody);

    pCurr->UpdateValues();
}

MainGLCanvas_cls* CameraPathGUI_cls::
GetPCActiveGLCanvas(){
    return mpCCtrl->mpCParentFrame->
	GetPCGLCanvas(mpCBook1->GetSelection());
}

void CameraPathGUI_cls::
HideAndShowView(int i){
    
}

void CameraPathGUI_cls::
HideAndShowPath(){
}

void CameraPathGUI_cls::
HideAndShowAll(){
}

void CameraPathGUI_cls::
OnNotebookChanged(wxNotebookEvent& event){
    //mspCParent->mActiveView = event.GetSelection();
    ReconnectViewAdv();
}

void CameraPathGUI_cls::
OnButtonShowItemAdv(wxCommandEvent &event){
    mpCMNumAdv->ShowPanel(TRUE);
    ResizeDialog();
}

void CameraPathGUI_cls::
OnButtonShowItemPath(wxCommandEvent &event){
    mpCMNumPath->ShowPanel(TRUE);
    ResizeDialog();
}

void CameraPathGUI_cls::
OnButtonViewDefault(wxCommandEvent& event){
    int i = mpCCtrl->mActiveView;
    MainGLCanvas_cls *pCGLCanvasLink = 
	mpCCtrl->mpCParentFrame->GetPCGLCanvas(i);
    int type = pCGLCanvasLink->mCDisplayMode.mBody;
    switch(type){
    case 0:{
	pCGLCanvasLink->SetCameraToDefaultPos();
	pCGLCanvasLink->mCCameraPos.UpdateYPR();
	break; }
    }
    UpdateValuesAll();
    wxCommandEvent com(wxEVT_COMMAND_MENU_SELECTED, menuEditRefreshGL);
    wxGetTopLevelParent(mspCThis->mpCMNumView[0])->
	GetParent()->AddPendingEvent(com);
}

void CameraPathGUI_cls::
FuncHookShow(void *pPanel, void *pArg){
    mspCThis->ResizeDialog();
}

void CameraPathGUI_cls::
FuncHookAssign(void *pPanel, void *pArg){
    mspCThis->mpCCtrl->mpCCameraPathLinked->mpCTranslate->
	SetKeyFrameIfChanged();
    mspCThis->mpCCtrl->mpCCameraPathLinked->mpCRotation->
	SetKeyFrameIfChanged();
    mspCThis->mpCCtrl->mpCAnchorPathLinked->mpCTranslate->
	SetKeyFrameIfChanged();

    wxCommandEvent com(wxEVT_COMMAND_MENU_SELECTED, menuEditRefreshGL);
    wxGetTopLevelParent(mspCThis->mpCMNumView[0])->
	GetParent()->AddPendingEvent(com);
}

void CameraPathGUI_cls::
UpdateValuesAll(){
    for (int i = 0; i < 4; i++)
	mpCMNumView[i]->UpdateValues();
    mpCMNumAdv->UpdateValues();
    mpCMNumPath->UpdateValues();
}

// --------

CameraPathCtrl_cls::
CameraPathCtrl_cls(FrameMain_cls* pCParentFrame, 
		   StereoGram_cls* pCStereoGram, 
		   CameraPath_cls* pCCameraPath,
		   AnchorPath_cls* pCAnchorPath){
    mpCGUI = NULL;
    mpCParentFrame = pCParentFrame;
    mpCCameraPathLinked = pCCameraPath;
    mpCAnchorPathLinked = pCAnchorPath;
    mpBitmapLink = mpCParentFrame->mpBitmapCamera;
    mActiveView = 0;

    InitGUI(pCParentFrame);
}

void CameraPathCtrl_cls::
InitGUI(wxWindow *pC){
    mpCGUI = new CameraPathGUI_cls(this, pC,
				   /*mCScreenRect.GetPosition(),*/
				   /*mCScreenRect.GetSize()*/
				   wxDefaultPosition, wxDefaultSize);
    SetPDialog(mpCGUI);
}

void CameraPathCtrl_cls::
UpdateItems(bool bForce){
    if (mpCGUI == NULL) return;

    /*
    double doubleIndex = GpCurrentGLPanel->mpCParentFrameLink->
	GetPCTimeCtrl()->GetDoubleIndex();
    GetActivePath()->mCPosition.GetPInterpolatedValue(doubleIndex);
    mpCGUI->mpCMNumPath->UpdateValues();
    mpCGUI->mpCMNumView->UpdateValues();
    mpCGUI->mpCMNumAdv->UpdateValues();
    */
}

CameraPathGUI_cls* CameraPathGUI_cls::mspCThis = 0;
