// -*- c -*-
// "clsLightSource.h"

#include "clsLightSource.h"

#include "../UNDO/clsUndoManager.h"
extern UndoManager_cls GCUndoList;
#include "../UNDO/clsUndoItemConstParam.h"

#ifdef WIN32
#define ENDLWX wxT("\r\n")
#else
#define ENDLWX wxT("\n")
#endif

#define LOOP_RGBA for (int rgba = 0; rgba < 4; rgba++)
#define X 0
#define Y 1
#define Z 2

LightSRC_cls::
LightSRC_cls(int selfIndex, wxString strTag) : SceneObj_cls(strTag) {
    
    mSelfIndex = selfIndex;
    mObjType   = _SCENEOBJ_LIGHT;
    mObjIndex  = _SCENEOBJ_IDBASE_LIGHT + selfIndex;
    mUpdateBit = _COMMAND_UPDATE_DISPLAY_TL | _COMMAND_UPDATE_DISPLAY_GL | _COMMAND_UPDATE_DISPLAY_LIGHT;
    
    if (mSelfIndex == 1){
	SetInitValueTrans(-1.0, -1.0, -1.41421356);
	SetInitValueRot  (0, -45, 45);
    }
    else{
	SetInitValueTrans(1.0, 1.0, 1.41421356);
	SetInitValueRot  (0, 45, 225);
    }

    mCConfList.Clear();
    mCConfList.Add(&mCLightType, strTag + wxT("Type="), wxT("3"));
    mCConfList.Add(&mCSpotAngle, strTag + wxT("SpotAngle="), wxT("35"));
    mCConfList.Add(&mCFadeType, strTag  + wxT("Fade="), wxT("0"));
    mCConfList.Add(&mCbCastShadow, strTag + wxT("CastShadow="), wxT("1"));
    mCConfList.Add(&mCTarget, strTag + wxT("Target="), wxT("0"));
    mCConfList.Add(&mCbVisible, strTag + wxT("Visible="), wxT("1"));
    
    wxString listTag[1] = {wxT("Strength")};
    int updateBit = _COMMAND_UPDATE_DISPLAY_GL | _COMMAND_UPDATE_DISPLAY_TL |
	_COMMAND_UPDATE_DISPLAY_LIGHT ;
    mpCLightStrength = new ParamGroup_cls(strTag + wxT(" Strength") , 1, listTag, updateBit, TRUE);
    
    if (mSelfIndex == 1){
	double pLightInit[1] = {0.1};
	mpCLightStrength->SetKeyFrame(pLightInit, TRUE);
    }
    else{
	double pLightInit[1] = {1};
	mpCLightStrength->SetKeyFrame(pLightInit, TRUE);
    }
};

LightSRC_cls::
~LightSRC_cls(){
    delete mpCLightStrength;
}

void LightSRC_cls::
CopyFromP(LightSRC_cls *pCFrom){
    CopyFromP_SceneObj(pCFrom);

    int newValue = pCFrom->mCLightType.mBody;
    int oldValue = mCLightType.mBody;
    GCUndoList.SetNewCommand
	(new CommandSetValue_cls(&mCLightType.mBody, newValue),
	 new CommandSetValue_cls(&mCLightType.mBody, oldValue));
    GCUndoList.ExecCurrent();

    newValue = pCFrom->mCFadeType.mBody;
    oldValue = mCFadeType.mBody;
    GCUndoList.SetNewCommand
	(new CommandSetValue_cls(&mCFadeType.mBody, newValue),
	 new CommandSetValue_cls(&mCFadeType.mBody, oldValue));
    GCUndoList.ExecCurrent();

    newValue = pCFrom->mCSpotAngle.mBody;
    oldValue = mCSpotAngle.mBody;
    GCUndoList.SetNewCommand
	(new CommandSetValue_cls(&mCSpotAngle.mBody, newValue, mUpdateBit),
	 new CommandSetValue_cls(&mCSpotAngle.mBody, oldValue, mUpdateBit));
    GCUndoList.ExecCurrent();
    
    mpCLightStrength->CopyFromP(pCFrom->mpCLightStrength);
}

void LightSRC_cls::
GetLightInfo(double doubleIndex, float pLightPos[4], float pLightPhiTheta[2],
	     bool bSV){
    std::vector < double > VPos;
    std::vector < double > VRot = 
	mpCRotation->GetInterpolatedValue(doubleIndex, 0);
    
    if (bSV == TRUE){
	VPos.push_back(cos(3.14159*VRot[1]/180) * cos(3.14159*(VRot[2]/180+1)));
	VPos.push_back(cos(3.14159*VRot[1]/180) * sin(3.14159*(VRot[2]/180+1)));
	VPos.push_back(sin(3.14159*VRot[1]/180));
	VPos.push_back(0);
    }
    else{
	VPos = mpCTranslate->GetInterpolatedValue(doubleIndex, 0);
	VPos.push_back(1);
    }

    for (int i = 0; i < 4; i++)	pLightPos[i] = VPos[i];

    pLightPhiTheta[0] = VRot[2];
    pLightPhiTheta[1] = VRot[1];

}

void LightSRC_cls::
GetLightColor(double doubleIndex, float  pLightColor[4]){
    std::vector < double > VStr = 
	mpCLightStrength->GetInterpolatedValue(doubleIndex, 0);
    for (int rgb = 0; rgb < 3; rgb++) pLightColor[rgb] = VStr[0];
    pLightColor[3] = 0;
}

void LightSRC_cls::
GLSetLightSourceComp(int GLLightIndex, 
		     double doubleIndex, bool bAmbient, bool bDiffuse){
    GLuint GLLightType;
    switch(/*mSelfIndex*/GLLightIndex){
    case 0: GLLightType = GL_LIGHT0; break;
    case 1: GLLightType = GL_LIGHT1; break;
    case 2: GLLightType = GL_LIGHT2; break;
    case 3: GLLightType = GL_LIGHT3; break;
    case 4: GLLightType = GL_LIGHT4; break;
    case 5: GLLightType = GL_LIGHT5; break;
    case 6: GLLightType = GL_LIGHT6; break;
    }

    glEnable(GLLightType);
    float pos[4], phiTheta[2], clr[4] = {0, 0, 0, 0};
    GetLightInfo(doubleIndex, pos, phiTheta, /*clr,*/ (mCLightType.mBody == 3));
    GetLightColor(doubleIndex, clr);
    glLightfv(GLLightType, GL_POSITION, pos);

    GLfloat pZero[4] = {0, 0, 0, 0};
    GLfloat pOne[4] = {1, 1, 1, 0};
    if (bAmbient == TRUE)
	glLightfv(GLLightType, GL_AMBIENT , pOne);
    else
	glLightfv(GLLightType, GL_AMBIENT , pZero);

    if (bDiffuse == TRUE){
	glLightfv(GLLightType, GL_DIFFUSE , clr);
	glLightfv(GLLightType, GL_SPECULAR, clr);
    }
    else{
	glLightfv(GLLightType, GL_DIFFUSE , pZero);
	glLightfv(GLLightType, GL_SPECULAR, pZero);
    }
}

void LightSRC_cls::
GLSetLightSource(double doubleIndex, bool bEnable, int shadowMode){

    GLuint GLLightType;
    switch(mSelfIndex){
    case 0: GLLightType = GL_LIGHT0; break;
    case 1: GLLightType = GL_LIGHT1; break;
    case 2: GLLightType = GL_LIGHT2; break;
    case 3: GLLightType = GL_LIGHT3; break;
    case 4: GLLightType = GL_LIGHT4; break;
    case 5: GLLightType = GL_LIGHT5; break;
    case 6: GLLightType = GL_LIGHT6; break;
    }

    if (bEnable == FALSE){
	glDisable(GLLightType);
	GLfloat pZero[4] = {0, 0, 0, 0};
	GLfloat pOne[4] = {1, 1, 1, 0};
	glLightfv(GLLightType, GL_AMBIENT , pZero);
	glLightfv(GLLightType, GL_DIFFUSE , pZero);
	glLightfv(GLLightType, GL_SPECULAR, pZero);
	return;
    }

    glEnable(GLLightType);
    
    float pos[4], phiTheta[2], clr[4] = {0, 0, 0, 0};
    GetLightInfo(doubleIndex, pos, phiTheta, /*clr,*/ (mCLightType.mBody == 3));
    GetLightColor(doubleIndex, clr);
    glLightfv(GLLightType, GL_POSITION, pos);

    if ((mSelfIndex == 0) && (shadowMode != 0) && (mCbCastShadow.mbBody == TRUE))
	LOOP_RGBA clr[rgba] = 0;
    
    GLfloat lightColorAmbient[4] = {0, 0, 0, 0};
    if (mSelfIndex == 0) 
	for (int i = 0; i < 3; i++) lightColorAmbient[i] = 1.0f;
    
    glLightfv(GLLightType, GL_AMBIENT , lightColorAmbient);
    glLightfv(GLLightType, GL_DIFFUSE , clr);
    glLightfv(GLLightType, GL_SPECULAR, clr);
}

wxString LightSRC_cls::
Serialize(){
    wxString strOut;
    strOut << wxT("---LightSRC ") << mSelfIndex << ENDLWX;
    strOut << mCConfList.SerializeAll();
    strOut << mpCTranslate->Serialize();
    strOut << mpCRotation->Serialize();
    strOut << mpCLightStrength->Serialize();
    return strOut;
}

void LightSRC_cls::
ReadStrLine(std::wistream &istream, wxString strLineTag){
    // ł---LightSRC n ̃C͓ǂ񂾁B
    std::wstring wstrLine;
    wxString strLine;
    while ( std::getline (istream, wstrLine)){
	strLine = wstrLine.c_str();
	if (strLine.Find(wxT("ParamGroup=")) != wxNOT_FOUND) break;
	mCConfList.ReadStrLineAll(strLine);
    }
    mpCTranslate->ReadStrLine(istream, strLine);
    std::getline (istream, wstrLine);
    strLine = wstrLine.c_str();
    mpCRotation->ReadStrLine(istream, strLine);
    std::getline (istream, wstrLine);
    strLine = wstrLine.c_str();
    mpCLightStrength->ReadStrLine(istream, strLine);
}
