// "clsLightSourceGUI.h"

#include "clsLightSourceGUI.h"
#include "clsLightSource.h"

#include "../UNDO/clsUndoManager.h"
extern UndoManager_cls GCUndoList;
#include "../UNDO/clsUndoItemConstParam.h"

#include "../MAINCANVAS/clsParamGUICtrl.h"
#include "../PARAMETER/clsParamGroupGUI.h"
#include "../WIDGETCTRL/clsMultiInputPanel.h"
#include "../WIDGETCTRL/clsHookCtrl.h"
#include "../WIDGETCTRL/clsIconSelector.h"

#include "../../clsFrameMain.h"
#include "../../clsMain.h"
#include "../../wxWidgetEnum.h"
#include "../../wxWidgetEnumMenu.h"

#define LOOP_XYZ for (int xyz = 0; xyz < 3; xyz++)
#define X 0
#define Y 1
#define Z 2

#ifdef WIN32
#define ENDLWX wxT("\r\n")
#else
#define ENDLWX wxT("\n")
#endif

BEGIN_EVENT_TABLE(LightSRCGUI_cls, wxDialog)
EVT_BUTTON(btnLightAdd,    LightSRCGUI_cls::OnButtonAdd)
EVT_BUTTON(btnLightRemove, LightSRCGUI_cls::OnButtonRemove)
END_EVENT_TABLE()

LightSRCGUI_cls::
LightSRCGUI_cls(LightSRCCtrl_cls* pCCtrl,
		  wxWindow *pC, wxPoint pos, wxSize size) :
wxDialogEXKey(pC, wxID_ANY, _("Light Source Editor"), 
	      pos, size, pCCtrl->mpBitmapLink) {
    mspCThis = this;
    mpCCtrl = pCCtrl;

    wxIcon iconPointLight, iconSpotLight;
    iconPointLight.CopyFromBitmap(wxBitmap(wxT("ICONS/LightSource.png"), wxBITMAP_TYPE_PNG));
    iconSpotLight.CopyFromBitmap(wxBitmap(wxT("ICONS/SpotLight.png"), wxBITMAP_TYPE_PNG));
    mVLightTypeIcon.resize(2);
    mVLightTypeIcon[0] = iconPointLight;
    mVLightTypeIcon[1] = iconSpotLight;

    wxBoxSizer *pBoxSizerSrc   = new wxBoxSizer(wxVERTICAL);
    SetMNumMain();
    SetMNumParm();
    pBoxSizerSrc->Add(mpCMInputMain, 0, wxEXPAND);
    pBoxSizerSrc->Add(mpCMInputLight, 0, wxEXPAND);
    
    SetSizerAndFit(pBoxSizerSrc);
}

void LightSRCGUI_cls::
SetMNumMain(){
    MultiInputPanel_cls* pCurr = 
	InitializeMNumPanel(&mpCMInputMain, this);
    
    int boxWidth = 3;
    pCurr->CombinePointer(_ITEM_BOXSIZERHORIZONTAL, &boxWidth);

    pCurr->mItemWidth = -1;
    mIndexIcon = pCurr->CombinePointer
	(_ITEM_UINT_ICON, &mpCCtrl->mActiveItem);
   
    pCurr->mVCItemList[mIndexIcon].mpVIcon = &mVIcon;
    pCurr->mVCItemList[mIndexIcon].SetRange(0, mpCCtrl->mCLightNum.mBody, 1);

    pCurr->mItemWidth = 24;
    mIndexAdd    = pCurr->CombinePointerButton(btnLightAdd, wxT("+"));
    mIndexDelete = pCurr->CombinePointerButton(btnLightRemove, wxT("-"));

    pCurr->FinishSetting(NULL);

    Connect(mpCMInputMain->mVCItemList[mIndexIcon].mId, 
	    wxEVT_COMMAND_COMBOBOX_SELECTED,
	    wxCommandEventHandler(LightSRCGUI_cls::OnSelector));

    /*
    pCurr->SetFuncHook(&MultiParameters_cls::FuncHookItemNumberChanged,
		       _MN_HOOK_ASSIGN, wxID_ANY,
		       (MultiParameters_cls*)mpCParent);
    */
}

void LightSRCGUI_cls::
SetMNumParm(){
    MultiInputPanel_cls* pCurr = 
	InitializeMNumPanel(&mpCMInputLight, this);
    
    pCurr->mUpdateBit = 
	_COMMAND_UPDATE_DISPLAY_CAMCONF | _COMMAND_UPDATE_DISPLAY_GL |
	_COMMAND_UPDATE_DISPLAY_LIGHT; 

#ifdef WIN32
    pCurr->mTextWidth = 90;
    pCurr->mComboWidth = -1;
#else
    pCurr->mTextWidth = 100;
    pCurr->mComboWidth = 120;
#endif

    int boxWidth = 2;
    mIndexType = pCurr->CombinePointer
	(_ITEM_BOXSIZERHORIZONTAL, &boxWidth, _("Type/Fade"));
    
    wxString strListType[4] = {
	_("Point Light"), _("Spot Light"), _("Cylinder Light"), _("Parallel Light") };

    pCurr->CombinePointerList
	(_ITEM_UINT_LIST, &mpCCtrl->mpCLightSRC[0]->mCLightType.mBody,
	 4, strListType);
    wxString strListFade[3] = { 
	_("No Fade"), _("power 1"), _("power 2")};
    pCurr->CombinePointerList
	(_ITEM_UINT_LIST, &mpCCtrl->mpCLightSRC[0]->mCFadeType.mBody,
	 3, strListFade);

    pCurr->mItemWidth = -1;
    wxString strListTarget[3] = {
	_("No Target"), _("Target Origin"), _("Target Anchor") };
    mIndexTarget = pCurr->CombinePointerList
	(_ITEM_UINT_LIST, &mpCCtrl->mpCLightSRC[0]->mCTarget.mBody,
	 3, strListTarget, _("Target"));
    
    pCurr->mItemWidth = 80;
    boxWidth = 3;
    mIndexTranslate = pCurr->CombinePointer
	(_ITEM_BOXSIZERHORIZONTAL, &boxWidth, wxT("Translate"));
    pCurr->CombinePointer(_ITEM_DOUBLE, &mpCCtrl->mpCLightSRC[0]->mpCTranslate->mVCParam[X].mInterpolatedValue);
    pCurr->CombinePointer(_ITEM_DOUBLE, &mpCCtrl->mpCLightSRC[0]->mpCTranslate->mVCParam[Y].mInterpolatedValue);
    pCurr->CombinePointer(_ITEM_DOUBLE, &mpCCtrl->mpCLightSRC[0]->mpCTranslate->mVCParam[Z].mInterpolatedValue);

    mIndexRotate = pCurr->CombinePointer
	(_ITEM_BOXSIZERHORIZONTAL, &boxWidth, wxT("Rotate"));
    pCurr->CombinePointer(_ITEM_DOUBLE, &mpCCtrl->mpCLightSRC[0]->mpCRotation->mVCParam[X].mInterpolatedValue);
    pCurr->CombinePointer(_ITEM_DOUBLE, &mpCCtrl->mpCLightSRC[0]->mpCRotation->mVCParam[Y].mInterpolatedValue);
    pCurr->CombinePointer(_ITEM_DOUBLE, &mpCCtrl->mpCLightSRC[0]->mpCRotation->mVCParam[Z].mInterpolatedValue);

    pCurr->mItemWidth = -1;
    boxWidth = 2;
    mIndexStrength = pCurr->CombinePointer
	(_ITEM_BOXSIZERHORIZONTAL, &boxWidth, _("L. Strength"));
    pCurr->CombinePointer(_ITEM_DOUBLE, &mpCCtrl->mpCLightSRC[0]->mpCLightStrength->mVCParam[0].mInterpolatedValue);
    pCurr->CombinePointer(_ITEM_DOUBLE_SLIDER, &mpCCtrl->mpCLightSRC[0]->mpCLightStrength->mVCParam[0].mInterpolatedValue);
    pCurr->SetRange(_ITEM_LASTITEM, 0, 200, 100);

    mIndexShadow = pCurr->CombinePointer
	(_ITEM_BOOL, &mpCCtrl->mpCLightSRC[0]->mCbCastShadow.mbBody, _("Cast Shadow"));
    
    pCurr->FinishSetting(NULL);

    pCurr->SetColorForXYZ(mIndexTranslate+1);
    pCurr->SetColorForXYZ(mIndexRotate+1);

    pCurr->mpCHookCtrl->SetFuncHook(&FuncHookAssign, _HOOK_ASSIGN);
    pCurr->mpCHookCtrl->SetFuncHook(&FuncHookType, _HOOK_COMBOBOX);

    RemakeIconList();
}

void LightSRCGUI_cls::
ReconnectParam(){
    MultiInputPanel_cls* pCurr = mpCMInputLight;
    int i = mpCCtrl->mActiveItem;

    if (mpCCtrl->mCLightNum.mBody == 0) {
	// !! _FIX_ME_ !!
	return;
    }
    
    pCurr->mVCItemList[mIndexType+1].CombinePointer
	(_ITEM_UINT_LIST, &mpCCtrl->mpCLightSRC[i]->mCLightType.mBody);
    pCurr->mVCItemList[mIndexType+2].CombinePointer
	(_ITEM_UINT_LIST, &mpCCtrl->mpCLightSRC[i]->mCFadeType.mBody);
	
    pCurr->mVCItemList[mIndexTarget].CombinePointer
	(_ITEM_UINT_LIST, &mpCCtrl->mpCLightSRC[i]->mCTarget.mBody);

    LOOP_XYZ{
	pCurr->mVCItemList[mIndexTranslate+1+xyz].CombinePointer
	    (_ITEM_DOUBLE, &mpCCtrl->mpCLightSRC[i]->mpCTranslate->mVCParam[xyz].mInterpolatedValue);
    }

    LOOP_XYZ{
	pCurr->mVCItemList[mIndexRotate+1+xyz].CombinePointer
	    (_ITEM_DOUBLE, &mpCCtrl->mpCLightSRC[i]->mpCRotation->mVCParam[xyz].mInterpolatedValue);
    }

    pCurr->mVCItemList[mIndexStrength+1].CombinePointer
	(_ITEM_DOUBLE, &mpCCtrl->mpCLightSRC[i]->mpCLightStrength->mVCParam[0].mInterpolatedValue);
    pCurr->mVCItemList[mIndexStrength+2].CombinePointer
	(_ITEM_DOUBLE_SLIDER, &mpCCtrl->mpCLightSRC[i]->mpCLightStrength->mVCParam[0].mInterpolatedValue);

    pCurr->mVCItemList[mIndexShadow].CombinePointer
	(_ITEM_BOOL, &mpCCtrl->mpCLightSRC[i]->mCbCastShadow.mbBody);

    pCurr->UpdateValues();
}

void LightSRCGUI_cls::
RemakeIconList(int selected){
    mVIcon.clear();
    for (int i = 0; i< mpCCtrl->mCLightNum.mBody; i++){
	if (mpCCtrl->mpCLightSRC[i]->mCLightType.mBody == _LIGHT_SPOT)
	    mVIcon.push_back(mVLightTypeIcon[1]);
	else
	    mVIcon.push_back(mVLightTypeIcon[0]);
    }

    if (selected > 0)
	((IconSelector_cls*)mpCMInputMain->mVCItemList[mIndexIcon].mpCCtrl)
	    ->SetValueInt(selected);
    if (selected == -2)
	((IconSelector_cls*)mpCMInputMain->mVCItemList[mIndexIcon].mpCCtrl)
	    ->SetValueInt(mpCCtrl->mActiveItem);
    ((IconSelector_cls*)mpCMInputMain->mVCItemList[mIndexIcon].mpCCtrl)
	->RemakeImageList(&mVIcon);
}

void LightSRCGUI_cls::
OnButtonAdd(wxCommandEvent &event){
    mpCCtrl->AddLight();
    RemakeIconList(mpCCtrl->mActiveItem);
}

void LightSRCGUI_cls::
OnButtonRemove(wxCommandEvent &event){
    if (mpCCtrl->mCLightNum.mBody == 0) return;
    mpCCtrl->RemoveLight();
    RemakeIconList();
}

void LightSRCGUI_cls::
OnSelector(wxCommandEvent& event){
    ReconnectParam();
    int i = mpCCtrl->mActiveItem;
    mpCCtrl->mpCParentFrameLink->mpCParent->
	SelectObject(mpCCtrl->mpCLightSRC[i]->mObjIndex);
}

void LightSRCGUI_cls::
FuncHookAssign(void *pPanel, void *pArg){
    int i = mspCThis->mpCCtrl->mActiveItem;
    mspCThis->mpCCtrl->mpCLightSRC[i]->mpCTranslate->    SetKeyFrameIfChanged();
    mspCThis->mpCCtrl->mpCLightSRC[i]->mpCRotation->     SetKeyFrameIfChanged();
    mspCThis->mpCCtrl->mpCLightSRC[i]->mpCLightStrength->SetKeyFrameIfChanged();
    
    wxCommandEvent com(wxEVT_COMMAND_MENU_SELECTED, menuEditRefreshGL);
    wxGetTopLevelParent(mspCThis->mpCMInputLight)->
	GetParent()->AddPendingEvent(com);
}

void LightSRCGUI_cls::
FuncHookType(void *pPanel, void *pArg){
    mspCThis->RemakeIconList();
}

LightSRCGUI_cls* LightSRCGUI_cls::mspCThis;

// ------

LightSRCCtrl_cls::
LightSRCCtrl_cls(FrameMain_cls* pCParentFrame){

    mpCGUI = NULL;
    mpCParentFrameLink = pCParentFrame;
    mpBitmapLink = mpCParentFrameLink->mpBitmapLight0;

    mCConfList.Clear();
    mCConfList.Add(&mCLightNum, wxT("LightNumber="), wxT("2"));
    mActiveItem = 0;
    mUpdateBit = _COMMAND_UPDATE_DISPLAY_TL | _COMMAND_UPDATE_DISPLAY_GL | _COMMAND_UPDATE_DISPLAY_LIGHT;

    for (int i = 0; i < _LIGHT_NUM_MAX; i++){
	wxString strTag;
	strTag << wxT("LightSRC") << i;
	mpCLightSRC[i]   = new LightSRC_cls(i, strTag);
	mpCParamGroup[i] = new ParamGroup_cls(strTag, 0, NULL, 0);
	mpCParamGroup[i]->mVpCParamGroup.push_back(mpCLightSRC[i]->mpCTranslate);
	mpCParamGroup[i]->mVpCParamGroup.push_back(mpCLightSRC[i]->mpCRotation);
	mpCParamGroup[i]->mVpCParamGroup.push_back(mpCLightSRC[i]->mpCLightStrength);
	mpCParamGroup[i]->mpCSceneObjLink = mpCLightSRC[i];
	if (i >= mCLightNum.mBody)
	    mpCParamGroup[i]->mbShownOnTL = FALSE;
    }
    
    InitGUI(pCParentFrame);
}

void LightSRCCtrl_cls::
InitGUI(wxWindow *pC){
    mpCGUI = new LightSRCGUI_cls(this, pC,
				 /*mCScreenRect.GetPosition(),*/
				 /*mCScreenRect.GetSize()*/
				 wxDefaultPosition, wxDefaultSize);
    SetPDialog(mpCGUI);
}

LightSRCCtrl_cls::
~LightSRCCtrl_cls(){
    for (int i = 0; i < _LIGHT_NUM_MAX; i++){
	delete mpCLightSRC[i];
	delete mpCParamGroup[i];
    }
}

void LightSRCCtrl_cls::
UpdateTLShown(){
    for (int i = 0; i < _LIGHT_NUM_MAX; i++){
	if (i < mCLightNum.mBody) 
	    mpCParamGroup[i]->mbShownOnTL = TRUE;
	else
	    mpCParamGroup[i]->mbShownOnTL = FALSE;
    }
    mpCParamGroup[0]->mpCGUI->mpCCtrlLink->HideAndShow();
}

void LightSRCCtrl_cls::
SelectLight(int index){
    mActiveItem = index;
    mpCGUI->ReconnectParam();
}

void LightSRCCtrl_cls::
AddLight(){
    if (mCLightNum.mBody < _LIGHT_NUM_MAX){
	unsigned int oldValue = mCLightNum.mBody;
	unsigned int newValue = mCLightNum.mBody + 1;
	GCUndoList.OpenBlock();
	GCUndoList.SetNewCommand
	    (new CommandSetValue_cls(&mCLightNum.mBody, newValue, mUpdateBit),
	     new CommandSetValue_cls(&mCLightNum.mBody, oldValue, mUpdateBit));
	GCUndoList.ExecCurrent();
	GCUndoList.SetNewCommand
	    (new CommandSetValue_cls(&mActiveItem, newValue - 1, mUpdateBit),
	     new CommandSetValue_cls(&mActiveItem, mActiveItem , mUpdateBit));
	GCUndoList.ExecCurrent();
	GCUndoList.CloseBlock();
    }
    mpCGUI->ReconnectParam();
    UpdateTLShown();
}

void LightSRCCtrl_cls::
RemoveLight(){
    GCUndoList.OpenBlock();
    for (int i = mActiveItem; i < _LIGHT_NUM_MAX - 1; i++)
	mpCLightSRC[i]->CopyFromP(mpCLightSRC[i+1]);
    unsigned int oldValue = mCLightNum.mBody;
    unsigned int newValue = mCLightNum.mBody - 1;
    GCUndoList.SetNewCommand
	    (new CommandSetValue_cls(&mCLightNum.mBody, newValue, mUpdateBit),
	     new CommandSetValue_cls(&mCLightNum.mBody, oldValue, mUpdateBit));
    GCUndoList.ExecCurrent();
    if (mActiveItem == newValue){
	GCUndoList.SetNewCommand
	    (new CommandSetValue_cls(&mActiveItem, newValue-1, mUpdateBit),
	     new CommandSetValue_cls(&mActiveItem, newValue, mUpdateBit));
	GCUndoList.ExecCurrent();
    }
    GCUndoList.CloseBlock();
    mpCGUI->ReconnectParam();
    UpdateTLShown();
}

GLuint LightSRCCtrl_cls::
GetLightType(int lightIndex){
    GLuint GLLightType;
    switch(lightIndex){
    case 0: GLLightType = GL_LIGHT0; break;
    case 1: GLLightType = GL_LIGHT1; break;
    case 2: GLLightType = GL_LIGHT2; break;
    case 3: GLLightType = GL_LIGHT3; break;
    case 4: GLLightType = GL_LIGHT4; break;
    case 5: GLLightType = GL_LIGHT5; break;
    case 6: GLLightType = GL_LIGHT6; break;
    }
    return GLLightType;
}

int LightSRCCtrl_cls::
GetNumberLightWithShadow(){
    int num = 0;
    for (int i = 0; i < mCLightNum.mBody; i++)
	if (mpCLightSRC[i]->mCbCastShadow.mbBody == TRUE) num++;
    return num;
}

int LightSRCCtrl_cls::
GetActiveShadow(){
    int index = 0;
    for (int i = 0; i < mCLightNum.mBody; i++)
	if (mpCLightSRC[i]->mCbCastShadow.mbBody == TRUE) return i;
    return index;
}

void LightSRCCtrl_cls::
GetLightInfo(double doubleIndex, float pLightPos[4], float pLightThetaPhi[2],
	     int index, bool bSV){
    mpCLightSRC[index]->
	GetLightInfo(doubleIndex, pLightPos, pLightThetaPhi, bSV);
}

void LightSRCCtrl_cls::
GLDrawPath(double doubleIndex){
    for (int i = 0; i < mCLightNum.mBody; i++)
	mpCLightSRC[i]->GLDrawPath(doubleIndex);
}

void LightSRCCtrl_cls::
GLDrawCurrent(double doubleIndex, double scale, int displayList){
    for (int i = 0; i < mCLightNum.mBody; i++){
	glPushName(mpCLightSRC[i]->mObjIndex);
	bool bDirection = FALSE;
	if (mpCLightSRC[i]->mCLightType.mBody == _LIGHT_PARALLEL)
	    bDirection = TRUE;
	mpCLightSRC[i]->
	    GLDrawCurrent(doubleIndex, scale, displayList, bDirection);
	glPopName();
    }
}

void LightSRCCtrl_cls::
GLSetLightSource(double doubleIndex, int shadowMode){
    for (int i = 0; i < _LIGHT_NUM_MAX; i++)
	mpCLightSRC[i]->GLSetLightSource
	    (doubleIndex, (i < mCLightNum.mBody), shadowMode);
}

void LightSRCCtrl_cls::
GLDisableAllLight(){
    for (int i = 0; i < _LIGHT_NUM_MAX; i++){
	glDisable(GetLightType(i));
	GLfloat pZero[4] = {0, 0, 0, 0};
	glLightfv(GL_LIGHT0, GL_AMBIENT , pZero);
	glLightfv(GL_LIGHT0, GL_DIFFUSE , pZero);
	glLightfv(GL_LIGHT0, GL_SPECULAR, pZero);
    }
}

void LightSRCCtrl_cls::
GLSetSingleDiffuse(int lightIndex, double doubleIndex){
    GLDisableAllLight();
    mpCLightSRC[lightIndex]->GLSetLightSourceComp(0, doubleIndex, FALSE, TRUE);
}

void LightSRCCtrl_cls::
SetNonShadowLight(double doubleIndex){
    GLDisableAllLight();
    for (int i = 0; i < mCLightNum.mBody; i++)
	mpCLightSRC[i]->GLSetLightSourceComp(i, doubleIndex, (i==0), TRUE);
}

void LightSRCCtrl_cls::
SetBasicLight(double doubleIndex, int shadowMode){

    for (int i = 1; i < _LIGHT_NUM_MAX; i++) 
	glDisable(GetLightType(i));

    if (mpCLightSRC[0]->mCbCastShadow.mbBody == TRUE)
	mpCLightSRC[0]->GLSetLightSourceComp(0, doubleIndex, TRUE, FALSE);
    else
	mpCLightSRC[0]->GLSetLightSourceComp(0, doubleIndex, TRUE, TRUE);

    // e𗎂ƂȂ̌ʂ
    for (int i = 1; i < _LIGHT_NUM_MAX; i++)
	mpCLightSRC[i]->GLSetLightSource
	    (doubleIndex, (i < mCLightNum.mBody) & (mpCLightSRC[i]->mCbCastShadow.mbBody == FALSE), shadowMode);
}

void LightSRCCtrl_cls::
UpdateItems(bool bForce){
    if (mpCGUI == NULL) return;
}

// ------

wxString LightSRCCtrl_cls::
Serialize(){
    wxString strOut;

    strOut << wxT("---AllLightSource") << ENDLWX;
    strOut << mCConfList.SerializeAll();

    for (int i = 0; i < mCLightNum.mBody; i++)
	strOut << mpCLightSRC[i]->Serialize();

    strOut << wxT("---AllLightSourceEnd") << ENDLWX;
    
    return strOut;
}

void LightSRCCtrl_cls::
ReadStrLine(std::wistream &istream, wxString strLineTag){
    if (strLineTag == wxT("---AllLightSource")){
	std::wstring wstrLine;
	wxString strLine;
	while ( std::getline (istream, wstrLine)){
	    strLine = wstrLine.c_str();
	    if (strLine.Find(wxT("---")) != wxNOT_FOUND) break;
	    mCConfList.ReadStrLineAll(strLine);
	}
	int lightIndex = 0;
	while (TRUE){
	    if (strLine == wxT("---AllLightSourceEnd")) break;
	    
	    if (strLine.Find(wxT("---LightSRC")) != wxNOT_FOUND){
		mpCLightSRC[lightIndex]->ReadStrLine(istream, strLine);
		lightIndex++;
	    }
	    std::getline(istream, wstrLine);
	    strLine = wstrLine.c_str();
	}
    }
}
