// -*- c -*-
// "clsCameraPath.h"

#include "clsSceneObj.h"

#include "../MYGL//clsGLGuides.h"
#include "../UNDO/clsUndoManager.h"
extern UndoManager_cls GCUndoList;
#include "../UNDO/clsUndoItemConstParam.h"

#define LOOP_XYZ for (int xyz = 0; xyz < 3; xyz++)
#define X 0
#define Y 1
#define Z 2

SceneObj_cls::
SceneObj_cls(wxString strObjName){
    wxString listTag[3] = {wxT("x"), wxT("y"), wxT("z")};
    wxString strObjTrans = strObjName;
    wxString strObjRotation = strObjName;
    strObjTrans    << wxT(" Translate");
    strObjRotation << wxT(" Rotation");
    int updateBit = _COMMAND_UPDATE_DISPLAY_GL | _COMMAND_UPDATE_DISPLAY_TL;
    mpCTranslate = new ParamGroup_cls(strObjTrans, 3, listTag, updateBit, TRUE);
    mpCRotation  = new ParamGroup_cls(strObjRotation, 3, listTag, updateBit, TRUE);
    
    mpCRotation->mVCParam[0].mCPeriodic.mBody = 360;
    mpCRotation->mVCParam[2].mCPeriodic.mBody = 360;
    mbSelected = FALSE;
    mbDragging = FALSE;
    mDragMode = -1;
}

SceneObj_cls::
~SceneObj_cls(){
    delete mpCTranslate;
    delete mpCRotation;
}

void SceneObj_cls::
SetInitValueTrans(double x, double y, double z){
    double pTransInit[3] = {x, y, z};
    mpCTranslate->SetKeyFrame(pTransInit, TRUE);
}

void SceneObj_cls::
SetInitValueRot(double x, double y, double z){
    double pRotInit[3] = {x, y, z};
    mpCRotation->SetKeyFrame(pRotInit, TRUE);
}

void SceneObj_cls::
CopyFromP_SceneObj(SceneObj_cls *pCFrom){
    int newValue = pCFrom->mObjType;
    int oldValue = mObjType;
    GCUndoList.SetNewCommand
	(new CommandSetValue_cls(&mObjType, newValue),
	 new CommandSetValue_cls(&mObjType, oldValue));
    GCUndoList.ExecCurrent();
    mpCTranslate->CopyFromP(pCFrom->mpCTranslate);
    mpCRotation ->CopyFromP(pCFrom->mpCRotation);
}

void SceneObj_cls::
UpdateCurrentObj(double doubleIndex){
    std::vector < double > VPos = 
	mpCTranslate->GetInterpolatedValue(doubleIndex, 0);
    std::vector < double > VRot = 
	mpCRotation->GetInterpolatedValue(doubleIndex, 0);
    
    Position_cls CPos(VPos[X], VPos[Y], VPos[Z]);
    mCCurrentObj.Set(CPos);
    double tgt[3] = {1, 0, 0};
    double top[3] = {0, 0, 1};
    mCCurrentObj.Setv(tgt, top);
    mCCurrentObj.RotateYPR(VRot[Z], 0, 0);
    mCCurrentObj.RotateYPR(0, VRot[Y], 0);
    mCCurrentObj.RotateYPR(0, 0, VRot[X]);

}

void SceneObj_cls::
Targetting(){
    switch(mCTarget.mBody){
    case 0:
	break;
    case 1:{
	double targetPos[3] = {0, 0, 0};
	mCCurrentObj.PointAt(targetPos);
	mCCurrentObj.UpdateYPR();
	LOOP_XYZ mpCRotation->mVCParam[xyz].mInterpolatedValue = 
	    mCCurrentObj.mYPR[xyz];
	mpCRotation->SetKeyFrameIfChanged();
	break; }
    }
}

void SceneObj_cls::
SetDragMode(int mode){
    mDragMode = mode;
}

void SceneObj_cls::
GLDrawGuide(double scale){
    PositionAndDirection_cls CPosOrigin;
    mspCGLGuideLink->DrawGuide(mCCurrentObj.mPos, scale, //mObjIndex,
			       CPosOrigin, mDragMode);
}

void SceneObj_cls::
GLDrawPath(double scale){
    if (mCbVisible.mbBody == FALSE) return;
    
    glColor4d(0.2, 1.0, 0.2, 1);
    glBegin(GL_POINTS);
    for (int i = 0; i < 100; i++){
	double doubleIndex = (double) i / 100;
	std::vector < double > VPos = 
	    mpCTranslate->GetInterpolatedValue(doubleIndex, 0);
	glVertex3dv(&VPos[0]);
    }
    glEnd();
}

void SceneObj_cls::
GLDrawCurrent(double doubleIndex, double scale, int displayList,
	      bool bDirection){
    if (mCbVisible.mbBody == FALSE) return;

    if (mbSelected == TRUE && mbDragging == FALSE) GLDrawGuide(scale);

    glColor4d(0.2, 1.0, 0.2, 1);
    UpdateCurrentObj(doubleIndex);

    double matrix[16];
    mCCurrentObj.GetGLMatrix(matrix);
    glPushMatrix();
    glMultMatrixd(matrix); 
    glScaled(scale*0.02, scale*0.02, scale*0.02);
    if (mObjType != _SCENEOBJ_LIGHT)
	glCallList(displayList);
    else{
	glEnable(GL_LINE_STIPPLE);
	glLineStipple(1,0xff00);
	glColor4f(0.7f, 0.7f, 0.2f, 1.0f);

	if (bDirection == TRUE){ 
	    switch(mCTarget.mBody){
	    case 0:
		glBegin(GL_LINES); // 
		glVertex3d(0, 0, 0);
		glVertex3d(3, 0, 0);
		glEnd();
		break;
	    case 1:
		glPopMatrix();
		glBegin(GL_LINES);
		glVertex3dv(mCCurrentObj.mPos);
		glVertex3d(0, 0, 0);
		glEnd();
		glPushMatrix();
		break;
	    }
	}
	glPopMatrix();

	glLineStipple(1,0xf0f0);
	glBegin(GL_LINES); // ⏕
	glVertex3d(mCCurrentObj.mPos[X], mCCurrentObj.mPos[Y], 0);
	glVertex3d(mCCurrentObj.mPos[X], mCCurrentObj.mPos[Y], mCCurrentObj.mPos[Z]);
	glEnd();
	glDisable(GL_LINE_STIPPLE);	

	// J̕ɂ́H
	GLfloat matrixB[16];
	glGetFloatv(GL_TRANSPOSE_MODELVIEW_MATRIX, matrixB);
	
	glPushMatrix();    
	
	for (int j = 0; j < 3; j++){
	    matrix[0+j] = matrixB[0+j];
	    matrix[4+j] = matrixB[4+j];
	    matrix[8+j] = matrixB[8+j];
	}

	glMultMatrixd(matrix); 
	glScaled(scale*0.01, scale*0.01, scale*0.01);
	glCallList(displayList);
    }
    glPopMatrix();



    /*
    glPushMatrix();
    glTranslated(VPos[X], VPos[Y], VPos[Z]);
    glRotated(VRot[Z], 0, 0, 1);
    glRotated(VRot[Y], 0, 1, 0);
    glRotated(VRot[X], 1, 0, 0);
    Camera_cls::Draw(_CAMERA_BASIC, scale * 0.02);
    glPopMatrix();
    */
}

GLGuides_cls* SceneObj_cls::mspCGLGuideLink;
GLGuides_cls* SceneObj_cls::mspCGLGuideYPRLink;
