// "clsTLPanel.h"

#include "clsTLPanelParam.h"
//#include "../clsDF3Sequence.h"
#include "clsTimeCtrl.h"

#include "../../wxWidgetEnumMenu.h"

#ifndef TRUE
#define TRUE true
#define FALSE false
#endif

// ---------------------------

BEGIN_EVENT_TABLE(TLPanel_cls, wxPanel)
     EVT_PAINT  (TLPanel_cls::OnPaint)
     EVT_CHAR   (TLPanel_cls::OnChar)
END_EVENT_TABLE()

TLPanel_cls::
TLPanel_cls(wxWindow *parent, wxWindowID id, wxPoint pos, wxSize size):
    wxPanel(parent, id, pos, size)
{
    mOffset = _MEM_PANEL_OFFSET;
    SetBackgroundColour(wxColour(216, 216, 216));
    SetBackgroundStyle(wxBG_STYLE_CUSTOM);
}

TLPanel_cls::
~TLPanel_cls(){
}

void TLPanel_cls::
UpdateTLFrame(){
    mCurrentFrame = mspCTimeLink->mFrame;
    mFrameMax   = mspCTimeLink->mCMaxFrame.mBody;
    mMinFrameWA = mspCTimeLink->mCWAMin.mBody;
    mMaxFrameWA = mspCTimeLink->mCWAMax.mBody;
    mAAMin      = mspCTimeLink->mCAAMin.mBody;
    mAAMax      = mspCTimeLink->mCAAMax.mBody;
    mStep       = mspCTimeLink->mCStep.mBody;
}

int TLPanel_cls::
CalcFrameFromPosition(int pos, unsigned int minFrame, unsigned int maxFrame){
    if (maxFrame == minFrame) return 0;
    
    double dIndex = 
	(double)(pos - mOffset) / (mWidth - 2.0 * mOffset);
    int frame = 
	(int)(dIndex * (maxFrame - minFrame) + minFrame + 0.5);
    if (frame < 0) frame = 0;
    if (frame > (int)maxFrame) frame = maxFrame;
    return frame;
}

// -- Region Selector --

void TLPanel_cls::
SetRange(unsigned int regionMin, unsigned int regionMax){
    mRegionMin = regionMin;
    mRegionMax = regionMax;
}

void TLPanel_cls::
SetInitRegion(wxMouseEvent& event,
	      unsigned int initMin, unsigned int initMax){
    mInitMin = initMin;
    mInitMax = initMax;
    
    mMouseFrame = CalcFrameFromPosition(event.GetX(),
					mRegionMin, mRegionMax);

    CalcCoordinateX0X1(mInitMin, mInitMax - mInitMin,
		       mRegionMin, mRegionMax);
}

void TLPanel_cls::
DrawRegionSliderX0X1(wxAutoBufferedPaintDC &dc){
    //CalcCoordinateX0X1(mMinFrameWA, mMaxFrameWA-mMinFrameWA, mAAMin, mAAMax);

    dc.SetBrush(wxBrush(wxT("AQUAMARINE"), wxSOLID));
    dc.SetPen(*wxBLACK_PEN);
    wxPoint *listPointA = new wxPoint[4];
    wxPoint *listPointB = new wxPoint[4];
    listPointA[0] = wxPoint(-1, 0);
    listPointA[1] = wxPoint(-1, mHeight-1);
    listPointA[2] = wxPoint(-7, mHeight-1);
    listPointA[3] = wxPoint(-7, 3);
    listPointB[0] = wxPoint(0, 0);
    listPointB[1] = wxPoint(0, mHeight-1);
    listPointB[2] = wxPoint(6, mHeight-1);
    listPointB[3] = wxPoint(6, 3);
    dc.DrawPolygon(4, listPointA, mX0);
    dc.DrawPolygon(4, listPointB, mX1);
    delete listPointA;
    delete listPointB;
}

void TLPanel_cls::
CalcSelected(wxMouseEvent &event){
    mSelected = 0; 
    mFrameInDragging = mCurrentFrame;
    int posX = event.GetX();
    if (posX > mXCurrent-5 && posX < mXCurrent + 5) {
	mSelected = _TL_SEL_SLIDER; return; }
    if (posX < mX0 && posX > mX0 - 7) mSelected = _TL_SEL_MIN;
    if (posX < mX1 + 7 && posX > mX1) mSelected = _TL_SEL_MAX;
    if (posX < mX1 && posX > mX0) mSelected = _TL_SEL_REGION;
}

bool TLPanel_cls::
ProcSelect(){
    if (mSelected == 0) return FALSE;
    
    //CaptureMouse();
    //mbCaptureMouse = TRUE;
    if (mSelected == _TL_SEL_MIN || mSelected == _TL_SEL_MAX)
	SetCursor(wxCURSOR_SIZEWE);
    if (mSelected == _TL_SEL_REGION){
	SetCursor(wxCURSOR_HAND);
	mFrameAtClick = mMouseFrame;
	mMinAtClick = mInitMin;
	mMaxAtClick = mInitMax;
    }
    
    return TRUE;
}

void TLPanel_cls::
ProcDragging(unsigned int *pMin, unsigned int *pMax){ 
    switch (mSelected){
    case _TL_SEL_MIN:
	*pMin = mMouseFrame; break;
    case _TL_SEL_MAX:
	*pMax = mMouseFrame; break;
    case _TL_SEL_REGION: {
	int min = mMinAtClick + (mMouseFrame - mFrameAtClick);
	int max = mMaxAtClick + (mMouseFrame - mFrameAtClick);
	if (min < (int)mRegionMin) min = mRegionMin;
	if (max > (int)mRegionMax) max = mRegionMax;
	*pMin = min;
	*pMax = max;
	break; }
    case _TL_SEL_SLIDER:
	mFrameInDragging = mMouseFrame; break;
    }
}

void TLPanel_cls::
ProcMouseEventBase(wxMouseEvent& event){

    if (event.ButtonDown(wxMOUSE_BTN_LEFT)){
	CalcSelected(event);
	if (ProcSelect() == TRUE){
	    CaptureMouse();
	    mbCaptureMouse = TRUE;
	}
	if (mSelected == _TL_SEL_SLIDER)
	    mspCTimeLink->SetIndex(mFrameInDragging);
    }

    if (event.ButtonUp(wxMOUSE_BTN_LEFT)){
	if (mbCaptureMouse == TRUE){
	    ReleaseMouse();
	    mbCaptureMouse = FALSE;
	    SetCursor(wxNullCursor);
	}
	if (mSelected == _TL_SEL_SLIDER)
	    mspCTimeLink->SetIndex(mFrameInDragging);
	mSelected = 0;
    }

    if (event.Dragging() == TRUE){
	ProcDragging(mpCRegionTgtMin, mpCRegionTgtMax);
	if (mSelected == _TL_SEL_SLIDER)
	    mspCTimeLink->SetIndex(mFrameInDragging);
    }

    if ((*mpCRegionTgtMin != mInitMin) ||
	(*mpCRegionTgtMax != mInitMax)){
	mspCTimeLink->CheckWorkingArea();
	mspCTimeLink->UpdateDF3SeqGUI();
	UpdateDisplay();
	//Refresh(FALSE);
	wxCommandEvent com(wxEVT_COMMAND_MENU_SELECTED, menuItemRefreshPanels);
	wxGetTopLevelParent(this)->AddPendingEvent(com);
    }

    mPosCurrent = event.GetX();
    mFrameInSlider = mMouseFrame;

    if (mFrameInDragging != mFrameInDisplay){
	UpdateDisplay();
	//Refresh(FALSE);
	wxCommandEvent com2(wxEVT_COMMAND_MENU_SELECTED, menuItemRefresh);
	wxGetTopLevelParent(this)->AddPendingEvent(com2);
	wxCommandEvent com1(wxEVT_COMMAND_MENU_SELECTED, menuItemRefreshPanels);
	wxGetTopLevelParent(this)->AddPendingEvent(com1);
	mFrameInDisplay = mFrameInDragging;
    }
}

// --------------

void TLPanel_cls::
OnPaint(wxPaintEvent& event){
    UpdateDisplay();
    event.Skip();
}

void TLPanel_cls::
OnChar(wxKeyEvent& event){
    GetParent()->AddPendingEvent(event);
}

TimeCtrl_cls* TLPanel_cls::mspCTimeLink = NULL;
