// -*- c++ -*-
// (setq c-basic-offset 4)

#include "clsTimeCtrl.h"

#include "clsPlayCtrlGUI.h"
#include "clsTimeCtrlGUI.h"
#include "clsTimeCtrlGUIAA.h"
#include "clsTimeCtrlGUIAtoi.h"
#include "clsTimeCtrlGUITics.h"

#include "../MAINCANVAS/clsParamGUICtrl.h"
#include "../PARAMETER/clsParameters.h"
#include "../PARTICLEDATA/clsParticleDataSeqCtrl.h"
#include "../UNDO/clsUndoManager.h"
extern UndoManager_cls GCUndoList;
#include "../UNDO/clsUndoItemTimeCtrl.h"

#include "../../clsMain.h"
#include "../../clsFrameMain.h"
#include "../../wxWidgetEnumMenu.h"

#ifdef WIN32
#define ENDLWX wxT("\r\n");
#else
#define ENDLWX wxT("\n");
#endif

TimeCtrl_cls::
TimeCtrl_cls(Main_cls *pCMain) //: mCTimeRemap(1, L"TimeRemap")
{
    mpCMainLink = pCMain;
    mFrame = 0;
    mInterpStep = 0;
    mCustomRegionMin = 0;
    mCustomRegionMax = 99;

    mbPlayTex = FALSE; 
    mPlayMode = _TIME_CTRL_PLAY_FULL;
    mPlayDirection = +1;
    mPrefetchRange = 3;
    mClockLastDraw = 0;
    mWaitDuration = 30;
    mbUndoGroupOpened = FALSE;

    mCConfListTime.Add(&mCMaxFrame, L"TimeMaxFrame=", L"100");
    mCConfListTime.Add(&mCWAMin, L"TimeWorkAreaMin=", L"0");
    mCConfListTime.Add(&mCWAMax, L"TimeWorkAreaMax=", L"100");
    mCConfListTime.Add(&mCAAMin, L"TimeActiveAreaMin=", L"0");
    mCConfListTime.Add(&mCAAMax, L"TImeActiveAreaMax=", L"100");

    mCConfListTime.Add(&mCbUseTimeRemap, L"TimeUseTimeRemap=", L"0");
    mCConfListTime.Add(&mCAnimStep, L"TimeAnimStep=", L"1");
    mCConfListTime.Add(&mCAnimInterp, L"TimeAnimInterp=", L"1");
    mCConfListTime.Add(&mCbRepeat, L"TimeRepeat=", L"0");
    
    mCConfListTime.Add(&mCSceneLengthType, L"TimeSceneLengthType=", L"0");
    
    //TimeLinePanel_cls::msVBSnippingTag.resize(99);

    mCStopWatchWait.Start();
}

void TimeCtrl_cls::
InstantiateGUIA(wxWindow *pCParent){
    mpCGUITics = new TimeCtrlGUITics_cls
	(pCParent, this, wxID_ANY, wxDefaultPosition, wxSize(-1, 12));
    mpCGUIAA = new TimeCtrlGUIAA_cls	
	(pCParent, this, wxID_ANY, wxDefaultPosition, wxSize(-1, 12));
    mpCGUIAtoi = new TimeCtrlGUIAtoi_cls	
	(pCParent, this, wxID_ANY, wxDefaultPosition, wxSize(73, 24));
    mpCGUI = new TimeCtrlGUI_cls
	(pCParent, this, wxID_ANY, wxDefaultPosition, wxSize(-1, 24));
}

void TimeCtrl_cls::
InstantiateGUIB(wxWindow *pCParent){
    mpCGUIPlay = new PlayCtrlGUI_cls
	(pCParent, this, wxID_ANY, wxDefaultPosition, wxSize(-1, -1));
}

void TimeCtrl_cls::SetValueCommand(int target, int valueNew, int valueOld){
    GCUndoList.SetNewCommand
	(new CommandSetTimeCtrlValue_cls(this, target, valueNew),
	 new CommandSetTimeCtrlValue_cls(this, target, valueOld));
    GCUndoList.ExecCurrent();
}

void TimeCtrl_cls::MoveFrameCommand(int dframe){
    int newframe = CalcNextFrame(_TIME_CTRL_CURRENT, dframe);
    if (newframe == mFrame) return;
    SetValueCommand(_COMMAND_TARGET_FRAME, newframe, mFrame);
}

void TimeCtrl_cls::MoveInterpStepCommand(int dstep){
    int newStep = CalcNextStep(_TIME_CTRL_CURRENT, dstep);
    int newFrame = mFrame;
    CalcNextFrameFromStep(&newFrame, &newStep);
    if (newFrame != mFrame || newStep != mInterpStep){
	GCUndoList.OpenBlock();
	SetValueCommand(_COMMAND_TARGET_INTERPSTEP, newStep, mInterpStep);
	SetValueCommand(_COMMAND_TARGET_FRAME, newFrame, mFrame);
	GCUndoList.CloseBlock();
    }
}

void TimeCtrl_cls::MoveFrameCommandLarge(int dframe){
    int newframe;
    int newstep = 0;
    if (dframe < 0) newframe = mCWAMin.mBody;
    else            newframe = mCWAMax.mBody;
    
    if (newframe == mFrame && newstep == mInterpStep) return;
    GCUndoList.OpenBlock();
    SetValueCommand(_COMMAND_TARGET_INTERPSTEP, newstep, mInterpStep);
    SetValueCommand(_COMMAND_TARGET_FRAME, newframe, mFrame);
    GCUndoList.CloseBlock();
}

void TimeCtrl_cls::MoveFrameCommandSnip(int direction){
    int newframe;
    int newstep = 0;
    if (direction > 0) newframe = GetSnipNext();
    else               newframe = GetSnipPrev();
    GCUndoList.OpenBlock();
    SetValueCommand(_COMMAND_TARGET_INTERPSTEP, newstep, mInterpStep);
    SetValueCommand(_COMMAND_TARGET_FRAME, newframe, mFrame);
    GCUndoList.CloseBlock();
}

void TimeCtrl_cls::SetFrameCommand(int frame, int frameAtClick){
    if (frameAtClick == _TIME_CTRL_CURRENT) frameAtClick = mFrame;
    SetValueCommand(_COMMAND_TARGET_FRAME, frame, frameAtClick);
}

void TimeCtrl_cls::SetFrameFromDoubleCommand(double doubleIndex){
    unsigned int newFrame = GetIndexFromDouble(doubleIndex);
    unsigned int newstep = 0;
    if (newFrame == mFrame && newstep == mInterpStep) return;
    GCUndoList.OpenBlock();
    SetValueCommand(_COMMAND_TARGET_INTERPSTEP, newstep, mInterpStep);
    SetValueCommand(_COMMAND_TARGET_FRAME, newFrame, mFrame);
    GCUndoList.CloseBlock();
}

void TimeCtrl_cls::SetInterpStepCommand(int step){
    SetValueCommand(_COMMAND_TARGET_INTERPSTEP, step, mInterpStep);
}

void TimeCtrl_cls::SetWAMinCommand(int min, int minAtClick){
    if (minAtClick == _TIME_CTRL_CURRENT) minAtClick = mCWAMin.mBody;
    SetValueCommand(_COMMAND_TARGET_WAMIN, min, minAtClick);
}

void TimeCtrl_cls::SetWAMaxCommand(int max, int maxAtClick){
    if (maxAtClick == _TIME_CTRL_CURRENT) maxAtClick = mCWAMax.mBody;
    SetValueCommand(_COMMAND_TARGET_WAMAX, max, maxAtClick);
}

void TimeCtrl_cls::SetAAMinCommand(int min, int minAtClick){
    if (minAtClick == _TIME_CTRL_CURRENT) minAtClick = mCAAMin.mBody;
    SetValueCommand(_COMMAND_TARGET_AAMIN, min, minAtClick);
}

void TimeCtrl_cls::SetAAMaxCommand(int max, int maxAtClick){
    if (maxAtClick == _TIME_CTRL_CURRENT) maxAtClick = mCAAMax.mBody;
    SetValueCommand(_COMMAND_TARGET_AAMAX, max, maxAtClick);
}

void TimeCtrl_cls::SetRepeatCommand(int repeat, int repeatPrev){
    SetValueCommand(_COMMAND_TARGET_REPEAT, repeat, repeatPrev);
}

void TimeCtrl_cls::SetAnimStepCommand(int animStep, int animStepPrev){
    GCUndoList.OpenBlock();
    SetValueCommand(_COMMAND_TARGET_ANIMSTEP, animStep, animStepPrev);
    if (mFrame != RoundFrame(mFrame))
	SetValueCommand(_COMMAND_TARGET_FRAME, RoundFrame(mFrame), mFrame);
    GCUndoList.CloseBlock();
}

void TimeCtrl_cls::SetAnimInterpCommand(int animInterp, int animInterpPrev){
    SetValueCommand(_COMMAND_TARGET_ANIMINTERP, animInterp, animInterpPrev);
}

void TimeCtrl_cls::SetPrefetchCommand(int prefetch, int prefetchPrev){
    SetValueCommand(_COMMAND_TARGET_PREFETCH, prefetch, prefetchPrev);
}

void TimeCtrl_cls::SetMaxFrameCommand(int maxFrame){
    SetValueCommand(_COMMAND_TARGET_MAXFRAME, maxFrame, mCMaxFrame.mBody);
}

void TimeCtrl_cls::SetFrameNative(int frame, int updateBit){
    mFrame = frame;
    Parameters_cls::msCurrentDoubleIndex = GetDoubleIndex();
    CommandItem_cls::UpdateBit(updateBit);
}

void TimeCtrl_cls::SetInterpStepNative(int step, int updateBit){
    mInterpStep = step;
    CommandItem_cls::UpdateBit(updateBit);
}

void TimeCtrl_cls::SetWAMinNative(int min){
    mCWAMin.mBody = min;
    wxCommandEvent com(wxEVT_COMMAND_MENU_SELECTED, menuEditRefreshTL);
    mpCMainLink->GetPTopLevelWindow()->AddPendingEvent(com);
}

void TimeCtrl_cls::SetWAMaxNative(int max){
    mCWAMax.mBody = max;
    wxCommandEvent com(wxEVT_COMMAND_MENU_SELECTED, menuEditRefreshTL);
    mpCMainLink->GetPTopLevelWindow()->AddPendingEvent(com);
}

void TimeCtrl_cls::SetAAMinNative(int min){
    mCAAMin.mBody = min;
    wxCommandEvent com(wxEVT_COMMAND_MENU_SELECTED, menuEditRefreshTL);
    mpCMainLink->GetPTopLevelWindow()->AddPendingEvent(com);
}

void TimeCtrl_cls::SetAAMaxNative(int max){
    mCAAMax.mBody = max;
    wxCommandEvent com(wxEVT_COMMAND_MENU_SELECTED, menuEditRefreshTL);
    mpCMainLink->GetPTopLevelWindow()->AddPendingEvent(com);
}

void TimeCtrl_cls::SetRepeatNative(int repeat){
    mCbRepeat.mbBody = repeat;
    wxCommandEvent com(wxEVT_COMMAND_MENU_SELECTED, menuEditRefreshTL);
    mpCMainLink->GetPTopLevelWindow()->AddPendingEvent(com);
}

void TimeCtrl_cls::SetAnimStepNative(int animStep){
    mCAnimStep.mBody = animStep;
    wxCommandEvent com(wxEVT_COMMAND_MENU_SELECTED, menuEditRefreshTL);
    mpCMainLink->GetPTopLevelWindow()->AddPendingEvent(com);
}

void TimeCtrl_cls::SetAnimInterpNative(int animInterp){
    mCAnimInterp.mBody = animInterp;
    Parameters_cls::SetFrameNumber(mCMaxFrame.mBody, mCAnimInterp.mBody);
    wxCommandEvent com(wxEVT_COMMAND_MENU_SELECTED, menuEditRefreshTL);
    mpCMainLink->GetPTopLevelWindow()->AddPendingEvent(com);
}

void TimeCtrl_cls::SetPrefetchNative(int prefetch){
    mPrefetchRange = prefetch;
    wxCommandEvent com(wxEVT_COMMAND_MENU_SELECTED, menuEditRefreshTL);
    mpCMainLink->GetPTopLevelWindow()->AddPendingEvent(com);
}

void TimeCtrl_cls::SetMaxFrameNative(int maxFrame){
    mCMaxFrame.mBody = maxFrame;
    CheckAfterMaxFrameChange();
    Parameters_cls::SetFrameNumber(maxFrame, mCAnimInterp.mBody);
    RefreshGUI();
}

void TimeCtrl_cls::
CheckAfterMaxFrameChange(){
    static unsigned int prevMaxFrame = 99;
    if (mCMaxFrame.mBody != prevMaxFrame){
	if (prevMaxFrame == mCAAMax.mBody)
	    mCAAMax.mBody = mCMaxFrame.mBody;
	if (prevMaxFrame == mCWAMax.mBody)
	    mCWAMax.mBody = mCMaxFrame.mBody;
	if (mCAAMax.mBody > mCMaxFrame.mBody)
	    mCAAMax.mBody = mCMaxFrame.mBody;
	if (mCWAMax.mBody > mCMaxFrame.mBody)
	    mCWAMax.mBody = mCMaxFrame.mBody;
	prevMaxFrame = mCMaxFrame.mBody;
    }
    
    mCustomRegionMin = 0; 
    mCustomRegionMax = mCMaxFrame.mBody;
    
    if (mFrame > mCMaxFrame.mBody) mFrame = 0;
    
    //TimeLinePanel_cls::msVBSnippingTag.resize(mCMaxFrame.mBody);
}

void TimeCtrl_cls::
RefreshGUI(){
    mpCGUI->Refresh(FALSE);
    mpCGUIAA->Refresh(FALSE);
    mpCGUIAtoi->Refresh(FALSE);
    mpCGUITics->Refresh(FALSE);
    mpCGUIPlay->UpdateDisplayedValues(); 

}

void TimeCtrl_cls::
SetWorkingArea(int maxIndex){
    mCWAMin.mBody = mCAAMin.mBody = 0;
    mCWAMax.mBody = mCAAMax.mBody = maxIndex;
}

void TimeCtrl_cls::
CheckWorkingArea(){
    if (mCWAMax.mBody > mCMaxFrame.mBody)
	mCWAMax.mBody = mCMaxFrame.mBody;
    if (mCWAMin.mBody > mCWAMax.mBody){
	unsigned int max = mCWAMax.mBody;
	mCWAMax.mBody = mCWAMin.mBody;
	mCWAMin.mBody = max;
    }

    if (mCAAMax.mBody > mCMaxFrame.mBody)
	mCAAMax.mBody = mCMaxFrame.mBody;
    if (mCAAMin.mBody > mCAAMax.mBody){
	unsigned int max = mCAAMax.mBody;
	mCAAMax.mBody = mCAAMin.mBody;
	mCAAMin.mBody = max;
    }
    
    if (mFrame < mCAAMin.mBody) 
	mFrame = mCAAMin.mBody;
    if (mFrame > mCAAMax.mBody)
	mFrame = mCAAMax.mBody;

    mFrame = CalcNextFrame(_TIME_CTRL_CURRENT, 0);
}

void TimeCtrl_cls::
RemapParameterKeys(double rate){
    /*
    Parameters_cls::SetFrameNumber(mCMaxFrame.mBody, mCAnimStep.mBody);
    GCUndoList.OpenBlock();
    mspCDF3SeqCtrlLink->mpCGUI->mpCPanelList->RemapKeys(rate);
    GCUndoList.CloseBlock();
    */
}

void TimeCtrl_cls::
SetBUseTimeRemap(bool bUse){
    /*
    mCbUseTimeRemap.mbBody = bUse;
    if (mCbUseTimeRemap.mbBody == TRUE){
	// \[Xłlast = (double)mMaxIndexǖȂH
	double init = 0, last = (double)mCMaxFrame.mBody;//, zero = 0;

	mCTimeRemap.mCbSelectable.mbBody = TRUE;
	mspCDF3SeqCtrlLink->mpCGUI->ShowItem(wxT("[Time Remap]"));
	
	GCUndoList.OpenBlock();
	mCTimeRemap.InitSet(&init);

	KeyFrame_cls CKey(1);//, CKeySlave(0);
	CKey.Set(GetDoubleIndex(last), _PARAM_HERMITE_AUTO, &last);
	mCTimeRemap.SetKey(CKey);
	
	GCUndoList.CloseBlock();
    }
    else{
	mCTimeRemap.mCbSelectable.mbBody = FALSE;
	mspCDF3SeqCtrlLink->mpCGUI->ShowItem(wxT("[Time Remap]"), FALSE);
    }
    mspCDF3SeqCtrlLink->mpCGUI->UpdateItemsVisivility();
    UpdateDF3SeqGUI(TRUE);
    */
}

int  TimeCtrl_cls::
GetExpectedPrefetchMax(){
    int averageMB = mpCMainLink->GetAverageMemoryMB();
    if (averageMB == -1) return -1;
    if (averageMB == 0) averageMB = 1;
    int memoryLimit = mpCMainLink->GetMemoryLimitMB();
    return memoryLimit / averageMB;
}

unsigned int TimeCtrl_cls::
GetRemappedIndex(int frame){
    if (frame == _TIME_CTRL_CURRENT) frame = mFrame;
    int remappedIndex = frame;
    if (mCbUseTimeRemap.mbBody == TRUE)
	remappedIndex = frame;
	/*
	remappedIndex = (int)(*mCTimeRemap.GetPInterpolatedValue
			      (GetDoubleIndex(frame)) + 1e-5);
	*/
    if (remappedIndex < 0) remappedIndex = 0;
    return remappedIndex;
}

double TimeCtrl_cls::
GetRemappedIndexDouble(int frame, int step){
    double remappedIndex;
    if (frame == _TIME_CTRL_CURRENT) frame = mFrame;
    if (step  == _TIME_CTRL_CURRENT) step = mInterpStep;
    if (mCbUseTimeRemap.mbBody == FALSE)
	remappedIndex = frame;
    else
	remappedIndex = frame;
	//remappedIndex = (*mCTimeRemap.GetPInterpolatedValue(GetDoubleIndex(frame)));
    remappedIndex += (double)step / mCAnimInterp.mBody;
    if (remappedIndex < 0) remappedIndex = 0;
    return remappedIndex;
}

double TimeCtrl_cls::
GetRemappedInterpStepDouble(int frame, int step){
    double remappedStep;
    if (frame == _TIME_CTRL_CURRENT) frame = mFrame;
    if (step  == _TIME_CTRL_CURRENT) step = mInterpStep;
    if (mCbUseTimeRemap.mbBody == FALSE)
	remappedStep = (double)step / mCAnimInterp.mBody;
    else
	remappedStep = (double)step / mCAnimInterp.mBody;
    return remappedStep;
}

int TimeCtrl_cls::
GetInterval(int frameNumber){
    int interval = 1;
    if (frameNumber > 1000)     interval = 10;
    else if (frameNumber > 500) interval = 5;
    else if (frameNumber > 200) interval = 2;
    return interval;
}

double TimeCtrl_cls::
GetDoubleIndex(int frame){
    if (mCMaxFrame.mBody == 0) return 0;
    if (frame == _TIME_CTRL_CURRENT)
	return ((double)mFrame + (double)mInterpStep/mCAnimInterp.mBody) / (mCMaxFrame.mBody);
    return (double)frame / (mCMaxFrame.mBody);
}

double TimeCtrl_cls::
GetDoubleIndexWAMin(){
    if (mCMaxFrame.mBody == 0) return 0;
    return (double)mCWAMin.mBody / mCMaxFrame.mBody;
}

double TimeCtrl_cls::
GetDoubleIndexWAMax(){
    if (mCMaxFrame.mBody == 0) return 0;
    return (double)mCWAMax.mBody / mCMaxFrame.mBody;
}

double TimeCtrl_cls::
GetDoubleIndexAAMin(){
    if (mCMaxFrame.mBody == 0) return 0;
    return (double)mCAAMin.mBody / mCMaxFrame.mBody;
}

double TimeCtrl_cls::
GetDoubleIndexAAMax(){
    if (mCMaxFrame.mBody == 0) return 0;
    return (double)mCAAMax.mBody / mCMaxFrame.mBody;
}

int TimeCtrl_cls::
GetOffsetForImgSeq(){
    return 
	(mFrame - mCWAMin.mBody) / mCAnimStep.mBody * mCAnimInterp.mBody
	+ mInterpStep;
}

int TimeCtrl_cls::
RoundFrame(int frame){
    int roundFrame = frame + mCAnimStep.mBody / 2;
    roundFrame /= mCAnimStep.mBody;
    roundFrame *= mCAnimStep.mBody;
    while (roundFrame > (int)mCMaxFrame.mBody)
	roundFrame -= mCAnimStep.mBody;
    return roundFrame;
}

void TimeCtrl_cls::
AdjustFrameByRange(int *pFrame, int *pInterpStep,
		   int increment, 
		   bool bForceWA, bool bCheckRepeat){

    if (*pFrame > (int)mCMaxFrame.mBody){
	if (increment < 0) 
	    if (bCheckRepeat && mCbRepeat.mbBody)
		*pFrame = mCMaxFrame.mBody;
	    else
		*pFrame = 0;
	else
	    if (bCheckRepeat && mCbRepeat.mbBody)
		*pFrame = 0;
	    else
		*pFrame = mCMaxFrame.mBody;
    }
    
    if (bForceWA == TRUE){
	if (*pFrame >= (int)mCWAMax.mBody){
	    *pFrame = mCWAMax.mBody; 
	    *pInterpStep = 0;
	}
	
	if (*pFrame < (int)mCWAMin.mBody) {
	    *pFrame = mCWAMin.mBody; 
	    *pInterpStep = 0;
	}
    }

    // ǂ?
    *pFrame /= mCAnimStep.mBody;
    *pFrame *= mCAnimStep.mBody;

    //return frame;
}

unsigned int TimeCtrl_cls::
CalcNextFrame(int frame, int increment, bool bForceWA, bool bCheckRepeat){
    if (frame == _TIME_CTRL_CURRENT) frame = mFrame;
    int nextFrame = frame + mCAnimStep.mBody * increment;
    int nextStep = mInterpStep;

    //nextStep = AdjustFrameByRange(nextStep, increment, bForceWA, bCheckRepeat);
    AdjustFrameByRange(&nextFrame, &nextStep, increment, bForceWA, bCheckRepeat);
    
    return nextFrame;
}

int TimeCtrl_cls::
CalcNextStep(int step, int dstep){
    if (step == _TIME_CTRL_CURRENT) step = mInterpStep;
    int nextStep = step + dstep;
    return nextStep;
}

void TimeCtrl_cls::
CalcNextFrameFromStep(int *pFrame, int *pStep, bool bForceWA, bool bCheckRepeat){
    int frame = *pFrame; int step = *pStep;
    int increment = 0;
    if (frame == _TIME_CTRL_CURRENT) frame = mFrame;
    if (step == -1) {
	increment = -1;
	frame -= mCAnimStep.mBody; 
	step = mCAnimInterp.mBody - 1; 
    }
    if (step >= mCAnimInterp.mBody){
	increment = +1;
	frame += mCAnimStep.mBody;
	step = 0;
    }
    //frame = 
    AdjustFrameByRange(&frame, &step, increment, bForceWA, bCheckRepeat);
    
    *pFrame = frame;
    *pStep = step;
    
    //return (increment != 0);
}

unsigned int TimeCtrl_cls::
GetIndexFromDouble(double doubleIndex){
    if (mCMaxFrame.mBody == 0) return 0;
    return (doubleIndex * mCMaxFrame.mBody) + 0.1;
}

bool TimeCtrl_cls::
IsTheFrameReady(int frame, bool bUseInterpolate){
    double indexDouble = GetRemappedIndexDouble(frame);
    int index = (int)indexDouble;
    double indexmod = indexDouble - index;
    if (bUseInterpolate == TRUE) indexmod = 1;
    return mpCMainLink->IsTheIndexReady(index, indexmod);

}

bool TimeCtrl_cls::
IsTheFrameLoaded(int frame){
    int index = GetRemappedIndex(frame);
    return mpCMainLink->IsTheIndexLoaded(index);
}

bool TimeCtrl_cls::
IsWorkingArea(int frame){
    if (frame < (int)mCWAMin.mBody) return FALSE;
    if (frame > (int)mCWAMax.mBody) return FALSE;
    if (frame > (int)mCMaxFrame.mBody) return FALSE;

    return TRUE;
}

bool TimeCtrl_cls::
IsPlayConditionSatisfied(int frame){
    // FIX_ME !!
    return IsWorkingArea(frame);
}

void TimeCtrl_cls::
SetIndexWithinWA(){
    if (mFrame > mCWAMax.mBody) mFrame = mCWAMax.mBody;
    if (mFrame < mCWAMin.mBody) mFrame = mCWAMin.mBody;
}

bool TimeCtrl_cls::
OpenCustomRegionDialog(wxWindow *pC){
    /*
    MultiNumericalInputs_cls *pCurr;

    pCurr = new MultiNumericalInputs_cls (&pCurr, pC, wxID_ANY, wxT("Render Area"));
    pCurr->SetDefaultButtonType(_MN_OK_CANCEL_DEFAULT);
    pCurr->mTextWidth = 80;
    int boxWidth = 2;
    pCurr->CombinePointer(_GP_BOXSIZERHORIZONTAL, &boxWidth, 
			  _("init/fin"));
    pCurr->CombinePointer(_GP_UINT_SPIN, &mCustomRegionMin);
    pCurr->SetRange(0, mCMaxFrame.mBody);
    pCurr->CombinePointer(_GP_UINT_SPIN, &mCustomRegionMax);
    pCurr->SetRange(0, mCMaxFrame.mBody);

    bool bRet = (pCurr->FinishSettingDLG(TRUE, _MN_DLG_MODAL, _MN_DLG_MAKEGEOM)
		 == wxID_OK);
    
    delete pCurr;
    */
    
    return TRUE;
}

void TimeCtrl_cls::
ZoomIn(){
    int widthR = mCAAMax.mBody - mFrame;
    int widthL = mFrame - mCAAMin.mBody;
    int center = mFrame;
    mCAAMax.mBody = center + widthR / 2;
    mCAAMin.mBody = center - widthL / 2;
    CheckWorkingArea();
    //UpdateDF3SeqGUI();
}

void TimeCtrl_cls::
ZoomOut(){
    int widthR = mCAAMax.mBody - mFrame; if (widthR == 0) widthR =1;
    int widthL = mFrame - mCAAMin.mBody; if (widthL == 0) widthL =1;
    int center = mFrame;
    mCAAMax.mBody = center + widthR * 2;
    if ((center - widthL * 2) < 0)
	mCAAMin.mBody = 0;
    else
	mCAAMin.mBody = center - widthL * 2;
    CheckWorkingArea();
}

void TimeCtrl_cls::
EmptySnippingTag(){
    if (mVBSnippingTag.size() != mCMaxFrame.mBody + 1)
	mVBSnippingTag.resize(mCMaxFrame.mBody + 1);
    for (unsigned int i = 0; i < mVBSnippingTag.size(); i++)
	mVBSnippingTag[i] = FALSE;
}

void TimeCtrl_cls::
UpdateSnippingTag(){
    EmptySnippingTag();
    mpCMainLink->mpCPDataSeqCtrl->ClearSnipping();
    mpCParamGUICtrlLink->UpdateSnippingTag(&mVBSnippingTag);
}

int TimeCtrl_cls::
GetSnipNext(){
    unsigned int i = mFrame + 1;
    int nextStep = mFrame;
    for (; i <= mCMaxFrame.mBody; i++)
	if (mVBSnippingTag[i] == TRUE) {
	    nextStep = i;
	    break;
	}
    if (mFrame == nextStep) nextStep = mCMaxFrame.mBody;

    return nextStep;
}

int TimeCtrl_cls::
GetSnipPrev(){
    int i = mFrame - 1;
    int prevStep = mFrame;
    for (; i >= 0; i--)
	if (mVBSnippingTag[i] == TRUE) {
	    prevStep = i; 
	    break;
	}
    if (mFrame == prevStep) prevStep = 0;
    
    return prevStep;
}

void TimeCtrl_cls::
SetWAInHere(){
    mCWAMin.mBody = mFrame;
    CheckWorkingArea();
}

void TimeCtrl_cls::
SetWAOutHere(){
    mCWAMax.mBody = mFrame;
    CheckWorkingArea();
}

void TimeCtrl_cls::
SetIndex(unsigned int index){
    mFrame = index;
    mFrame = CalcNextFrame(_TIME_CTRL_CURRENT, 0);
    CheckWorkingArea();
}

void TimeCtrl_cls::
StartPlayCommand(int direction){
    if (mbPlayTex == FALSE){
	// ĐJn̂݋L^Ƃ
	GCUndoList.SetNewCommand(new CommandStartAnim_cls(this, direction),
				 new CommandStopAnim_cls(this, mFrame));
	GCUndoList.ExecCurrent();
	mpCGUIPlay->UpdateButtonColor();
    }
    else{
	mCStopWatchA.Pause();
	ProcStopPlaying();
    }
}

void TimeCtrl_cls::
StartPlay(int direction){
    mbPlayTex = TRUE;
    mPlayDirection = direction;
    int nextStep = CalcNextFrame(_TIME_CTRL_CURRENT, direction);
    if (IsPlayConditionSatisfied(nextStep) == FALSE){
	MoveFrameCommand(direction);
	mpCGUIPlay->RefreshGLPanels();
	return;
    }
    mCStopWatchA.Start();
}

void TimeCtrl_cls::
StopAnimationAt(int frame){
    mbPlayTex = FALSE;
    SetFrameNative(frame);
}

void TimeCtrl_cls::
ProcStopPlaying(){
    mbPlayTex = FALSE;

    // ̈[ɂł̏IJԂ̏ꍇ
    if (mCbRepeat.mbBody == TRUE && 
	((mPlayDirection > 0 && mFrame == mCWAMax.mBody) ||
	 (mPlayDirection < 0 && mFrame == mCWAMin.mBody))){
	mbPlayTex = TRUE;
	mpCMainLink->mpCFrame->mbRepeatPlay = TRUE;
	if (mPlayDirection > 0) mFrame = mCWAMin.mBody;
	else                    mFrame = mCWAMax.mBody;
    }

    mpCGUIPlay->UpdateButtonColor();
	
    // Aԕۑ̃{^̐FXV邽߂ɂx
    wxCommandEvent com(wxEVT_COMMAND_MENU_SELECTED, menuItemPlayStop);
    mpCMainLink->GetPTopLevelWindow()->AddPendingEvent(com);
}

bool TimeCtrl_cls::
ProcPlaying(){
    int nextStep = CalcNextStep(_TIME_CTRL_CURRENT, mPlayDirection);
    int nextFrame = mFrame;
    CalcNextFrameFromStep(&nextFrame, &nextStep);
    
    if (nextFrame == mFrame && nextStep == mInterpStep){
	ProcStopPlaying();
	return FALSE;
    }

    while(TRUE){
	if ((mCStopWatchWait.Time() - mClockLastDraw) >= mWaitDuration) break;
	wxMilliSleep(3);
    }
    
    mClockLastDraw = mCStopWatchWait.Time();
    
    SetFrameNative(nextFrame);
    SetInterpStepNative(nextStep);
    mpCGUIPlay->RefreshGLPanels();

    if (mFrame == mCMaxFrame.mBody) {
	ProcStopPlaying();
	return FALSE;
    }

    return TRUE;
}

// -----

void TimeCtrl_cls::
ReadStrLine(/*std::wistream &istream,*/ wxString strLine){
    if (mCConfListTime.ReadStrLineAll(strLine) == TRUE){
	Parameters_cls::SetFrameNumber(mCMaxFrame.mBody, mCAnimInterp.mBody);
	mpCGUIPlay->UpdateDisplayedValues();
    }
    /*
    std::wstring::size_type loc = strLine.find(L"--TimeRemap");
    if (loc != std::wstring::npos){
	std::wstring strLine;
	std::getline(istream, strLine);
	mCTimeRemap.ReadStrLine(istream, strLine);
    }
    */
}

wxString TimeCtrl_cls::
Serialize(){
    wxString sOut;
    sOut << mCConfListTime.SerializeAll();
    /*
    if (mCbUseTimeRemap.mbBody == TRUE){
	sOut << wxT("--TimeRemap\n");
	sOut << mCTimeRemap.Serialize();
    }
    */
    return sOut;
}

//DF3SeqGUI_cls* TimeCtrl_cls::mspCDF3SeqGUI = NULL;
//DF3SeqCtrl_cls* TimeCtrl_cls::mspCDF3SeqCtrlLink = NULL;
