
#ifndef _UNDO_MANAGER_H_
#define _UNDO_MANAGER_H_

#include <vector>

#include <stdio.h>

#define _COMMAND_UNDO true
#define _COMMAND_REDO false

#define _COMMAND_TYPE_NORMAL 0
#define _COMMAND_TYPE_BLOCK_OPEN 1
#define _COMMAND_TYPE_BLOCK_CLOSE 2

#define _COMMAND_UPDATE_DISPLAY_ATTRIB 1
#define _COMMAND_UPDATE_DISPLAY_GL     2
#define _COMMAND_UPDATE_DISPLAY_TL     4
#define _COMMAND_UPDATE_DISPLAY_IMGSEQ 8
#define _COMMAND_UPDATE_DISPLAY_CAMCONF 16
#define _COMMAND_UPDATE_DISPLAY_COLOR 32
#define _COMMAND_UPDATE_DISPLAY_EXPORTER 64
#define _COMMAND_UPDATE_DISPLAY_DRAWOPTIONS 128
#define _COMMAND_UPDATE_DISPLAY_LIGHT 256
#define _COMMAND_UPDATE_DISPLAY_EXPOSURE 512

class CommandItem_cls; 

#ifdef _DEBUG
#include <stdlib.h>
#include <crtdbg.h>
#define new new(_NORMAL_BLOCK, __FILE__, __LINE__)
#endif

class CommandItem_cls {
public:
    CommandItem_cls();
    virtual ~CommandItem_cls();
    
    virtual void Execute(bool bFlag = _COMMAND_REDO) = 0;
    
    void UpdateBit();
    static void UpdateBit(int updateBit);
    int mCommandType;
    int mUpdateBit;
};

// ----------- UNDO BLOCK -------------

class ComBlockOpen : public CommandItem_cls {
public:
    ComBlockOpen();
    void Execute(bool bFlag = _COMMAND_REDO);
};

class ComBlockClose : public CommandItem_cls {
public:
    ComBlockClose();
    void Execute(bool bFlag = _COMMAND_REDO);
};

// -----------------------------------

class UndoManager_cls {
 public:
    UndoManager_cls();
    ~UndoManager_cls();

    std::vector < CommandItem_cls* > mVPCRedoList;
    std::vector < CommandItem_cls* > mVPCUndoList;
    unsigned int mCurrentIndex;

    int  mBlockDepth;
    int  mUpdateAccumulateBit;
    
    void Clear();
    void RemoveItem(unsigned int i);
    void OpenBlock();
    void CloseBlock();
    void SendEventToFrame(int comID);
    void ExecUpdate();

    // ̃R}hǉĂ䂭ɂ́AUsetpushĂ䂭 
    void SetNewCommand (CommandItem_cls* pComRedo, CommandItem_cls* pComUndo);
    void PushNewCommand(CommandItem_cls* pComRedo, CommandItem_cls* pComUndo);

    int  GetRedoCommandType(int level = 1);
    int  GetUndoCommandType(int level = 1); 
    void SkipCurrent();
    void ExecCurrent();

    void UndoCurrent();
};

#endif
