// -*- c -*- 
// "clsGraphPanel.h"

#include "clsGraphPanel.h"
#include "../PARAMETER/clsParamGroup.h"

#include "../WIDGETCTRL/EX/clsColourDialogEX.h"
extern wxColourDialogEX* GpCClrDialog;

#include "../../wxWidgetEnum.h"
#include "../../wxWidgetEnumMenu.h"

extern wxFrame  *GpCMainFrameCLS;

BEGIN_EVENT_TABLE(GraphPanel_cls, wxPanel)
EVT_PAINT       (GraphPanel_cls::OnPaint)
EVT_KEY_DOWN    (GraphPanel_cls::OnKeyDown)
EVT_MOUSE_EVENTS(GraphPanel_cls::OnMouse)
EVT_MENU        (GraphMenuInsertKey, GraphPanel_cls::OnInsertKey)
EVT_MENU        (GraphMenuDeleteKey, GraphPanel_cls::OnDeleteKey)
EVT_MENU        (GraphMenuColor,     GraphPanel_cls::OnColor)
END_EVENT_TABLE()

#define X 0
#define Y 1

#define LOOP_RGB for (int rgb = 0; rgb < 3; rgb++)

GraphPanel_cls::
GraphPanel_cls(wxWindow *parent, wxWindowID id, 
	       wxPoint pos, wxSize size, bool bColor) :
wxPanel(parent, id, pos, size, wxFULL_REPAINT_ON_RESIZE)
{
    mViewMinY = 0;
    mViewMaxY = 1;
    mValueMinY = 0;
    mValueMaxY = 1;
    mpCParamGroupLinked = NULL;
    mColorType = _GRAPH_RGBA;

    mpCPopupMenu = new wxMenu;
    mpCPopupMenu->Append(GraphMenuInsertKey, _("Insert Key"));
    mpCPopupMenu->Append(GraphMenuDeleteKey, _("Delete Key"));

    if (bColor == TRUE)
	mpCPopupMenu->Append(GraphMenuColor, _("Select Color"));
}

GraphPanel_cls::
~GraphPanel_cls(){
    delete mpCPopupMenu;
}

void GraphPanel_cls::
SetParamGroup(ParamGroup_cls *pCParamGroup){
    mpCParamGroupLinked = pCParamGroup;
}

void GraphPanel_cls::
ShowMenu(){
#ifdef WIN32
    PopupMenu(mpCPopupMenu, mCoordX, mCoordY);
#else
    PopupMenu(mpCPopupMenu);
#endif
    if (HasCapture() == TRUE) ReleaseMouse();
}

int GraphPanel_cls::
GetCoordX(double valueX){
    return valueX * (mWidth - 2) + 1;
}

int GraphPanel_cls::
GetCoordY(double valueY){
    double doubleY = (valueY - mViewMinY) / (mViewMaxY - mViewMinY);
    return (mHeight - 2) - doubleY * (mHeight - 2) + 1;
}

double GraphPanel_cls::
GetDoubleX(int coordX){
    return ((double)coordX - 1) / (mWidth - 2);
}

double GraphPanel_cls::
GetDoubleY(int coordY){
    return 1.0 - ((double)coordY - 1) / (mHeight - 2);
}

void GraphPanel_cls::
SetPenColor(wxBufferedPaintDC &dc, int i){
    wxPen pen = wxPen(msColorRGBA[i]);
    if (i == mpCParamGroupLinked->mControlMode) pen.SetWidth(2);
    if (mColorType == _GRAPH_RGBA) dc.SetPen(pen);
}

void GraphPanel_cls::
SetBrushColor(wxBufferedPaintDC &dc, int i){
    if (mColorType == _GRAPH_RGBA) dc.SetBrush(wxBrush(msColorRGBA[i]));
}

void GraphPanel_cls::
DrawGraphOfParamGroup(wxBufferedPaintDC &dc){
    
    int dim = mpCParamGroupLinked->mCDimension.mBody;
    std::vector < std::vector < wxPoint > > VVPoint(dim);
    int xstep = 1;
    if (mWidth > 320) xstep = 2;
    if (mWidth > 640) xstep = 4;
    for (int x = 0; x < mWidth; x += xstep){
	double valueX = (double)x/mWidth;
	std::vector < double > VValue = 
	    (mpCParamGroupLinked->GetInterpolatedValue(valueX, 0));
	for (int i = 0; i < dim; i++)
	    VVPoint[i].push_back(wxPoint(x, GetCoordY(VValue[i])));
    }
    
    for (int i = 0; i < dim; i++){
	SetPenColor(dc, i);
	for (int x = 0; x < mWidth; x+=xstep)
	    dc.DrawLines(mWidth / xstep, VVPoint[i].data());
    }
    
    for (int i = 0; i < dim; i++){
	SetPenColor(dc, i);
	Parameters_cls* pCParam = &(mpCParamGroupLinked->mVCParam[i]);
	for (int key = 0; key < pCParam->GetKeyNumber(); key++){
	    int coordX = GetCoordX(pCParam->mVCKeys[key].mDoubleIndex);
	    int coordY = GetCoordY(pCParam->mVCKeys[key].mValue);
	    if (pCParam->mVCKeys[key].mbSelected == TRUE){
		SetBrushColor(dc, i);
		if (HasCapture() == TRUE)
		    dc.DrawRectangle(coordX-3 + (mCoordX - mCoordXAtClick),
				     coordY-3 + (mCoordY - mCoordYAtClick),
				     7, 7);
	    }
	    else
		dc.SetBrush(*wxTRANSPARENT_BRUSH);
	    dc.DrawRectangle(coordX-3, coordY-3, 7, 7);
	}
    }
}

void GraphPanel_cls::
OnPaint(wxPaintEvent &event){
    wxBufferedPaintDC dc(this);

    wxSize sz = GetClientSize();
    mWidth = sz.x;
    mHeight = sz.y;
    wxRect rectToDraw (0, 0, mWidth, mHeight);
    
    dc.SetPen(*wxBLACK_PEN);
    dc.SetBrush(wxBrush(*wxLIGHT_GREY));
    dc.DrawRectangle(rectToDraw);

    dc.SetPen(*wxGREY_PEN);
    dc.DrawLine(mCoordX, 0, mCoordX, mHeight);

    if (mpCParamGroupLinked != NULL) DrawGraphOfParamGroup(dc);

}

void GraphPanel_cls::
SetValueX(double valueX){
    mValueX = valueX;
    mCoordX = GetCoordX(mValueX);
}

void GraphPanel_cls::
OnMouse(wxMouseEvent &event){
    if (mpCParamGroupLinked == NULL) return;
    int x = event.GetX();
    int y = event.GetY();

    mCoordX = x; mCoordY = y;
    mValueX = GetDoubleX(x); 
    mValueY = GetDoubleY(y);

    bool bRedraw = FALSE;
    if (event.LeftDown() == TRUE)
	if (mpCParamGroupLinked->
	    SelectKeyByXY(mValueX, 0.03, mValueY, 0.03) == FALSE)
	    mpCParamGroupLinked->DeselectAll();

    if (event.ButtonDown(wxMOUSE_BTN_LEFT)){
	mCoordXAtClick = x; mCoordYAtClick = y;
	mValueXAtClick = mValueX; mValueYAtClick = mValueY;
	CaptureMouse();
	SetFocus();
    }
    
    if (event.ButtonDown(wxMOUSE_BTN_RIGHT)) ShowMenu();

    if (event.ButtonUp(wxMOUSE_BTN_LEFT)){
	if (HasCapture() == TRUE){
	    ReleaseMouse();
	    bRedraw |= mpCParamGroupLinked->
		MoveSelectedKeyByXY(mValueX - mValueXAtClick,
				    mValueY - mValueYAtClick,
				    0.01, 0, 1, mValueMinY, mValueMaxY);
	    mpCParamGroupLinked->DeselectAll();
	}
    }
    
    if (bRedraw == FALSE) Refresh(FALSE);
    
    /*
    if (bRefresh == TRUE) Refresh(FALSE);
    if (bRedraw == TRUE) {
	wxPaintEvent com;
	GetParent()->AddPendingEvent(com);
    }
    */

}

void GraphPanel_cls::
OnInsertKey(wxCommandEvent& event){
    mpCParamGroupLinked->InsertKey(_PARAM_ALL, mValueX);
}

void GraphPanel_cls::
OnDeleteKey(wxCommandEvent& event){
    mpCParamGroupLinked->SelectKeyByXY(mValueX, 0.03, mValueY, 0.03);
    mpCParamGroupLinked->DeleteSelectedKey();
}

void GraphPanel_cls::
OnColor(wxCommandEvent& event){
    std::vector < double > VValue = 
	mpCParamGroupLinked->GetInterpolatedValue(mValueX, 0);
    float clr[3] = {VValue[0], VValue[1], VValue[2]};
    GpCClrDialog->SetColourFloat(clr);
    Freeze();
    if (GpCClrDialog->ShowModal() == wxID_OK){
	double newclr[4] = {0, 0, 0, 1};
	LOOP_RGB newclr[rgb] = GpCClrDialog->GetPColorFloat()[rgb];
	mpCParamGroupLinked->SetKeyFrame(newclr, TRUE, mValueX);
    }
    Thaw();
}

void GraphPanel_cls::
SetMode(int mode){
    if (mpCParamGroupLinked->mControlMode == mode)
	mpCParamGroupLinked->mControlMode = -1;
    else
	mpCParamGroupLinked->mControlMode = mode;
}

void GraphPanel_cls::
OnKeyDown(wxKeyEvent& event){
    if (event.GetModifiers() == wxMOD_NONE){ // _FIX_ME!! F̂Ƃ
	if (event.GetKeyCode() == 'R') SetMode(0);
	else if (event.GetKeyCode() == 'G') SetMode(1);
	else if (event.GetKeyCode() == 'B') SetMode(2);
	else if (event.GetKeyCode() == 'A') SetMode(3);
	else SetMode(-1);
	Refresh(FALSE);
    }
    if (event.GetModifiers() == wxMOD_CONTROL)
	switch(event.GetKeyCode()){
	case 'Z': {
	    wxCommandEvent com(wxEVT_COMMAND_MENU_SELECTED, menuEditUndo);
	    GpCMainFrameCLS->AddPendingEvent(com);
	    break; }
	case 'Y': {
	    wxCommandEvent com(wxEVT_COMMAND_MENU_SELECTED, menuEditRedo);
	    GpCMainFrameCLS->AddPendingEvent(com);
	    break; }
	}
    event.Skip();
}

wxColour GraphPanel_cls::msColorRGBA[4] = {
    wxColour(255, 0, 0), wxColour(0, 255, 0), wxColour(0, 0, 255), 
    wxColour(255,255,255) };
