// -*- c++ -*- 
// (setq c-basic-offset 4)
// "clsMultiInputPanel.h"

#include "clsMultiInputPanel.h"

#include "clsHookCtrl.h"
#include "EX/clsCheckBoxEX.h"
#include "EX/clsButtonEX.h"
#include "EX/clsStaticTextEX.h"

#include "../../wxWidgetEnumMenu.h"

#include <wx/valgen.h>

extern int GShortcutModifier;
extern int GShortcutKeycode;
extern wxFrame  *GpCMainFrameCLS;

#define wxComEH wxCommandEventHandler
#define wxMouseEH wxMouseEventHandler

MultiInputBase_cls::
MultiInputBase_cls(void **ppToThis, wxWindow *pCParent) {
    mType = _MN_PANEL;
    mppToThis = ppToThis;
    mpCParent = pCParent;
    mTextWidth = 40;
    mCheckBoxWidth = -1;
    mItemWidth = 80;
    mComboWidth = 90;
    mMinimumWidth = 250;
    mUpdateBit = 0;
    mbUndoCommand = TRUE;
    mSizerIndex = mSizerCounter = 0;
    mpBoxSizerSrc = NULL;
    mpCHookCtrl = new HookCtrl_cls(ppToThis);
}

MultiInputBase_cls::
~MultiInputBase_cls(){
    mppToThis = NULL;
    delete mpCHookCtrl;
}

InputItem_cls& MultiInputBase_cls::
GetLastItem(){
    return mVCItemList[mVCItemList.size() - 1];
}

void MultiInputBase_cls::
SetColorForXYZ(unsigned int index){
    mVCItemList[index+0].GetPCwxWindow()->
	SetBackgroundColour(wxColour(255, 244, 244));
    mVCItemList[index+1].GetPCwxWindow()->
	SetBackgroundColour(wxColour(244, 255, 244));
    mVCItemList[index+2].GetPCwxWindow()->
	SetBackgroundColour(wxColour(244, 244, 255));
}

void MultiInputBase_cls::
ProcHorizontalBoxSizer(int currIndex, int type, void *pValue){
    // TCU[ݒ肳ꂽn񕪂̏c
    if (mSizerCounter > 0){
	mVCItemList[currIndex].mbConnectSizer = TRUE;
	mVCItemList[currIndex].mSizerIndex = mSizerIndex;
	mSizerCounter--;
    }
    // TCU[ݒ肳ꂽ玟n͂Ɍт
    if (type == _ITEM_BOXSIZERHORIZONTAL){
	mSizerIndex = currIndex;
	mSizerCounter = *(int*)pValue;
    }
}

int MultiInputBase_cls::
CombinePointerButton(int id, wxString strLabel, int type, bool enabled){
    unsigned int currIndex = (unsigned int)mVCItemList.size();
    mVCItemList.push_back(InputItem_cls());
    mVCItemList[currIndex].CombinePointerButton(id, strLabel, type, enabled);
    mVCItemList[currIndex].SetParameter
	(mSizerCounter, mTextWidth, mCheckBoxWidth, mComboWidth, mItemWidth); 
    mVCItemList[currIndex].mUpdateBit = mUpdateBit;
    mVCItemList[currIndex].mbUndoCommand = mbUndoCommand;

    ProcHorizontalBoxSizer(currIndex, _ITEM_BUTTON, NULL);
    return currIndex;
}

int MultiInputBase_cls::
CombinePointer(int type, void *pValue, wxString strLabel, bool enable){

    // ACeXgɃp[^ݒ肵Ă䂭
    unsigned int currIndex = (unsigned int)mVCItemList.size();
    mVCItemList.push_back(InputItem_cls());
    mVCItemList[currIndex].CombinePointer(type, pValue, strLabel, enable);
    mVCItemList[currIndex].SetParameter
	(mSizerCounter, mTextWidth, mCheckBoxWidth, mComboWidth, mItemWidth);
    mVCItemList[currIndex].mUpdateBit = mUpdateBit;
    mVCItemList[currIndex].mbUndoCommand = mbUndoCommand;
    
    ProcHorizontalBoxSizer(currIndex, type, pValue);
    return currIndex;
}

int MultiInputBase_cls::
CombinePointerList(int type, void *pPointerVal, 
		   unsigned int listMax, wxString *pChoiceList, 
		   wxString strLabel, bool enable){
    int currIndex = CombinePointer(type, pPointerVal, strLabel, enable);
    mVCItemList[currIndex].CombineList(listMax, pChoiceList);
    return currIndex;
}

void MultiInputBase_cls::
SetRange(int index, double min, double max, int scale){
    if (index == _ITEM_LASTITEM)
	mVCItemList[mVCItemList.size() - 1].SetRange(min, max, scale);
    else
	mVCItemList[index].SetRange(min, max, scale);
}

void MultiInputBase_cls::
UpdateOtherItems(int index){
    for (int i = 0; i < (int)mVCItemList.size(); i++){
	if (i == index) continue;
	if (mVCItemList[i].mpValue == mVCItemList[index].mpValue)
	    mVCItemList[i].UpdateDisplay();
    }
}

void MultiInputBase_cls::
AssignBase(int index, bool bUseHookChange, bool bForceHookChange){
    
    for (int i = 0; i < (int)mVCItemList.size(); i++){
	if (index != _MN_ASSIGN_ALL) i = index;
	if (mVCItemList[i].mpValue == NULL) continue;

	if (mVCItemList[i].Assign() == TRUE || 
	    bForceHookChange){
	    mpCHookCtrl->CheckHook(_HOOK_CHANGE, &index, mVCItemList[i].mId,
				   bUseHookChange);
	    UpdateOtherItems(i);
	}
	
	if (index != _MN_ASSIGN_ALL) break;
    }
    
    if (index != _MN_ASSIGN_ALL)
	mpCHookCtrl->CheckHook(_HOOK_ASSIGN, &mVCItemList[index]);
}

void MultiInputBase_cls::
AssignBaseById(int id){
    for (unsigned int index = 0; index < mVCItemList.size(); index++)
	if (id == mVCItemList[index].mId)
	    AssignBase(index);
}

void MultiInputBase_cls::
ShowItem(unsigned int index, bool bShow){
    if (mVCItemList[index].mType == _ITEM_BOXSIZERHORIZONTAL){
	mVCItemList[index].Show(bShow);
	for (int i = index + 1; i < mVCItemList.size(); i++)
	    if (mVCItemList[i].mSizerIndex == index)
		mVCItemList[i].Show(bShow);
    }
    else
	mVCItemList[index].Show(bShow);
}

void MultiInputBase_cls::
HideItem(unsigned int index){
    ShowItem(index, FALSE);
}

void MultiInputBase_cls::
EnableItem(int index, bool bEnable){
    if (mVCItemList[index].mType == _ITEM_BOXSIZERHORIZONTAL){
	mVCItemList[index].Enable(bEnable);
	for (int i = index + 1; i < mVCItemList.size(); i++)
	    if (mVCItemList[i].mSizerIndex == index)
		mVCItemList[i].Enable(bEnable);
    }
    else
	mVCItemList[index].Enable(bEnable);
}

void MultiInputBase_cls::
OnCheckBoxBase(wxCommandEvent& event){
    bool bChange = FALSE;
    int id = event.GetId();
    for (unsigned int i = 0; i < mVCItemList.size(); i++)
	if ((mVCItemList[i].mType == _ITEM_BOOL) &&
	    mVCItemList[i].mId   == id){
	    int orgValue = mVCItemList[i].GetValueBool();
	    AssignBase(i);
	    int newValue = mVCItemList[i].GetValueBool();
	    if (orgValue != newValue) bChange = TRUE;
	}
    /*
    if (bChange) mpCHookCtrl->CheckHook(_HOOK_COMBOBOX, &event);
    else         mpCHookCtrl->CheckHook(_HOOK_COMBOBOX_CHANGE, &event);
    */
    
    event.Skip();
}

void MultiInputBase_cls::
OnComboBoxBase(wxCommandEvent& event){
    bool bChange = FALSE;
    int id = event.GetId();
    for (unsigned int i = 0; i < mVCItemList.size(); i++)
	if ((mVCItemList[i].mType == _ITEM_UINT_LIST ||
	     mVCItemList[i].mType == _ITEM_UINT_ICON) &&
	    mVCItemList[i].mId   == id){
	    int orgValue = mVCItemList[i].GetValueInt();
	    AssignBase(i);
	    int newValue = mVCItemList[i].GetValueInt();
	    if (orgValue != newValue) bChange = TRUE;
	}
    
    if (bChange) mpCHookCtrl->CheckHook(_HOOK_COMBOBOX, &event);
    else         mpCHookCtrl->CheckHook(_HOOK_COMBOBOX_CHANGE, &event);
    
    event.Skip();
}

void MultiInputBase_cls::
OnScrollBase(wxScrollEvent& event){
    int id = event.GetId();
    for (unsigned int i = 0; i < mVCItemList.size(); i++)
	if ((mVCItemList[i].mType == _ITEM_INTEGER_SLIDER ||
	     mVCItemList[i].mType == _ITEM_DOUBLE_SLIDER ||
	     mVCItemList[i].mType == _ITEM_FLOAT_SLIDER ||
	     mVCItemList[i].mType == _ITEM_DUALSLIDER)
	    && mVCItemList[i].mId   == id)
	    AssignBase(i, FALSE);

    event.Skip();
}

void MultiInputBase_cls::
OnScrollBaseChanged(wxScrollEvent& event){
    int id = event.GetId();
    for (unsigned int i = 0; i < mVCItemList.size(); i++)
	if ((mVCItemList[i].mType == _ITEM_INTEGER_SLIDER ||
	     mVCItemList[i].mType == _ITEM_DOUBLE_SLIDER ||
	     mVCItemList[i].mType == _ITEM_FLOAT_SLIDER ||
	     mVCItemList[i].mType == _ITEM_DUALSLIDER)
	    && mVCItemList[i].mId   == id)
	    AssignBase(i, TRUE, TRUE);
    event.Skip();
}

void MultiInputBase_cls::
MakeNewSizer(int index, wxBoxSizer *pBoxSizerSrc, wxWindow *pCMInputPanel){
    int id = mVCItemList[index].
	ConnectBoxSizer(pBoxSizerSrc, pCMInputPanel);
    if (mType == _MN_PANEL && id != -1)
	pCMInputPanel->Connect
	    (id, wxEVT_LEFT_DOWN,
	     wxMouseEventHandler(MultiInputPanel_cls::OnMouse));
}

void MultiInputBase_cls::
MakeGeometryBase(wxBoxSizer *pBoxSizerSrc, wxWindow *pCMInputPanel){
    
     for (unsigned int i = 0; i < mVCItemList.size(); i++){
	 
	 // ̃TCU[pӂB
	 if (mVCItemList[i].mType == _ITEM_BOXSIZERHORIZONTAL){
	     MakeNewSizer(i, pBoxSizerSrc, pCMInputPanel);
	     continue;
	 }
	 
	 wxBoxSizer *pBSCurrent = NULL;
	 if (mVCItemList[i].mbConnectSizer == TRUE){
	     // ̃TCU[Ɍт
	     pBSCurrent = (wxBoxSizer*)
		 mVCItemList[mVCItemList[i].mSizerIndex].mpCCtrl;
	 }
	 else{
	     pBSCurrent = mVCItemList[i].GetPNewSizer(pCMInputPanel);
	     if (mType == _MN_PANEL &&
		 mVCItemList[i].mpCTextLabel != NULL)
		 pCMInputPanel->Connect
		     (mVCItemList[i].mpCTextLabel->GetId(),
		      wxEVT_LEFT_DOWN,
		      wxMouseEH(MultiInputPanel_cls::OnMouse));
	 }

	 switch(mVCItemList[i].mType){
	 case _ITEM_INTEGER: case _ITEM_UINT:
	     mVCItemList[i].ItemConnect_Int(pBSCurrent, pCMInputPanel);
	     break;
	 case _ITEM_INTEGER_SPIN: case _ITEM_UINT_SPIN:
	     mVCItemList[i].ItemConnect_IntSpin(pBSCurrent, pCMInputPanel);
	     break;
	 case _ITEM_INTEGER_SLIDER:
	     mVCItemList[i].ItemConnect_IntSlider(pBSCurrent, pCMInputPanel);
	     ConnectScroll(pCMInputPanel, mVCItemList[i].mId);
	     break;
	 case _ITEM_FLOAT:
	     mVCItemList[i].ItemConnect_Float(pBSCurrent, pCMInputPanel);
	     break;
	 case _ITEM_FLOAT_SLIDER:
	     mVCItemList[i].ItemConnect_FloatSlider(pBSCurrent, pCMInputPanel);
	     ConnectScroll(pCMInputPanel, mVCItemList[i].mId);
	     break;
	 case _ITEM_DOUBLE:
	     mVCItemList[i].ItemConnect_Double(pBSCurrent, pCMInputPanel);
	     break;
	 case _ITEM_DOUBLE_SLIDER:
	     mVCItemList[i].ItemConnect_DoubleSlider(pBSCurrent, pCMInputPanel);
	     ConnectScroll(pCMInputPanel, mVCItemList[i].mId);
	     break;
	 case _ITEM_DUALSLIDER: {
	     double value0 = *(double*)(mVCItemList[i].mpValue);
	     double value1 = *((double*)mVCItemList[i].mpValue + 1);
	     mVCItemList[i].ItemConnect_DualSlider(pBSCurrent, pCMInputPanel);
	     ConnectScroll(pCMInputPanel, mVCItemList[i].mId);
	     break; }
	 case _ITEM_BOOL:
	     mVCItemList[i].ItemConnect_Bool(pBSCurrent, pCMInputPanel);
	     ConnectBool(pCMInputPanel, mVCItemList[i].mId);
	     break;
	 case _ITEM_UINT_LIST: 
	     mVCItemList[i].ItemConnect_List
		 (pBSCurrent, pCMInputPanel/*, /*mComboWidth*//*108-1*/);
	     ConnectComboBox(pCMInputPanel, mVCItemList[i].mId);
	     break; 
	 case _ITEM_UINT_ICON:
	     mVCItemList[i].ItemConnect_Icon(pBSCurrent, pCMInputPanel);
	     ConnectComboBox(pCMInputPanel, mVCItemList[i].mId);
	     break;
	 case _ITEM_STRING:
	     mVCItemList[i].ItemConnect_String(pBSCurrent, pCMInputPanel);
	     break;
	 case _ITEM_WXPANEL:
	     mVCItemList[i].ItemConnect_WXPanel(pBSCurrent);
	     ConnectClick(pCMInputPanel, mVCItemList[i].mId);
	     break;
	 case _ITEM_BUTTON:
	     mVCItemList[i].ItemConnect_Button(pBSCurrent, pCMInputPanel);
	     ConnectButton(pCMInputPanel, mVCItemList[i].mId); 
	     break;
	 case _ITEM_BUTTON_HIDE_PANEL:
	     mVCItemList[i].ItemConnect_Button(pBSCurrent, pCMInputPanel, TRUE);
	     pCMInputPanel->Connect
		 (mVCItemList[i].mId, wxEVT_COMMAND_BUTTON_CLICKED,
		  wxComEH(MultiInputPanel_cls::OnHide ));
	     pCMInputPanel->Connect
		 (mVCItemList[i].mId, wxEVT_RIGHT_DOWN,
		  wxMouseEH(MultiInputPanel_cls::OnMouseOnHide));
	     break;
	 }

	 mVCItemList[i].ApplyRange();
	
	 /*
	 if (mVCItemList[i].mbEnabled == FALSE)
	     mVCItemList[i].Disable();
	 */
	 
	 if (mVCItemList[i].mbConnectSizer == FALSE)
	     pBoxSizerSrc->Add(pBSCurrent, 
			       mVCItemList[i].GetSizerFlagV()/*.Align(wxALIGN_CENTER_VERTICAL)*/);
     }

     mpPadding = pBoxSizerSrc->Add(mMinimumWidth, -1, 0, 0);
}

void MultiInputBase_cls::
UpdateValues(int index){
    for (int i = 0; i < (int)mVCItemList.size(); i++){
	if (index != _MN_ALLITEM) i = index;
	//mVCItemList[i].UpdateLabel();
	mVCItemList[i].UpdateDisplay();
	if (index != _MN_ALLITEM) return;
    }
}
     
// --------------

BEGIN_EVENT_TABLE(MultiInputDialog_cls, wxDialog)
EVT_BUTTON (wxID_OK, MultiInputDialog_cls::OnOK)
//EVT_CHAR       (MultiInputPanel_cls::OnChar)
END_EVENT_TABLE()

MultiInputDialog_cls::
MultiInputDialog_cls(MultiInputDialog_cls **ppToThis,
		     wxWindow *pCParent, wxWindowID id, 
		     wxString strLabel,
		     wxPoint pos, wxSize size) :
    wxDialogEXKey(pCParent, id, strLabel, pos, size)
{
    mpCUserDefinedClue = NULL;
    SetBackgroundColour(wxColour(wxT("WHEAT")));
    SetWindowStyle(GetWindowStyle() | (wxTAB_TRAVERSAL
				       & ~wxMINIMIZE_BOX
				       & ~wxMAXIMIZE_BOX));
    //mTitle = title;
    mppToThis = ppToThis;

    mpCPanel = new MultiInputPanel_cls
	(&mpCPanel, this, wxID_ANY, wxDefaultPosition, wxDefaultSize);
    mpCPanel->mpCParentDLG = this;
    mpCHookCtrlLink = mpCPanel->mpCHookCtrl;
    
    
    //

    wxBoxSizer *pBoxSizerSrc = new wxBoxSizer(wxVERTICAL);
    
    pBoxSizerSrc->Add(mpCPanel, 0, wxEXPAND);

    wxBoxSizer *pBoxSizerBTN = new wxBoxSizer(wxHORIZONTAL);

    mpBTNOK     = new wxButtonEX(this, wxID_OK, wxT("OK"));
    mpBTNCancel = new wxButtonEX(this, wxID_CANCEL, wxT("Calcel"));

    pBoxSizerBTN->Add(mpBTNOK,     1, wxEXPAND);
    pBoxSizerBTN->Add(mpBTNCancel, 1, wxEXPAND);

    pBoxSizerSrc->Add(pBoxSizerBTN, 0, wxEXPAND);

    SetSizer(pBoxSizerSrc);
}

int MultiInputDialog_cls::
CombinePointer(int type, void *pValue, wxString strLabel, bool enable){
    return mpCPanel->CombinePointer(type, pValue, strLabel, enable);
}

int MultiInputDialog_cls::
CombinePointerList(int type, void *pValue, 
		   unsigned int listMax, wxString *pChoiceList, 
		   wxString strLabel, bool enable){
    return mpCPanel->CombinePointerList
	(type, pValue, listMax, pChoiceList, strLabel, enable);
}

void MultiInputDialog_cls::
SetRange(int index, double min, double max, int scale){
    mpCPanel->SetRange(index, min, max, scale);
}

void MultiInputDialog_cls::
ShowItem(int index){
    mpCPanel->ShowItem(index);
}

void MultiInputDialog_cls::
HideItem(int index){
    mpCPanel->HideItem(index);
}

void MultiInputDialog_cls::
OnOK(wxCommandEvent& event){
    mpCPanel->AssignBase(_MN_ASSIGN_ALL);
    event.Skip();
}

void MultiInputDialog_cls::
MakeGeometry(){
    mpCPanel->MakeGeometry();
    ResizeDialog();
}

void MultiInputDialog_cls::
UpdateValues(int index){
    mpCPanel->UpdateValues(index);
}

// --------------

BEGIN_EVENT_TABLE(MultiInputPanel_cls, wxPanel)
EVT_CHAR       (MultiInputPanel_cls::OnChar)
EVT_KEY_DOWN   (MultiInputPanel_cls::OnKeyDown)
END_EVENT_TABLE()

MultiInputPanel_cls::
MultiInputPanel_cls(MultiInputPanel_cls **ppToThis,
		    wxWindow *pCParent, wxWindowID id, 
		    wxPoint pos, wxSize size) :
    MultiInputBase_cls((void**)ppToThis, pCParent),
    wxPanel(pCParent, id, pos, size, wxBORDER_SIMPLE)
{
    mpCParentDLG = NULL;
    mpBTNMinimize = NULL;
    mpCheckBoxValid = NULL;
    SetBackgroundColour(wxColour(wxT("LIGHT BLUE")));
    //SetBackgroundColour(wxColour(wxT("WHEAT")));
    SetWindowStyle(GetWindowStyle() | wxTAB_TRAVERSAL);
}

void MultiInputPanel_cls::
SetPaddingWidth(int paddingWidth){
    mpPadding->SetInitSize(paddingWidth, -1);
    Fit();
}

int MultiInputPanel_cls::
GetPaddingWidth(int minimumWidth){
    int width = GetClientSize().GetWidth();
    if (width > minimumWidth) minimumWidth = width;
    return minimumWidth;
}

void MultiInputPanel_cls::
MakeGeometry(){
    if (mpBoxSizerSrc == NULL)
	mpBoxSizerSrc = new wxBoxSizer(wxVERTICAL);
    MakeGeometryBase(mpBoxSizerSrc, this);
    this->SetSizer(mpBoxSizerSrc);
}

void MultiInputPanel_cls::
ConnectButton(wxWindow *pWindow, int id){
    pWindow->Connect(id, wxEVT_COMMAND_BUTTON_CLICKED,
		     wxComEH(MultiInputPanel_cls::OnButton));
}

void MultiInputPanel_cls::
ConnectClick(wxWindow *pWindow, int id){
    pWindow->Connect(id, wxEVT_LEFT_DOWN,
		     wxMouseEH(MultiInputPanel_cls::OnMouseOnItem));
    pWindow->Connect(id, wxEVT_RIGHT_DOWN,
		     wxMouseEH(MultiInputPanel_cls::OnMouseOnItem));
}

void MultiInputPanel_cls::
ConnectBool(wxWindow *pWindow, int id){
    pWindow->Connect(id, wxEVT_COMMAND_CHECKBOX_CLICKED,
		     wxComEH(MultiInputPanel_cls::OnCheckBox));
}

void MultiInputPanel_cls::
ConnectComboBox(wxWindow *pWindow, int id){
    pWindow->Connect(id, wxEVT_COMMAND_COMBOBOX_SELECTED,
		     wxComEH(MultiInputPanel_cls::OnComboBox));
}

void MultiInputPanel_cls::
ConnectScroll(wxWindow *pWindow, int id){
    pWindow->Connect(id, wxEVT_SCROLL_THUMBTRACK,
		     wxScrollEventHandler(MultiInputPanel_cls::OnScroll));
    pWindow->Connect(id, wxEVT_SCROLL_THUMBRELEASE, 
		     wxScrollEventHandler(MultiInputPanel_cls::OnScrollChanged));
}
   
void MultiInputPanel_cls::
OnButton(wxCommandEvent& event){
    int id = event.GetId();
    for (unsigned int i = 0; i < mVCItemList.size(); i++)
	if (mVCItemList[i].mType == _ITEM_BUTTON && mVCItemList[i].mId   == id)
	    GetParent()->AddPendingEvent(event);
    
    mpCHookCtrl->CheckHook(_HOOK_BUTTON, &event, event.GetId());
}

void MultiInputPanel_cls::
OnChar(wxKeyEvent& event){
    if (event.GetKeyCode() == WXK_RETURN || event.GetKeyCode() == WXK_SPACE){
	AssignBaseById(event.GetId()); 
	//UpdateValues();
    }
}

void MultiInputPanel_cls::
OnKeyDown(wxKeyEvent& event){
    GShortcutModifier = event.GetModifiers();
    GShortcutKeycode  = event.GetKeyCode();

    wxCommandEvent com(wxEVT_COMMAND_MENU_SELECTED, menuEditShortcut);
    GpCMainFrameCLS->AddPendingEvent(com);
}

void MultiInputPanel_cls::
OnCheckBox(wxCommandEvent& event){
    OnCheckBoxBase(event);
}

void MultiInputPanel_cls::
OnComboBox(wxCommandEvent& event){
    OnComboBoxBase(event);
}

void MultiInputPanel_cls::
OnHide(wxCommandEvent& event){
    ShowPanel(FALSE);
    wxSizeEvent com;
    GetParent()->AddPendingEvent(com);
}

void MultiInputPanel_cls::
OnMouse(wxMouseEvent& event){
}

#include "../MYAPI/MsgBox.h"

void MultiInputPanel_cls::
OnMouseOnItem(wxMouseEvent& event){
    mpCHookCtrl->CheckHook(_HOOK_PANELCLICK, &event);
}

void MultiInputPanel_cls::
OnMouseOnHide(wxMouseEvent& event){
    if (mpBTNMinimize == NULL) return;
    event.SetId(mpBTNMinimize->GetId());
    mpBTNMinimize->AddPendingEvent(event);
}

void MultiInputPanel_cls::
OnScroll(wxScrollEvent& event){
    OnScrollBase(event);
}

void MultiInputPanel_cls::
OnScrollChanged(wxScrollEvent& event){
    OnScrollBaseChanged(event);
}

//--ŜB̏


void MultiInputPanel_cls:: 
AssignMinimizeButtonBoth(wxButtonEX* pBTNMinimize){
    mpBTNMinimize = pBTNMinimize;
    mIndexMinimize = CombinePointerButton(pBTNMinimize->GetId(), 
					  pBTNMinimize->GetLabel(), _ITEM_BUTTON_HIDE_PANEL);
}

void MultiInputPanel_cls::
ShowPanel(bool mode){
    bool bShownPrev = IsShownOnScreen();

    if (mpBTNMinimize != NULL) mpBTNMinimize->Show(!mode);
    if (mpCheckBoxValid != NULL){
	mpCheckBoxValid->GetValidator()->TransferToWindow();
	mpCheckBoxValid->Show(!mode);
    }
    Show(mode);
    
    if (bShownPrev == mode) return;
    for (unsigned int index = 0; index < mVCItemList.size(); index++)
	mpCHookCtrl->CheckHook(_HOOK_SHOW, &mVCItemList[index]);
}

//--

void MultiInputPanel_cls::
AddThisAndHideButtonToSizer(wxBoxSizer *pBoxSizer){
    if (pBoxSizer == NULL) return;
    wxSizerFlags sflag0(0), sflag0E(0), sflag1E(1);
    sflag0          .Border(wxALL, 2);
    sflag0E.Expand().Border(wxALL, 2);
    sflag1E.Expand();

    pBoxSizer->Add(this, sflag0);
    if (mpBTNMinimize != NULL)
	if (mpCheckBoxValid != NULL){
	    wxBoxSizer *pBoxSizerChild = new wxBoxSizer(wxHORIZONTAL);
	    pBoxSizerChild->Add(mpCheckBoxValid,sflag0.Align(wxALIGN_CENTER_VERTICAL));
	    pBoxSizerChild->Add(mpBTNMinimize,0, wxALIGN_CENTER_VERTICAL);
	    pBoxSizer->Add(pBoxSizerChild, 0, wxEXPAND);
	}
	else
	    pBoxSizer->Add(mpBTNMinimize, sflag0E);
}

int MultiInputPanel_cls::
FinishSetting(wxBoxSizer *pBoxSizer, bool vis){
    MakeGeometry();
    ShowPanel(vis);
    AddThisAndHideButtonToSizer(pBoxSizer);
    //UpdateValuesAll(TRUE);
    return 0;
}
