// -*- c++ -*-
// (setq c-basic-offset 4)
// "clsMain.h"

#include "clsMain.h"
#include "clsFrameMain.h"

#include "SRC/COMMON/clsFilename.h"
#include "SRC/GLATTRIB/clsColorCurveGUI.h"
#include "SRC/GLATTRIB/clsColorCurve.h"
#include "SRC/GLCANVAS/clsMainGLCanvas.h"
#include "SRC/GRAPHIC/clsImgSequence.h"
#include "SRC/MAINCANVAS/clsDrawOption.h"
#include "SRC/MAINCANVAS/clsExposure.h"
#include "SRC/MAINCANVAS/clsOverlay.h"
#include "SRC/MAINCANVAS/clsParamGUICtrl.h"
#include "SRC/MAINCANVAS/clsBtmHalfPanel.h"
#include "SRC/PANELS/clsDataLoader.h"
#include "SRC/PANELS/clsDataExporter.h"
#include "SRC/PANELS/clsScene.h"
#include "SRC/PARAMETER/clsParamTL.h"
#include "SRC/PARTICLEDATA/clsGLDrawData.h"
#include "SRC/PARTICLEDATA/clsParticleDataSeqCtrl.h"
#include "SRC/PARTICLEDATA/clsParticleDataSequence.h"
#include "SRC/PARTICLEDATA/clsParticleDataBlock.h"
#include "SRC/PARTICLEDATA/clsSnapData.h"
#include "SRC/PARTICLEDATA/clsThreadFileReader.h"
extern ThreadFileReader_cls   *GpCThreadReader;
extern bool                    GbThreadReaderFinished;
#include "SRC/PARTICLEDATA/clsThreadInterpolate.h"
extern ThreadInterpolator_cls *GpCThreadInterpolate;
extern bool                    GbThreadInterpolateFinished;

#include "SRC/SCENEOBJ/clsCameraPath.h"
#include "SRC/SCENEOBJ/clsCameraPathGUI.h"
#include "SRC/SCENEOBJ/clsLightSource.h"
#include "SRC/SCENEOBJ/clsLightSourceGUI.h"
#include "SRC/TIMELINE/clsTimeCtrl.h"
#include "SRC/TIMELINE/clsTLPanelBase.h"

#include "SRC/UNDO/clsUndoManager.h"
extern UndoManager_cls GCUndoList;
#include "SRC/UNDO/clsUndoItemDataIO.h"

extern bool GbGLLockProjectLoading;

#include "wxWidgetEnum.h"
#include "wxWidgetEnumMenu.h"
#include "wxWidgetString.h"

#include "SRC/MYAPI/MsgBox.h"
#include <wx/tooltip.h>
#include <wx/file.h>
#include <wx/gdicmn.h>
#include <wx/progdlg.h>
#include <wx/stdpaths.h>

class wxFileDialogEX;

//O[oϐ
wxFileDialogEX*   GpCFileDialog = NULL;

#define LOOP_XYZ for(int xyz = 0; xyz < 3; xyz++)

#ifdef WIN32
#define _WXDELIM_ wxT("\\")
#else
#define _WXDELIM_ wxT("/")
#endif

#ifdef WIN32
#include <process.h>
#else
#include <sys/types.h>
#include <unistd.h>
#endif

Main_cls::
Main_cls() 
{
    SplitPath_cls CPath(wxStandardPathsBase::Get().GetExecutablePath());
    mstrExecuteDir = CPath.GetPathStr(_FN_DRIVE_DIR);
	
#ifdef WIN32
    mPID = _getpid();
#else
    mPID = getpid();
#endif

#ifdef __WAMAC__
    int posx = 10, posy = 10;
#else
    int posx = 10, posy = 10;
#endif
    int width = 1200, height = 900;
    wxRect rect = wxGetClientDisplayRect();
    int clientDisplayRightPos = rect.x + rect.width;
    int clientDisplayBottomPos = rect.y + rect.height;
    if (posx + width > clientDisplayRightPos) { 
	posx = rect.x; width = rect.width; }
    if (posy + height > clientDisplayBottomPos) {
	posy = rect.y; height = rect.height; }

    mpCDrawOption   = new DrawOption_cls;
    mpCExposure     = new Exposure_cls;
    mpCOverlay      = new Overlay_cls;
    mpCPDataSeqCtrl = new PDataSeqCtrl_cls(mpCDrawOption);
    mpCDataLoader   = new DataLoader_cls(this);
    mpCDataExporter = new DataExporter_cls(this);
    mpCCameraPath   = new CameraPath_cls;
    mpCAnchorPath   = new AnchorPath_cls;

    mpCTime  = new TimeCtrl_cls(this);

    mpCFrame = new FrameMain_cls
	(_FRAME_MAIN_APPNAME, this, posx, posy, width, height);

    mpCCameraPathCtrl = new CameraPathCtrl_cls
	(mpCFrame, NULL/*STEREO*/, mpCCameraPath, mpCAnchorPath);
    mpCLightSRCCtrl   = new LightSRCCtrl_cls (mpCFrame);
    mpCColorCurveCtrl = new ColorCurveCtrl_cls (mpCFrame);
    
    mpCImgSeq = new ImgSequence_cls(mpCFrame);

    // p[^[plݒ
    mpCParamGUICtrlLink = mpCFrame->mpCBtmHalfPanel->mpCParamGUICtrl;

    mpCParamGroupCameraPath =
	new ParamGroup_cls(wxT("Camera Path"), 0, NULL, 0);
    mpCParamGroupCameraPath->
	mpCSceneObjLink = mpCCameraPath;

    mpCParamGroupCameraPath->mVpCParamGroup.push_back(mpCCameraPath->mpCTranslate);
    mpCParamGroupCameraPath->mVpCParamGroup.push_back(mpCCameraPath->mpCRotation);
    mpCParamGroupCameraPath->mVpCParamGroup.push_back(mpCAnchorPath->mpCTranslate);
    mpCParamGroupCameraPath->mVpCParamGroup[2]->
	mpCSceneObjLink = mpCAnchorPath;

    mpCParamGUICtrlLink->AddParamGroup(mpCParamGroupCameraPath);
    mpCParamGroupCameraPath->HideItems();

    for (int i = 0; i < _LIGHT_NUM_MAX; i++){
	mpCParamGUICtrlLink->AddParamGroup(mpCLightSRCCtrl->mpCParamGroup[i]);
	mpCLightSRCCtrl->mpCParamGroup[i]->HideItems();
    }

    mpCParamGUICtrlLink->ResizeTL();

    mpCTime->mpCParamGUICtrlLink = mpCParamGUICtrlLink;
    
    mpCScene = new Scene_cls(mpCFrame, mpCTime, mpCFrame->mpBitmapScene);
    
    // ÓIȃNX̃N
    Parameters_cls::mspCTimeCtrlLink      = mpCTime;
    Parameters_cls::mspCPDataSeqCtrlLink  = mpCPDataSeqCtrl;
    ParamTL_cls::mspCTimeCtrlLink         = mpCTime;
    PDataBlock_cls::mspCDataLoaderLink    = mpCDataLoader;
    ColorCurveBtn_cls::mspCColorCurveCtrlLinked = mpCColorCurveCtrl;
    
    ReadConfigure(mstrExecuteDir + _WXDELIM_ + wxT("Zindaiji3Configure.txt"));

    wxCommandEvent com(wxEVT_COMMAND_MENU_SELECTED, menuEditRefreshTL);
    mpCFrame->AddPendingEvent(com);
    wxCommandEvent com2(wxEVT_COMMAND_MENU_SELECTED, menuEditRefreshGL);
    mpCFrame->AddPendingEvent(com2);

    mpCFrame->FitSplitterWindow(TRUE);
}

Main_cls::
~Main_cls(){
    delete mpCDrawOption;
    delete mpCOverlay;
    delete mpCPDataSeqCtrl;
    delete mpCDataLoader;
    delete mpCExposure;
    delete mpCDataExporter;
    delete mpCCameraPath;
    delete mpCColorCurveCtrl;
    delete mpCAnchorPath;
    delete mpCTime;
    delete mpCLightSRCCtrl;
    delete mpCCameraPathCtrl;
    delete mpCImgSeq;
    delete mpCScene;
    delete mpCParamGroupCameraPath;
}

wxWindow* Main_cls::
GetPTopLevelWindow(){
    return mpCFrame;
}

int  Main_cls::
GetUsedTotalMemoryMB(){
    return mpCPDataSeqCtrl->mTotalUsedMemoryMB;
}

int  Main_cls::
GetAverageMemoryMB(){
    int totalMB = GetUsedTotalMemoryMB();
    int totalFrame = mpCPDataSeqCtrl->GetLoadedFrameNumber();
    if (totalFrame == 0) return -1;
    return totalMB / totalFrame;
}

int  Main_cls::
GetMemoryLimitMB(){
    return mpCPDataSeqCtrl->mMemoryLimitationMB;
}

void Main_cls::
SelectObject(int objectIndex, int guideIndex){
    MainGLCanvas_cls::mspCSelectedObject = 
	mpCParamGUICtrlLink->SelectObject(objectIndex, guideIndex);
    //return MainGLCanvas_cls::mspCSelectedObject;
}

void Main_cls::
ThisObjectIsSelected(int objectIndex){
    if (objectIndex >= _SCENEOBJ_IDBASE_LIGHT &&
	objectIndex < _SCENEOBJ_IDBASE_LIGHT + 7)
	mpCLightSRCCtrl->SelectLight(objectIndex - _SCENEOBJ_IDBASE_LIGHT);
}

bool Main_cls::
IsTheIndexReady(int index, double indexmod){
    return mpCPDataSeqCtrl->IsTheIndexReady(index, indexmod);
}

bool Main_cls::
IsTheIndexLoaded(int index){
    return mpCPDataSeqCtrl->IsTheIndexLoaded(index);
}

void Main_cls::
WaitThreadFinish(){
    // FIX_ME!! KXbhI܂őҋ@ɂ́H
    if (GpCThreadReader != NULL){
	while (GbThreadReaderFinished == FALSE)
	    wxMilliSleep(10);
	GpCThreadReader = NULL;
    }

    if (GpCThreadInterpolate != NULL){
	while (GbThreadInterpolateFinished == FALSE)
	    wxMilliSleep(10);
	GpCThreadInterpolate = NULL;
    }
}

void Main_cls::
OpenDataFileCommand(wxString strFilePath){
    GCUndoList.OpenBlock();
    GCUndoList.SetNewCommand(new CommandLoadData_cls(this, wxT("")),
			     new CommandLoadData_cls(this, mpCPDataSeqCtrl->GetStrActiveFileSequencePath()));
    GCUndoList.ExecCurrent();
    GCUndoList.SetNewCommand(new CommandLoadData_cls(this, strFilePath),
			     new CommandLoadData_cls(this, wxT("")));
    GCUndoList.ExecCurrent();
    GCUndoList.CloseBlock();
}

void Main_cls::
OpenDataFile(wxString strFilePath){
    mpCPDataSeqCtrl->OpenDataFile(strFilePath);
    int maxFrame = mpCPDataSeqCtrl->GetActiveFileSequenceLength()-1;
    if (maxFrame < 0) maxFrame = 0;
    if (maxFrame >= _MAX_PDATA_INDEX){
	MsgBox(mpCFrame, _("Max file sequence number is 8000"), _("Warning"));
	mpCPDataSeqCtrl->GetPCActiveFileSequence()->
	    mCFileSequence.mSeqLength = 8000;
	maxFrame = 7999;
    }
    
    mpCTime->SetMaxFrameNative(maxFrame);
    wxCommandEvent com(wxEVT_COMMAND_MENU_SELECTED, menuEditRefreshGL);
    mpCFrame->AddPendingEvent(com);
}

void Main_cls::
CloseDataCommand(){
    GCUndoList.SetNewCommand(new CommandLoadData_cls(this, wxT("")),
			     new CommandLoadData_cls(this, mpCPDataSeqCtrl->GetStrActiveFileSequencePath()));
    GCUndoList.ExecCurrent();
}

void Main_cls::
CloseData(){
    WaitThreadFinish();
    mpCPDataSeqCtrl->ClearData(_ACTIVE_SEQUENCE,_SEQUENCE_ALL_INDEX, TRUE);
    if (GLDrawData_cls::mspCSnapShot != NULL){
	delete GLDrawData_cls::mspCSnapShot;
	GLDrawData_cls::mspCSnapShot = NULL;
    }
}

void Main_cls::
Undo(){
    GCUndoList.UndoCurrent();
}

void Main_cls::
Redo(){
    GCUndoList.ExecCurrent();
}

void Main_cls::
ReadConfigure(wxString strFileIn){

    std::wifstream ifstream(strFileIn.char_str(), std::ios::in);
    
#ifdef WIN32
    ifstream.imbue(std::locale("japanese"));
#endif

    if (ifstream.fail()){
	printf("Read Configure Fails"); 
	return;
    }

    GbGLLockProjectLoading = TRUE;
    std::wstring wstrLine;
    while ( std::getline (ifstream, wstrLine)){
	wxString strLine = wstrLine.c_str();
	mpCDataLoader->ReadStrLine(strLine);
	mpCDataExporter->ReadStrLine(strLine);
	mpCFrame->ReadStrLine(strLine);
	mpCTime->ReadStrLine(strLine);
	mpCDrawOption->ReadStrLine(strLine);
	mpCOverlay->ReadStrLine(strLine);
	mpCPDataSeqCtrl->ReadStrLine(ifstream, strLine);
	mpCCameraPath->ReadStrLine(ifstream, strLine);
	mpCAnchorPath->ReadStrLine(ifstream, strLine);
	mpCLightSRCCtrl->ReadStrLine(ifstream, strLine);
    }

    printf("%ls\n", mpCDataLoader->mCLastOpenedFile.mstrBody.c_str());
    mpCLightSRCCtrl->UpdateTLShown();

    GCUndoList.Clear();
    GbGLLockProjectLoading = FALSE;
}

bool Main_cls::
SaveConfigure(wxString strFileOut, bool flag){
    wxFile CFile(strFileOut, wxFile::write);
    if (CFile.IsOpened() == FALSE) {
	return FALSE;
    }

    if (! strFileOut.Matches(wxT("*Zindaiji3Configure.txt")))
	mpCFrame->mCLastOpenedFileZJP.mstrBody = strFileOut;
    
    wxString sOut;
    
    sOut << mpCDataLoader->Serialize();
    sOut << mpCDataExporter->Serialize();
    sOut << mpCFrame->SerializeForConfigure();

    if (flag == _FM_PROJECT_CONF){
	sOut << mpCFrame->Serialize();
	sOut << mpCTime->Serialize();
	sOut << mpCDrawOption->Serialize();
	sOut << mpCOverlay->Serialize();
	sOut << mpCPDataSeqCtrl->Serialize();
	sOut << mpCCameraPath->Serialize();
	sOut << mpCAnchorPath->Serialize();
	sOut << mpCLightSRCCtrl->Serialize();
    }

    CFile.Write(sOut, wxConvFile);

    return TRUE;
}

void Main_cls::
QuitProcess(){
    SaveConfigure(mstrExecuteDir + _WXDELIM_ + wxT("Zindaiji3Configure.txt"), 
		  _FM_DEFAULT_CONF);
}
